#define BRUTIL
/*
 MD5 related functions:
 AUTHOR:        Bob Trower 03/11/05
 PROJECT:       Crypt Data Packaging
 COPYRIGHT:     Copyright (c) Trantor Standard Systems Inc., 2001, 2005
 NOTES:         This is free software. This source code may be used as 
                you wish, subject to the LGPL license.  See the
                LICENCE section below.
                http://toogles.sourceforge.net
                Message-Digest Algorithm described in RFC1321
                (http://www.faqs.org/rfcs/rfc1321.html).
 7.Aug.2008
 Adapted by Dusan Peterc, Arahne, d.o.o. for use in Arahne browsing libraries
 1.May.2011
 fixed for 64 bits LP64 "unsigned long" > "unsigned int"*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <libgen.h>
#include <math.h>
#include "../llib/loadimg.h"
#include "acolor.h"
#include "browseio.h"

#define floatEqual(f1, f2, d)	(fabs((f1)-(f2)) <= fabs(d) )

typedef struct {
 unsigned int hash[4];
 unsigned int bits[2];
 unsigned char data[64];
} MD5Context;

/* Basic MD5 functions */
#define F1(x, y, z) (z ^ (x & (y ^ z)))
#define F2(x, y, z) (y ^ (z & (x ^ y)))
#define F3(x, y, z) (x ^ y ^ z)
#define F4(x, y, z) (y ^ (x | ~z))

/* This is the central step in the MD5 algorithm. */
#define TRANSFORM(f, w, x, y, z, data, s) \
        ( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )

/* md5_transform
** The MD5 "basic transformation". Updates the hash
** based on the data block passed.*/
static void md5_transform(unsigned int hash[4], const unsigned int data[16])
{
 unsigned int a = hash[0], b = hash[1], c = hash[2], d = hash[3];

  /* Round 1 */
 TRANSFORM( F1, a, b, c, d, data[ 0] + 0xd76aa478,  7);
 TRANSFORM( F1, d, a, b, c, data[ 1] + 0xe8c7b756, 12);
 TRANSFORM( F1, c, d, a, b, data[ 2] + 0x242070db, 17);
 TRANSFORM( F1, b, c, d, a, data[ 3] + 0xc1bdceee, 22);
 TRANSFORM( F1, a, b, c, d, data[ 4] + 0xf57c0faf,  7);
 TRANSFORM( F1, d, a, b, c, data[ 5] + 0x4787c62a, 12);
 TRANSFORM( F1, c, d, a, b, data[ 6] + 0xa8304613, 17);
 TRANSFORM( F1, b, c, d, a, data[ 7] + 0xfd469501, 22);
 TRANSFORM( F1, a, b, c, d, data[ 8] + 0x698098d8,  7);
 TRANSFORM( F1, d, a, b, c, data[ 9] + 0x8b44f7af, 12);
 TRANSFORM( F1, c, d, a, b, data[10] + 0xffff5bb1, 17);
 TRANSFORM( F1, b, c, d, a, data[11] + 0x895cd7be, 22);
 TRANSFORM( F1, a, b, c, d, data[12] + 0x6b901122,  7);
 TRANSFORM( F1, d, a, b, c, data[13] + 0xfd987193, 12);
 TRANSFORM( F1, c, d, a, b, data[14] + 0xa679438e, 17);
 TRANSFORM( F1, b, c, d, a, data[15] + 0x49b40821, 22);

  /* Round 2 */
 TRANSFORM( F2, a, b, c, d, data[ 1] + 0xf61e2562,  5);
 TRANSFORM( F2, d, a, b, c, data[ 6] + 0xc040b340,  9);
 TRANSFORM( F2, c, d, a, b, data[11] + 0x265e5a51, 14);
 TRANSFORM( F2, b, c, d, a, data[ 0] + 0xe9b6c7aa, 20);
 TRANSFORM( F2, a, b, c, d, data[ 5] + 0xd62f105d,  5);
 TRANSFORM( F2, d, a, b, c, data[10] + 0x02441453,  9);
 TRANSFORM( F2, c, d, a, b, data[15] + 0xd8a1e681, 14);
 TRANSFORM( F2, b, c, d, a, data[ 4] + 0xe7d3fbc8, 20);
 TRANSFORM( F2, a, b, c, d, data[ 9] + 0x21e1cde6,  5);
 TRANSFORM( F2, d, a, b, c, data[14] + 0xc33707d6,  9);
 TRANSFORM( F2, c, d, a, b, data[ 3] + 0xf4d50d87, 14);
 TRANSFORM( F2, b, c, d, a, data[ 8] + 0x455a14ed, 20);
 TRANSFORM( F2, a, b, c, d, data[13] + 0xa9e3e905,  5);
 TRANSFORM( F2, d, a, b, c, data[ 2] + 0xfcefa3f8,  9);
 TRANSFORM( F2, c, d, a, b, data[ 7] + 0x676f02d9, 14);
 TRANSFORM( F2, b, c, d, a, data[12] + 0x8d2a4c8a, 20);

  /* Round 3 */
 TRANSFORM( F3, a, b, c, d, data[ 5] + 0xfffa3942,  4);
 TRANSFORM( F3, d, a, b, c, data[ 8] + 0x8771f681, 11);
 TRANSFORM( F3, c, d, a, b, data[11] + 0x6d9d6122, 16);
 TRANSFORM( F3, b, c, d, a, data[14] + 0xfde5380c, 23);
 TRANSFORM( F3, a, b, c, d, data[ 1] + 0xa4beea44,  4);
 TRANSFORM( F3, d, a, b, c, data[ 4] + 0x4bdecfa9, 11);
 TRANSFORM( F3, c, d, a, b, data[ 7] + 0xf6bb4b60, 16);
 TRANSFORM( F3, b, c, d, a, data[10] + 0xbebfbc70, 23);
 TRANSFORM( F3, a, b, c, d, data[13] + 0x289b7ec6,  4);
 TRANSFORM( F3, d, a, b, c, data[ 0] + 0xeaa127fa, 11);
 TRANSFORM( F3, c, d, a, b, data[ 3] + 0xd4ef3085, 16);
 TRANSFORM( F3, b, c, d, a, data[ 6] + 0x04881d05, 23);
 TRANSFORM( F3, a, b, c, d, data[ 9] + 0xd9d4d039,  4);
 TRANSFORM( F3, d, a, b, c, data[12] + 0xe6db99e5, 11);
 TRANSFORM( F3, c, d, a, b, data[15] + 0x1fa27cf8, 16);
 TRANSFORM( F3, b, c, d, a, data[ 2] + 0xc4ac5665, 23);

  /* Round 4 */
 TRANSFORM( F4, a, b, c, d, data[ 0] + 0xf4292244,  6);
 TRANSFORM( F4, d, a, b, c, data[ 7] + 0x432aff97, 10);
 TRANSFORM( F4, c, d, a, b, data[14] + 0xab9423a7, 15);
 TRANSFORM( F4, b, c, d, a, data[ 5] + 0xfc93a039, 21);
 TRANSFORM( F4, a, b, c, d, data[12] + 0x655b59c3,  6);
 TRANSFORM( F4, d, a, b, c, data[ 3] + 0x8f0ccc92, 10);
 TRANSFORM( F4, c, d, a, b, data[10] + 0xffeff47d, 15);
 TRANSFORM( F4, b, c, d, a, data[ 1] + 0x85845dd1, 21);
 TRANSFORM( F4, a, b, c, d, data[ 8] + 0x6fa87e4f,  6);
 TRANSFORM( F4, d, a, b, c, data[15] + 0xfe2ce6e0, 10);
 TRANSFORM( F4, c, d, a, b, data[ 6] + 0xa3014314, 15);
 TRANSFORM( F4, b, c, d, a, data[13] + 0x4e0811a1, 21);
 TRANSFORM( F4, a, b, c, d, data[ 4] + 0xf7537e82,  6);
 TRANSFORM( F4, d, a, b, c, data[11] + 0xbd3af235, 10);
 TRANSFORM( F4, c, d, a, b, data[ 2] + 0x2ad7d2bb, 15);
 TRANSFORM( F4, b, c, d, a, data[ 9] + 0xeb86d391, 21);

 hash[0] += a;
 hash[1] += b;
 hash[2] += c;
 hash[3] += d;
}

/* md5_init
** Initialise md5 context structure*/
static void md5_init(MD5Context *ctx)
{
 ctx->hash[0] = 0x67452301;
 ctx->hash[1] = 0xefcdab89;
 ctx->hash[2] = 0x98badcfe;
 ctx->hash[3] = 0x10325476;

 ctx->bits[0] = 0;
 ctx->bits[1] = 0;
}

/* md5_update
** Update context with the next buffer from the stream of data.
** Call with each block of data to update the md5 hash.*/
static void md5_update(MD5Context *ctx, const unsigned char *buf, unsigned int buflen)
{
 unsigned int idx;

 /* Update bitcount */
 idx = ctx->bits[0];
 ctx->bits[0] = idx + (buflen << 3);
 if ( ctx->bits[0] < idx )
     ctx->bits[1]++;         /* Carry from low to high */
 ctx->bits[1] += buflen >> 29;

 idx = (idx >> 3) & 0x3f;    /* Bytes already in ctx->data */

 /* Handle any leading odd-sized chunks */
 if( idx != 0 )
  {
   unsigned char *p = (unsigned char *) ctx->data + idx;
   idx = 64 - idx;
   if (buflen < idx)
    memcpy( p, buf, (size_t) buflen );
   else
    {
     memcpy( p, buf, (size_t) idx );
     md5_transform( ctx->hash, (unsigned int *) ctx->data );
     buf += idx;
     buflen -= idx;
    }
  }
 if( buflen >= idx )
  {
   while (buflen >= 64)
    {
     memcpy(ctx->data, buf, 64);
     md5_transform(ctx->hash, (unsigned int *) ctx->data);
     buf += 64;
     buflen -= 64;
    }
   memcpy(ctx->data, buf, (size_t) buflen);
  }
}

/* md5_final
** Finalize creation of md5 hash and copy to digest buffer.*/
static void md5_final(MD5Context *ctx, unsigned char digest[16])
{
 unsigned int count;
 unsigned char *pad;

 count = (ctx->bits[0] >> 3) & 0x3F; /* Number of bytes mod 64 */
 pad = ctx->data + count;
 *pad++ = 0x80;

  /* Bytes of padding needed to make 64 bytes */
 count = 64 - 1 - count;

  /* Pad out to 56 mod 64 */
 if(count < 8)
  {
   /* Two lots of padding:  Pad the first block to 64 bytes */
   memset( pad, 0, (size_t) count );
   md5_transform( ctx->hash, (unsigned int *) ctx->data );

    /* Now fill the next block with 56 bytes */
   memset(ctx->data, 0, 56);
  }
 else /* Pad block to 56 bytes */
  memset(pad, 0, (size_t) (count - 8));

  /* Append length in bits and transform */
 ((unsigned int *) ctx->data)[14] = ctx->bits[0];
 ((unsigned int *) ctx->data)[15] = ctx->bits[1];
 md5_transform(ctx->hash, (unsigned int *) ctx->data);
 memcpy(digest, ctx->hash, 16);
}

/* Compute the md5 hash on a buffer and return md5 as 32 bytes long ascii encoded hex string */
DBRUTIL char *md5string(const unsigned char *buf, int buflen)
{
 MD5Context ctx;
 unsigned char digest[16];
 static char digest_string[33];
 int i;

 md5_init(&ctx);
 md5_update(&ctx, buf, buflen);
 md5_final(&ctx, digest);

 digest_string[32] = 0;
 for (i = 0; i < 16; i++)
  sprintf(&(digest_string[i*2]), "%2.2x", digest[i]);
 return(digest_string);
}

/* insert the string on the top of the list, do not allow duplicates, take care of maximum entries, free the extra string if we have too many */
static int insertTopList(char *entry, char **list, int entries, int maxEntries)
{
 int i;
 int duplicatePos = -1;

 if (maxEntries<1)
  return(0);
 for (i=0; i<entries; i++)
  if (!strcmp(entry, list[i]))
   {
    duplicatePos = i;
    i=entries; // bailout
   }
 if (duplicatePos==0)
  return(entries); // keep it intact
 if (duplicatePos<0) // insert the new one on top
  {
   entries++;
   if (entries>maxEntries)
    {
     entries--;
     free(list[entries-1]);
    }
   for (i=entries-1; i>=1; i--)
    list[i]=list[i-1];
   list[0]=strdup(entry);
  }
 if (duplicatePos>0) // shift other entries down and put the new one on top
  {
   char *duplicateEntry = list[duplicatePos];

   for (i=duplicatePos; i; i--)
    list[i] = list[i-1];
   list[0]= duplicateEntry;
  }
 return(entries);
}

static int getNumEntriesList(char **list, int maxEntries)
{
 int i;
 
 for (i=0; i<maxEntries; i++)
  if (!list[i])
   return(i);
 return(maxEntries);
}

/* save the information about recently read or written files. */
/* we pass the history file, since the same functions are used for files of types */
/* pass maxHistory to prevent reading the list files over the allocated space */
DBRUTIL void saveRecentFile(char *fileName, int onlyRead, char *historyFile, int maxHistory)
{
 char *readFileList[maxHistory], *writeFileList[maxHistory], *dirList[maxHistory];
 int numRead, numWrite, numDir;
 int i;
 FILE *fp;
 char *tmp, *dirName;

 if (!maxHistory)
  return;
 /* in this way, fileName will not be modified by dirname function, and we will not free the result of dirname */
 /* crappy basename/dirname functions may or may not modify parameters and returns pointers which may be modified later and may not be freed */
 tmp = strdup(fileName);
 dirName = strdup(dirname(tmp));
 free(tmp);
 memset(readFileList, 0, sizeof(char *)*maxHistory);
 memset(writeFileList, 0, sizeof(char *)*maxHistory);
 memset(dirList, 0, sizeof(char *)*maxHistory);
 /* read current history file, put the new one on top, eliminate duplicates, and save it */
 /* since multiple programs could be reading/writing it at the same time */

//printf("save %d recent file: %s into %s maxHistory %d\n", onlyRead, fileName, historyFile, maxHistory);
 if (getRecentFiles(readFileList, writeFileList, dirList, historyFile, maxHistory))
  {
   numRead = getNumEntriesList(readFileList, maxHistory);
   numWrite = getNumEntriesList(writeFileList, maxHistory);
   numDir = getNumEntriesList(dirList, maxHistory);

   if (onlyRead)
    numRead = insertTopList(fileName, readFileList, numRead, maxHistory);
   else
    numWrite = insertTopList(fileName, writeFileList, numWrite, maxHistory);
   numDir = insertTopList(dirName, dirList, numDir, maxHistory);
   free(dirName);
  }
 else /* setup initial fake data... */
  {
   numDir = 1;
   if (onlyRead)
    {
     numWrite = 0;
     numRead = 1;
     readFileList[0] = strdup(fileName);
    }
   else
    {
     numWrite = 1;
     numRead = 0;
     writeFileList[0] = strdup(fileName);
    }
   dirList[0] = dirName;
  }
 
 if ((fp = fopen(historyFile, "w" )) == NULL)
  return;
 fprintf(fp, "%d %d %d\n", numWrite, numRead, numDir);
 for (i=0; i<numWrite; i++)
  fprintf(fp, "%s\n", writeFileList[i]);
 for (i=0; i<numRead; i++)
  fprintf(fp, "%s\n", readFileList[i]);
 for (i=0; i<numDir; i++)
  fprintf(fp, "%s\n", dirList[i]);
 fclose(fp);

 /* now free all the temporary allocated strings */
 for (i=0; i<numWrite; i++)
  if (writeFileList[i])
   free(writeFileList[i]);
 for (i=0; i<numRead; i++)
  if (readFileList[i])
   free(readFileList[i]);
 for (i=0; i<numDir; i++)
  if (dirList[i])
   free(dirList[i]);
}

#define MAX_LINE	1000

/* get the current recently used file history */
/* separate sections for files which have been written, read, and the directories */
/* list of read and written files are used from the menu of recently used files, while list of directories is used for browsers */
/* eliminate the files which can not be accessed at the time of reading */
/* list of arrays to pointers for readFileList and saveFileList must be pre-allocated and empty */
/* pass maxHistory to prevent reading the list files over the allocated space */
/* getRecentFiles already eliminates files, which can not be accessed */
/* caller is responsible for freeing the allocated strings, after they are no longer needed */
/* you can query individually just readFileList, writeFileList of dirList, by passing NULL ot some of the parameters */
/* For example, if you are only interested in directories, pass NULL into readFileList and writeFileList parameters */
DBRUTIL int getRecentFiles(char **readFileList, char **writeFileList, char **dirList, char *historyFile, int maxHistory)
{
 FILE *fp;
 char line[MAX_LINE];
 int numW, numR, numD;
 int i, j, k;

 if (!maxHistory)
  return(FALSE);
 if ((fp = fopen(historyFile, "r" )) == NULL)
  return(FALSE);
 if (fgets(line, MAX_LINE, fp)==NULL)
  { fclose(fp);return(FALSE); }
 if (sscanf(line, "%2d %2d %2d\n", &numW, &numR, &numD)!=3)
  { fclose(fp);return(FALSE); }

 if ((numW<0)|| (numR<0)||(numD<0)||(numW>MAX_FILE_HISTORY)|| (numR>MAX_FILE_HISTORY)||(numD>MAX_FILE_HISTORY))
  { fclose(fp);return(FALSE); }
 j=0;
 for (i=0; i<numW; i++)
  {
   if (fgets(line, MAX_LINE, fp)==NULL)
    { fclose(fp);return(FALSE); }
   if (strlen(line))
    line[strlen(line)-1]=0; /* cutoff CR */
   if ((j<maxHistory) && writeFileList)
    if (access(line, F_OK)==0)
     writeFileList[j++]=strdup(line);
  }
 if (numW>maxHistory)
  numW=maxHistory;
 j=0;
 for (i=0; i<numR; i++)
  {
   int notWritten = TRUE;

   if (fgets(line, MAX_LINE, fp)==NULL)
    { fclose(fp);return(FALSE); }
   if (strlen(line))
    line[strlen(line)-1]=0; /* cutoff CR */
   /* check that read file is different from the written file, since we do not want it duplicated */
   if (writeFileList)
    for (k=0; k<numW; k++)
     if (writeFileList[k]) /* we could have less files in read list, since some files could be deleted */
      if (!strcmp(line, writeFileList[k]))
       notWritten= FALSE;
   if ( (j<maxHistory) && notWritten && readFileList)
     if (access(line, F_OK)==0)
      readFileList[j++]=strdup(line);
  }
 j=0;
 for (i=0; i<numD; i++)
  {
   if (fgets(line, MAX_LINE, fp)==NULL)
    { fclose(fp);return(FALSE); }
   if (strlen(line))
    line[strlen(line)-1]=0; /* cutoff CR */
   if ((j<maxHistory) && dirList)
    if (access(line, F_OK)==0) /* XXX here we should check that it is acutally a directory, not just that the file exists... */
     dirList[j++]=strdup(line);
  }
 fclose(fp);
 
 return(TRUE);
}

/* clear the history of recently saved or loaded files */
DBRUTIL void clearRecentFiles(char *historyFile)
{
 remove(historyFile);
}

/* in-memory image icon reduction by integer divisor factors, copying of reduced icon to new location in selected RGB packing mode */
/* the source icon is damaged due to in-memory reduction, so icon must be created just for this purpose and discarded afterwards */
/* source icon must always be in 24 bpp r,g,b mode */
/* parameters wid, hei are the final reduced icon size, so the input icon size must be at least wid*divideFactor and hei*divideFactor */
/* function does not perform any range checking or memory allocations - all sizes must be big enough */
/* paddedWid is only used for copying the icon into destination, where it is made wider by that area and filled with backgound color */
/* in this way icons of various width can be made to fill the same width and line up vertically in the menu */
DBRUTIL void reduceIconByFactor(int divideFactor, unsigned char *iconData, int iconBPL, int wid, int hei, int paddedWid,
 unsigned char *backgroundRGB, unsigned char *resultData, int resultBPL, int resultBitsPerPixel)
{
 int x, y, r, g, b, i, j;
 unsigned char *srcData, *srcPix;
 unsigned int *dst32, backCol32 = ((int)backgroundRGB[0]<<16) + ((int)backgroundRGB[1]<<8) + backgroundRGB[2];
 unsigned short *dst16, backCol16 = (unsigned short) ((((int)backgroundRGB[0] & 248) << 8) + (((int)backgroundRGB[1] & 252) << 3) + (backgroundRGB[2] >> 3));
 int divideFactor2 = divideFactor*divideFactor;
 int paddingOffset = (paddedWid-wid)/2;

 for (y=0; y<hei; y++)
  {
   srcData  = &iconData[(iconBPL*y + paddingOffset*3)*divideFactor];

   /* it is not so optimized, but icons are already small, so it is not so much work to calculate */
   /* and we have very generic code */
   if (divideFactor>1)
    {
     unsigned char *dstData  = &iconData[iconBPL*y];

     for (x=0; x<wid; x++)
      {
       r=g=b=0;
       srcPix = &srcData[x*3*divideFactor];
       for (i=divideFactor; i; i--)
	{
	 for (j=divideFactor; j; j--)
	  {
	   r += *srcPix++;
	   g += *srcPix++;
	   b += *srcPix++;
	  }
	 srcPix += iconBPL - 3*divideFactor;
	}
       *dstData++ = (unsigned char) (r/divideFactor2);
       *dstData++ = (unsigned char) (g/divideFactor2);
       *dstData++ = (unsigned char) (b/divideFactor2);
      }
    }
   switch (resultBitsPerPixel)
    {
     case 32:
      srcData  = &iconData[iconBPL*y];
      dst32  = (unsigned int *) &resultData[resultBPL*y];
      for (x=0; x<paddingOffset; x++)
       *dst32++  = backCol32;
      for (x=0; x<wid; x++)
       {
	*dst32  = (unsigned int) (*srcData++ <<16);
	*dst32 += (unsigned int) (*srcData++ <<8);
	*dst32++ += (unsigned int) (*srcData++);
       }
      for (x=paddingOffset+wid; x<paddedWid; x++)
       *dst32++  = backCol32;
      break;
     case 24:
      {
       unsigned char *dstData = &resultData[resultBPL*y];

       for (x=0; x<paddingOffset; x++)
	{
	 *dstData++ = backgroundRGB[0];
	 *dstData++ = backgroundRGB[1];
	 *dstData++ = backgroundRGB[2];
	}
       memcpy(&resultData[resultBPL*y + paddingOffset*3], &iconData[iconBPL*y], wid*3);
       dstData = &resultData[resultBPL*y + (paddingOffset+wid)*3];
       for (x=paddingOffset+wid; x<paddedWid; x++)
	{
	 *dstData++ = backgroundRGB[0];
	 *dstData++ = backgroundRGB[1];
	 *dstData++ = backgroundRGB[2];
	}
       }
      break;
     case 16:
      srcData  = &iconData[iconBPL*y];
      dst16  = (unsigned short *) &resultData[resultBPL*y];
      for (x=0; x<paddingOffset; x++)
       *dst16++  = backCol16;
      for (x=0; x<wid; x++)
       {
	*dst16  = (unsigned short) ((*srcData++ & 248) << 8);
	*dst16 += (unsigned short) ((*srcData++ & 252) << 3);
	*dst16++ += (unsigned short) (*srcData++ >> 3);
       }
      for (x=paddingOffset+wid; x<paddedWid; x++)
       *dst16++  = backCol16;
      break;
    }
  }
}

/* RGB to HSL transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
DBRUTIL void RGBtoHSL (int *rgb, int *hsl)
{
 double v, m, vm, r1, g1, b1, h1, s1, l1, r2, g2, b2;

 r1 = (double) rgb[0]/255.0;
 g1 = (double) rgb[1]/255.0;
 b1 = (double) rgb[2]/255.0;

 v = max(r1, g1);
 v = max(v, b1);
 m = min(r1, g1);
 m = min(m, b1);

 l1 = (m+v)/2.0;
 if (l1<=0.0)
  {
   hsl[0] = 0;
   hsl[1] = 0;
   hsl[2] = (int) (l1*1000);
   return;
  }

 s1 = vm = v - m;
 if (s1 > 0.0)
  { s1 /= (l1 <= 0.5) ? (v + m) : (2.0 - v - m); }
 else
  {
   hsl[0] = 0;
   hsl[1] = (int) (s1*1000);
   hsl[2] = (int) (l1*1000);
   return;
  }

 r2 = (v - r1)/vm;
 g2 = (v - g1)/vm;
 b2 = (v - b1)/vm;

 if (floatEqual(r1, v, 0.0001))
  h1 = (floatEqual(g1, m, 0.0001) ? 5.0 + b2 : 1.0 - g2);
 else if (floatEqual(g1, v, 0.0001))
  h1 = (floatEqual(b1, m, 0.0001) ? 1.0 + r2 : 3.0 - b2);
 else
  h1 = (floatEqual(r1, m, 0.0001) ? 3.0 + g2 : 5.0 - r2);

 h1 /= 6.0;

 hsl[0] = (int) (h1*359 + 0.5);
 hsl[1] = (int) (s1*1000 + 0.5);
 hsl[2] = (int) (l1*1000 + 0.5);
}

/* HSL to RGB transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
DBRUTIL void HSLtoRGB (int *hsl, int *rgb)
{
 double h1, s1, l1, r1, g1, b1, v;

 h1 = hsl[0]/360.0;
 s1 = hsl[1]/1000.0;
 l1 = hsl[2]/1000.0;

 v = (l1<=0.5) ? (l1*(1.0 + s1)) : (l1+s1 - l1*s1);
 r1 = g1 = b1 = 0.0;
 if (v>0)
  {
   double m, sv, fract, vsf, mid1, mid2;
   int sextant;

   m = l1 + l1 - v;
   sv = (v - m)/v;
   h1 *= 6.0;
   sextant = (int) h1;
   fract = h1 - sextant;
   vsf = v * sv * fract;
   mid1 = m + vsf;
   mid2 = v - vsf;
   switch (sextant)
    {
     case 0: r1=v; g1=mid1; b1=m; break;
     case 1: r1=mid2; g1=v; b1=m; break;
     case 2: r1=m; g1=v; b1=mid1; break;
     case 3: r1=m; g1=mid2; b1=v; break;
     case 4: r1=mid1; g1=m; b1=v; break;
     case 5: r1=v; g1=m; b1=mid2; break;
    }
  }

 rgb[0] = (int) (r1*255 + 0.5);
 rgb[1] = (int) (g1*255 + 0.5);
 rgb[2] = (int) (b1*255 + 0.5);
}
