/* lwif.c loads wif fabric files, extracts the weave and passes it as an image */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "loadimg.h"

static int readSofar, srcbpl, shafts, treadles, xdim, ydim;
static unsigned char *bitData=NULL;

#define MAX_LINE	1000
#define LIFTING_BPL	40 /* enough space for 320 shafts */
#define MAX_CARD	320
#define MAX_DESIGN	MAX_LOAD_IMAGE

#define SECTION_DONTCARE	0
#define SECTION_WIF		1
#define SECTION_CONTENTS	2
#define SECTION_WEAVING		3
#define SECTION_WARP		4
#define SECTION_WEFT		5
#define SECTION_THREADING	6
#define SECTION_LIFTPLAN	7
#define SECTION_TIEUP		8
#define SECTION_TREADLING	9

static void freePtr(void **ptr)
{
 free(*ptr);
 *ptr=NULL;
}

static int getWifSection(char *line)
{
 if (strncmp(line, "[contents]", 10)==0)
  return(SECTION_CONTENTS);
 if (strncmp(line, "[weaving]", 9)==0)
  return(SECTION_WEAVING);
 if (strncmp(line, "[warp]", 6)==0)
  return(SECTION_WARP);
 if (strncmp(line, "[weft]", 6)==0)
  return(SECTION_WEFT);
 if (strncmp(line, "[threading]", 11)==0)
  return(SECTION_THREADING);
 if (strncmp(line, "[liftplan]", 10)==0)
  return(SECTION_LIFTPLAN);
 if (strncmp(line, "[tieup]", 7)==0)
  return(SECTION_TIEUP);
 if (strncmp(line, "[treadling]", 11)==0)
  return(SECTION_TREADLING);

 return(SECTION_DONTCARE);
}

static int readWifBoolean(char *dummy)
{
 if (strncmp(dummy, "yes", 3)==0)
  return(TRUE);
 if (dummy[0]=='1')
  return(TRUE);
 if (strncmp(dummy, "on", 2)==0)
  return(TRUE);
 if (strncmp(dummy, "true", 4)==0)
  return(TRUE);
 return(FALSE);
}

static void readLifting(unsigned char *resultData, int weftLine, char *data)
{
 int positions[LIFTING_BPL*8];
 int numEntries=0;
 int i, value;
 int length = strlen(data);

 positions[numEntries++]=0;
 for (i=0; i<length; i++)
  if (data[i]==',')
   { positions[numEntries++]=i+1; data[i]=0; } /* null terminate each block */
 for (i=0; i<numEntries; i++)
  {
   value = atoi(&data[positions[i]])-1;
   if ((value>=0)&&(value<MAX_CARD))
    resultData[weftLine*LIFTING_BPL+(value>>3)]|=(128>>(value&7));
  }
}

static int tryLoad(char *filename, int *loadType, int *xpixels,
 int *ypixels, int *xdpi, int *ydpi, unsigned char *cmap, char *info,
 char *reverse, int *numInterlace, int *coreFile, int realLoad)
{
 FILE *fp;
 char dummy[MAX_LINE+1], *datastr;
 int intval, num1, num2, num3;
 int len = strlen(filename);
 int section = SECTION_DONTCARE;
 int haveTieup = FALSE, haveThreading=FALSE, haveTreadling=FALSE, haveLiftplan=FALSE;
 int linesNoWif = 0;
 int wifMarkerFound = FALSE;

 /* discard all files whose filename does not end in .wif */
 if (len<=4)
  return(LOAD_FILETYPE_UNKNOWN);

 if ((filename[len-4]!='.')||(tolowerAZ(filename[len-3])!='w')||(tolowerAZ(filename[len-2])!='i')||(tolowerAZ(filename[len-1])!='f'))
  return(LOAD_FILETYPE_UNKNOWN);

 *xpixels = *ypixels = shafts = treadles = 0;
 freePtr((void *)&bitData);
 
 if ((fp = fopen(filename, "r" )) == NULL)
  return(LOAD_IO_ERROR);

 memset(dummy, 0, MAX_LINE);

 fgetsDosMac(dummy, MAX_LINE, fp);
 stringToLower(dummy);
 wifMarkerFound = strncmp(dummy, "[wif]", 5);
 while ((linesNoWif++<10) && (!wifMarkerFound) && (!feof(fp)))
  {
   fgetsDosMac(dummy, MAX_LINE, fp);
   stringToLower(dummy);
   wifMarkerFound = strncmp(dummy, "[wif]", 5);
  }

 if (!wifMarkerFound)
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }
 else
  {
   section = SECTION_WIF;
   while(!feof(fp)) /* first read the whole crap to get some info on size and stuff */
    {
     fgetsDosMac(dummy, MAX_LINE, fp);
     stringToLower(dummy);
     datastr=removeHead(dummy);

     if (dummy[0]=='[') /* detect new section */
      {
       section = getWifSection(dummy);
       fgetsDosMac(dummy, MAX_LINE, fp);
       stringToLower(dummy);
       datastr=removeHead(dummy);     
      }
     switch (section)
      {
       case SECTION_CONTENTS:
	if (strncmp(dummy, "tieup=", 6)==0)
	 haveTieup = readWifBoolean(datastr);
	if (strncmp(dummy, "threading=", 10)==0)
	 haveThreading = readWifBoolean(datastr);
	if (strncmp(dummy, "treadling=", 10)==0)
	 haveTreadling = readWifBoolean(datastr);
	if (strncmp(dummy, "liftplan=", 9)==0)
	 haveLiftplan = readWifBoolean(datastr);
	break;
       case SECTION_WEAVING:
	if (strncmp(dummy, "shafts=", 7)==0)
	 if (sscanf(datastr, "%d", &intval)==1)
	  *xpixels=shafts=intval;
	if (strncmp(dummy, "treadles=", 9)==0)
	 if (sscanf(datastr, "%d", &intval)==1)
	  treadles=intval; /* since we do not use treadles, this will be ignored */
	break;
       case SECTION_WARP:
	if (strncmp(dummy, "threads=", 8)==0)
	 if (sscanf(datastr, "%d", &intval)==1)
	  *xpixels=intval;
	break;
       case SECTION_WEFT:
	if (strncmp(dummy, "threads=", 8)==0)
	 if (sscanf(datastr, "%d", &intval)==1)
	  *ypixels=intval;
	break;
      }
    }

//printf("It's a WIF! %d shafts %d treadles x %d y %d treading %d tieup %d \n", shafts, treadles, *xpixels, *ypixels, haveTreadling, haveTieup);

   if ((shafts<2) || (treadles<2) || (*xpixels<2) || (*ypixels<2))
    { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }

   srcbpl = (*xpixels+7)/8;
   *loadType = LOAD_BW;
   *coreFile = TRUE;
   *xdpi = *ydpi = 0;
   *reverse = TRUE;
   *numInterlace = 1; /* no interlace */
   strcpy(info, "WIF weave");
   sscanf(dummy, "%d %d\n", xpixels, ypixels);

   if (realLoad)
    {
     int i,x,y;
     short *threading1, *threading2, *treadling;
     unsigned char *lifting; /* binary coded lifting */
     unsigned char *tieup; /* binary coded tieup */

     xdim = *xpixels;
     ydim = *ypixels;
     threading1 = malloc(sizeof(short)*MAX_DESIGN);
     threading2 = malloc(sizeof(short)*MAX_DESIGN);
     treadling = malloc(sizeof(short)*MAX_DESIGN);
     lifting = malloc(sizeof(unsigned char)*LIFTING_BPL*MAX_DESIGN);
     tieup = malloc(sizeof(unsigned char)*LIFTING_BPL*MAX_CARD);
     bitData = calloc(srcbpl, ydim);
     if (!(threading1 && threading2 && treadling && lifting && tieup && bitData))
      {
       if (threading1)
        free(threading1);
       if (threading2)
        free(threading2);
       if (treadling)
        free(treadling);
       if (lifting)
        free(lifting);
       if (tieup)
        free(tieup);
       freePtr((void *)&bitData);
       fclose(fp);
       return (LOAD_NO_MEMORY);
      }

     memset(lifting, 0, LIFTING_BPL*MAX_DESIGN);
     memset(tieup, 0, LIFTING_BPL*MAX_CARD);

     /* initialise tieup to straight, if we do not have it */
     if (!haveTieup)
      for (i=0; i<MAX_CARD; i++)
       tieup[i*LIFTING_BPL+(i>>3)] |=  (128>>(i&7));

     /* initialise both threading and treadling to sensible defaults */
     for (i=(max(xdim, ydim)); i>=0; i--)
      {
       threading1[i] = treadling[i] = (short) i;
       threading2[i] = -1;
      }


     /* now the second pass; actually read the data */
     rewind(fp);

     section = SECTION_WIF;
     while(!feof(fp)) /* read the whole file again */
      {
       fgetsDosMac(dummy, MAX_LINE, fp);
       stringToLower(dummy);
       datastr=removeHead(dummy);

       if (dummy[0]=='[') /* detect new section */
	{
	 section = getWifSection(dummy);
	 fgetsDosMac(dummy, MAX_LINE, fp);
	 stringToLower(dummy);
	 datastr=removeHead(dummy);     
	}
       switch (section)
	{
	 case SECTION_THREADING:
	  if (strrchr(dummy, ',')) /* first try the crazy one thread into two shafts */
	   {
	    if (sscanf(dummy, "%d=%d,%d", &num1, &num2, &num3)==3)
	     {
              num1--; num2--; num3--;
	      if ((num1>=0)&&(num1<MAX_DESIGN))
	       {
		if ((num2>=0)&&(num2<MAX_CARD))	    
		 threading1[num1] = (short) num2;
		if ((num3>=0)&&(num3<MAX_CARD))	    
		 threading2[num1] = (short) num3;
	       }
	     }
	   }
	  else
	   if (sscanf(dummy, "%d=%d", &num1, &num2)==2)
	    {
             num1--; num2--;
	     if ((num1>=0)&&(num2>=0)&&(num1<MAX_DESIGN)&&(num2<MAX_CARD))	    
	      threading1[num1] = (short) num2;
	    }
	    
	  break;
	 case SECTION_TREADLING:
	  if (sscanf(dummy, "%d=%d", &num1, &num2)==2)
	   {
            num1--; num2--;
	    if ((num1>=0)&&(num2>=0)&&(num1<MAX_DESIGN)&&(num2<MAX_CARD))	    
	     treadling[num1] = (short) num2;
	   }
	  break;
	 case SECTION_LIFTPLAN:
	  if (sscanf(dummy, "%d=", &intval)==1)
	   {
	    intval--;
	    if ((intval>=0)&&(intval<MAX_DESIGN))
             readLifting(lifting, intval, datastr);
	   }
	  break;
	 case SECTION_TIEUP:
	  if (sscanf(dummy, "%d=", &intval)==1)
	   {
	    intval--;
	    if ((intval>=0)&&(intval<MAX_CARD))
             readLifting(tieup, intval, datastr);
	   }
	  break;
	}
      }

    if (haveLiftplan)
     {
      for (y=0; y<ydim; y++)
       for (x=0; x<xdim; x++)
        {
	 if (lifting[y*LIFTING_BPL + (threading1[x]>>3)]&(128>>(threading1[x]&7)))
          bitData[y*srcbpl + (x>>3)] |= (128>>(x&7));
	 if (threading2[x]>=0)
	  if (lifting[y*LIFTING_BPL + (threading2[x]>>3)]&(128>>(threading2[x]&7)))
           bitData[y*srcbpl + (x>>3)] |= (128>>(x&7));
	}
     }
    else
     if (haveTreadling || haveTieup || haveThreading)
       for (y=0; y<ydim; y++)
	for (x=0; x<xdim; x++)
	 {
	  if (tieup[treadling[y]*LIFTING_BPL + (threading1[x]>>3)]&(128>>(threading1[x]&7)))
           bitData[y*srcbpl + (x>>3)] |= (128>>(x&7));
 	  if (threading2[x]>=0)
	   if (tieup[treadling[y]*LIFTING_BPL + (threading2[x]>>3)]&(128>>(threading2[x]&7)))
            bitData[y*srcbpl + (x>>3)] |= (128>>(x&7));
	 }

     readSofar = 0;

     free(threading1);
     free(threading2);
     free(treadling);
     free(lifting);
     free(tieup);
    }

   fclose(fp);
   return(LOAD_OK);
  }
}

/* loader reads a band */
static int loadBand(unsigned char *data, int  lines, int  bpl)
{
 int i;

 for (i=0; i<lines; i++)
  memcpy(&data[bpl*i], &bitData[(i+readSofar)*srcbpl], bpl);
 readSofar+=lines;
 return(LOAD_OK);
}

/* closes the input file and releases any temporary storage */
static int endLoad()
{
 freePtr((void *)&bitData);
 return(LOAD_OK);
}

/* register wif image loader */
void registerLoadWIF()
{
 registerLoader(*tryLoad, *loadBand, *endLoad);
}
