/***************************************************************************
                          apaint.cpp  -  description
                             -------------------
    begin                : Mon Jan 27 11:15:43 CET 2003
    copyright            : (C) 2003-2013 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qvaluevector.h>
#include <qaccel.h>

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libgen.h>
#include "../../iolib/acolor.h"
#include "../../iolib/browseio.h"
#include "../../llib/loadimg.h"
}

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <stdlib.h>

#include <qprocess.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qclipboard.h>
#include <qpainter.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include "toolBSpline.h"
#include "apaint.h"
#include "apaintconfig.h"
#include "dialogColorPalette.h"
#include "dialogColorPaletteUtil.h"
#include "qtabwidget.h"
#include "qiconset.h"
#include "dialogIconView.h"
#include "toolLayerMove.h"
#include "apaintclipboard.h"
#include "dialogContour.h"
#include "toolCleanup.h"
#include "apaintio.h"
#include "toolInsRemPixels.h"
#include "dialogGuessWeave.h"
#include "dialogPrint.h"
#include "apaintUtil.h"
#include "dialogBrushList.h"
#include "apaintBrushList.h"
#include "toolMagicWand.h"
#include "apaintimageUtil.h"
#include "toolUnRedo.h"
#include "toolFreeTransform.h"

extern char *getCommonPNG(const char *pixName);

#define VERSION_NUM "4.1r"

ApaintApp::ApaintApp()
{
 apaint = this;
 lastOperation = -1;
 multipleLayers = true;
 canPaste = false;
 canUndo = false;
 canRedo = false;
 setCaption(tr("ArahPaint " VERSION_NUM));
 setGeometry(0, 0, ApaintConfig::initialWindowSizeXpix(), ApaintConfig::initialWindowSizeYpix());

 imageScale = 100;
 lineSize = 1;
 mainLayerLoaded = false;
 undoRedoEnable = true;

 setIcon(QPixmap(getCommonPNG("ArahPaint4-one")));

 ///////////////////////////////////////////////////////////////////
 // call inits to invoke all other construction parts

 initActions();
 initMenuBar();
 initToolBar();

 initMainObjects();

 /*set up all modules connections*/ 
 initConnections();

 /*set focus stuff*/
 view->setFocusPolicy(StrongFocus);
 setFocusProxy(view);

 /*init pen and brush after connections has been made*/  
 pbUtil->initPenBrush();

 viewInteger->setOn(true);
 viewAntialias->setOn(false);
 viewRepeatDelimiter->setOn(false);
 crossHair = FALSE;
 viewCrosshair->setOn(crossHair);
 
 keepOriginalCmap=ApaintConfig::getKeepColormap();
 keepOriginalCmapAct->setOn(keepOriginalCmap);

 viewToolBar->setOn(true);
 slotViewStatusBar(false);
 slotViewToolBar(false);
 sgrid->setOn(true);
 shRuler->setOn(false);
 showRuler(false);
 repeatView->setOn(false);
 smallFont = QFont(font().family(), ApaintConfig::getSmallFontSize());
 if (!ApaintConfig::getUseXFT())
  smallFont.setPointSize(ApaintConfig::getBestPointSizeMainWin(smallFont, ApaintConfig::getPixelSize(ApaintConfig::getSmallFontSize())));
 iconDlg->setFont(smallFont);
 vruler->setFont(smallFont);
 hruler->setFont(smallFont);
 rInfo->setFont(smallFont);
 if (ApaintConfig::isRulerOn() && !shRuler->isOn())
  shRuler->toggle();
}

ApaintApp::~ApaintApp()
{
 delete unredo;
 ApaintIO::deleteObject(); //must because of shared memory
 ApaintBrushList::deleteObject();
}

/** initializes all QActions of the application */
void ApaintApp::initActions()
{
  fileNew = new QAction(ApaintConfig::getText(NEW_PICTURE_MSG)+"...", QAccel::stringToKey(tr("Ctrl+N")), this);
  fileNew->setIconSet(QPixmap(getCommonPNG("document-new")));
  connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

  fileOpen = new QAction(ApaintConfig::getText(LOAD_PICTURE_MSG)+"...", QAccel::stringToKey(tr("Ctrl+O")), this);
  fileOpen->setIconSet(QPixmap(getCommonPNG("document-open")));
  connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

  fileDocumentBrowse = new QAction(ApaintConfig::getText(ApaintConfig::getHideLoadMenu() ? LOAD_PICTURE_MSG : 657)+"...", QAccel::stringToKey(tr("Ctrl+Shift+O")), this);
  fileDocumentBrowse->setIconSet(QPixmap(getCommonPNG("document-browse")));
  connect(fileDocumentBrowse, SIGNAL(activated()), this, SLOT(slotFileBrowse()));

  saveLayer = new QAction(ApaintConfig::getText(SAVE_LAYER_MSG)+"...", 0, this);
  saveLayer->setIconSet(QPixmap(getCommonPNG("document-save-as")));
  connect(saveLayer, SIGNAL(activated()), this, SLOT(slotLayerSave()));

  fileReload = new QAction(ApaintConfig::getText(758)+"...", QAccel::stringToKey(tr("Ctrl+U")), this);
  fileReload->setIconSet(QPixmap(getCommonPNG("reload")));
  connect(fileReload, SIGNAL(activated()), this, SLOT(revertToSaved()));

  loadLayer = new QAction(ApaintConfig::getText(LOAD_LAYER_MSG)+"...", QAccel::stringToKey(tr("Ctrl+L")), this);
  loadLayer->setIconSet(QPixmap(getCommonPNG("document-open")));
  loadLayer->setEnabled(false);
  connect(loadLayer, SIGNAL(activated()), this, SLOT(slotLayerLoad()));
  
  fileSave = new QAction(ApaintConfig::getText(SAVE_PICTURE_MSG), QAccel::stringToKey(tr("Ctrl+S")), this);
  fileSave->setEnabled(false);
  fileSave->setIconSet(QPixmap(getCommonPNG("document-save")));
  connect(fileSave, SIGNAL(activated()), this, SLOT(slotFileSave()));   

  fileSaveAs = new QAction(ApaintConfig::getText(SAVE_PICTURE_MSG_AS)+"...", QAccel::stringToKey(tr("Ctrl+Shift+S")), this);
  fileSaveAs->setEnabled(false);
  fileSaveAs->setIconSet(QPixmap(getCommonPNG("document-save-as")));
  connect(fileSaveAs, SIGNAL(activated()), this, SLOT(slotFileSaveAs()));

  fileBrowseLayer = new QAction(ApaintConfig::getText(ApaintConfig::getHideLoadMenu() ? LOAD_LAYER_MSG : 764)+"...", QAccel::stringToKey(tr("Ctrl+Shift+L")), this);
  fileBrowseLayer->setIconSet(QPixmap(getCommonPNG("document-browse")));
  connect(fileBrowseLayer, SIGNAL(activated()), this, SLOT(slotLayerBrowse()));

  filePrint = new QAction(ApaintConfig::getText(16)+"...", QAccel::stringToKey(tr("Ctrl+P")), this);
  filePrint->setIconSet(QPixmap(getCommonPNG("document-print")));
  connect(filePrint, SIGNAL(activated()), this, SLOT(slotFilePrint()));

  fileQuit = new QAction(ApaintConfig::getText(PROGRAM_EXIT_MSG), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setIconSet(QPixmap(getCommonPNG("exit")));
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

  editUndo = new QAction(ApaintConfig::getText(UNDO_MSG), QAccel::stringToKey(tr("Ctrl+Z")), this);
  editUndo->setIconSet(QPixmap(getCommonPNG("edit-undo")));
  connect(editUndo, SIGNAL(activated()), this, SLOT(slotEditUndo()));

  editRedo = new QAction(ApaintConfig::getText(REDO_MSG), QAccel::stringToKey(tr("Ctrl+Y")), this);
  editRedo->setIconSet(QPixmap(getCommonPNG("edit-redo")));
  connect(editRedo, SIGNAL(activated()), this, SLOT(slotEditRedo()));
  
  editCut = new QAction(ApaintConfig::getText(CUT_MSG), QAccel::stringToKey(tr("Ctrl+X")), this);
  editCut->setIconSet(QPixmap(getCommonPNG("edit-cut")));
  connect(editCut, SIGNAL(activated()), this, SLOT(slotEditCut()));

  editCopy = new QAction(ApaintConfig::getText(COPY_MSG), QAccel::stringToKey(tr("Ctrl+C")), this);
  editCopy->setIconSet(QPixmap(getCommonPNG("edit-copy")));
  connect(editCopy, SIGNAL(activated()), this, SLOT(slotEditCopy()));

  editPaste = new QAction(ApaintConfig::getText(PASTE_MSG), QAccel::stringToKey(tr("Ctrl+V")), this);
  editPaste->setIconSet(QPixmap(getCommonPNG("edit-paste")));
  connect(editPaste, SIGNAL(activated()), this, SLOT(slotEditPaste()));

  editSelectAll = new QAction(ApaintConfig::getText(SELECT_ALL_MSG), QAccel::stringToKey(tr("Ctrl+A")), this);
  connect(editSelectAll, SIGNAL(activated()), this, SLOT(slotEditSelectAll()));

  editClear = new QAction(ApaintConfig::getText(745), QAccel::stringToKey(tr("Ctrl+K")), this);
  editClear->setIconSet(QPixmap(getCommonPNG("edit-clear")));
  connect(editClear, SIGNAL(activated()), this, SLOT(slotEditClear()));

  editScale = new QAction(ApaintConfig::getText(REDUCE_AREA_MSG), QAccel::stringToKey(tr("C")), this);
  editScale->setIconSet(QPixmap(getCommonPNG("crop")));
  connect(editScale, SIGNAL(activated()), this, SLOT(slotEditScale()));

  colorNum = new QAction(ApaintConfig::getText(SET_NUM_COL_MSG)+"...", QAccel::stringToKey(tr("Alt+X")), this);
  connect(colorNum, SIGNAL(activated()), this, SLOT(showColNumDlg()));

  colorReduce = new QAction(ApaintConfig::getText(REDUCE_COL_MSG)+"...", QAccel::stringToKey(tr("Alt+C")), this);
  connect(colorReduce, SIGNAL(activated()), this, SLOT(showReduceColDlg()));

  colorRemap = new QAction(ApaintConfig::getText(COLOR_REMAP_MSG)+"...", QAccel::stringToKey(tr("Shift+C")), this);
  colorRemap->setIconSet(QPixmap(getCommonPNG("object-colorize")));
  connect(colorRemap, SIGNAL(activated()), this, SLOT(showColorRemapDialog()));

  viewRepeat = new QAction(ApaintConfig::getText(DRAW_REPEAT_MSG)+"...", QAccel::stringToKey(tr("Alt+D")), this);
  viewRepeat->setIconSet(QPixmap(getCommonPNG("repeat")));
  connect(viewRepeat, SIGNAL(activated()), this, SLOT(slotViewRepeat()));

  editDelete = new QAction(ApaintConfig::getText(DELETE_LAYER_MSG), QAccel::stringToKey(tr("Del")), this);
  editDelete->setIconSet(QPixmap(getCommonPNG("edit-delete")));
  connect(editDelete, SIGNAL(activated()), this, SLOT(slotEditDelete()));

  editPasteMultiple = new QAction(ApaintConfig::getText(MULTI_PASTE_MSG)+"...", QAccel::stringToKey(tr("Ctrl+Shift+V")), this);
  connect(editPasteMultiple, SIGNAL(activated()), this, SLOT(slotEditMultiplePaste()));

  sortColorsPopularity = new QAction(ApaintConfig::getText(POPULARITY_MSG), 0, this);
  sortColorsPopularity->setIconSet(QPixmap(getCommonPNG("sort-popularity")));
  connect(sortColorsPopularity, SIGNAL(activated()), this, SLOT(slotColorsPopylarity()));

  sortColorsHue = new QAction(ApaintConfig::getText(HUE_MSG), 0, this);
  sortColorsHue->setIconSet(QPixmap(getCommonPNG("color")));
  connect(sortColorsHue, SIGNAL(activated()), this, SLOT(slotColorsHue()));

  sortColorsSaturation = new QAction(ApaintConfig::getText(SATURATION_MSG), 0, this);
  sortColorsSaturation->setIconSet(QPixmap(getCommonPNG("saturation")));
  connect(sortColorsSaturation, SIGNAL(activated()), this, SLOT(slotColorsSaturation()));

  sortColorsLightness = new QAction(ApaintConfig::getText(LIGHT_MSG), 0, this);
  sortColorsLightness->setIconSet(QPixmap(getCommonPNG("gradient-linear")));
  connect(sortColorsLightness, SIGNAL(activated()), this, SLOT(slotColorsLightness()));

  sortColorsSimilarity = new QAction(ApaintConfig::getText(SIMILARITY_MSG), 0, this);
  connect(sortColorsSimilarity, SIGNAL(activated()), this, SLOT(slotColorsSimilarity()));

  colorCountHorizontal = new QAction(ApaintConfig::getText(HORIZONTAL_MSG), QAccel::stringToKey(tr("Ctrl+H")), this);
  connect(colorCountHorizontal, SIGNAL(activated()), this, SLOT(slotColorCountHorizontal()));

  colorCountVertical = new QAction(ApaintConfig::getText(VERTICAL_MSG), QAccel::stringToKey(tr("Ctrl+Shift+H")), this);
  connect(colorCountVertical, SIGNAL(activated()), this, SLOT(slotColorCountVertical()));

  colorInvertProtected = new QAction(ApaintConfig::getText(INVERT_MSG), QAccel::stringToKey(tr("Ctrl+M")), this);
  colorInvertProtected->setIconSet(QPixmap(getCommonPNG("invert")));
  connect(colorInvertProtected, SIGNAL(activated()), this, SLOT(slotColorInvertProtected()));

  colorClearProtected = new QAction(ApaintConfig::getText(CLEAR_MSG), QAccel::stringToKey(tr("Shift+Ctrl+M")), this);
  colorClearProtected->setIconSet(QPixmap(getCommonPNG("edit-clear")));
  connect(colorClearProtected, SIGNAL(activated()), this, SLOT(slotColorClearProtected()));

  colorInvertTransparent = new QAction(ApaintConfig::getText(INVERT_MSG), QAccel::stringToKey(tr("Ctrl+B")), this);
  colorInvertTransparent->setIconSet(QPixmap(getCommonPNG("invert")));
  connect(colorInvertTransparent, SIGNAL(activated()), this, SLOT(slotColorInvertTransparent()));

  colorClearTransparent = new QAction(ApaintConfig::getText(CLEAR_MSG), QAccel::stringToKey(tr("Ctrl+Shift+B")), this);
  colorClearTransparent->setIconSet(QPixmap(getCommonPNG("edit-clear")));
  connect(colorClearTransparent, SIGNAL(activated()), this, SLOT(slotColorClearTransparent()));

  colorInvertAll = new QAction(ApaintConfig::getText(ALL_MSG), QAccel::stringToKey(tr("CTRL+I")), this);
  colorInvertAll->setIconSet(QPixmap(getCommonPNG("circle-RGB")));
  connect(colorInvertAll, SIGNAL(activated()), this, SLOT(slotColorInvertAll()));

  colorInvertRed = new QAction(ApaintConfig::getText(RED_MSG), 0, this);
  colorInvertRed->setIconSet(QPixmap(getCommonPNG("circle-red")));
  connect(colorInvertRed, SIGNAL(activated()), this, SLOT(slotColorInvertRed()));

  colorInvertGreen = new QAction(ApaintConfig::getText(GREEN_MSG), 0, this);
  colorInvertGreen->setIconSet(QPixmap(getCommonPNG("circle-green")));
  connect(colorInvertGreen, SIGNAL(activated()), this, SLOT(slotColorInvertGreen()));

  colorInvertBlue = new QAction(ApaintConfig::getText(BLUE_MSG), 0, this);
  colorInvertBlue->setIconSet(QPixmap(getCommonPNG("circle-blue")));
  connect(colorInvertBlue, SIGNAL(activated()), this, SLOT(slotColorInvertBlue()));

  colorInvertCyan = new QAction(ApaintConfig::getText(CYAN_MSG), 0, this);
  colorInvertCyan->setIconSet(QPixmap(getCommonPNG("circle-cyan")));
  connect(colorInvertCyan, SIGNAL(activated()), this, SLOT(slotColorInvertCyan()));

  colorInvertYellow = new QAction(ApaintConfig::getText(YELLOW_MSG), 0, this);
  colorInvertYellow->setIconSet(QPixmap(getCommonPNG("circle-yellow")));
  connect(colorInvertYellow, SIGNAL(activated()), this, SLOT(slotColorInvertYellow()));

  colorInvertMagenta = new QAction(ApaintConfig::getText(MAGENTA_MSG), 0, this);
  colorInvertMagenta->setIconSet(QPixmap(getCommonPNG("circle-magenta")));
  connect(colorInvertMagenta, SIGNAL(activated()), this, SLOT(slotColorInvertMagenta()));

  colorInvertGrey = new QAction(ApaintConfig::getText(GREY_MSG), 0, this);
  colorInvertGrey->setIconSet(QPixmap(getCommonPNG("circle-grey")));
  connect(colorInvertGrey, SIGNAL(activated()), this, SLOT(slotColorInvertGrey()));

  colorInvertSaturation = new QAction(ApaintConfig::getText(SATURATION_MSG), 0, this);
  colorInvertSaturation->setIconSet(QPixmap(getCommonPNG("color")));
  connect(colorInvertSaturation, SIGNAL(activated()), this, SLOT(slotColorInvertSaturation()));

  colorSwapPalette = new QAction(ApaintConfig::getText(COLOR_PALETTE), QAccel::stringToKey(tr(".")), this);
  colorSwapPalette->setIconSet(QPixmap(getCommonPNG("convert-indexed")));
  connect(colorSwapPalette, SIGNAL(activated()), this, SLOT(slotColorSwapPalette()));

  colorSwapRGB_GBR = new QAction(ApaintConfig::getText(RED_MSG)+"->"+ApaintConfig::getText(GREEN_MSG)+"->"+ApaintConfig::getText(BLUE_MSG)+"->"+ApaintConfig::getText(RED_MSG), 0, this);
  colorSwapRGB_GBR->setIconSet(QPixmap(getCommonPNG("RGB2GBR")));
  connect(colorSwapRGB_GBR, SIGNAL(activated()), this, SLOT(slotColorSwapRGB_GBR()));

  colorSwapRGB_BRG = new QAction(ApaintConfig::getText(RED_MSG)+"->"+ApaintConfig::getText(BLUE_MSG)+"->"+ApaintConfig::getText(GREEN_MSG)+"->"+ApaintConfig::getText(RED_MSG), 0, this);
  colorSwapRGB_BRG->setIconSet(QPixmap(getCommonPNG("RGB2BRG")));
  connect(colorSwapRGB_BRG, SIGNAL(activated()), this, SLOT(slotColorSwapRGB_BRG()));

  colorSwapRG = new QAction(ApaintConfig::getText(RED_MSG)+"<->"+ApaintConfig::getText(GREEN_MSG), 0, this);
//  colorSwapRG->setIconSet(QPixmap(getCommonPNG("RGB2GRB")));
  connect(colorSwapRG, SIGNAL(activated()), this, SLOT(slotColorSwapRG()));

  colorSwapRB = new QAction(ApaintConfig::getText(RED_MSG)+"<->"+ApaintConfig::getText(BLUE_MSG), 0, this);
//  colorSwapRB->setIconSet(QPixmap(getCommonPNG("RGB2BGR")));
  connect(colorSwapRB, SIGNAL(activated()), this, SLOT(slotColorSwapRB()));

  colorSwapGB = new QAction(ApaintConfig::getText(GREEN_MSG)+"<->"+ApaintConfig::getText(BLUE_MSG), 0, this);
//  colorSwapGB->setIconSet(QPixmap(getCommonPNG("RGB2RBG")));
  connect(colorSwapGB, SIGNAL(activated()), this, SLOT(slotColorSwapGB()));

  colorSwapSL = new QAction(ApaintConfig::getText(SATURATION_MSG)+"<->"+ApaintConfig::getText(LIGHT_MSG), 0, this);
//  colorSwapSL->setIconSet(QPixmap(getCommonPNG("RGB2RBG")));
  connect(colorSwapSL, SIGNAL(activated()), this, SLOT(slotColorSwapSL()));

  contrastStretch = new QAction(ApaintConfig::getText(CONTRAST_STRETCH_MSG), QAccel::stringToKey(tr("Alt+E")), this);
  contrastStretch->setIconSet(QPixmap(getCommonPNG("brightness-contrast")));
  connect(contrastStretch, SIGNAL(activated()), this, SLOT(slotContrastStretch()));

  findRepeat = new QAction(ApaintConfig::getText(FIND_REPEAT_MSG), QAccel::stringToKey(tr("Ctrl+Alt+F")), this);
  findRepeat->setIconSet(QPixmap(getCommonPNG("find-repeat")));
  connect(findRepeat, SIGNAL(activated()), this, SLOT(slotFindRepeat()));

  findExactRepeat = new QAction(ApaintConfig::getText(FIND_EXACT_REPEAT_MSG), QAccel::stringToKey(tr("Alt+F")), this);
  connect(findExactRepeat, SIGNAL(activated()), this, SLOT(slotFindExactRepeat()));

  duplicateX1 = new QAction(ApaintConfig::getText(HORIZONTAL_MSG), QAccel::stringToKey(tr("(")), this);
  duplicateX1->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal")));
  connect(duplicateX1, SIGNAL(activated()), this, SLOT(slotDuplicateX1()));

  duplicateX2 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" X", QAccel::stringToKey(tr("Alt+(")), this);
  duplicateX2->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-mirrorX")));
  connect(duplicateX2, SIGNAL(activated()), this, SLOT(slotDuplicateX2()));

  duplicateX3 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" X - 1", 0, this);
  duplicateX3->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-mirrorX")));
  connect(duplicateX3, SIGNAL(activated()), this, SLOT(slotDuplicateX3()));

  duplicateX4 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" Y", 0, this);
  duplicateX4->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-mirrorY")));
  connect(duplicateX4, SIGNAL(activated()), this, SLOT(slotDuplicateX4()));

  duplicateX5 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" XY", 0, this);
  duplicateX5->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-mirrorXY")));
  connect(duplicateX5, SIGNAL(activated()), this, SLOT(slotDuplicateX5()));

  duplicateX6 = new QAction(ApaintConfig::getText(HALFDROP_MSG), QAccel::stringToKey(tr("Ctrl+(")), this);
  duplicateX6->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-halfdrop")));
  connect(duplicateX6, SIGNAL(activated()), this, SLOT(slotDuplicateX6()));

  duplicateX7 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" X", 0, this);
  duplicateX7->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-halfdrop-mirrorX")));
  connect(duplicateX7, SIGNAL(activated()), this, SLOT(slotDuplicateX7()));

  duplicateX8 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" Y", 0, this);
  duplicateX8->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-halfdrop-mirrorY")));
  connect(duplicateX8, SIGNAL(activated()), this, SLOT(slotDuplicateX8()));

  duplicateX9 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" XY", 0, this);
  duplicateX9->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-halfdrop-mirrorXY")));
  connect(duplicateX9, SIGNAL(activated()), this, SLOT(slotDuplicateX9()));

  duplicateX10 = new QAction(ApaintConfig::getText(DIAMOND_MSG), 0, this);
  duplicateX10->setIconSet(QPixmap(getCommonPNG("duplicate-horizontal-diamond")));
  connect(duplicateX10, SIGNAL(activated()), this, SLOT(slotDuplicateX10()));

  duplicateY1 = new QAction(ApaintConfig::getText(VERTICAL_MSG), QAccel::stringToKey(tr(")")), this);
  duplicateY1->setIconSet(QPixmap(getCommonPNG("duplicate-vertical")));
  connect(duplicateY1, SIGNAL(activated()), this, SLOT(slotDuplicateY1()));

  duplicateY2 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" X", 0, this);
  duplicateY2->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-mirrorX")));
  connect(duplicateY2, SIGNAL(activated()), this, SLOT(slotDuplicateY2()));

  duplicateY3 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" Y", QAccel::stringToKey(tr("Alt+)")), this);
  duplicateY3->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-mirrorY")));
  connect(duplicateY3, SIGNAL(activated()), this, SLOT(slotDuplicateY3()));

  duplicateY4 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" Y - 1", 0, this);
  duplicateY4->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-mirrorY")));
  connect(duplicateY4, SIGNAL(activated()), this, SLOT(slotDuplicateY4()));

  duplicateY5 = new QAction(ApaintConfig::getText(MIRROR_MSG)+" XY", 0, this);
  duplicateY5->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-mirrorXY")));
  connect(duplicateY5, SIGNAL(activated()), this, SLOT(slotDuplicateY5()));

  duplicateY6 = new QAction(ApaintConfig::getText(HALFDROP_MSG), QAccel::stringToKey(tr("Ctrl+)")), this);
  duplicateY6->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-halfdrop")));
  connect(duplicateY6, SIGNAL(activated()), this, SLOT(slotDuplicateY6()));

  duplicateY7 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" X", 0, this);
  duplicateY7->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-halfdrop-mirrorX")));
  connect(duplicateY7, SIGNAL(activated()), this, SLOT(slotDuplicateY7()));

  duplicateY8 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" Y", 0, this);
  duplicateY8->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-halfdrop-mirrorY")));
  connect(duplicateY8, SIGNAL(activated()), this, SLOT(slotDuplicateY8()));

  duplicateY9 = new QAction(ApaintConfig::getText(HALFDROP_MSG)+" "+ApaintConfig::getText(MIRROR_MSG)+" XY", 0, this);
  duplicateY9->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-halfdrop-mirrorXY")));
  connect(duplicateY9, SIGNAL(activated()), this, SLOT(slotDuplicateY9()));

  duplicateY10 = new QAction(ApaintConfig::getText(DIAMOND_MSG), 0, this);
  duplicateY10->setIconSet(QPixmap(getCommonPNG("duplicate-vertical-diamond")));
  connect(duplicateY10, SIGNAL(activated()), this, SLOT(slotDuplicateY10()));

  scale2X = new QAction("2X", QAccel::stringToKey(tr("Ctrl+Alt+2")), this);
  connect(scale2X, SIGNAL(activated()), this, SLOT(slotScale2X()));

  scale3X = new QAction("3X", QAccel::stringToKey(tr("Ctrl+Alt+3")), this);
  connect(scale3X, SIGNAL(activated()), this, SLOT(slotScale3X()));

  scale4X = new QAction("4X", QAccel::stringToKey(tr("Ctrl+Alt+4")), this);
  connect(scale4X, SIGNAL(activated()), this, SLOT(slotScale4X()));

  shrink2X = new QAction("/2", QAccel::stringToKey(tr("Shift+F2")), this);
  connect(shrink2X, SIGNAL(activated()), this, SLOT(slotShrink2X()));

  shrink3X = new QAction("/3", QAccel::stringToKey(tr("Shift+F3")), this);
  connect(shrink3X, SIGNAL(activated()), this, SLOT(slotShrink3X()));

  shrink4X = new QAction("/4", QAccel::stringToKey(tr("Shift+F4")), this);
  connect(shrink4X, SIGNAL(activated()), this, SLOT(slotShrink4X()));

  halveX = new QAction(ApaintConfig::getText(HALVE_MSG)+" X", 0, this);
  halveX->setIconSet(QPixmap(getCommonPNG("cutY")));
  connect(halveX, SIGNAL(activated()), this, SLOT(slotHalveX()));

  halveY = new QAction(ApaintConfig::getText(HALVE_MSG)+" Y", 0, this);
  halveY->setIconSet(QPixmap(getCommonPNG("cutX")));
  connect(halveY, SIGNAL(activated()), this, SLOT(slotHalveY()));

  multiplyX2 = new QAction("X 2", QAccel::stringToKey(tr("Ctrl+2")), this);
  connect(multiplyX2, SIGNAL(activated()), this, SLOT(slotMultiplyX2()));
  multiplyX3 = new QAction("X 3", QAccel::stringToKey(tr("Ctrl+3")), this);
  connect(multiplyX3, SIGNAL(activated()), this, SLOT(slotMultiplyX3()));
  multiplyX4 = new QAction("X 4", QAccel::stringToKey(tr("Ctrl+4")), this);
  connect(multiplyX4, SIGNAL(activated()), this, SLOT(slotMultiplyX4()));
  multiplyX5 = new QAction("X 5", QAccel::stringToKey(tr("Ctrl+5")), this);
  connect(multiplyX5, SIGNAL(activated()), this, SLOT(slotMultiplyX5()));
  multiplyX6 = new QAction("X 6", QAccel::stringToKey(tr("Ctrl+6")), this);
  connect(multiplyX6, SIGNAL(activated()), this, SLOT(slotMultiplyX6()));
  multiplyX7 = new QAction("X 7", QAccel::stringToKey(tr("Ctrl+7")), this);
  connect(multiplyX7, SIGNAL(activated()), this, SLOT(slotMultiplyX7()));
  multiplyX8 = new QAction("X 8", QAccel::stringToKey(tr("Ctrl+8")), this);
  connect(multiplyX8, SIGNAL(activated()), this, SLOT(slotMultiplyX8()));
  multiplyX9 = new QAction("X 9", QAccel::stringToKey(tr("Ctrl+9")), this);
  connect(multiplyX9, SIGNAL(activated()), this, SLOT(slotMultiplyX9()));
  multiplyX10 = new QAction("X 10", QAccel::stringToKey(tr("Ctrl+0")), this);
  connect(multiplyX10, SIGNAL(activated()), this, SLOT(slotMultiplyX10()));
 
  multiplyY2 = new QAction("X 2", QAccel::stringToKey(tr("Alt+2")), this);
  connect(multiplyY2, SIGNAL(activated()), this, SLOT(slotMultiplyY2()));
  multiplyY3 = new QAction("X 3", QAccel::stringToKey(tr("Alt+3")), this);
  connect(multiplyY3, SIGNAL(activated()), this, SLOT(slotMultiplyY3()));
  multiplyY4 = new QAction("X 4", QAccel::stringToKey(tr("Alt+4")), this);
  connect(multiplyY4, SIGNAL(activated()), this, SLOT(slotMultiplyY4()));
  multiplyY5 = new QAction("X 5", QAccel::stringToKey(tr("Alt+5")), this);
  connect(multiplyY5, SIGNAL(activated()), this, SLOT(slotMultiplyY5()));
  multiplyY6 = new QAction("X 6", QAccel::stringToKey(tr("Alt+6")), this);
  connect(multiplyY6, SIGNAL(activated()), this, SLOT(slotMultiplyY6()));
  multiplyY7 = new QAction("X 7", QAccel::stringToKey(tr("Alt+7")), this);
  connect(multiplyY7, SIGNAL(activated()), this, SLOT(slotMultiplyY7()));
  multiplyY8 = new QAction("X 8", QAccel::stringToKey(tr("Alt+8")), this);
  connect(multiplyY8, SIGNAL(activated()), this, SLOT(slotMultiplyY8()));
  multiplyY9 = new QAction("X 9", QAccel::stringToKey(tr("Alt+9")), this);
  connect(multiplyY9, SIGNAL(activated()), this, SLOT(slotMultiplyY9()));
  multiplyY10 = new QAction("X 10", QAccel::stringToKey(tr("Alt+0")), this);
  connect(multiplyY10, SIGNAL(activated()), this, SLOT(slotMultiplyY10()));

  shrinkX = new QAction(ApaintConfig::getText(SHRINK_MSG)+" X", QAccel::stringToKey(tr("Ctrl+1")), this);
  shrinkX->setIconSet(QPixmap(getCommonPNG("shrink-horizontal")));
  connect(shrinkX, SIGNAL(activated()), this, SLOT(slotShrinkX()));

  shrinkY = new QAction(ApaintConfig::getText(SHRINK_MSG)+" Y", QAccel::stringToKey(tr("Alt+1")), this);
  shrinkY->setIconSet(QPixmap(getCommonPNG("shrink-vertical")));
  connect(shrinkY, SIGNAL(activated()), this, SLOT(slotShrinkY()));

  colorEdit = new QAction(ApaintConfig::getText(EDIT_MSG), QAccel::stringToKey(tr("Shift+Space")), this);
  colorEdit->setIconSet(QPixmap(getCommonPNG("applications-graphics")));
  connect(colorEdit, SIGNAL(activated()), this, SLOT(slotColorEdit()));

  colorTrueToCmap = new QAction(QString(tr("24->8")), QAccel::stringToKey(tr("Ctrl+J")), this);
  colorTrueToCmap->setIconSet(QPixmap(getCommonPNG("convert-indexed")));
  connect(colorTrueToCmap, SIGNAL(activated()), this, SLOT(slotColorTrueToCmap()));

  colorCmapToTrue = new QAction(QString(tr("8->24")), QAccel::stringToKey(tr("Ctrl+J")), this);
  colorCmapToTrue->setIconSet(QPixmap(getCommonPNG("convert-rgb")));
  connect(colorCmapToTrue, SIGNAL(activated()), this, SLOT(slotColorCmapToTrue()));

  loadColormap = new QAction(ApaintConfig::getText(LOAD_COLORMAP_MSG)+"...", QAccel::stringToKey(tr("Ctrl+Alt+C")), this);
  loadColormap->setIconSet(QPixmap(getCommonPNG("convert-indexed")));
  connect(loadColormap, SIGNAL(activated()), this, SLOT(slotLoadColormap()));

  loadColormapDither = new QAction(ApaintConfig::getText(LOAD_COLORMAP_DITHER_MSG)+"...", QAccel::stringToKey(tr("Shift+Alt+C")), this);
  connect(loadColormapDither, SIGNAL(activated()), this, SLOT(slotLoadColormapDither()));

  toolContour = new QAction(ApaintConfig::getText(CONTOURS_MSG)+"...", QAccel::stringToKey(tr("U")), this);
  connect(toolContour, SIGNAL(activated()), this, SLOT(slotToolsContour()));

  toolContourRemove = new QAction(ApaintConfig::getText(REMOVE_CONT_MSG), QAccel::stringToKey(tr("Ctrl+E")), this);
  connect(toolContourRemove, SIGNAL(activated()), this, SLOT(slotContourRemove()));

  toolContourThin = new QAction(ApaintConfig::getText(THIN_CONT_MSG), QAccel::stringToKey(tr("Ctrl+W")), this);
  connect(toolContourThin, SIGNAL(activated()), this, SLOT(slotContourThin()));

  toolContourPrune = new QAction(ApaintConfig::getText(39), QAccel::stringToKey(tr("Alt+W")), this);
  connect(toolContourPrune, SIGNAL(activated()), this, SLOT(slotContourPrune()));

  toolInvert = new QAction(ApaintConfig::getText(735), QAccel::stringToKey(tr("CTRL+Shift+I")), this);
  toolInvert->setIconSet(QPixmap(getCommonPNG("invert")));
  connect(toolInvert, SIGNAL(activated()), this, SLOT(invertLayer()));

  toolFlatten = new QAction(ApaintConfig::getText(762), QAccel::stringToKey(tr("CTRL+Shift+E")), this);
  connect(toolFlatten, SIGNAL(activated()), this, SLOT(slotToolsFlatten()));

  toolLayerToBrush = new QAction(ApaintConfig::getText(724), QAccel::stringToKey(tr(">")), this);
  toolLayerToBrush->setIconSet(QPixmap(getCommonPNG("1rightarrow")));
  connect(toolLayerToBrush, SIGNAL(activated()), this, SLOT(layerToBrushSlot()));

  toolBrushToLayer = new QAction(ApaintConfig::getText(725), QAccel::stringToKey(tr("<")), this);
  toolBrushToLayer->setIconSet(QPixmap(getCommonPNG("1leftarrow")));
  connect(toolBrushToLayer, SIGNAL(activated()), this, SLOT(brushToLayerSlot()));

  filtersRepeatLast1 = new QAction("X 1", QAccel::stringToKey(tr("Ctrl+F")), this);
  connect(filtersRepeatLast1, SIGNAL(activated()), this, SLOT(slotFiltersRepeatLast1()));

  filtersRepeatLast10 = new QAction("X 10", QAccel::stringToKey(tr("Shift+Ctrl+F")), this);
  connect(filtersRepeatLast10, SIGNAL(activated()), this, SLOT(slotFiltersRepeatLast10()));

  filtersContourAverage = new QAction(ApaintConfig::getText(AVERAGING_CONT_MSG), QAccel::stringToKey(tr("Shift+M")), this);
  connect(filtersContourAverage, SIGNAL(activated()), this, SLOT(slotFiltersContourAverage()));

  despecleMenu = new QPopupMenu();
  filtersContourDespecleAll = new QAction(ApaintConfig::getText(ALL_MSG), QAccel::stringToKey(tr("Shift+A")), this);
  connect(filtersContourDespecleAll, SIGNAL(activated()), this, SLOT(slotFiltersContourDespecleAll()));

  filtersContourDespecleSelected = new QAction(ApaintConfig::getText(SELECTED_MSG), QAccel::stringToKey(tr("Shift+S")), this);
  connect(filtersContourDespecleSelected, SIGNAL(activated()), SLOT(slotFiltersContourDespecleSelected()));

  zoomInBtn = new QAction(ApaintConfig::getText(ZOOM_IN_MSG), QAccel::stringToKey(tr("+")), this);
  zoomInBtn->setIconSet(QPixmap(getCommonPNG("zoom-in")));
  connect(zoomInBtn, SIGNAL(activated()), this, SLOT(zoomInBtnClicked()));

  zoomOutBtn = new QAction(ApaintConfig::getText(ZOOM_OUT_MSG), QAccel::stringToKey(tr("-")), this);
  zoomOutBtn->setIconSet(QPixmap(getCommonPNG("zoom-out")));
  connect(zoomOutBtn, SIGNAL(activated()), this, SLOT(zoomOutBtnClicked()));

  removeGuides = new QAction(ApaintConfig::getText(REMOVE_GUIDES_MSG), 0, this);
  connect(removeGuides, SIGNAL(activated()), this, SLOT(slotRemoveGuides()));

  repeatView = new QAction(tr("repeatview"), ApaintConfig::getText(54), QAccel::stringToKey(tr("R")), this, 0, true);
  connect(repeatView, SIGNAL(toggled(bool)), this, SLOT(setRepeatMode(bool)));
          
  viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

  sgrid = new QAction(tr("Show Grid"), ApaintConfig::getText(SHOW_GRID_MSG), QAccel::stringToKey(tr("Ctrl+G")), this, 0, true);
  connect(sgrid, SIGNAL(toggled(bool)), this, SLOT(showViewGrid(bool)));

  shRuler = new QAction(tr("Show ruler"), ApaintConfig::getText(656), QAccel::stringToKey(tr("Ctrl+R")), this, 0, true);
  connect(shRuler, SIGNAL(toggled(bool)), this, SLOT(showRuler(bool)));
  
  repeatd = new QAction(ApaintConfig::getText(REPEAT_DIM_MSG)+"...", QAccel::stringToKey(tr("Alt+R")), this);
  connect(repeatd, SIGNAL(activated()), this, SLOT(showRepeatDlg()));  
  
  helpAboutApp = new QAction(ApaintConfig::getText(ABOUT_MSG)+"...", 0, this);
  helpAboutApp->setIconSet(QPixmap(getCommonPNG("aicon")));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

  selByColor = new QAction(ApaintConfig::getText(729), QAccel::stringToKey(tr("Shift+W")), this);
  selByColor->setIconSet(QPixmap(getCommonPNG("select-by-color")));
  connect(selByColor, SIGNAL(activated()), this, SLOT(selectByColorSlot()));

  viewRepeatDelimiter = new QAction(tr("Draw delimiter lines"), ApaintConfig::getText(181), QAccel::stringToKey(tr("Alt+G")), this, 0, true);
  connect(viewRepeatDelimiter, SIGNAL(toggled(bool)), this, SLOT(setRepeatDelimiterView(bool)));

  viewAntialias = new QAction(tr("Antialias on zoom out"), ApaintConfig::getText(786), QAccel::stringToKey(tr("Ctrl+Shift+A")), this, 0, true);
  connect(viewAntialias, SIGNAL(toggled(bool)), this, SLOT(setAntialiasView(bool)));

  viewInteger = new QAction(tr("Show integer view"), ApaintConfig::getText(733), QAccel::stringToKey(tr("A")), this, 0, true);
  connect(viewInteger, SIGNAL(toggled(bool)), this, SLOT(setIntegerView(bool)));

  viewCrosshair = new QAction(tr("Show crosshair cursor"), ApaintConfig::getText(787), QAccel::stringToKey(tr("Z")), this, 0, true);
  connect(viewCrosshair, SIGNAL(toggled(bool)), this, SLOT(setCrosshairView(bool)));

  keepOriginalCmapAct = new QAction(tr("Keep original colormap"), ApaintConfig::getText(821), QAccel::stringToKey(tr("Ctrl+Shift+C")), this, 0, true);
  connect(keepOriginalCmapAct, SIGNAL(toggled(bool)), this, SLOT(setKeepOriginalCmap(bool)));

  imgRotate = new QAction(ApaintConfig::getText(85)+"...", QAccel::stringToKey(tr("Shift+R")), this);
  imgRotate->setIconSet(QPixmap(getCommonPNG("object-rotate-right")));
  connect(imgRotate, SIGNAL(activated()), this, SLOT(imageRotate()));
  imgResize = new QAction(ApaintConfig::getText(RESIZE_IMAGE_MSG)+"...", QAccel::stringToKey(tr("Shift+T")), this);
  imgResize->setIconSet(QPixmap(getCommonPNG("transform-scale")));
  connect(imgResize, SIGNAL(activated()), this, SLOT(imageResize()));
}

void ApaintApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu=new QPopupMenu();
  fileNew->addTo(fileMenu);

  if (!ApaintConfig::getHideLoadMenu())
   fileOpen->addTo(fileMenu);

  fileRecent = new QPopupMenu();
  layerRecent = new QPopupMenu();
  connect(fileRecent, SIGNAL(aboutToShow()), this, SLOT(slotRecent()));
  connect(layerRecent, SIGNAL(aboutToShow()), this, SLOT(slotRecent()));
  connect(fileRecent, SIGNAL(activated(int)), this, SLOT(slotRecentFile(int)));
  connect(layerRecent, SIGNAL(activated(int)), this, SLOT(slotRecentLayer(int)));

  fileDocumentBrowse->addTo(fileMenu);
  /* if specified in the configuration file, hide recent files */
  if (!ApaintConfig::getHideLoadRecent())
   fileMenu->insertItem(QPixmap(getCommonPNG("document-open-recent")), ApaintConfig::getText(757), fileRecent);
  fileMenu->insertSeparator();

  /* if specified in the configuration file, hide save menu. */
  if (!ApaintConfig::getHideSaveMenu())
   fileSave->addTo(fileMenu);
  fileSaveAs->addTo(fileMenu);

  fileMenu->insertSeparator();
  fileReload->addTo(fileMenu);

  fileMenu->insertSeparator();

  if (!ApaintConfig::getHideLoadMenu())
   loadLayer->addTo(fileMenu);
  fileBrowseLayer->addTo(fileMenu);
  fileBrowseLayer->setEnabled(false);
  /* if specified in the configuration file, hide recent files */
  if (!ApaintConfig::getHideLoadRecent())
   fileMenu->insertItem(QPixmap(getCommonPNG("document-open-recent")), ApaintConfig::getText(757), layerRecent);
  layerRecent->setEnabled(false);

  saveLayer->addTo(fileMenu);

  fileMenu->insertSeparator();
  filePrint->addTo(fileMenu);

  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  editMenu=new QPopupMenu();
  editUndo->addTo(editMenu);
  editRedo->addTo(editMenu);
  editMenu->insertSeparator();
  editCut->addTo(editMenu);
  editCopy->addTo(editMenu);
  editPaste->addTo(editMenu);
  editMenu->insertSeparator();
  editSelectAll->addTo(editMenu);
  editClear->addTo(editMenu);
  editDelete->addTo(editMenu);
  editMenu->setEnabled(false);
  editPaste->setEnabled(false);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry imageMenu
  imageMenu=new QPopupMenu();

  viewRepeat->addTo(imageMenu);

  imageMenu->insertSeparator();

  duplicateXMenu = new QPopupMenu();
  duplicateX1->addTo(duplicateXMenu);
  duplicateX2->addTo(duplicateXMenu);
  duplicateX3->addTo(duplicateXMenu);
  duplicateX4->addTo(duplicateXMenu);
  duplicateX5->addTo(duplicateXMenu);
  duplicateX6->addTo(duplicateXMenu);
  duplicateX7->addTo(duplicateXMenu);
  duplicateX8->addTo(duplicateXMenu);
  duplicateX9->addTo(duplicateXMenu);
  duplicateX10->addTo(duplicateXMenu);

  duplicateYMenu = new QPopupMenu();
  duplicateY1->addTo(duplicateYMenu);
  duplicateY3->addTo(duplicateYMenu);
  duplicateY4->addTo(duplicateYMenu);
  duplicateY2->addTo(duplicateYMenu);
  duplicateY5->addTo(duplicateYMenu);
  duplicateY7->addTo(duplicateYMenu);
  duplicateY6->addTo(duplicateYMenu);
  duplicateY8->addTo(duplicateYMenu);
  duplicateY9->addTo(duplicateYMenu);
  duplicateY10->addTo(duplicateYMenu);

  imageMenu->insertItem(QPixmap(getCommonPNG("duplicate-horizontal")), ApaintConfig::getText(DUPLICATE_MSG)+" X", duplicateXMenu);
  imageMenu->insertItem(QPixmap(getCommonPNG("duplicate-vertical")), ApaintConfig::getText(DUPLICATE_MSG)+" Y", duplicateYMenu);

  multiplyXMenu = new QPopupMenu();
  multiplyX2->addTo(multiplyXMenu);
  multiplyX3->addTo(multiplyXMenu);
  multiplyX4->addTo(multiplyXMenu);
  multiplyX5->addTo(multiplyXMenu);
  multiplyX6->addTo(multiplyXMenu);
  multiplyX7->addTo(multiplyXMenu);
  multiplyX8->addTo(multiplyXMenu);
  multiplyX9->addTo(multiplyXMenu);
  multiplyX10->addTo(multiplyXMenu);

  multiplyYMenu = new QPopupMenu();
  multiplyY2->addTo(multiplyYMenu);
  multiplyY3->addTo(multiplyYMenu);
  multiplyY4->addTo(multiplyYMenu);
  multiplyY5->addTo(multiplyYMenu);
  multiplyY6->addTo(multiplyYMenu);
  multiplyY7->addTo(multiplyYMenu);
  multiplyY8->addTo(multiplyYMenu);
  multiplyY9->addTo(multiplyYMenu);
  multiplyY10->addTo(multiplyYMenu);

  imageMenu->insertItem(QPixmap(getCommonPNG("object-flip-horizontal")), ApaintConfig::getText(MULTIPLY_MSG)+" X", multiplyXMenu);
  imageMenu->insertItem(QPixmap(getCommonPNG("object-flip-vertical")), ApaintConfig::getText(MULTIPLY_MSG)+" Y", multiplyYMenu);

  smoothMultiplyMenu = new QPopupMenu();
  scale2X->addTo(smoothMultiplyMenu);
  scale3X->addTo(smoothMultiplyMenu);
  scale4X->addTo(smoothMultiplyMenu);
  imageMenu->insertItem(QPixmap(getCommonPNG("transform-scale")), ApaintConfig::getText(SMOOTH_MULTIPLY_MSG), smoothMultiplyMenu);

  imageMenu->insertSeparator();

  shrinkX->addTo(imageMenu);
  shrinkY->addTo(imageMenu);

  smoothShrinkMenu = new QPopupMenu();
  shrink2X->addTo(smoothShrinkMenu);
  shrink3X->addTo(smoothShrinkMenu);
  shrink4X->addTo(smoothShrinkMenu);
  imageMenu->insertItem(QPixmap(getCommonPNG("transform-reduce")), ApaintConfig::getText(SMOOTH_SHRINK_MSG), smoothShrinkMenu);

  halveX->addTo(imageMenu);
  halveY->addTo(imageMenu);

  imageMenu->insertSeparator();

  editScale->addTo(imageMenu);
  imageMenu->insertItem(ApaintConfig::getText(45), REDUCE_SEL_BORDER);

  imageMenu->insertSeparator();

  imgResize->addTo(imageMenu);
  imgRotate->addTo(imageMenu);

  imageMenu->setEnabled(false);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu=new QPopupMenu();
  zoomInBtn->addTo(viewMenu);
  zoomOutBtn->addTo(viewMenu);

  viewMenu->insertItem(QPixmap(getCommonPNG("form-table-control")), ApaintConfig::getText(GRID_PROP_MSG)+"...", GRID_PROP);
  repeatd->addTo(viewMenu);
  
  removeGuides->addTo(viewMenu);
  
  viewMenu->insertSeparator();
  sgrid->addTo(viewMenu);
  shRuler->addTo(viewMenu);
  repeatView->addTo(viewMenu);
  viewRepeatDelimiter->addTo(viewMenu);
  viewMenu->insertSeparator();
  viewAntialias->addTo(viewMenu);
  viewInteger->addTo(viewMenu);
  viewCrosshair->addTo(viewMenu);
  viewMenu->setEnabled(false);
 
  ///////////////////////////////////////////////////////////////////

  convertMenu = new QPopupMenu();
  colorTrueToCmap->addTo(convertMenu);
  colorCmapToTrue->addTo(convertMenu);
  convertMenu->insertItem(QPixmap(getCommonPNG("color-gradient")), QString(ApaintConfig::getText(159)), IMAGE_TO_GREYSCALE);
  colorMenu = new QPopupMenu();  
  colorNum->addTo(colorMenu);
  colorReduce->addTo(colorMenu);
  keepOriginalCmapAct->addTo(colorMenu);
  loadColormap->addTo(colorMenu);
  loadColormapDither->addTo(colorMenu);
  
  colorRemap->addTo(colorMenu);

  sortColorsMenu = new QPopupMenu();
  sortColorsPopularity->addTo(sortColorsMenu);
  sortColorsHue->addTo(sortColorsMenu);
  sortColorsSaturation->addTo(sortColorsMenu);
  sortColorsLightness->addTo(sortColorsMenu);
  sortColorsSimilarity->addTo(sortColorsMenu);

  colorMenu->insertItem(QPixmap(getCommonPNG("view-sort-descending")), ApaintConfig::getText(SORT_COLORS_MSG), sortColorsMenu);
  colorMenu->insertItem(QPixmap(getCommonPNG("histogram")), ApaintConfig::getText(647)+"...", HISTOGRAM_DIALOG);

  countColorsMenu = new QPopupMenu();
  colorCountHorizontal->addTo(countColorsMenu);
  colorCountVertical->addTo(countColorsMenu);

  colorMenu->insertItem(ApaintConfig::getText(COUNT_COLORS_MSG), countColorsMenu);
  colorMenu->insertItem(ApaintConfig::getText(MAKE_COLOR_SHADES_MSG), MAKE_COLOR_SHADES);

  protectedColorsMenu = new QPopupMenu();
  colorInvertProtected->addTo(protectedColorsMenu);
  colorClearProtected->addTo(protectedColorsMenu);
  colorMenu->insertItem(QPixmap(getCommonPNG("lock")), ApaintConfig::getText(PROTECTED_COLORS_MSG), protectedColorsMenu);

  transparentColorsMenu = new QPopupMenu();
  colorInvertTransparent->addTo(transparentColorsMenu);
  colorClearTransparent->addTo(transparentColorsMenu);
  colorMenu->insertItem(QPixmap(getCommonPNG("transparency")), ApaintConfig::getText(TRANSPARENT_COLORS_MSG), transparentColorsMenu);

  colorInvertMenu = new QPopupMenu();
  colorInvertAll->addTo(colorInvertMenu);
  colorInvertRed->addTo(colorInvertMenu);
  colorInvertGreen->addTo(colorInvertMenu);
  colorInvertBlue->addTo(colorInvertMenu);
  colorInvertCyan->addTo(colorInvertMenu);
  colorInvertYellow->addTo(colorInvertMenu);
  colorInvertMagenta->addTo(colorInvertMenu);
  colorInvertGrey->addTo(colorInvertMenu);
  colorInvertSaturation->addTo(colorInvertMenu);
  colorMenu->insertItem(QPixmap(getCommonPNG("invert")), ApaintConfig::getText(INVERT_MSG), colorInvertMenu);


  colorSwapMenu = new QPopupMenu();
  colorSwapPalette->addTo(colorSwapMenu);
  colorSwapRGB_GBR->addTo(colorSwapMenu);
  colorSwapRGB_BRG->addTo(colorSwapMenu);
  colorSwapRG->addTo(colorSwapMenu);
  colorSwapRB->addTo(colorSwapMenu);
  colorSwapGB->addTo(colorSwapMenu);
  colorSwapSL->addTo(colorSwapMenu);
  colorMenu->insertItem(QPixmap(getCommonPNG("exchange")), ApaintConfig::getText(SWAP_MSG), colorSwapMenu);

  contrastStretch->addTo(colorMenu);
  colorEdit->addTo(colorMenu);


  colorMenu->insertItem(ApaintConfig::getText(CONVERT_MSG), convertMenu);  
  colorMenu->setEnabled(false);

  toolsMenu = new QPopupMenu();
  toolContour->addTo(toolsMenu);
  toolContourRemove->addTo(toolsMenu);
  toolContourThin->addTo(toolsMenu);
  toolContourPrune->addTo(toolsMenu);

  toolsMenu->insertSeparator(); 

  selByColor->addTo(toolsMenu);

  toolInvert->addTo(toolsMenu);
  toolInvert->setEnabled(false);

  toolFlatten->addTo(toolsMenu);
  toolFlatten->setEnabled(false);


  toolsMenu->insertSeparator(); 
  toolLayerToBrush->addTo(toolsMenu);
  toolBrushToLayer->addTo(toolsMenu);
  toolBrushToLayer->setEnabled(false);

  toolsMenu->insertSeparator(); 

  findRepeat->addTo(toolsMenu);
  findExactRepeat->addTo(toolsMenu);

  editPasteMultiple->addTo(toolsMenu);
  toolsMenu->insertItem(ApaintConfig::getText(195)+"...", GUESS_WEAVE_DLG);
  toolsMenu->setEnabled(false);

  filtersMenu = new QPopupMenu();

  repeatLastMenu = new QPopupMenu();
  filtersRepeatLast1->addTo(repeatLastMenu);
  filtersRepeatLast10->addTo(repeatLastMenu);
  filtersMenu->insertItem(ApaintConfig::getText(REPEAT_LAST_MSG), repeatLastMenu);

  filtersContourAverage->addTo(filtersMenu);

  despecleMenu = new QPopupMenu();
  filtersContourDespecleAll->addTo(despecleMenu);
  filtersContourDespecleSelected->addTo(despecleMenu);

  filtersMenu->insertItem(ApaintConfig::getText(DESPECLE_CONT_MSG), despecleMenu);
  filtersMenu->insertSeparator();

  noiseMenu = new QPopupMenu();
  noiseMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), CREATE_NOISE_XY);
  noiseMenu->insertItem(ApaintConfig::getText(HORIZONTAL_MSG), CREATE_NOISE_X);
  noiseMenu->insertItem(ApaintConfig::getText(VERTICAL_MSG), CREATE_NOISE_Y);
  filtersMenu->insertItem(QPixmap(getCommonPNG("noise")), ApaintConfig::getText(NOISE_MSG), noiseMenu);
  filtersMenu->setEnabled(false);

  selfAvoidingMotifsMenu = new QPopupMenu();
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-square-filled")), "",        SELF_AVOID_MOTIFS1);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-diamond-filled")), "",       SELF_AVOID_MOTIFS2);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-circle-filled")), "",        SELF_AVOID_MOTIFS3);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-equilateral")), "", SELF_AVOID_MOTIFS4);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-square")), "",      SELF_AVOID_MOTIFS5);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line")), "",                 SELF_AVOID_MOTIFS6);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line-back")), "",            SELF_AVOID_MOTIFS7);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-X")), "",                    SELF_AVOID_MOTIFS8);
  selfAvoidingMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-cross")), "",                SELF_AVOID_MOTIFS9);
  filtersMenu->insertItem(QPixmap(getCommonPNG("self-avoiding-placement")), ApaintConfig::getText(SELF_AVOIDING_MOTIFS_MSG), selfAvoidingMotifsMenu);

  randomMotifsMenu = new QPopupMenu();
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-square-filled")),        "", RANDOM_MOTIFS1);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-diamond-filled")),       "", RANDOM_MOTIFS2);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-circle-filled")),        "", RANDOM_MOTIFS3);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-equilateral")), "", RANDOM_MOTIFS4);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-square")),      "", RANDOM_MOTIFS5);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line")),                 "", RANDOM_MOTIFS6);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line-back")),            "", RANDOM_MOTIFS7);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-X")),                    "", RANDOM_MOTIFS8);
  randomMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-cross")),                "", RANDOM_MOTIFS9);
  filtersMenu->insertItem(QPixmap(getCommonPNG("random-placement")), ApaintConfig::getText(RANDOM_MOTIFS_MSG), randomMotifsMenu);

  placeMotifsMenu = new QPopupMenu();
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-square-filled")),        "", PLACE_MOTIFS1);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-diamond-filled")),       "", PLACE_MOTIFS2);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-circle-filled")),        "", PLACE_MOTIFS3);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-equilateral")), "", PLACE_MOTIFS4);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-triangle-square")),      "", PLACE_MOTIFS5);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line")),                 "", PLACE_MOTIFS6);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-line-back")),            "", PLACE_MOTIFS7);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-X")),                    "", PLACE_MOTIFS8);
  placeMotifsMenu->insertItem(QPixmap(getCommonPNG("draw-cross")),                "", PLACE_MOTIFS9);
  filtersMenu->insertItem(QPixmap(getCommonPNG("space-placement")), ApaintConfig::getText(PLACE_MOTIFS_MSG), placeMotifsMenu);

  ikatMenu = new QPopupMenu();
  ikatMenu->insertItem(QPixmap(getCommonPNG("ikat")), ApaintConfig::getText(HORIZONTAL_MSG), IKAT_X);
  ikatMenu->insertItem(QPixmap(getCommonPNG("ikat2")), ApaintConfig::getText(VERTICAL_MSG), IKAT_Y);
  ikatMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), IKAT_XY);
  filtersMenu->insertItem(QPixmap(getCommonPNG("ikat")), ApaintConfig::getText(IKAT_MSG), ikatMenu);

  marbleMenu = new QPopupMenu();
  marbleMenu->insertItem(QPixmap(getCommonPNG("marble")), ApaintConfig::getText(HORIZONTAL_MSG), MARBLE_X);
  marbleMenu->insertItem(QPixmap(getCommonPNG("marble2")), ApaintConfig::getText(VERTICAL_MSG), MARBLE_Y);
  marbleMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), MARBLE_XY);
  filtersMenu->insertItem(QPixmap(getCommonPNG("marble")), ApaintConfig::getText(MARBLE_MSG), marbleMenu);

  zigzagMenu = new QPopupMenu();
  zigzagMenu->insertItem(QPixmap(getCommonPNG("zigzag")), ApaintConfig::getText(HORIZONTAL_MSG), ZIGZAG_X);
  zigzagMenu->insertItem(QPixmap(getCommonPNG("zigzag2")), ApaintConfig::getText(VERTICAL_MSG), ZIGZAG_Y);
  zigzagMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), ZIGZAG_XY);
  filtersMenu->insertItem(QPixmap(getCommonPNG("zigzag")), ApaintConfig::getText(782), zigzagMenu);

  waveMenu = new QPopupMenu();
  waveMenu->insertItem(QPixmap(getCommonPNG("wave")), ApaintConfig::getText(HORIZONTAL_MSG), WAVE_X);
  waveMenu->insertItem(QPixmap(getCommonPNG("wave2")), ApaintConfig::getText(VERTICAL_MSG), WAVE_Y);
  waveMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), WAVE_XY);
  filtersMenu->insertItem(QPixmap(getCommonPNG("wave")), ApaintConfig::getText(783), waveMenu);

  mirrorMenu = new QPopupMenu();
  mirrorMenu->insertItem(QPixmap(getCommonPNG("mirror-segment")), ApaintConfig::getText(HORIZONTAL_MSG), MIRROR_Y);
  mirrorMenu->insertItem(QPixmap(getCommonPNG("mirror-segment2")), ApaintConfig::getText(VERTICAL_MSG), MIRROR_X);
  filtersMenu->insertItem(QPixmap(getCommonPNG("mirror-segment")), ApaintConfig::getText(MIRROR_SEGMENT_MSG), mirrorMenu);

  gradientMenu = new QPopupMenu();
  gradientMenu->insertItem(ApaintConfig::getText(HORIZONTAL_MSG), GRADIENT_X);
  gradientMenu->insertItem(ApaintConfig::getText(VERTICAL_MSG), GRADIENT_Y);
  filtersMenu->insertItem(QPixmap(getCommonPNG("color-gradient")), ApaintConfig::getText(GRADIENT_MSG), gradientMenu);

  filtersMenu->insertItem(QPixmap(getCommonPNG("contour")), ApaintConfig::getText(CONTOURS_MSG), CONTOUR);

  coralMenu = new QPopupMenu();
  coralMenu->insertItem(QPixmap(getCommonPNG("coral1")), "1", CORAL_1);
  coralMenu->insertItem(QPixmap(getCommonPNG("coral2")), "2", CORAL_2);
  coralMenu->insertItem(QPixmap(getCommonPNG("coral3")), "3", CORAL_3);
  coralMenu->insertSeparator(); 
  coralMenu->insertItem(QPixmap(getCommonPNG("coral4")), "4", CORAL_4);
  coralMenu->insertItem(QPixmap(getCommonPNG("coral5")), "5", CORAL_5);
  coralMenu->insertItem(QPixmap(getCommonPNG("coral6")), "6", CORAL_6);
  filtersMenu->insertItem(QPixmap(getCommonPNG("coral1")), ApaintConfig::getText(CORAL_MSG), coralMenu);

  colorShorterInsideMenu = new QPopupMenu();
  colorShorterInsideMenu->insertItem(ApaintConfig::getText(HORIZONTAL_MSG), COLOR_SHORTER_INSIDE_X);
  colorShorterInsideMenu->insertItem(ApaintConfig::getText(VERTICAL_MSG), COLOR_SHORTER_INSIDE_Y);
  filtersMenu->insertItem(ApaintConfig::getText(765), colorShorterInsideMenu);

  colorShorterOutsideMenu = new QPopupMenu();
  colorShorterOutsideMenu->insertItem(ApaintConfig::getText(HORIZONTAL_MSG), COLOR_SHORTER_OUTSIDE_X);
  colorShorterOutsideMenu->insertItem(ApaintConfig::getText(VERTICAL_MSG), COLOR_SHORTER_OUTSIDE_Y);
  filtersMenu->insertItem(ApaintConfig::getText(766), colorShorterOutsideMenu);

  mixMenu = new QPopupMenu();
  mixMenu->insertItem(ApaintConfig::getText(HORIZONTAL_VERTICAL_MSG), MIX_XY);
  mixMenu->insertItem(ApaintConfig::getText(HORIZONTAL_MSG), MIX_X);
  mixMenu->insertItem(ApaintConfig::getText(VERTICAL_MSG), MIX_Y);
  filtersMenu->insertItem(QPixmap(getCommonPNG("draw-airbrush")), ApaintConfig::getText(767), mixMenu);
  
  ///////////////////////////////////////////////////////////////////
  // EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();

  saveSetupMenu = new QPopupMenu();
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("China")), "Chinese (simplified)", SET_LANGUAGE_CHINESE_SIMPLIFIED);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Taiwan")), "Chinese (traditional)", SET_LANGUAGE_CHINESE_TRADITIONAL);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Netherlands")), "Dutch", SET_LANGUAGE_DUTCH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("English")), "English", SET_LANGUAGE_ENGLISH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Finland")), "Finnish", SET_LANGUAGE_FINNISH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("France")), "French", SET_LANGUAGE_FRENCH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("German")), "German", SET_LANGUAGE_GERMAN);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Italy")), "Italian", SET_LANGUAGE_ITALIAN);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Poland")), "Polish", SET_LANGUAGE_POLISH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Slovenia")), "Slovenian", SET_LANGUAGE_SLOVENIAN);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Spain")), "Spanish", SET_LANGUAGE_SPANISH);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Thailand")), "Thai", SET_LANGUAGE_THAI);
  saveSetupMenu->insertItem(QPixmap(getCommonPNG("Turkey")), "Turkish", SET_LANGUAGE_TURKISH);
  helpMenu->insertItem(QPixmap(getCommonPNG("preferences-system")), ApaintConfig::getText(SAVE_SETUP_MSG), saveSetupMenu);

  helpMenu->insertItem(QPixmap(getCommonPNG("manual")), ApaintConfig::getText(USER_MANUAL_MSG)+"...", OPEN_PDF_MANUAL);
  helpMenu->insertItem(QPixmap(getCommonPNG("shortcuts")), ApaintConfig::getText(SHORTCUTS_MSG)+"...", OPEN_PDF_SHORTCUTS);
  helpMenu->insertItem(QPixmap(getCommonPNG("pdf")), ApaintConfig::getText(FAST_PATTERN_MSG)+"...", OPEN_PDF_FAST_PATTERN);
  helpMenu->insertItem(QPixmap(getCommonPNG("paragraph")), ApaintConfig::getText(LICENSE_MSG)+"...", OPEN_LICENSE_WEB_PAGE);
  helpMenu->insertItem(QPixmap(getCommonPNG("emblem-web")), "Arahne...", OPEN_ARAHNE_WEB_PAGE);
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  mBar = new QMenuBar(this);
  mBar->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed, FALSE);
  // MENUBAR CONFIGURATION
  mBar->insertItem(ApaintConfig::getText(FILE_MSG), fileMenu);
  mBar->insertItem(ApaintConfig::getText(EDIT_MSG), editMenu);
  mBar->insertItem(ApaintConfig::getText(COLORS_MSG), colorMenu);
  mBar->insertItem(ApaintConfig::getText(IMAGE_MSG), imageMenu);
  mBar->insertItem(ApaintConfig::getText(TOOLS_MSG), toolsMenu);
  mBar->insertItem(ApaintConfig::getText(FILTERS_MSG), filtersMenu);
  mBar->insertItem(ApaintConfig::getText(VIEW_MSG), viewMenu);
  mBar->insertSeparator();
  mBar->insertItem(ApaintConfig::getText(HELP_MSG), helpMenu);
  mBar->setMaximumWidth(600);
}

void ApaintApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  fileToolbar = new QToolBar(QString("Toolbar"),0,this,FALSE,"file operations",0);
  fileNew->addTo(fileToolbar);
  fileOpen->addTo(fileToolbar);
  fileSave->addTo(fileToolbar);

  statusBar= new QStatusBar(this,"status bar");
  fileToolbar->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum, FALSE);
}

void ApaintApp::setImagePath(QString path)
{
  imgPath = path;
  doc->setDirName(imgPath);
}           

void ApaintApp::initMainObjects()
{
  //////////////////////////WATCH OUT THE ORDER OF OBJECT CREATION!
 
  /*init doc*/
  doc=new ApaintDoc(this);
  // set the main widget here
  centralWidget = new QWidget(this, "centralWidget", WRepaintNoErase|WResizeNoErase|WStaticContents);
  centralContainer = new QGridLayout(centralWidget, 2, 2);
  //create main view obj
  view=new ApaintView(centralWidget, doc);

  /*create file dialog*/
  fileDlg = new QFileDialog(this, "fileDialog");
  doc->setFileDlg(fileDlg);

  /*icon view dialog*/
  iconDlg = new IconViewDlg(this, "iconViewDialog", doc);
  
  // create vruler&hruler
  vruler = new Ruler(Qt::Vertical, centralWidget);
  vruler->drawNumRevers(true);
  vruler->setEndOffset(0);
  
  hruler = new Ruler(Qt::Horizontal, centralWidget);
  hruler->setEndOffset(0);  
  
  rInfo = new RulerInfo(centralWidget, "rInfo", Ruler::Pixel);
  centralContainer->addWidget(rInfo, 0, 0);
  centralContainer->addWidget(hruler, 0, 1);
  centralContainer->addWidget(vruler, 1, 0);
  centralContainer->addWidget(view, 1, 1);

  /*init draw object*/
  draw= new ApaintDraw(doc, view);

  /*drawing int repeat*/
  drawRep = new DrawInRepeat(doc, draw);

  /*init tool control object*/
  tCtrl = new ToolsCtrl(drawRep, doc);
  /*init repDim dialog*/
  repDim = new RepeatDimDlg(this, "repeat dim");
  /*init scaleRot dialog*/
  srot = new ScaleRotDialog(this, "scaleRotDialog", repDim);

  /*color manip dialogs*/
  cnDlg = new ColNumDlg(this, "color num dialog", doc);
  rcDlg = new ReduceColorDlg(this, "reduce color dialog", doc, fileDlg, iconDlg);
  colRemapDialog = new ColorRemapDialog(this, "color remap dialog", doc);
  vertWid = new VerticalyAlignedWidget(this, "vertical align");
  drawToolBar = new DrawToolBar(vertWid, "draw operations",tCtrl);
  pbUtil = new PenBrushUtil(vertWid, "colorPos", doc);
  toolTab = new QTabWidget(vertWid, "toolTab");

  vertWid->setMaximumWidth(193);
  vertWid->addWidget(drawToolBar);
  vertWid->addWidget(pbUtil);
  colP = new ColPaletteDlg(toolTab, "ColorPalette", doc);

  /*init layer selection dialog*/
  layerSel = new LayerSelDlg(toolTab, "layer selection dialog", doc);
  layerSelSimple = new LayerSelSimpleWid(vertWid, "layer selection simple", doc);
  /*init brush list widget*/
  bListWid = new BrushListWidget(toolTab, "brush list widget", doc);
  /*tooltab stuff*/
  QIconSet *iset = new QIconSet(QPixmap(getCommonPNG("applications-graphics")));
  toolTab->addTab(colP, *iset, "");
  toolTab->setTabToolTip(colP, ApaintConfig::getText(61)+"   F6");

  QIconSet *iset2 = new QIconSet(QPixmap(getCommonPNG("layers")));  
  toolTab->addTab(layerSel, *iset2, "");
  toolTab->setTabToolTip(layerSel, ApaintConfig::getText(645)+"   F5");

  QIconSet *iset3 = new QIconSet(QPixmap(getCommonPNG("draw-clone")));  
  toolTab->addTab(bListWid, *iset3, "");
  toolTab->setTabToolTip(bListWid, ApaintConfig::getText(722)+"   F7");

  vertWid->addWidget(layerSelSimple);
  vertWid->addWidget(toolTab);

  drawToolBar->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed, FALSE);
  pbUtil->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed, FALSE);
  toolTab->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding, FALSE);
  vertWid->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding, FALSE);
  QVBoxLayout *vertLay = new QVBoxLayout(0,"vboxlay");
  vertLay->addWidget(mBar);
  vertLay->addWidget(fileToolbar);
  vertLay->addWidget(centralWidget);
  vertLay->addWidget(statusBar);

 QHBoxLayout *mainLay =new QHBoxLayout(this);
  mainLay->addLayout(vertLay);
  mainLay->addWidget(vertWid,0,0);
  /*init tools properties*/
  sprayProp= new SprayPropWid(pbUtil, "Spray prop.");
  CommonProp::createObject(pbUtil, "commonProp");
  cProp = CommonProp::getObject();
  stProp = new StitchPropWid(pbUtil, "stitchProp");
  cleanupProp = new CleanupProp(pbUtil, "cleanupProp");
  brushProp = new BrushToolProp(pbUtil, "brushProp");
  fillProp = new FillToolProp(pbUtil, "fillProp");
  measureToolProp =  new MeasureToolProp(pbUtil, "measureProp");
  selectProp = new SelectProp(pbUtil, "SelectToolProp");
  pbUtil->setSelectProp(selectProp);
  rectProp = new RectProp(pbUtil, "RectToolProp");
  circleProp = new RectProp(pbUtil, "CircleToolProp");
  ellipseProp = new RectProp(pbUtil, "EllipseToolProp");
  quadProp = new RectProp(pbUtil, "QuadToolProp");
  magicWandProp =  new MagicToolProp(pbUtil, "magicWandProp", selectProp);
  bezierProp = new BezierToolProp(pbUtil, "bezierProp");
 
  pbUtil->putInWidget(cProp);
  pbUtil->putInWidget(sprayProp);
  pbUtil->putInWidget(stProp);
  pbUtil->putInWidget(cleanupProp);
  pbUtil->putInWidget(brushProp);
  pbUtil->putInWidget(fillProp);
  pbUtil->putInWidget(measureToolProp);
  pbUtil->putInWidget(selectProp);
  pbUtil->putInWidget(rectProp);
  pbUtil->putInWidget(circleProp);
  pbUtil->putInWidget(ellipseProp);
  pbUtil->putInWidget(quadProp);
  pbUtil->putInWidget(magicWandProp);
  pbUtil->putInWidget(bezierProp);
  
  sprayProp->hide();
  stProp->hide();
  cleanupProp->hide();
  brushProp->hide();
  fillProp->hide();
  measureToolProp->hide();
  magicWandProp->hide();
  bezierProp->hide();
  rectProp->hide();
  circleProp->hide();
  ellipseProp->hide();
  quadProp->hide();

  /*PropObject's ...*/
  pSpray = new PropObject(sprayProp);
  pStitchTool = new PropObject(stProp);
  pCleanup = new PropObject(cleanupProp);
  pBrushProp = new PropObject(brushProp);
  pFillProp = new PropObject(fillProp);
  pMagicWand = new PropObject(magicWandProp);
  pMeasureProp = new PropObject(measureToolProp);
  pSelectProp = new PropObject(selectProp);
  pRectProp = new PropObject(rectProp);
  pCircleProp = new PropObject(circleProp);
  pEllipseProp = new PropObject(ellipseProp);
  pQuadProp = new PropObject(quadProp);
  pBezierProp = new PropObject(bezierProp);

  /*dialog for inserting and removing pixels*/
  dlgInsRemPix = new InsertRemovePixelsDialog(this, "InsertRemovePixelsDialog");
  //FreeTransform dialog
  fdlg = new FreeTransformDialog(this, "FreeTransform dialog");

  /*create tools*/
  straightLineTool = new StraightLineTool(drawRep, doc, 0);
  bezierTool = new BezierTool(drawRep, doc, bezierProp, pBezierProp);
  bezierProp->setTool(bezierTool);
  measureTool = new LineTool(drawRep, doc, DrawTool, measure, pMeasureProp);
  circleTool = new RectTool(drawRep, doc,  DrawTool, CircleRect,  pCircleProp);
  ellipseTool = new RectTool(drawRep, doc, DrawTool, EllipseRect, pEllipseProp);
  rectTool = new RectTool(drawRep, doc, DrawTool, NormalRect, pRectProp);
  quadTool = new RectTool(drawRep, doc, DrawTool, QuadRect, pQuadProp);
  brushTool = new BrushTool(drawRep, DrawTool, pBrushProp, doc);
  fillTool = new FillTool(drawRep, doc, DrawTool, pFillProp);
  sprayTool = new SprayTool(drawRep, DrawTool, NormalSpray, doc, pSpray);
  mixTool   = new SprayTool(drawRep, DrawTool, MixSpray, doc, pSpray);
  pLineTool = new LineTool(drawRep, doc, DrawTool, rbLine, 0);
  eArcTool = new ElipArcTool(drawRep, DrawTool, 0);
  selTool = new SelectTool(SelectionTool, Rectangle, doc, pSelectProp);
  magicWandTool = new MagicWandTool(drawRep, doc, SelectionTool, pMagicWand);
  transTool = new TransformTool(SelectionTool, doc, 0, srot);
  hshift = new ShiftImageTool(drawRep, SelectionTool, doc, HorShift, 0);
  vshift = new ShiftImageTool(drawRep, SelectionTool, doc, VertShift, 0);
  freehand = new FreeHandSelect(SelectionTool, doc, pSelectProp);
  mirrorxTool = new SimpleTransformTool(ActionTool, Mirrorx, doc);
  mirroryTool = new SimpleTransformTool(ActionTool, Mirrory, doc);
  explodeTool = new SimpleTransformTool(ActionTool, Explode, doc);
  flipTool = new SimpleTransformTool(ActionTool, Flip, doc);
  writeTool = new WriteTool(this, SelectionTool,drawRep, doc, view, pbUtil, pSelectProp);
  stTool = new StitchTool(drawRep, doc, DrawTool, pStitchTool);
  zoomin = new ZoomTool(ActionTool, ZoomInType);
  zoomout = new ZoomTool(ActionTool, ZoomOutType);
  bsTool = new BSplineTool(drawRep, DrawTool, 0);
  lMoveTool = new LayerMoveTool(SelectionTool, doc, pSelectProp);
  unselTool = new UnselectTool(ActionTool, doc);
  cleanTool = new CleanupTool(drawRep, DrawTool, doc,  pCleanup);
  insRemPixTool = new InsertRemovePixelsTool(SelectionTool, drawRep, doc, dlgInsRemPix, 0);
  undoTool = new UnRedoTool(ActionTool, UndoOp);
  redoTool = new UnRedoTool(ActionTool, RedoOp);
  freeTransTool = new FreeTransformTool(SelectionTool, doc, fdlg, 0);
  
  /*add them to tool win*/
  drawToolBar->addTool(selTool, ApaintConfig::getText(683)+"   M");
  drawToolBar->addTool(freehand, ApaintConfig::getText(684)+"   L");
  drawToolBar->addTool(magicWandTool, ApaintConfig::getText(728)+"   W");
  drawToolBar->addTool(unselTool, ApaintConfig::getText(685)+"   Ctrl+D");
  drawToolBar->addTool(lMoveTool, ApaintConfig::getText(686)+"   V");

//  drawToolBar->addTool(lineTool, ApaintConfig::getText(688)+"   I");
  drawToolBar->addTool(straightLineTool, ApaintConfig::getText(688)+"   I");
  drawToolBar->addTool(pLineTool, ApaintConfig::getText(689)+"   N");
  drawToolBar->addTool(eArcTool, ApaintConfig::getText(690)+"   D");
  drawToolBar->addTool(bsTool, ApaintConfig::getText(691)+"   Shift+D");
  drawToolBar->addTool(bezierTool, ApaintConfig::getText(792)+"   Shift+B");

  drawToolBar->addTool(circleTool, ApaintConfig::getText(693)+"   O");
  drawToolBar->addTool(ellipseTool, ApaintConfig::getText(694)+"   Shift+O");
  drawToolBar->addTool(rectTool, ApaintConfig::getText(695)+"   Q");
  drawToolBar->addTool(quadTool, ApaintConfig::getText(696)+"   Shift+Q");    
  drawToolBar->addTool(brushTool, ApaintConfig::getText(692)+"   P");

  drawToolBar->addTool(sprayTool, ApaintConfig::getText(697)+"   J");
  drawToolBar->addTool(mixTool, ApaintConfig::getText(698)+"   Shift+J");
  drawToolBar->addTool(cleanTool, ApaintConfig::getText(699)+"   E");  
  drawToolBar->addTool(hshift, ApaintConfig::getText(700)+"   H");
  drawToolBar->addTool(vshift, ApaintConfig::getText(701)+"   Shift+H");

  drawToolBar->addTool(flipTool, ApaintConfig::getText(702)+"   Y");
  drawToolBar->addTool(mirrorxTool, ApaintConfig::getText(MIRROR_HORIZONTAL_MSG)+"   Shift+F");
  drawToolBar->addTool(mirroryTool, ApaintConfig::getText(MIRROR_VERTICAL_MSG)+"   F");
  drawToolBar->addTool(stTool, ApaintConfig::getText(707)+"   Shift+K");
  drawToolBar->addTool(writeTool, ApaintConfig::getText(708)+"   T");

  drawToolBar->addTool(transTool, ApaintConfig::getText(687)+"   Ctrl+T");
  drawToolBar->addTool(freeTransTool, ApaintConfig::getText(736)+"   Shift+P");
  drawToolBar->addTool(insRemPixTool, ApaintConfig::getText(680)+"   G");
  drawToolBar->addTool(measureTool, ApaintConfig::getText(769)+"   Shift+I");
  drawToolBar->addTool(fillTool, ApaintConfig::getText(668)+"   K");

  drawToolBar->addTool(zoomin, ApaintConfig::getText(601)+"   +");
  drawToolBar->addTool(zoomout, ApaintConfig::getText(602)+"   -");
  drawToolBar->addTool(explodeTool, ApaintConfig::getText(REPEAT_FROM_CENTER_MSG)+"   Shift+E");
  drawToolBar->addTool(undoTool, ApaintConfig::getText(26)+"   Ctrl+Z");
  drawToolBar->addTool(redoTool, ApaintConfig::getText(600)+"   Ctrl+Y");

  drawToolBar->endToolAdd();

  /*init unredo object*/
  unredo= new ApaintUnRedo(doc, transTool, colP);      

  /*multiple paste dialog*/
  mpasteDlg = new MultiplePasteDlg(this, "multiple paste dlg", false, doc);

  /*init new file dialog*/
  nfile = new NewFileDlg(this, "new file dlg", true, doc, pbUtil);
  
  /*grid properties dialog*/
  gridProp = new GridPropDlg(this, "gridProperties");
  gridProp->setGridSize(8, 8);

  repeatDlg = new DrawingInRepeatDlg(this, "DrawingInRepeatDlg", doc);
    

  /*clipboard object*/
//  clipB = QApplication::clipboard();

  /*Histogram dialog*/
  histoDlg = new HistogramDlg(this, "HistoDlg", doc);

  /*about box*/

  aboutBox = new QMessageBox(this, "aboutBox");
  aboutBox->setCaption(ApaintConfig::getText(ABOUT_MSG));
  aboutBox->setText(QString::fromUtf8("ArahPaint®\nVersion " VERSION_NUM "\n"__DATE__ " " __TIME__ "\n© 2003-2014 Arahne d.o.o.\n\nwritten by:\nMatjaž Gutenberger\nwith contributions by:\nDušan Peterc\nNedeljko Stefanović"));

  aboutBox->setIconPixmap(QPixmap(getCommonPNG("alogo")));

  dlgContour = new DialogContour(this, "ContouR Dialog", doc, colP);

  gweaveDlg = new GuessWeaveDialog(this, doc, "GuessWeaveDialog");

  printDlg = new PrintDialog(this, "PrintDialog", doc);

  dummyTool = new ToolBase(SelectionTool, 0);
  saveTool = dummyTool;
} 


/*SETUP CONNECTIONS*/
void ApaintApp::initConnections()
{
  /*view<--->*/
  connect(view, SIGNAL(docPropChanged(double, double)), this, SLOT(slotShowCaption(double, double)));

  connect(view, SIGNAL(mouseMoveSig(QMouseEvent *, QPainter*)),
          tCtrl, SLOT(mouseMoveSlot(QMouseEvent *, QPainter*)));
  connect(view, SIGNAL(mousePressSig(QMouseEvent *, QPainter*)),
          tCtrl, SLOT(mousePressSlot(QMouseEvent *, QPainter*)));
  connect(view, SIGNAL(mouseReleaseSig(QMouseEvent *, QPainter*)),
          tCtrl, SLOT(mouseReleaseSlot(QMouseEvent *, QPainter*)));
  connect(view, SIGNAL(mouseDoubleClickSig(QMouseEvent *, QPainter*)),
          tCtrl, SLOT(mouseDoubleClickSlot(QMouseEvent *, QPainter*)));          

  connect(view, SIGNAL(repaintSig(QPainter*)), tCtrl, SLOT(repaintSlot(QPainter*)));
  connect(bezierTool, SIGNAL(repaintSig()), view, SLOT(repaint()));

  connect(view, SIGNAL(beforeRefresh(QPainter*)), tCtrl, SLOT(beforeRefreshSlot(QPainter*)));
  connect(view, SIGNAL(afterRefresh(QPainter*)), tCtrl, SLOT(afterRefreshSlot(QPainter*)));
  connect(view, SIGNAL(leaveEventSig(QEvent*, QPainter *)), tCtrl, SLOT(leaveEventSlot(QEvent*, QPainter *)));
  connect(view, SIGNAL(enterEventSig(QEvent*, QPainter *)), tCtrl, SLOT(enterEventSlot(QEvent*, QPainter *)));
  
  connect(view, SIGNAL(zoomChanged(double, double)), tCtrl, SLOT(slotZoomChanged(double, double)));
  connect(view, SIGNAL(zoomChanged(double, double)), draw, SLOT(zoomChangedSlot(double, double)));
  connect(view, SIGNAL(zoomChanged(double, double)), histoDlg, SLOT(zoomChanged(double, double)));
  connect(view, SIGNAL(viewZoomChanged(double, double)), vruler, SLOT(setScale(double, double)));
  connect(view, SIGNAL(viewZoomChanged(double, double)), hruler, SLOT(setScale(double, double)));

  connect(hruler, SIGNAL(mouseDoubleClicked(QPoint*, Orientation)), this, SLOT(rulerClicked(QPoint*, Orientation)));
  connect(vruler, SIGNAL(mouseDoubleClicked(QPoint*, Orientation)), this, SLOT(rulerClicked(QPoint*, Orientation)));

  connect(view, SIGNAL(keyPressSig(QKeyEvent*, QPainter*)), tCtrl, SLOT(keyPressSlot(QKeyEvent*, QPainter*)));
  connect(view, SIGNAL(keyReleaseSig(QKeyEvent*, QPainter*)), tCtrl, SLOT(keyReleaseSlot(QKeyEvent*, QPainter*)));

  connect(view, SIGNAL(contentsSizeChanged(int, int)), this, SLOT(viewSizeChanged(int, int)));
  connect(view, SIGNAL(viewOffsetChanged(int, int)), this, SLOT(viewMoving(int, int)));
  connect(view, SIGNAL(mouseMoveSig(QMouseEvent *, QPainter*)), this, SLOT(mouseMoveSlot(QMouseEvent *, QPainter*)));

  /*doc<---->*/
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));  
  connect(doc, SIGNAL(newDocSig()), this, SLOT(newDocSlot()));
  connect(doc, SIGNAL(newDocSig()), bezierTool, SLOT(newDocSlot()));
  
  connect(doc, SIGNAL(layerChanged(Layer *)), repDim, SLOT(layerChanged(Layer *)));
  connect(doc, SIGNAL(layerChanged(Layer *)), pbUtil, SLOT(layerChangedSlot(Layer *)));
  connect(doc, SIGNAL(layerMovedSig(QRect)), this, SLOT(layerMovedSlot(QRect)));

  
  connect(doc, SIGNAL(newLayerSig(Layer *)), layerSel, SLOT(refreshLayerTab(Layer *)));
  connect(doc, SIGNAL(removeLayer(int)), layerSel, SLOT(removeLayerSlot(int)));
  connect(doc, SIGNAL(layerChanged(Layer *)), histoDlg, SLOT(layerChanged(Layer *)));

  connect(doc, SIGNAL(protectLayerSig(bool)), this, SLOT(protectLayer(bool)));

  /*draw<--->*/
  connect(draw, SIGNAL(unRedoData(QRect &, QRect &)), unredo, SLOT(putImageData(QRect &, QRect &)));
  connect(draw, SIGNAL(colorTableChanged(Layer *)), colP, SLOT(layerChangedSlot(Layer *))); 
 
  /*unredo <---> draw*/
  connect(unredo, SIGNAL(refresh(int, int, int, int, int)), draw, SLOT(screenRefresh(int, int, int, int, int)));
  connect(unredo, SIGNAL(refreshRepeat(int, int, int, int)), draw, SLOT(screenRefreshRepeat(int, int, int, int)));
  connect(unredo, SIGNAL(refreshRepeat(Layer*, int, int, int, int)), draw, SLOT(screenRefreshRepeat(Layer*, int, int, int, int)));

  connect(unredo, SIGNAL(redo()), bezierTool, SLOT(slotRedo()));
  connect(unredo, SIGNAL(undo()), bezierTool, SLOT(slotUndo()));
  connect(bezierTool, SIGNAL(putToStack()), unredo, SLOT(putSpecificData()));
  connect(bezierTool, SIGNAL(clearStacks()), unredo, SLOT(clearSpecificData()));
 
  /*tCtrl<--->*/
  connect(tCtrl, SIGNAL(setCur(QCursor&)), view, SLOT(setViewCur(QCursor&)));
  connect(tCtrl, SIGNAL(unsetCur()), view, SLOT(unsetViewCur()));
  connect(tCtrl, SIGNAL(scrollBySig(int, int)), view, SLOT(scrollBy(int, int)));
  connect(tCtrl, SIGNAL(setAutoScroll(bool)), view, SLOT(setAutoScroll(bool)));

  connect(tCtrl, SIGNAL(coordsPos(int, int)), pbUtil, SLOT(showCoords(int, int)));
  connect(tCtrl, SIGNAL(curRGB(uchar, uchar, uchar, uchar)), pbUtil, SLOT(setRGB(uchar, uchar, uchar, uchar)));
  connect(tCtrl, SIGNAL(clearSize()), pbUtil, SLOT(clearSizeSlot()));

  connect(tCtrl, SIGNAL(setColorSig(QRgb)), pbUtil, SLOT(setColorSlot(QRgb)));
  connect(tCtrl, SIGNAL(setColorSigIndx(int)), colP, SLOT(setColorPos(int)));
  connect(tCtrl, SIGNAL(drawStarted()), colP, SLOT(disableUsedColors()));     

  /*rectTool<--->*/
  connect(rectTool, SIGNAL(showSize(QRect)), pbUtil, SLOT(showSizeSlot(QRect)));
  connect(rectTool, SIGNAL(clearSize()), pbUtil, SLOT(clearSizeSlot()));

  /*quadTool<--->draw*/
  connect(quadTool, SIGNAL(showSize(QRect)), pbUtil, SLOT(showSizeSlot(QRect)));
  connect(quadTool, SIGNAL(clearSize()), pbUtil, SLOT(clearSizeSlot()));
   
  /*colorUtil<--->*/
  connect(pbUtil, SIGNAL(fgChangedIndx(int)), doc, SLOT(setFgIndx(int)));
  connect(pbUtil, SIGNAL(bgChangedIndx(int)), doc, SLOT(setBgIndx(int)));
  connect(pbUtil, SIGNAL(fgChanged(QRgb)), doc, SLOT(setFgCol(QRgb)));
  connect(pbUtil, SIGNAL(bgChanged(QRgb)), doc, SLOT(setBgCol(QRgb)));
   
  // it works faster... I, think
  connect(pbUtil, SIGNAL(fgChanged(QRgb)), draw, SLOT(setFgColor(QRgb)));
  connect(pbUtil, SIGNAL(bgChanged(QRgb)), draw, SLOT(setBgColor(QRgb)));  
  connect(pbUtil, SIGNAL(fgChangedIndx(int)), draw, SLOT(setFgColorIndx(int)));
  connect(pbUtil, SIGNAL(bgChangedIndx(int)), draw, SLOT(setBgColorIndx(int)));
  
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), tCtrl, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), bezierTool, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), circleTool, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), ellipseTool, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), rectTool, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), quadTool, SLOT(lineSizeChgSlot(const QPen&)));
  connect(pbUtil, SIGNAL(lineSizeChg(const QPen &)), this, SLOT(lineSizeChgSlot(const QPen&)));
  
  connect(pbUtil, SIGNAL(setColorSigIndx(int)), colP, SLOT(setColorPos(int)));
  connect(pbUtil, SIGNAL(setColorSig(QRgb, bool)), colP, SLOT(setColor(QRgb, bool))); 

  /*COLOR PALETTE colP*/

  connect(colP, SIGNAL(colorChanged(QRgb, int)), pbUtil, SLOT(setColorSlot(QRgb, int)));
  connect(colP, SIGNAL(bgFgChanged(int , QRgb , int , QRgb )), pbUtil, SLOT(setBgFgIndxCol(int , QRgb , int , QRgb )));
  
  connect(colP, SIGNAL(colorChanged(QRgb, int)), this, SLOT(insertColor(QRgb, int)));
  connect(colP, SIGNAL(putColorUndo(QRgb, int)), unredo, SLOT(putColorData(QRgb, int)));
  
  /*selTool<--->*/
  connect(selTool, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));
  connect(selTool, SIGNAL(unsetCursor()), view, SLOT(unsetViewCur()));
  connect(selTool, SIGNAL(showSize(QRect)), pbUtil, SLOT(showSizeSlot(QRect)));
  connect(selTool, SIGNAL(clearSize()), pbUtil, SLOT(clearSizeSlot()));


  /*freehand<--->*/
  connect(freehand, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));
  connect(freehand, SIGNAL(unsetCursor()), view, SLOT(unsetViewCur()));  

  /*magicWandTool<--->*/
  connect(magicWandTool, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));
  connect(magicWandTool, SIGNAL(unsetCursor()), view, SLOT(unsetViewCur()));

                                            
  /*transTool<---->*/  
  connect(transTool, SIGNAL(putUndoStack(double, double, double, int, int)), unredo, SLOT(putTransformData(double, double, double, int, int)));
  connect(transTool, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));
  connect(transTool, SIGNAL(unsetCursor()), view, SLOT(unsetViewCur()));

  /*repDim<--->view*/
  connect(repDim, SIGNAL(scaleFactor(double, double)), view, SLOT(scaleFactorSlot(double, double)));
  connect(repDim, SIGNAL(scaleFactor(double, double)), this, SLOT(slotAspectChanged(double, double)));
  connect(repDim, SIGNAL(scaleFactorChanged()), this, SLOT(setAspectRatioDraw()));
  
  /*colorMenu<----------->this*/
  connect(colorMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  
  connect(convertMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  connect(protectedColorsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(transparentColorsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(colorInvertMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(sortColorsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(countColorsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  /*imageSizeMenu<----------->this*/
  connect(editMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  connect(duplicateXMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(duplicateYMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  connect(multiplyXMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(multiplyYMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  /*srot<----->this*/
  connect(srot, SIGNAL(closeSig()), this, SLOT(toolsDialogCloseSlot()));

  
  connect(toolsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(filtersMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(noiseMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(selfAvoidingMotifsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(randomMotifsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(placeMotifsMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(ikatMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(marbleMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(zigzagMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(waveMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(mirrorMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(gradientMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(coralMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(colorShorterInsideMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(colorShorterOutsideMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(mixMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
  connect(despecleMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));

  /*helpMenu*/
  connect(helpMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  
  connect(saveSetupMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  

  /*viewMenu*/
  connect(viewMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  

  /*imageMenu*/
  connect(imageMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  

  /*filemenu*/
  connect(fileMenu, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));  
      
  /*writeTool*/
  connect(writeTool, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));

  /*gridProp*/
  connect(gridProp, SIGNAL(okBtnClicked()), this, SLOT(setGridProp()));  

  /*repeatDlg--------------*/
  connect(repeatDlg, SIGNAL(setTileUndo()), unredo, SLOT(putTileData()));
  
  /*layerSel----------*/
  connect(layerSel, SIGNAL(putLayerNum(int)), unredo, SLOT(putLayerNum(int)));
  connect(layerSel, SIGNAL(loadLayerSig()), this, SLOT(slotLayerLoad()));
  connect(layerSel, SIGNAL(browseLayerSig()), this, SLOT(slotLayerBrowse()));
  connect(layerSel, SIGNAL(newLayerSig()), this, SLOT(slotLayerNew()));

  /*layerSelSimple-------- */
  connect(layerSelSimple, SIGNAL(putLayerNum(int)), unredo, SLOT(putLayerNum(int)));

  /*zoomin, zoomout*/
  connect(zoomin, SIGNAL(scaleUp(int, int, bool)), view, SLOT(scaleUp(int, int, bool)));
  connect(zoomout, SIGNAL(scaleDown(int, int, bool)), view, SLOT(scaleDown(int, int, bool)));
  
  /*CommonProp------*/
  connect(cProp, SIGNAL(setThickLinesSig(bool)), this, SLOT(setFatLines(bool)));
  connect(cProp, SIGNAL(aspectDrawChangedSig()), this, SLOT(setAspectRatioDraw()));
  
   
  /*rulerInfo------------*/
  connect(rInfo, SIGNAL(styleChanged(Ruler::MetricStyle)), this, SLOT(rulerStyleChanged(Ruler::MetricStyle)));

  /*lMoveTool-------------------------*/
  connect(lMoveTool, SIGNAL(setCursor(QCursor &)), view, SLOT(setViewCur(QCursor &)));
  connect(lMoveTool, SIGNAL(unsetCursor()), view, SLOT(unsetViewCur()));

  /*iconDlg---------------------------*/
  connect(iconDlg, SIGNAL(loadLayerSig(QString&)), this, SLOT(loadLayerFile(QString&)));
  connect(iconDlg, SIGNAL(loadBrushSig(QString&)), this, SLOT(loadBrushFile(QString&)));
  connect(iconDlg, SIGNAL(setImagePathSig(QString)), this, SLOT(setImagePath(QString)));
  connect(iconDlg, SIGNAL(loadDocSig(QString&)), this, SLOT(loadDocSlot(QString&)));
   
  /*lMoveTool--------------------------------------------*/
  connect(lMoveTool, SIGNAL(putLayerNum(int)), unredo, SLOT(putLayerNum(int)));

  /*insRemPixTool--------------------------------------------*/
  connect(insRemPixTool, SIGNAL(putUndo(bool, int, int, bool)), unredo, SLOT(putInsRemData(bool, int, int, bool)));
  /*dlgInsRemPix--------------------------------------------*/
  connect(dlgInsRemPix, SIGNAL(closeSig()), this, SLOT(InsRemDlgClose()));
  
  /*fill tool*/
  connect(fillTool, SIGNAL(colorTableChanged(Layer *)), colP, SLOT(layerChangedSlot(Layer *)));
  /*brush tool*/
  connect(brushTool, SIGNAL(colorTableChanged(Layer *)), colP, SLOT(layerChangedSlot(Layer *)));

  /*bListWid*/
  connect(bListWid, SIGNAL(loadFileBrush()), this, SLOT(loadBrushSlot()));
  connect(bListWid, SIGNAL(browseFileBrush()), this, SLOT(browseBrushSlot()));
  connect(bListWid, SIGNAL(layerToBrushSig()), this, SLOT(layerToBrushSlot()));
  connect(bListWid, SIGNAL(brushToLayerSig()), this, SLOT(brushToLayerSlot()));

  /*UnRedoTool--------------------------------------------*/
  connect(undoTool, SIGNAL(doUndo()), this, SLOT(slotEditUndo()));
  connect(redoTool, SIGNAL(doRedo()), this, SLOT(slotEditRedo()));
  /*fdlg---------------------------------------------------*/
  connect(fdlg, SIGNAL(closeSig()), this, SLOT(toolsDialogCloseSlot()));
  documentChanged(false);
}

int ApaintApp::queryExit()
{
  int exit = QMessageBox::warning(this, ApaintConfig::getText(9), ApaintConfig::getText(PICTURE_CHANGED_SAVE_MSG),
                                        ApaintConfig::getText(1), ApaintConfig::getText(2), ApaintConfig::getText(CANCEL_MSG));  
  return exit;
}

int ApaintApp::queryRevert()
{
  int exit = QMessageBox::warning(this, ApaintConfig::getText(9), ApaintConfig::getText(759),
                                        ApaintConfig::getText(1), ApaintConfig::getText(2), ApaintConfig::getText(CANCEL_MSG));  
  return exit;
}

bool ApaintApp::transparentMove() const
{
 return selectProp->isTransparentMove();
}

bool ApaintApp::transparentPaste() const
{
 return selectProp->isTransparentPaste();
}

bool ApaintApp::transparentSelection(void *layer) const
{
	return (selectProp->isTransparentMove() && selectionInMoving.find(layer)!=selectionInMoving.end())
		|| (selectProp->isTransparentPaste()
//				&& selectionInMoving.find(layer)!=selectionInMoving.end()
				);
}

void ApaintApp::selectionMoving(bool moving, void *layer)
{
 if (moving != (selectionInMoving.find(layer)!=selectionInMoving.end()))
  {
   if (moving)
    selectionInMoving.insert(layer);
   else
    selectionInMoving.erase(layer);
   doc->redrawLayers();
  }
}

/*
void ApaintApp::selectionPasting(bool pasting, void *layer)
{
 if (pasting != (selectionInPasting.find(layer)!=selectionInPasting.end()))
  {
   if (pasting)
    selectionInPasting.insert(layer);
   else
    selectionInPasting.erase(layer);
  }
}
*/

void ApaintApp::transparentButtonToggled()
{
 doc->redrawLayers();
}

void ApaintApp::alignSelection(int align)
{
 QRect sel = doc->getLayerPos();
 int moveX=0, moveY=0;

 switch (align)
  {
   case ALIGN_LEFT:
    if (sel.x())
     moveX = sel.x();
    else
     moveX = sel.width()/2;
    break;
   case ALIGN_RIGHT:
    moveX = sel.x() + sel.width() - imageSize.width();
    if (moveX==0)
     moveX = sel.x() + sel.width()/2 - imageSize.width();
    break;
   case ALIGN_TOP:
    if (sel.y())
     moveY = sel.y();
    else
     moveY = sel.height()/2;
    break;
   case ALIGN_BOTTOM:
    moveY = sel.y() + sel.height() - imageSize.height();
    if (moveY==0)
     moveY = sel.y() + sel.height()/2 - imageSize.height();
    break;
   case ALIGN_CENTER_X:
    moveX = sel.x() + (sel.width() - imageSize.width())/2;
    break;
   case ALIGN_CENTER_Y:
    moveY = sel.y() + (sel.height() - imageSize.height())/2;
    break;
  }
 ApaintDocUtil::moveCurLayer(doc, moveX, moveY, 2);
}

void ApaintApp::moveSelectionX(int pos)
{
 QRect sel = doc->getLayerPos();
 int currentX;

 if (doc->getLayer() == doc->getMainLayer())
  return;

 if (doc->getLayerResizeable())
  return;

 currentX = sel.x()+1;
 if (pos!=currentX)
  ApaintDocUtil::moveCurLayer(doc, currentX-pos, 0, 2);
}

void ApaintApp::moveSelectionY(int pos)
{
 QRect sel = doc->getLayerPos();
 int mainY = doc->getMainLayer()->layerPos().height();
 int currentY;

 if (doc->getLayer() == doc->getMainLayer())
  return;

 if (doc->getLayerResizeable())
  return;

 currentY = mainY - sel.y()-sel.height()+1;
 if (pos!=currentY)
  ApaintDocUtil::moveCurLayer(doc, 0, pos-currentY, 2);
}

bool ApaintApp::repeatDelimiterView() const
{
 return repeatDelimiterView();
}

bool ApaintApp::antialiasView() const
{
 return view->antialias();
}

QRgb ApaintApp::getFgColor()
{
 return pbUtil->foreground();
}

QRgb ApaintApp::getBgColor()
{
 return pbUtil->background();
}

bool ApaintApp::crossHairCursor() const
{
 return crossHair;
}

bool ApaintApp::keepOriginalColormap() const
{
 return keepOriginalCmap;
}

QSize ApaintApp::getTotalSize() const
{
 int sx, sy, offx, offy;

 if (doc->getMainLayer() == 0) return QSize(0, 0);

 doc->mainLayer->getTileInfo(sx, sy, offx, offy);

 return QSize(doc->mainLayer->imageBuf.width()*sx*view->getScaleX(), doc->mainLayer->imageBuf.height()*sy*view->getScaleY());
}

QSize ApaintApp::getWindowSize() const
{
 return(size());
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void ApaintApp::setAspectRatioDraw()
{
  double fx, fy;

  repDim->getScaleFactor(fx, fy, true);
  if (cProp->doAspectDraw())
  {    
    doc->setAspectRatio(fx, fy);
    tCtrl->slotAspectChanged(1, 1);
  }
  else
  {
    doc->setAspectRatio(1, 1);
    tCtrl->slotAspectChanged(fx, fy);
  }
}

void ApaintApp::setIntegerView(bool enable)
{
  view->setIntegerView(enable);
  repDim->setIntegerView(enable);
}

void ApaintApp::setRepeatDelimiterView(bool enable)
{
 if (doc->mainLayer)
  {
   doc->mainLayer->setDrawDelimiterLines(enable);
   doc->refreshLayer();
  }
}

void ApaintApp::setAntialiasView(bool enable)
{
 view->setAntialiasView(enable);
}

void ApaintApp::setCrosshairView(bool enable)
{
 crossHair = enable;
// doc->redrawLayers();
 doc->refreshLayer();
}

void ApaintApp::setKeepOriginalCmap(bool enable)
{
 keepOriginalCmap = enable;
}

void ApaintApp::InsRemDlgClose()
{
  drawToolBar->setTool(brushTool, true);
}

void ApaintApp::lineSizeChgSlot(const QPen& pen)
{
 lineSize = pen.width();
}

void ApaintApp::setFatLines(bool enable)
{
 if (lineSize<2 && enable)
  draw->setFatLines(true);
 else
  draw->setFatLines(false);
}

void ApaintApp::toolsDialogCloseSlot()
{
  drawToolBar->setTool(lMoveTool, true);
}

void ApaintApp::setRepeatMode(bool enable)
{  
 int sx, sy, offx, offy;
 if (!doc->mainLayer) return;
 
 doc->mainLayer->getTileInfo(sx, sy, offx, offy);

 if (enable && repeatDlg->getRepX() == 1 && repeatDlg->getRepY() == 1)
  doc->createMainLayerTiles(10, 10, offx, offy, false);
 else if (enable)
	 repeatDlg->doit();
 else
  doc->enableMainLayerTiles(enable);
 viewRepeatDelimiter->setEnabled(enable);
}

static int getLayerRepeatX(Layer *layer, int width, int height)
{
 int cmapImage = (layer->getImageType() == LOAD_CMAP);
 unsigned long diff[width], minDiff;
 int i, j, x, y, repeat, d, numColors=1;
 int minRepeat=width/3; /* maximum reduction three times */
 int maxRepeat=(width*19)/20; /* minimum reduction 5% */
 unsigned char *linePtr, *ptr1, *ptr2;
 unsigned char cmap[256*4];
 unsigned short *cmapDiffTable=NULL;

 if (width<10)
  return(width); 
 for (i=0; i<width; i++)
  diff[i]=0;

 if (cmapImage)
  {
   numColors = layer->numColors();
   for (i=0; i<numColors; i++)
    {
     QRgb color = layer->imageBuf.color(i);
     cmap[i*4]   = qRed(color);
     cmap[i*4+1] = qGreen(color);
     cmap[i*4+2] = qBlue(color);
    }
   cmapDiffTable = (unsigned short *) malloc(numColors*numColors*sizeof(unsigned short));
   if (!cmapDiffTable)
    return(width);
   for (i=0; i<numColors; i++)
    for (j=0; j<numColors; j++)
     cmapDiffTable[i*numColors + j] = abs(cmap[i<<2]-cmap[j<<2]) + abs(cmap[(i<<2)+1]-cmap[(j<<2)+1]) + abs(cmap[(i<<2)+2]-cmap[(j<<2)+2]);
  }

 for (y=0; y<height; y++)
  {
   linePtr = layer->imageBuf.scanLine(y);
   for (repeat=minRepeat; repeat<width; repeat++)
    {
     d=0;
     ptr1 = linePtr;
     if (cmapImage)
      {
       ptr2 = &linePtr[repeat];
       for (x=repeat; x<width; x++)
	d += cmapDiffTable[(*ptr1++ * numColors) + *ptr2++];
      }
     else
      {
       ptr2 = &linePtr[repeat*3];
       for (x=repeat; x<width; x++)
	{
         d += abs(*ptr1++ - *ptr2++);
         d += abs(*ptr1++ - *ptr2++);
         d += abs(*ptr1++ - *ptr2++);
	}
      }
     diff[repeat] += d/(width-repeat);
    }
  }

 if (cmapImage)
  free(cmapDiffTable);

 minDiff = diff[minRepeat];
 for (x=minRepeat; x<width; x++)
  {
   if (minDiff > diff[x])
    {
     minRepeat = x;
     minDiff = diff[x];
    }
  }
 return(minRepeat<maxRepeat ? minRepeat : width);
}

static int getLayerRepeatY(Layer *layer, int width, int height)
{
 int cmapImage = (layer->getImageType() == LOAD_CMAP);
 unsigned long diff[height], minDiff;
 int i, j, x, y, repeat, d, numColors=1;
 int minRepeat=height/3; /* maximum reduction three times */
 int maxRepeat=(height*19)/20; /* minimum reduction 5% */
 unsigned char *ptr1, *ptr2;
 unsigned char cmap[256*4];
 unsigned short *cmapDiffTable=NULL;
 ApaintImage imgBuf = layer->imageBuf;

 if (height<10)
  return(height); 
 for (i=0; i<height; i++)
  diff[i]=0;

 if (cmapImage)
  {
   numColors = layer->numColors();
   for (i=0; i<numColors; i++)
    {
     QRgb color = layer->imageBuf.color(i);
     cmap[i*4]   = qRed(color);
     cmap[i*4+1] = qGreen(color);
     cmap[i*4+2] = qBlue(color);
    }
   cmapDiffTable = (unsigned short *) malloc(numColors*numColors*sizeof(unsigned short));
   if (!cmapDiffTable)
    return(height);
   for (i=0; i<numColors; i++)
    for (j=0; j<numColors; j++)
     cmapDiffTable[i*numColors + j] = abs(cmap[i<<2]-cmap[j<<2]) + abs(cmap[(i<<2)+1]-cmap[(j<<2)+1]) + abs(cmap[(i<<2)+2]-cmap[(j<<2)+2]);
  }

 for (x=0; x<width; x++)
  {
   for (repeat=minRepeat; repeat<height; repeat++)
    {
     d=0;
     if (cmapImage)
      for (y=repeat; y<height; y++)
       d += cmapDiffTable[imgBuf.scanLine(y)[x]*numColors + imgBuf.scanLine(y-repeat)[x]];
     else
      for (y=repeat; y<height; y++)
       {
	ptr1 = &layer->imageBuf.scanLine(y)[x*3];
	ptr2 = &layer->imageBuf.scanLine(y-repeat)[x*3];
        d += abs(*ptr1++ - *ptr2++);
        d += abs(*ptr1++ - *ptr2++);
        d += abs(*ptr1++ - *ptr2++);
       }
     diff[repeat] += d/(height-repeat);
    }
  }

 if (cmapImage)
  free(cmapDiffTable);

 minDiff = diff[minRepeat];
 for (y=minRepeat; y<height; y++)
  {
   if (minDiff > diff[y])
    {
     minRepeat = y;
     minDiff = diff[y];
    }
  }
 return(minRepeat<maxRepeat ? minRepeat : height);
}
   
void ApaintApp::menuActivated(int indx)
{
 {
  QProcess *proc;
  char filename[500];
  char url[500]="http://www.arahne.eu";

  /* functions, which should work even without any image */
  switch (indx)
   {
    case OPEN_PDF_MANUAL:
     proc = new QProcess( this );
     proc->addArgument(ApaintConfig::getAcrobatCommand() );
     sprintf(filename, "../pdf/apaint4-%s.pdf", ApaintConfig::getManualLanguage());
     if (access(filename, R_OK))
      {
       strcat(url, &filename[2]);
       strcpy(filename, url);
      }
     proc->addArgument(QString(filename));
     proc->start();
     return;
     break;
 
    case OPEN_PDF_SHORTCUTS:
     proc = new QProcess( this );
     proc->addArgument(ApaintConfig::getAcrobatCommand() );
     sprintf(filename, "../pdf/apaint4-shortcuts-%s.pdf", ApaintConfig::getManualLanguage());
     if (access(filename, R_OK))
      {
       strcat(url, &filename[2]);
       strcpy(filename, url);
      }
     proc->addArgument(QString(filename));
     proc->start();
     return;
     break;

    case OPEN_PDF_FAST_PATTERN:
     proc = new QProcess( this );
     proc->addArgument(ApaintConfig::getAcrobatCommand() );
     sprintf(filename, "../pdf/fastpattern-%s.pdf", ApaintConfig::getManualLanguage());
     if (access(filename, R_OK))
      {
       strcat(url, &filename[2]);
       strcpy(filename, url);
      }
     proc->addArgument(QString(filename));
     proc->start();
     return;
     break;

    case OPEN_ARAHNE_WEB_PAGE:
     proc = new QProcess( this );
     proc->addArgument(ApaintConfig::getWwwBrowserCommand() );
     proc->addArgument("http://www.arahne.si");
     proc->start();
     return;
     break;
    case OPEN_LICENSE_WEB_PAGE:
     proc = new QProcess( this );
     proc->addArgument(ApaintConfig::getWwwBrowserCommand() );
     proc->addArgument("http://www.gnu.org/licenses/gpl.txt");
     proc->start();
     return;
     break;

    case SET_LANGUAGE_CHINESE_SIMPLIFIED:
    case SET_LANGUAGE_CHINESE_TRADITIONAL:
    case SET_LANGUAGE_DUTCH:
    case SET_LANGUAGE_ENGLISH:
    case SET_LANGUAGE_FINNISH:
    case SET_LANGUAGE_FRENCH:
    case SET_LANGUAGE_GERMAN:
    case SET_LANGUAGE_ITALIAN:
    case SET_LANGUAGE_POLISH:
    case SET_LANGUAGE_SLOVENIAN:
    case SET_LANGUAGE_SPANISH:
    case SET_LANGUAGE_THAI:
    case SET_LANGUAGE_TURKISH:
     {
      char lang[4];

      switch (indx)
       {
	case SET_LANGUAGE_CHINESE_SIMPLIFIED:
	 strcpy(lang, "chi");
	 break;
	case SET_LANGUAGE_CHINESE_TRADITIONAL:
	 strcpy(lang, "cht");
	 break;
	case SET_LANGUAGE_DUTCH:
	 strcpy(lang, "dut");
	 break;
	default:
	case SET_LANGUAGE_ENGLISH:
	 strcpy(lang, "eng");
	 break;
	case SET_LANGUAGE_FINNISH:
	 strcpy(lang, "fin");
	 break;
	case SET_LANGUAGE_FRENCH:
	 strcpy(lang, "fre");
	 break;
	case SET_LANGUAGE_GERMAN:
	 strcpy(lang, "ger");
	 break;
	case SET_LANGUAGE_ITALIAN:
	 strcpy(lang, "ita");
	 break;
	case SET_LANGUAGE_POLISH:
	 strcpy(lang, "pol");
	 break;
	case SET_LANGUAGE_SLOVENIAN:
	 strcpy(lang, "slo");
	 break;
	case SET_LANGUAGE_SPANISH:
	 strcpy(lang, "spa");
	 break;
	case SET_LANGUAGE_THAI:
	 strcpy(lang, "tha");
	 break;
	case SET_LANGUAGE_TURKISH:
	 strcpy(lang, "tur");
	 break;
       }
      ApaintConfig::saveCfgData(lang);
      QMessageBox::information(this, ApaintConfig::getText(91), ApaintConfig::getText(LANGUAGE_CHANGE_RESTART_MSG), ApaintConfig::getText(3));  
      return;
     }
   }
 }

 /* exit if we have no layer loaded! */ 
 if (!doc->getLayer())
  return;

 switch (indx)
 {
   case IMAGE_TO_GREYSCALE:
    doc->putUndoStack(ChangeLayerForm);
    doc->setLayerImageType(LOAD_GREYSCALE, false);
    break;
   case GRID_PROP:
    gridProp->showDlg();
    break;
   case HISTOGRAM_DIALOG:
    histoDlg->startDlg();
    break;
   case GUESS_WEAVE_DLG:
    gweaveDlg->showDlg();
    break;
   case REMOVE_CONTOUR:
    doc->putUndoStack(ChangeLayerForm);
    doc->layerContourRemove(pbUtil->fgIndex());
    lastOperation = indx;
    break;
   case THIN_CONTOUR:
    doc->putUndoStack(ChangeLayerForm);
    doc->layerContourThin(pbUtil->fgIndex());
    lastOperation = indx;
    break;
   case PRUNE_CONTOUR:
    doc->putUndoStack(ChangeLayerForm);
    doc->layerContourPrune(pbUtil->fgIndex());
    lastOperation = indx;
    break;
   case DUPLICATE_X1:
   case DUPLICATE_X2:
   case DUPLICATE_X3:
   case DUPLICATE_X4:
   case DUPLICATE_X5:
   case DUPLICATE_X6:
   case DUPLICATE_X7:
   case DUPLICATE_X8:
   case DUPLICATE_X9:
   case DUPLICATE_X10:
   case DUPLICATE_Y1:
   case DUPLICATE_Y2:
   case DUPLICATE_Y3:
   case DUPLICATE_Y4:
   case DUPLICATE_Y5:
   case DUPLICATE_Y6:
   case DUPLICATE_Y7:
   case DUPLICATE_Y8:
   case DUPLICATE_Y9:
   case DUPLICATE_Y10:
    if (indx<DUPLICATE_Y1)
     {
      if ((doc->getLayer()->layerPos().width()*2)>MAX_IMAGE)
       { QApplication::beep(); return; }
     }
    else
     {
      if ((doc->getLayer()->layerPos().height()*2)>MAX_IMAGE)
       { QApplication::beep(); return; }
     }
    doc->layerDuplicate(indx);
    lastOperation = indx;
    break;
   case SCALE_2X:
    if ( ((doc->getLayer()->layerPos().width()*2)>MAX_IMAGE) || ((doc->getLayer()->layerPos().height()*2)>MAX_IMAGE))
     { QApplication::beep(); return; }
    doc->layerScale2X();
    lastOperation = indx;
    break;

   case SCALE_3X:
    if ( ((doc->getLayer()->layerPos().width()*3)>MAX_IMAGE) || ((doc->getLayer()->layerPos().height()*3)>MAX_IMAGE))
     { QApplication::beep(); return; }
    doc->layerScale3X();
    lastOperation = indx;
    break;

   case SCALE_4X:
    if ( ((doc->getLayer()->layerPos().width()*4)>MAX_IMAGE) || ((doc->getLayer()->layerPos().height()*4)>MAX_IMAGE))
     { QApplication::beep(); return; }
    doc->layerScale4X();
    lastOperation = indx;
    break;

   case SHRINK_2X:
    if ( (doc->getLayer()->layerPos().width()<2) || (doc->getLayer()->layerPos().height()<2))
     { QApplication::beep(); return; }
    doc->layerShrink2X();
    lastOperation = indx;
    break;

   case SHRINK_3X:
    if ( (doc->getLayer()->layerPos().width()<3) || (doc->getLayer()->layerPos().height()<3))
     { QApplication::beep(); return; }
    doc->layerShrink3X();
    lastOperation = indx;
    break;

   case SHRINK_4X:
    if ( (doc->getLayer()->layerPos().width()<4) || (doc->getLayer()->layerPos().height()<4))
     { QApplication::beep(); return; }
    doc->layerShrink4X();
    lastOperation = indx;
    break;

   case HALVE_X:
    if (doc->getLayer()->layerPos().width()<2)
     { QApplication::beep(); return; }
    doc->layerHalve(HALVE_X);
    lastOperation = indx;
    break;
   case HALVE_Y:
    if (doc->getLayer()->layerPos().height()<2)
     { QApplication::beep(); return; }
    doc->layerHalve(HALVE_Y);
    lastOperation = indx;
    break;
   case SHRINK_X:
    if (doc->getLayer()->layerPos().width()<2)
     { QApplication::beep(); return; }
    doc->layerShrink(SHRINK_X);
    lastOperation = indx;
    break;
   case SHRINK_Y:
    if (doc->getLayer()->layerPos().height()<2)
     { QApplication::beep(); return; }
    doc->layerShrink(SHRINK_Y);
    lastOperation = indx;
    break;
   case MULTIPLY_X2:
   case MULTIPLY_X3:
   case MULTIPLY_X4:
   case MULTIPLY_X5:
   case MULTIPLY_X6:
   case MULTIPLY_X7:
   case MULTIPLY_X8:
   case MULTIPLY_X9:
   case MULTIPLY_X10:
    if ((doc->getLayer()->layerPos().width()*(indx-MULTIPLY_X2+2))>MAX_IMAGE)
     { QApplication::beep(); return; }
    doc->layerMultiplyX(indx-MULTIPLY_X2+2);
    lastOperation = indx;
    break;

   case MULTIPLY_Y2:
   case MULTIPLY_Y3:
   case MULTIPLY_Y4:
   case MULTIPLY_Y5:
   case MULTIPLY_Y6:
   case MULTIPLY_Y7:
   case MULTIPLY_Y8:
   case MULTIPLY_Y9:
   case MULTIPLY_Y10:
    if ((doc->getLayer()->layerPos().height()*(indx-MULTIPLY_Y2+2))>MAX_IMAGE)
     { QApplication::beep(); return; }
    doc->layerMultiplyY(indx-MULTIPLY_Y2+2);
    lastOperation = indx;
    break;

   case REDUCE_SEL_BORDER:
    {
     Layer *layer = doc->getLayer();

     if (!layer)
      return;
     if (layer->getImageType() == LOAD_TRUE_COL)
      return;
     if (layer->getType() == MainLayer)
      {
       /* avoid making huge selections in repeat mode */
       if (layer->isUsingTiles())
	return;

       QRect sel = QRect(0,0, doc->getLayer()->layerPos().width(), doc->getLayer()->layerPos().height());
       doc->createNewLayer(sel.x(), sel.y(), sel.width(), sel.height(), SelectionLayer, RectangleG);
       doc->sendSignals(true, false);
       doc->refreshLayer(sel);
      }
     /* normal reduction of area */
     doc->putUndoStack(ChangeLayerForm);
     doc->reduceSelectionBorder();
     if (layer->getType() == MainLayer)
      doc->selectionToMainLayer();
    }

    break;

   case SELF_AVOID_MOTIFS1:
   case SELF_AVOID_MOTIFS2:
   case SELF_AVOID_MOTIFS3:
   case SELF_AVOID_MOTIFS4:
   case SELF_AVOID_MOTIFS5:
   case SELF_AVOID_MOTIFS6:
   case SELF_AVOID_MOTIFS7:
   case SELF_AVOID_MOTIFS8:
   case SELF_AVOID_MOTIFS9:
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_SELF_AVOID_MOTIFS1 + indx - SELF_AVOID_MOTIFS1, (doc->getLayer()->getImageType() == LOAD_CMAP) ? pbUtil->fgIndex() : pbUtil->foreground(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    break;

   case RANDOM_MOTIFS1:
   case RANDOM_MOTIFS2:
   case RANDOM_MOTIFS3:
   case RANDOM_MOTIFS4:
   case RANDOM_MOTIFS5:
   case RANDOM_MOTIFS6:
   case RANDOM_MOTIFS7:
   case RANDOM_MOTIFS8:
   case RANDOM_MOTIFS9:
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_RANDOM_MOTIFS1 + indx - RANDOM_MOTIFS1, (doc->getLayer()->getImageType() == LOAD_CMAP) ? pbUtil->fgIndex() : pbUtil->foreground(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    break;

   case PLACE_MOTIFS1:
   case PLACE_MOTIFS2:
   case PLACE_MOTIFS3:
   case PLACE_MOTIFS4:
   case PLACE_MOTIFS5:
   case PLACE_MOTIFS6:
   case PLACE_MOTIFS7:
   case PLACE_MOTIFS8:
   case PLACE_MOTIFS9:
    if (doc->getLayer()->getImageType() != LOAD_CMAP)
     { QApplication::beep(); return; }
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_PLACE_MOTIFS1 + indx - PLACE_MOTIFS1, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    break;

   case CREATE_NOISE_XY:
   case CREATE_NOISE_X:
   case CREATE_NOISE_Y:
    if (doc->getLayer()->getImageType() != LOAD_CMAP)
     { QApplication::beep(); return; }
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_NOISE_XY + indx - CREATE_NOISE_XY, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    break;

   case COLOR_SHORTER_INSIDE_X:
   case COLOR_SHORTER_INSIDE_Y:
   case COLOR_SHORTER_OUTSIDE_X:
   case COLOR_SHORTER_OUTSIDE_Y:
    if (doc->getLayer()->getImageType() != LOAD_CMAP)
     { QApplication::beep(); return; }
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_COLOR_SHORTER_INSIDE_X + indx - COLOR_SHORTER_INSIDE_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    break;

   case TILT_LEFT:
   case TILT_RIGHT:
   case TILT_UP:
   case TILT_DOWN:
    drawToolBar->setTool(measureTool, true);
    unredo->putLayerFilterData(FILTER_TILT_RIGHT + indx - TILT_RIGHT, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue());
    doc->layerApplyFilter(FILTER_TILT_RIGHT + indx - TILT_RIGHT, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case SPLIT_X:
   case JOIN_X:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getSizeY()<2)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
//    unredo->putLayerFilterData(FILTER_SPLIT_X + indx - SPLIT_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue());
    doc->layerApplyFilter(FILTER_SPLIT_X + indx - SPLIT_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case SPLIT_Y:
   case JOIN_Y:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getSizeX()<2)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
//    unredo->putLayerFilterData(FILTER_SPLIT_X + indx - SPLIT_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue());
    doc->layerApplyFilter(FILTER_SPLIT_X + indx - SPLIT_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case MOVE_LEFT:
   case MOVE_RIGHT:
   case MOVE_UP:
   case MOVE_DOWN:
    unredo->putLayerFilterData(FILTER_MOVE_LEFT + indx - MOVE_LEFT, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue());
    doc->layerApplyFilter(FILTER_MOVE_LEFT + indx - MOVE_LEFT, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case IKAT_X:
   case IKAT_Y:
   case IKAT_XY:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getValue()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_IKAT_X + indx - IKAT_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    break;

   case MARBLE_X:
   case MARBLE_Y:
   case MARBLE_XY:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getValue()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_MARBLE_X + indx - MARBLE_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;

   case ZIGZAG_X:
   case ZIGZAG_Y:
   case ZIGZAG_XY:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getValue()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_ZIGZAG_X + indx - ZIGZAG_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;

   case WAVE_X:
   case WAVE_Y:
   case WAVE_XY:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getValue()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_WAVE_X + indx - WAVE_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;

   case MIRROR_X:
   case MIRROR_Y:
    drawToolBar->setTool(measureTool, true);
    if ( ((indx == MIRROR_X) && (measureToolProp->getSizeX()<1)) || ((indx == MIRROR_Y) && (measureToolProp->getSizeY()<1)) )
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_MIRROR_X + indx - MIRROR_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case GRADIENT_X:
   case GRADIENT_Y:
    {
     int fg, bg;

     drawToolBar->setTool(measureTool, true);
     if (doc->getLayer()->getImageType() == LOAD_CMAP)
      {
       fg = pbUtil->fgIndex();
       bg = pbUtil->bgIndex();
      }
     else
      {
       fg = pbUtil->foreground();
       bg = pbUtil->background();
      }
     if (fg == bg)
      { QApplication::beep(); return; }
     doc->putUndoStack(ChangeLayerForm);
     doc->layerApplyFilter(FILTER_GRADIENT_X + indx - GRADIENT_X, fg, bg, measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), FALSE, measureToolProp->doGradient());
     lastOperation = indx;
    }
    return;

   case CONTOUR:
    drawToolBar->setTool(measureTool, true);
    if ((measureToolProp->getValue()<1) || (doc->getLayer()->getImageType() != LOAD_CMAP))
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_CONTOUR, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;

   case CORAL_1:
   case CORAL_2:
   case CORAL_3:
   case CORAL_4:
   case CORAL_5:
   case CORAL_6:
    if (doc->getLayer()->getImageType() != LOAD_CMAP)
     { QApplication::beep(); return; }
    drawToolBar->setTool(measureTool, true);
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_CORAL_1 + indx - CORAL_1, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
    lastOperation = indx;
    return;

   case MIX_X:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getSizeX()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_MIX_X, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;
   case MIX_Y:
    drawToolBar->setTool(measureTool, true);
    if (measureToolProp->getSizeY()<1)
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_MIX_Y, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    return;
   case MIX_XY:
    drawToolBar->setTool(measureTool, true);
    if ( (measureToolProp->getSizeX()<1) || (measureToolProp->getSizeY()<1))
     { QApplication::beep(); return; }
    doc->putUndoStack(ChangeLayerForm);
    doc->layerApplyFilter(FILTER_MIX_XY, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), measureToolProp->doVarColors(), measureToolProp->doVarSize(), measureToolProp->doGradient());
    lastOperation = indx;
    break;

   case MAKE_COLOR_SHADES:
    {
     int firstColor = pbUtil->fgIndex();
     int lastColor = pbUtil->bgIndex();

     if (firstColor>lastColor)
      {
       firstColor = pbUtil->bgIndex();
       lastColor = pbUtil->fgIndex();
      }
     if ((lastColor-firstColor)>1)
      doc->layerMakeColorShades(firstColor, lastColor);
     else
      { QApplication::beep(); return; }   
    }
    break;
   case FIND_EXACT_REPEAT:
     /* it will crash if we are in repeat view */
/*
     repeatView->setOn(false);
     repeatX = getLayerRepeatX(doc->getLayer(), doc->getLayer()->layerPos().width(), doc->getLayer()->layerPos().height());
     repeatY = getLayerRepeatY(doc->getLayer(), repeatX, doc->getLayer()->layerPos().height());
printf("found repeat %d %d\n", repeatX, repeatY);
*/
    break;
   case FIND_REPEAT:
    {
     int repeatX, repeatY;

     /* it will crash if we are in repeat view */
     repeatView->setOn(false);

     doc->setActiveLayer(0);
     repeatX = getLayerRepeatX(doc->getLayer(), doc->getLayer()->layerPos().width(), doc->getLayer()->layerPos().height());
     repeatY = getLayerRepeatY(doc->getLayer(), repeatX, doc->getLayer()->layerPos().height());

     if ( (repeatX!=doc->getLayer()->layerPos().width()) || (repeatY!=doc->getLayer()->layerPos().height()) )
      {
       QRect sel = QRect(0,0, repeatX, repeatY);

       doc->putUndoStack(doc->getLayer(), CreateLayer);

       doc->createNewLayer(sel.x(), sel.y(), sel.width(), sel.height(), SelectionLayer, RectangleG);
       Layer* selLayer = doc->getLayer();
       if (!selLayer)
	return;
       doc->sendSignals(true, false);
       doc->refreshLayer(sel);
      }
     else
      QApplication::beep();
    }
   break;
 }
}

void ApaintApp::invertLayer()
{
 Layer *layer = doc->getLayer();
 if (!layer) return;
 if (layer->getType() == MainLayer) return;
 if (layer->getIsResizeable())
  {
   QImage newMask = QImage(doc->mainLayer->layerPos().width(), doc->mainLayer->layerPos().height(), 1, 2, QImage::LittleEndian);
   newMask.fill(0);
   copy1BitImage(newMask, layer->layerPos().x(), layer->layerPos().y(), layer->getMask(), 0, 0, 
                 layer->layerPos().width(), layer->layerPos().height(), false, 0, 0);
   newMask = invertQImage(newMask);
   doc->putUndoStack(DestroyLayer);
   doc->deleteLayer();
   doc->createNewLayer(&newMask, doc->mainLayer->layerPos());
   doc->putUndoStack(CreateLayer);
   doc->refreshLayer();
  }
}

void ApaintApp::layerToBrushSlot()
{
 Layer *layer = doc->getLayer();
 if (!layer) return;
 ApaintBrushList *bList = ApaintBrushList::getObject();
 bList->addBrush(layer);
 if (layer->getType() != MainLayer)
  {
   doc->putUndoStack(DestroyLayer);
   doc->deleteLayer(doc->getLayerNum());
  }
}

void ApaintApp::brushToLayerSlot()
{
 ApaintBrushList *bList = ApaintBrushList::getObject();  
 if (bList->isValid())
  {
   QRect posView = view->getContentsSize();
   ApaintImage img = bList->getBrush();
   QImage mask = bList->getBrushMask();
   doc->createNewLayer(img, &mask, posView.x()+posView.width()/2, posView.y()+posView.height()/2);
   doc->putUndoStack(CreateLayer);
   doc->refreshLayer();
   bListWid->removeBrush();
  } 
}

void ApaintApp::slotEditMultiplePaste()
{
 if (!doc->getLayer())
  return;
 mpasteDlg->showDlg();
 mpasteDlg->adjustSize(); 
}

void ApaintApp::newDocSlot()
{
 if (!tCtrl->getActiveTool())
   drawToolBar->setTool(brushTool, true);
 if (doc->getLayer()->getImageType() == LOAD_CMAP)
  colP->setColorPos(1);
 undoRedoEnable = true;
 setRepeatMode(false);
 editMenu->setEnabled(true);
 colorMenu->setEnabled(true);
 toolsMenu->setEnabled(true);
 filtersMenu->setEnabled(true);
 viewMenu->setEnabled(true);
 imageMenu->setEnabled(true);
 emptyUndo(true);
 emptyRedo(true);
 loadLayer->setEnabled(true);
 fileSave->setEnabled(true);
 fileSaveAs->setEnabled(true);
 layerRecent->setEnabled(true);
 fileBrowseLayer->setEnabled(true);
 filePrint->setEnabled(true);
 drawToolBar->enableTool(selTool, true);
 drawToolBar->enableTool(freehand, true);
 drawToolBar->enableTool(magicWandTool, true);
 drawToolBar->enableTool(straightLineTool, true);
 drawToolBar->enableTool(pLineTool, true);
 drawToolBar->enableTool(eArcTool, true);
 drawToolBar->enableTool(bsTool, true);
 drawToolBar->enableTool(bezierTool, true);
 drawToolBar->enableTool(circleTool, true);
 drawToolBar->enableTool(ellipseTool, true);
 drawToolBar->enableTool(rectTool, true);
 drawToolBar->enableTool(quadTool, true);
 drawToolBar->enableTool(brushTool, true);
 drawToolBar->enableTool(sprayTool, true);
 drawToolBar->enableTool(mixTool, true);
 drawToolBar->enableTool(cleanTool, true);
 drawToolBar->enableTool(hshift, true);
 drawToolBar->enableTool(vshift, true);
 drawToolBar->enableTool(flipTool, true);
 drawToolBar->enableTool(mirrorxTool, true);
 drawToolBar->enableTool(mirroryTool, true);
 drawToolBar->enableTool(stTool, true);
 drawToolBar->enableTool(writeTool, true);
 drawToolBar->enableTool(transTool, true);
 drawToolBar->enableTool(freeTransTool, true);
 drawToolBar->enableTool(insRemPixTool, true);
 drawToolBar->enableTool(measureTool, true);
 drawToolBar->enableTool(fillTool, true);
 drawToolBar->enableTool(zoomin, true);
 drawToolBar->enableTool(zoomout, true);
 drawToolBar->enableTool(explodeTool, true);
}


void ApaintApp::layerMovedSlot(QRect pos)
{
 pbUtil->showSizeSlot(pos);
 //view->setVisible(pos);
}
                               
void ApaintApp::layerChangedSlot(Layer *sel)
{
 int cmapImage;

  if (!sel)
   {
    qDebug("Layer is null: Apaint.cpp\n");
    return;
   }
  setFocus();
  /*get selection info*/
  if (sel->getType() == MainLayer)
  {
    imageName = sel->imageBuf.imageName();
    setSizeCaption(sel->realLayerPos());    
    viewSizeChanged(sel->layerPos().width(), sel->layerPos().height());
    slotAspectChanged(0, 0); // params not used in this slot
    disconnect(repeatView, SIGNAL(toggled(bool)), this, SLOT(setRepeatMode(bool)));
    repeatView->setOn(sel->isUsingTiles());
    connect(repeatView, SIGNAL(toggled(bool)), this, SLOT(setRepeatMode(bool)));
    mainLayerLoaded = true;  
  }
 
  cmapImage = (sel->imageBuf.type() == LOAD_CMAP);

  colorRemap->setEnabled(cmapImage);
  colorMenu->setItemEnabled(HISTOGRAM_DIALOG, cmapImage);
  colorCountHorizontal->setEnabled(cmapImage);
  colorCountVertical->setEnabled(cmapImage);
  colorMenu->setItemEnabled(MAKE_COLOR_SHADES, cmapImage);
  colorTrueToCmap->setEnabled(!cmapImage);
  colorCmapToTrue->setEnabled(cmapImage);
  countColorsMenu->setEnabled(cmapImage);
  toolContour->setEnabled(cmapImage);
  toolsMenu->setItemEnabled(GUESS_WEAVE_DLG, cmapImage);
  filtersMenu->setItemEnabled(CONTOUR, cmapImage);
  despecleMenu->setEnabled(cmapImage);
  noiseMenu->setEnabled(cmapImage);
  placeMotifsMenu->setEnabled(cmapImage);
  coralMenu->setEnabled(cmapImage);
  colorShorterInsideMenu->setEnabled(cmapImage);
  colorShorterOutsideMenu->setEnabled(cmapImage);
  filtersMenu->setItemEnabled(CREATE_NOISE_XY, cmapImage);
  filtersMenu->setItemEnabled(CREATE_NOISE_X, cmapImage);
  filtersMenu->setItemEnabled(CREATE_NOISE_Y, cmapImage);
  selByColor->setEnabled(cmapImage);
  transparentColorsMenu->setEnabled(cmapImage);
  protectedColorsMenu->setEnabled(cmapImage);
  sortColorsMenu->setEnabled(cmapImage);
  colP->layerChangedSlot(sel);
}

void ApaintApp::setGridProp()
{
  view->setGridColor(gridProp->getGridColor());
  view->setGridSize(gridProp->getWid(), gridProp->getHei());
  view->refresh(-1, -1, -1, -1);
}

void ApaintApp::showReduceColDlg()
{
 rcDlg->startDlg();
}

void ApaintApp::showColNumDlg()
{
 cnDlg->startDlg();
}

void ApaintApp::emptyUndo(bool empty)
{
 canUndo = !empty;
 drawToolBar->enableTool(undoTool, !empty);
 editUndo->setEnabled(!empty);
}

void ApaintApp::emptyRedo(bool empty)
{
 canRedo = !empty;
 drawToolBar->enableTool(redoTool, !empty);
 editRedo->setEnabled(!empty);
}

void ApaintApp::layerOp()
{
 if ((doc->numLayers() > 1) != multipleLayers)
  {
   multipleLayers = !multipleLayers;
   editCut->setEnabled(multipleLayers);
   editCopy->setEnabled(multipleLayers);
   saveLayer->setEnabled(multipleLayers);
   editDelete->setEnabled(multipleLayers);
   editScale->setEnabled(multipleLayers);
   editPasteMultiple->setEnabled(multipleLayers);
   drawToolBar->enableTool(lMoveTool, multipleLayers);
  }
 selectProp->setAlignEnabled(multipleLayers && (doc->getLayer() != doc->getMainLayer()));
 drawToolBar->enableTool(unselTool, multipleLayers && (doc->getLayer() != doc->getMainLayer()));
 toolInvert->setEnabled(doc->numLayers() > 1);
 toolFlatten->setEnabled(doc->numLayers() > 1);
}

void ApaintApp::documentChanged(bool changed)
{
 fileReload->setEnabled(changed && (doc->getMainLayer()->getFilename() != ""));
}

void ApaintApp::hasBrushes(bool has)
{
 toolBrushToLayer->setEnabled(has);
}

void ApaintApp::showColorRemapDialog()
{
  colRemapDialog->startDlg();
}

void ApaintApp::insertColor(QRgb rgb, int num)
{
 view->setFocus();
 if (rcDlg->isOn())
  rcDlg->showColor(num);
 if (colRemapDialog->isOn())
  colRemapDialog->insertMainColor(num);
}

void ApaintApp::showViewGrid(bool toggle)
{
 view->showGrid(toggle);
}

void ApaintApp::showRepeatDlg()
{
 repDim->showDlg();
}

void ApaintApp::slotFileNew()
{
 if (canLoadNewDoc())
   nfile->getNewFile();
}

void ApaintApp::loadBrushSlot()
{
 fileDlg->setCaption(ApaintConfig::getText(LOAD_LAYER_MSG));
 fileDlg->setDir(imgPath);
 fileDlg->setSelection("");
 if (fileDlg->exec() == QDialog::Accepted)
  {
   fileName = fileDlg->selectedFile();
   loadBrushFile(fileName);
  }
 imgPath = fileDlg->dirPath(); 
}

void ApaintApp::loadBrushFile(QString &fname)
{
 ApaintBrushList *bList = ApaintBrushList::getObject();
 if (!fname.isEmpty())
  {
   Layer *brushLayer = new  Layer(fname, ImageLayer, doc->getAspectX(), doc->getAspectY()) ;
   if (!brushLayer) return;
   bList->addBrush(brushLayer);
   delete brushLayer;
  }
}

void ApaintApp::slotLayerLoad()
{
 fileDlg->setCaption(ApaintConfig::getText(LOAD_LAYER_MSG));
 fileDlg->setDir(imgPath);
 fileDlg->setSelection("");
 if (fileDlg->exec() == QDialog::Accepted)
  {
   fileName = fileDlg->selectedFile(); 
   if (!fileName.isEmpty())
    loadLayerFile(fileName);
  }
 imgPath = fileDlg->dirPath();
}

void ApaintApp::slotLayerNew()
{
 nfile->getNewLayer();
}

void ApaintApp::slotLayerSave()
{
 doc->saveLayer();
}

void ApaintApp::slotFileOpen()
{ 
 if (!canLoadNewDoc())
  return;
 fileDlg->setCaption(ApaintConfig::getText(LOAD_PICTURE_MSG));
 fileDlg->setDir(imgPath);
 fileDlg->setSelection("");
 if (fileDlg->exec() == QDialog::Accepted)
  {   
   fileName = fileDlg->selectedFile();
   if (!fileName.isEmpty())
    loadDocSlot(fileName);
  }
 imgPath = fileDlg->dirPath();
}

//#include <iostream>
//using namespace std;

namespace
{
 QString correctFileName(char *fileName, QImage &image)
 {
  QString ret(QString::fromLocal8Bit(fileName));
  imageInfo iI;

  if (!ApaintConfig::getRecentFilesWithPath())
   for (int i=ret.length(); i>0; --i)
    if (ret[i-1] == '/')
     {
      ret.remove(0, i);
      break;
     }

  memset(&iI, 0, sizeof(iI));

  int result;
  struct stat buf;
  aColor c[256];

  stat(fileName, &buf);
  iI.modifiedTime = buf.st_mtime;
  setImageBrowseCmap(c);
  result = loadImageI(fileName, &iI);

  if (result < 1 || iI.iconData == 0)
  {
   char *fileNameForDir = strdup(fileName);

   if (iI.Fname != 0)
    free(iI.Fname);
   iI.Fname = strdup(basename(fileName));
   int useless;
   setBrowsePath(dirname(fileNameForDir));
   result = loadImageMakeIcon(&iI, c, &useless);
   free(fileNameForDir);
  }

  if (result > 0)
   {
    uLongf uncompressedIconLen = iI.iconY*((iI.iconType==LOAD_TRUE_COL) ? iI.iconX*3 : ((iI.iconType==LOAD_BW) ? (iI.iconX+7)/8 : iI.iconX));
    unsigned char iData[uncompressedIconLen];

    uncompress(iData, &uncompressedIconLen, iI.iconData, iI.compressedIconLen);

    uchar *data = iData;
    image.create(iI.iconX, iI.iconY, 32);

    switch(iI.iconType)
     {
      case LOAD_TRUE_COL:
       for (int y=0; y<iI.iconY; y++)
	for (int x=0; x<iI.iconX; x++)
	 {
	  image.setPixel(x, y, qRgb(data[0], data[1], data[2]));
	  data += 3;
	 }
       break;
      case LOAD_CMAP:
       for (int y=0; y<iI.iconY; y++)
	for (int x=0; x<iI.iconX; x++)
	 {
	  uchar *col = iI.iconCmap+3*(*data);
	  image.setPixel(x, y, qRgb(col[0], col[1], col[2]));
	  data++;
	 }
       break;
      case LOAD_GREY:
      case LOAD_GREYSCALE:
       for (int y=0; y<iI.iconY; y++)
	for (int x=0; x<iI.iconX; x++)
	 {
	  image.setPixel(x, y, qRgb(*data, *data, *data));
	  data++;
	 }
       break;
     }
  }

  if (ApaintConfig::getShowRecentFilesDetail() || ApaintConfig::getShowRecentFilesDate())
   {
    if (ApaintConfig::getShowRecentFilesDetail())
     {
      ret += QString("\n%1 (%2x%3")
	  .arg(returnLoaderString(iI.loaderID))
	  .arg(iI.imageX)
	  .arg(iI.imageY);
      if (iI.imageType==LOAD_CMAP)
       {
	ret += QString(" %1 %2 ")
	 .arg(iI.usedColors ? iI.usedColors : 255)
	 .arg(ApaintConfig::getText(COLORS_MSG));
       }
      if (iI.imageType==LOAD_RGBA)
       ret += " "+ApaintConfig::getText(636);
      ret += QString(")");
     }

    struct stat statBuf;
    struct tm *tmTime;

    if (ApaintConfig::getShowRecentFilesDate() && !stat(fileName, &statBuf))
     {
      char smallStr[100];
      tmTime = localtime(&statBuf.st_mtime);
      strftime(smallStr, 40, "%d.%b.%Y %H:%M", tmTime);

      if (ApaintConfig::getShowRecentFilesDetail()) ret += QString(" ");
      else ret += "\n";

      ret += smallStr;
     }
   }

  free(fileName);
  freeImageI(&iI);
  return ret;
 }

 QStringList readFiles, writeFiles;
 QStringList readFilesFullPath, writeFilesFullPath;
 QValueVector<QImage> readImages, writeImages;
// changed to fetch directory history along with read and write file paths

 QStringList directories;

 void getRecentFileNameLists()
 {
  char *readFileList[MAX_FILE_HISTORY], *writeFileList[MAX_FILE_HISTORY];
  char *directoryList[MAX_FILE_HISTORY];

  QString historyFile = getImageIconPath();

  historyFile += "/history";
  readFiles.clear();
  writeFiles.clear();

  directories.clear();;

  readImages.clear();
  writeImages.clear();
  readFilesFullPath.clear();
  writeFilesFullPath.clear();
  memset(readFileList, 0, sizeof(char*)*MAX_FILE_HISTORY);
  memset(writeFileList, 0, sizeof(char*)*MAX_FILE_HISTORY);
  memset(directoryList, 0, sizeof(char*)*MAX_FILE_HISTORY);
  getRecentFiles(readFileList, writeFileList, directoryList , const_cast<char*>(historyFile.local8Bit().data()), ApaintConfig::getNumRecentFiles());

  for (int i=0; i<MAX_FILE_HISTORY && readFileList[i] != 0; i++)
   {
    readFilesFullPath.push_back(QString::fromLocal8Bit(readFileList[i]));
    readImages.push_back(QImage());
    readFiles.push_back(correctFileName(readFileList[i], readImages.last()));
   }

  for (int i=0; i<MAX_FILE_HISTORY && writeFileList[i] != 0; i++)
   {
    writeFilesFullPath.push_back(QString::fromLocal8Bit(writeFileList[i]));
    writeImages.push_back(QImage());
    writeFiles.push_back(correctFileName(writeFileList[i], writeImages.last()));
   }

  for (int i=0; i<MAX_FILE_HISTORY && directoryList[i] != 0; i++)
   directories.push_back(QString::fromLocal8Bit(directoryList[i]));
 }
}

class RecentFileMenuItem : public QCustomMenuItem
{
	QString str;
	int width;
	int height;
	QImage &image;

public:
	RecentFileMenuItem(QString str, QImage &im, int size) : image(im)
	{
	 this->str = str;
	 width = size;
	 height = size;

	 if (image.width() > 0 && image.height() > 0)
	 {
	  if (image.width() > image.height()) height = round(1.0*size*image.height()/image.width());
	  else width = round(1.0*size*image.width()/image.height());
	 }
	}
	/* we need this othwerise we have extra space at the beginning */
	bool fullSpan() const
	{
	 return true;
	}
	void paint(QPainter* p, const QColorGroup& /*cg*/, bool /*act*/, bool /*enabled*/, int x, int y, int w, int h)
	{
#define DISTANCE 4
#define MARGIN 4
	 if (width > 0 && height > 0)
	  p->drawImage(QRect(x+MARGIN+((QMAX(width,height)-width)>>1), y+((h-height)>>1), width, height), image);
	 p->drawText(x+MARGIN+DISTANCE+QMAX(width, height), y, w, h, AlignLeft | AlignVCenter | ShowPrefix | DontClip, str);
	}

    QSize sizeHint()
    {
     QSize size = QFontMetrics(QFont()).size(AlignLeft | AlignVCenter | ShowPrefix | DontClip, str);
     int w = size.width()+DISTANCE+MARGIN+QMAX(width, height);
     int h = height+2;
#undef MARGIN
#undef DISTANCE

     if (h < size.height())
      h = size.height();

     return QSize(w, h);
    }
};

void ApaintApp::slotRecent()
{
 QStringList &write = writeFiles;
 QStringList &read = readFiles;

 getRecentFileNameLists();
 fileRecent->clear();
 layerRecent->clear();
 int size = IMAGE_ICON_SIZE / ApaintConfig::getDivideRecentFilesIcon();

 for (uint i=0; i<write.size(); i++)
  {
   fileRecent->insertItem(new RecentFileMenuItem(write[i], writeImages[i], size));
   layerRecent->insertItem(new RecentFileMenuItem(write[i], writeImages[i], size));
  }

 if (read.size() > 0)
  {
   if (write.size() > 0)
    {
     fileRecent->insertSeparator(fileRecent->count());
     layerRecent->insertSeparator(fileRecent->count());
    }

   for (uint i=0; i<read.size(); i++)
    {
     fileRecent->insertItem(new RecentFileMenuItem(read[i], readImages[i], size));
     layerRecent->insertItem(new RecentFileMenuItem(read[i], readImages[i], size));
    }
  }
}

void ApaintApp::slotRecentFile(int id)
{
 uint num;

 for (num=0; num<writeFilesFullPath.size(); num++)
  if (fileRecent->idAt(num) == id)
   {
    loadDocSlot(writeFilesFullPath[num]);
    return;
   }

 int offset = writeFilesFullPath.size() + ((readFilesFullPath.size() > 0 && writeFilesFullPath.size() > 0) ? 1 : 0);

 for (num=0; num<readFilesFullPath.size(); num++)
  if (fileRecent->idAt(num+offset) == id)
   {
    loadDocSlot(readFilesFullPath[num]);
    return;
   }
}

void ApaintApp::slotRecentLayer(int id)
{
 uint num;

 for (num=0; num<writeFilesFullPath.size(); num++)
  if (layerRecent->idAt(num) == id)
   {
    loadLayerFile(writeFilesFullPath[num]);
    return;
   }

 int offset = writeFilesFullPath.size() + ((readFilesFullPath.size() > 0 && writeFilesFullPath.size() > 0) ? 1 : 0);

 for (num=0; num<readFilesFullPath.size(); num++)
  if (layerRecent->idAt(num+offset) == id)
   {
    loadLayerFile(readFilesFullPath[num]);
    return;
   }
}

void ApaintApp::loadDocSlot(QString &fname)
{
 drawToolBar->setTool(brushTool, true);
 if (canLoadNewDoc())
  {
   char *fnameAscii =strdup(fname.local8Bit().data()); /* dirname below changes source, so we need a copy! */
   bool wasRepeat = FALSE;
   
   if (doc->getLayer())
    wasRepeat = doc->getLayer()->isUsingTiles();

   doc->load(fname);
   unredo->resetDocChanged();
   imgPath = QString("%1").arg(QString::fromLocal8Bit(dirname(fnameAscii)));
   free(fnameAscii);
   if (wasRepeat)
    setRepeatMode(TRUE);
  }
}

void ApaintApp::loadLayerFile(QString& fname)
{
 QRect pos= view->getContentsSize();
 if (!doc->loadLayer(fname))
  return;
 doc->setLayerPos(pos.x()+pos.width()/2, pos.y()+pos.height()/2);
 doc->putUndoStack(CreateLayer);
 doc->refreshLayer();
 drawToolBar->setTool(lMoveTool, true);   
}

void ApaintApp::setSizeCaption(QRect size)
{
 imageSize = size;
 showDocCaption();
}

void ApaintApp::showDocCaption()
{
 label=QString("%1 %2  (%3x%4) %5%").arg("ArahPaint " VERSION_NUM).arg(ApaintConfig::getFilenamePathInWindowTitle() ? imageName : ApaintUtil::getBaseName(imageName)).arg(imageSize.width()).arg(imageSize.height()).arg(imageScale);
 if (unredo->isDocChanged())
  {
   label+=QString(" ");
   label+=ApaintConfig::getText(MODIFIED_MSG);
  }
 setCaption(label); 
}

void ApaintApp::slotShowCaption(double scaleX, double scaleY)
{
 imageScale =  (int)(scaleX*100);
 showDocCaption();
}

void ApaintApp::slotFileSave()
{
 QString name; // return name

 if (doc->save(false, name))
  unredo->resetDocChanged();
 showDocCaption();
}

void ApaintApp::slotFileSaveAs()
{
 QString name;

 if (doc->save(true, name))
  unredo->resetDocChanged();
 showDocCaption();
}

void ApaintApp::slotFilePrint()
{
 printDlg->showDlg();
}

/*clipboard data change*/
void ApaintApp::clipBChanged()
{  
 QMimeSource *ms = clipB->data(); 

 if (ms->provides(APAINT_MIME_SIG))
  {      
   editPaste->setEnabled(true);
   editPasteMultiple->setEnabled(true);
  }
 else
  {
   editPaste->setEnabled(false);
   editPasteMultiple->setEnabled(false);
  }
}

bool ApaintApp::doStuffOnExit()
{
 if (unredo->isDocChanged())
  {
   switch (queryExit())
    {
     case 0:
       slotFileSave();
       return true;
     case 1:
       return true;
     case 2:
       return false; //cancel
    }
  }
 return true;
}

bool ApaintApp::canLoadNewDoc()
{
  Layer *sel = doc->getLayer();
  if (!sel)
   return true;

  return doStuffOnExit();
}

void ApaintApp::revertToSaved()
{
  Layer *sel = doc->getLayer();
  if (!sel)
   return;

 QString fileName = doc->getMainLayer()->imageBuf.imageName();

 if(unredo->isDocChanged() && queryRevert() == 0) doc->load(fileName);
}

void ApaintApp::slotFileQuit()
{
  ///////////////////////////////////////////////////////////////////
  // exits the Application  
  if (doStuffOnExit())
   qApp->quit();
}

void ApaintApp::closeEvent(QCloseEvent *e)
{
 if (doStuffOnExit())
  QWidget::closeEvent(e);
}

                                                                  
void ApaintApp::slotEditUndo()
{
  if (undoRedoEnable)
    unredo->undoLast();
}

void ApaintApp::slotEditRedo()
{
  if (undoRedoEnable)
    unredo->redoLast();
}

void ApaintApp::slotEditCut()
{
  Layer *layer = doc->getLayer();
  Layer *mainLayer = doc->getMainLayer();
  if (!layer || !mainLayer) return;
  //doc->cutSelectedLayer(); old way...not very good
  
  if (layer->getType() !=MainLayer)
  {
    QRect pos = layer->layerPos();
    bool useMask = layer->usingMask();
    ApaintClipboard::putOnClipboard(layer);
    if (layer->getIsResizeable())
    {
      if (mainLayer->getImageType() == LOAD_TRUE_COL)
        mainLayer->fill(pos.x(), pos.y(), pos.width(), pos.height(),
                        pbUtil->background(), useMask, &layer->getMask());
      else
        mainLayer->fill(pos.x(), pos.y(), pos.width(), pos.height(),
                        pbUtil->bgIndex(), useMask, &layer->getMask());
    }
    doc->putUndoStack(DestroyLayer);
    doc->deleteLayer();
    doc->refreshLayer(pos);
    canPaste = true;
    editPaste->setEnabled(true);
  }
}

void ApaintApp::slotEditCopy()
{
  Layer *layer = doc->getLayer();
  if (!layer) return;

  if (layer->getType() !=MainLayer)
  {
    canPaste = true;
    editPaste->setEnabled(true);
    ApaintClipboard::putOnClipboard(layer);
  }
}

void ApaintApp::slotEditPaste()
{
 bool transCol[256];
 Layer *layer = doc->getLayer();
 if (!layer) return;
 QRect pos= view->getContentsSize();

 if (layer->getImageType() == LOAD_CMAP)
  memcpy(transCol, layer->getTransColors(), 256*sizeof(bool)); 

 //doc->pasteSelectedLayer(pos.x()+pos.width()/2, pos.y()+pos.height()/2);
 ApaintClipboard::getFromClipboard(doc, pos.x()+pos.width()/2, pos.y()+pos.height()/2);

 /* layer pointer has changed in above function, so we need to get a new one */
 layer = doc->getLayer();
 if (layer->getImageType() == LOAD_CMAP)
  {
   memcpy(layer->getTransColors(), transCol, 256*sizeof(bool));
   doc->sendSignals(true, false);
   doc->refreshLayer();
  }
 drawToolBar->setTool(lMoveTool, true);
}

void ApaintApp::slotEditSelectAll()
{
 Layer *layer = doc->getLayer();

 if (!layer)
  return;

 if (layer->getType() == MainLayer)
  {
   /* avoid making huge selections in repeat mode */
   if (layer->isUsingTiles())
    return;

   QRect sel = QRect(0,0, doc->getLayer()->layerPos().width(), doc->getLayer()->layerPos().height());
   doc->createNewLayer(sel.x(), sel.y(), sel.width(), sel.height(), SelectionLayer, RectangleG);
   doc->sendSignals(true, false);
   doc->refreshLayer(sel);
  }
}

void ApaintApp::slotEditDelete()
{
 Layer *layer = doc->getLayer();

 if (!layer)
  return;

 if (layer->getType() != MainLayer)
  {
   doc->putUndoStack(DestroyLayer);
   doc->deleteLayer(doc->getLayerNum());
  }
}

void ApaintApp::slotEditClear()
{
 if (doc->getLayer() == 0) return;

 doc->putUndoStack(ChangeLayerForm);
 if (doc->getLayer()->getImageType() == LOAD_CMAP)
  doc->layerApplyFilter(FILTER_CLEAR_IMAGE, pbUtil->fgIndex(), pbUtil->bgIndex(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
 else
  doc->layerApplyFilter(FILTER_CLEAR_IMAGE, (int) pbUtil->foreground(), (int) pbUtil->background(), measureToolProp->getSizeX(), measureToolProp->getSizeY(), measureToolProp->getValue(), FALSE, FALSE, FALSE);
}

void ApaintApp::slotEditScale()
{
 doc->putUndoStack(ChangeLayerPos);
 doc->putUndoStack(doc->getMainLayer(), ChangeSelToMainLayer);
 doc->selectionToMainLayer();
}

void ApaintApp::slotColorsPopylarity()
{
 doc->layerSortColors(SORT_COLORS_POPULARITY);
}

void ApaintApp::slotColorsHue()
{
 doc->layerSortColors(SORT_COLORS_HUE);
}

void ApaintApp::slotColorsSaturation()
{
 doc->layerSortColors(SORT_COLORS_SATURATION);
}

void ApaintApp::slotColorsLightness()
{
 doc->layerSortColors(SORT_COLORS_LIGHTNESS);
}

void ApaintApp::slotColorsSimilarity()
{
 doc->layerSortColors(SORT_COLORS_SIMILARITY);
}

void ApaintApp::slotColorCountHorizontal()
{
  int numColors;
  int haveColors, colors, maxColors;
  int totalX, totalY;
  uchar usedColors[256];
  
  /* it will crash if we are in repeat view */
  repeatView->setOn(false);

  doc->setActiveLayer(0);
  if (doc->getLayer()->getImageType() != LOAD_CMAP)
   return;
  numColors =  doc->getLayer()->numColors();
  if (numColors == 256)
   return;
  doc->putUndoStack(doc->getLayer(), CreateLayer);
  totalX = doc->getLayer()->layerPos().width();
  totalY = doc->getLayer()->layerPos().height();

  /* first pass: find out the maximum number of horizontal colors in the image */
  maxColors = 1;
  for (int x = 0; x < totalX; x++)
   {
    memset(usedColors, FALSE, numColors);
    for (int y = 0; y<totalY; y++)
     usedColors[doc->getMainLayer()->imageBuf.scanLine(y)[x]]=TRUE;
    haveColors = 0;
    for (colors = 0; colors<numColors; colors++)
     if (usedColors[colors])
	 haveColors++;
    if (haveColors>maxColors)
     maxColors = haveColors;
   }

  QRect posView= view->getContentsSize();
  QRect sel = QRect(0,posView.y(), totalX, maxColors);
  doc->createNewLayer(sel.x(), sel.y(), sel.width(), sel.height(), SelectionLayer, RectangleG);
  Layer* selLayer = doc->getLayer();
  if (!selLayer)
   return;
  selLayer->setNumColors(selLayer->numColors() + 1, false);
  selLayer->setTransColor(selLayer->numColors() - 1, true);
  for (int x = 0; x < totalX; x++)
   {
    memset(usedColors, FALSE, numColors);
    for (int y = 0; y<totalY; y++)
     usedColors[doc->getMainLayer()->imageBuf.scanLine(y)[x]]=TRUE;
    haveColors = 0;
    for (colors = 0; colors<numColors; colors++)
     if (usedColors[colors])
      selLayer->imageBuf.scanLine(haveColors++)[x] = colors;
    /* set the ending area to transparent color, so we know which colors are used */
    for (int y = haveColors; y<maxColors; y++)
     selLayer->imageBuf.scanLine(y)[x]=numColors;
   }
  memcpy(selLayer->origBuf.scanLine(0), selLayer->imageBuf.scanLine(0), sel.width()*sel.height());
  doc->sendSignals(true, false);
  doc->refreshLayer(sel);
}

void ApaintApp::slotColorCountVertical()
{
  int numColors;
  int haveColors, colors, maxColors;
  int totalX, totalY;
  uchar *selPtr, *mainPtr;
  uchar usedColors[256];

  /* it will crash if we are in repeat view */
  repeatView->setOn(false);

  doc->setActiveLayer(0);
  if (doc->getLayer()->getImageType() != LOAD_CMAP)
   return;
  numColors =  doc->getLayer()->numColors();
  if (numColors == 256)
   return;
  doc->putUndoStack(doc->getLayer(), CreateLayer);
  totalX = doc->getLayer()->layerPos().width();
  totalY = doc->getLayer()->layerPos().height();

  QRect posView= view->getContentsSize();
  /* first pass: find out the maximum number of horizontal colors in the image */
  maxColors = 1;
  for (int y = 0; y < totalY; y++)
   {
    memset(usedColors, FALSE, numColors);
    mainPtr = doc->getMainLayer()->imageBuf.scanLine(y);
    for (int x = totalX; x; x--)
     usedColors[*mainPtr++]=TRUE;
    haveColors = 0;
    for (int x = 0; x < numColors; x++)
     if (usedColors[x])
	 haveColors++;
    if (haveColors>maxColors)
     maxColors = haveColors;
   }

  QRect sel = QRect(posView.x(),0, maxColors, totalY);
  doc->createNewLayer(sel.x(), sel.y(), sel.width(), sel.height(), SelectionLayer, RectangleG);
  Layer* selLayer = doc->getLayer();
  if (!selLayer)
   return;
  selLayer->setNumColors(selLayer->numColors() + 1, false);
  selLayer->setTransColor(selLayer->numColors() - 1, true);
  for (int y = 0; y < totalY; y++)
   {
    memset(usedColors, FALSE, numColors);
    selPtr = selLayer->imageBuf.scanLine(y);
    mainPtr = doc->getMainLayer()->imageBuf.scanLine(y);
    for (int x = totalX; x; x--)
     usedColors[*mainPtr++]=TRUE;
    haveColors = 0;
    for (colors = 0; colors<numColors; colors++)
     if (usedColors[colors])
      selPtr[haveColors++] = colors;
    /* set the ending area to transparent color, so we know which colors are used */
    for (int x = haveColors; x<maxColors; x++)
     selPtr[x]=numColors;
   }
  memcpy(selLayer->origBuf.scanLine(0), selLayer->imageBuf.scanLine(0), sel.width()*sel.height());
  doc->sendSignals(true, false);
  doc->refreshLayer(sel);
}

void ApaintApp::slotColorInvertProtected()
{
 doc->layerInvertProtectedColors();
}

void ApaintApp::slotColorClearProtected()
{
 doc->layerClearProtectedColors();
}

void ApaintApp::slotColorInvertTransparent()
{
 doc->layerInvertTransparentColors();
}

void ApaintApp::slotColorClearTransparent()
{
 doc->layerClearTransparentColors();
}

void ApaintApp::slotColorEdit()
{
 pbUtil->showColorDlg();
}

void ApaintApp::slotColorInvertAll()
{
 doc->layerInvertColors(INVERT_COLORS_ALL);
}

void ApaintApp::slotColorInvertRed()
{
 doc->layerInvertColors(INVERT_COLORS_RED);
}

void ApaintApp::slotColorInvertGreen()
{
 doc->layerInvertColors(INVERT_COLORS_GREEN);
}

void ApaintApp::slotColorInvertBlue()
{
 doc->layerInvertColors(INVERT_COLORS_BLUE);
}

void ApaintApp::slotColorInvertCyan()
{
 doc->layerInvertColors(INVERT_COLORS_CYAN);
}

void ApaintApp::slotColorInvertYellow()
{
 doc->layerInvertColors(INVERT_COLORS_YELLOW);
}

void ApaintApp::slotColorInvertMagenta()
{
 doc->layerInvertColors(INVERT_COLORS_MAGENTA);
}

void ApaintApp::slotColorInvertGrey()
{
 doc->layerInvertColors(INVERT_COLORS_GREY);
}

void ApaintApp::slotColorInvertSaturation()
{
 doc->layerInvertColors(INVERT_COLORS_SATURATION);
}

void ApaintApp::slotColorSwapPalette()
{
 doc->layerInvertColors(SWAP_COLORS_PALETTE);
}

void ApaintApp::slotColorSwapRGB_GBR()
{
 doc->layerInvertColors(SWAP_COLORS_RGB_GBR);
}

void ApaintApp::slotColorSwapRGB_BRG()
{
 doc->layerInvertColors(SWAP_COLORS_RGB_BRG);
}

void ApaintApp::slotColorSwapRG()
{
 doc->layerInvertColors(SWAP_COLORS_RG);
}

void ApaintApp::slotColorSwapRB()
{
 doc->layerInvertColors(SWAP_COLORS_RB);
}

void ApaintApp::slotColorSwapGB()
{
 doc->layerInvertColors(SWAP_COLORS_GB);
}

void ApaintApp::slotColorSwapSL()
{
 doc->layerInvertColors(SWAP_COLORS_SL);
}

void ApaintApp::slotContrastStretch()
{
 doc->layerContrastStretch();
}

void ApaintApp::slotFindRepeat()
{
 menuActivated(FIND_REPEAT);
}

void ApaintApp::slotFindExactRepeat()
{
 menuActivated(FIND_EXACT_REPEAT);
}


void ApaintApp::slotRemoveGuides()
{
// should remove guides here
}

void ApaintApp::slotDuplicateX1()  { menuActivated(DUPLICATE_X1); }
void ApaintApp::slotDuplicateX2()  { menuActivated(DUPLICATE_X2); }
void ApaintApp::slotDuplicateX3()  { menuActivated(DUPLICATE_X3); }
void ApaintApp::slotDuplicateX4()  { menuActivated(DUPLICATE_X4); }
void ApaintApp::slotDuplicateX5()  { menuActivated(DUPLICATE_X5); }
void ApaintApp::slotDuplicateX6()  { menuActivated(DUPLICATE_X6); }
void ApaintApp::slotDuplicateX7()  { menuActivated(DUPLICATE_X7); }
void ApaintApp::slotDuplicateX8()  { menuActivated(DUPLICATE_X8); }
void ApaintApp::slotDuplicateX9()  { menuActivated(DUPLICATE_X9); }
void ApaintApp::slotDuplicateX10() { menuActivated(DUPLICATE_X10); }

void ApaintApp::slotDuplicateY1()  { menuActivated(DUPLICATE_Y1); }
void ApaintApp::slotDuplicateY2()  { menuActivated(DUPLICATE_Y2); }
void ApaintApp::slotDuplicateY3()  { menuActivated(DUPLICATE_Y3); }
void ApaintApp::slotDuplicateY4()  { menuActivated(DUPLICATE_Y4); }
void ApaintApp::slotDuplicateY5()  { menuActivated(DUPLICATE_Y5); }
void ApaintApp::slotDuplicateY6()  { menuActivated(DUPLICATE_Y6); }
void ApaintApp::slotDuplicateY7()  { menuActivated(DUPLICATE_Y7); }
void ApaintApp::slotDuplicateY8()  { menuActivated(DUPLICATE_Y8); }
void ApaintApp::slotDuplicateY9()  { menuActivated(DUPLICATE_Y9); }
void ApaintApp::slotDuplicateY10() { menuActivated(DUPLICATE_Y10); }

void ApaintApp::slotScale2X() { menuActivated(SCALE_2X); }
void ApaintApp::slotScale3X() { menuActivated(SCALE_3X); }
void ApaintApp::slotScale4X() { menuActivated(SCALE_4X); }

void ApaintApp::slotShrink2X() { menuActivated(SHRINK_2X); }
void ApaintApp::slotShrink3X() { menuActivated(SHRINK_3X); }
void ApaintApp::slotShrink4X() { menuActivated(SHRINK_4X); }

void ApaintApp::slotHalveX() { menuActivated(HALVE_X); }
void ApaintApp::slotHalveY() { menuActivated(HALVE_Y); }

void ApaintApp::slotShrinkX() { menuActivated(SHRINK_X); }
void ApaintApp::slotShrinkY() { menuActivated(SHRINK_Y); }

void ApaintApp::slotMultiplyX2() { menuActivated(MULTIPLY_X2); }
void ApaintApp::slotMultiplyX3() { menuActivated(MULTIPLY_X3); }
void ApaintApp::slotMultiplyX4() { menuActivated(MULTIPLY_X4); }
void ApaintApp::slotMultiplyX5() { menuActivated(MULTIPLY_X5); }
void ApaintApp::slotMultiplyX6() { menuActivated(MULTIPLY_X6); }
void ApaintApp::slotMultiplyX7() { menuActivated(MULTIPLY_X7); }
void ApaintApp::slotMultiplyX8() { menuActivated(MULTIPLY_X8); }
void ApaintApp::slotMultiplyX9() { menuActivated(MULTIPLY_X9); }
void ApaintApp::slotMultiplyX10() { menuActivated(MULTIPLY_X10); }

void ApaintApp::slotMultiplyY2() { menuActivated(MULTIPLY_Y2); }
void ApaintApp::slotMultiplyY3() { menuActivated(MULTIPLY_Y3); }
void ApaintApp::slotMultiplyY4() { menuActivated(MULTIPLY_Y4); }
void ApaintApp::slotMultiplyY5() { menuActivated(MULTIPLY_Y5); }
void ApaintApp::slotMultiplyY6() { menuActivated(MULTIPLY_Y6); }
void ApaintApp::slotMultiplyY7() { menuActivated(MULTIPLY_Y7); }
void ApaintApp::slotMultiplyY8() { menuActivated(MULTIPLY_Y8); }
void ApaintApp::slotMultiplyY9() { menuActivated(MULTIPLY_Y9); }
void ApaintApp::slotMultiplyY10() { menuActivated(MULTIPLY_Y10); }

void ApaintApp::slotColorTrueToCmap()
{
 doc->putUndoStack(ChangeLayerForm);
 doc->setLayerImageType(LOAD_CMAP, false);
}

void ApaintApp::slotColorCmapToTrue()
{
 doc->putUndoStack(ChangeLayerForm);
 doc->setLayerImageType(LOAD_TRUE_COL, false);
}

void ApaintApp::slotLoadColormap()
{
 iconDlg->showDlg(imgPath, IconViewDlg::ColormapImage); 
}

void ApaintApp::slotLoadColormapDither()
{
 iconDlg->showDlg(imgPath, IconViewDlg::ColormapDitherImage); 
}

void ApaintApp::slotToolsContour()
{
 dlgContour->showDlg();
}

void ApaintApp::slotContourRemove()
{
 menuActivated(REMOVE_CONTOUR);
}

void ApaintApp::slotContourThin()
{
 menuActivated(THIN_CONTOUR);
}

void ApaintApp::slotContourPrune()
{
 menuActivated(PRUNE_CONTOUR);
}

void ApaintApp::slotViewRepeat()
{
 repeatDlg->showDlg();
 viewRepeatDelimiter->setEnabled(true);
}

void ApaintApp::slotToolsFlatten()
{
 int i, haveLayers = doc->numLayers();

 /* we do not use function collapsLayers, so that we have undo... */
 for (i=1; i<haveLayers; i++)
  {
   doc->setLayerNum(1);
   ApaintDocUtil::destroyCurLayer(doc);
  }
}

void ApaintApp::slotFiltersRepeatLast1()
{
 if (lastOperation == -1)
  {
   QApplication::beep();
   return;
  }
 switch (lastOperation)
  {
   case CONTOUR_AVERAGE : slotFiltersContourAverage(); break;
   case CONTOUR_DESPECLE_ALL : slotFiltersContourDespecleAll(); break;
   case CONTOUR_DESPECLE_SELECTED : slotFiltersContourDespecleSelected(); break;
   default : menuActivated(lastOperation);
  }
}

void ApaintApp::slotFiltersRepeatLast10()
{
 if (lastOperation == -1)
  {
   QApplication::beep();
   return;
  }
 for (int i=0; i<10; i++)
  switch (lastOperation)
   {
    case CONTOUR_AVERAGE : slotFiltersContourAverage(); break;
    case CONTOUR_DESPECLE_ALL : slotFiltersContourDespecleAll(); break;
    case CONTOUR_DESPECLE_SELECTED : slotFiltersContourDespecleSelected(); break;
    default : menuActivated(lastOperation);
   }
}

void ApaintApp::slotFiltersContourAverage()
{
 doc->putUndoStack(ChangeLayerForm);
 doc->layerContourAverage(pbUtil->getLineSize(), pbUtil->getLineSize());
 lastOperation = CONTOUR_AVERAGE;
}

void ApaintApp::slotFiltersContourDespecleAll()
{
 doc->putUndoStack(ChangeLayerForm);
 doc->layerContourDespecle(-1, pbUtil->getLineSize());
 lastOperation = CONTOUR_DESPECLE_ALL;
}

void ApaintApp::slotFiltersContourDespecleSelected()
{
 doc->putUndoStack(ChangeLayerForm);
 doc->layerContourDespecle(pbUtil->fgIndex(), pbUtil->getLineSize());
 lastOperation = CONTOUR_DESPECLE_SELECTED;
}

void ApaintApp::slotViewToolBar(bool toggle)
{
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off

  if (toggle== false)
   fileToolbar->hide();
  else
   fileToolbar->show();
}

void ApaintApp::slotViewStatusBar(bool toggle)
{
 ///////////////////////////////////////////////////////////////////
 //turn Statusbar on or off

 if (toggle == false)
   statusBar->hide();
 else
   statusBar->show();
}

void ApaintApp::selectByColorSlot()
{
  Layer *layer = doc->getLayer();
  if (!layer) return;
  if (layer->getImageType() != LOAD_CMAP) return;
  
  uchar fgIndx = layer->getFgIndx();
  QRect area;
  QImage mask = ApaintImageUtil::getMaskByColor(layer->imageBuf, fgIndx, area);
  doc->createNewLayer(layer, &mask, area, (layer->getType() != MainLayer));
  doc->putUndoStack(CreateLayer);
  doc->refreshLayer();
  drawToolBar->setTool(lMoveTool, true);
}


void ApaintApp::slotHelpAbout()
{
 aboutBox->show();
}

void ApaintApp::zoomInBtnClicked()
{
 view->scaleUp(-1,-1, false);
}

void ApaintApp::zoomOutBtnClicked()
{
 view->scaleDown(-1,-1, false);
}

void ApaintApp::keyPressEvent(QKeyEvent *e)
{
  int x, y, ncol;
  Layer *sel = doc->getLayer();
  ApaintBrushList *bList = ApaintBrushList::getObject();
  QRect refRec = bList->getPos();
  tCtrl->getGlobalMousePos(x, y);  
  switch (e->key())
  {
    case Qt::Key_1:
    	    if (e->state() == NoButton) view->scale(1, 1, false, x, y);
            break;
    case Qt::Key_2:
    	    if (e->state() == NoButton) view->scale(2, 2, false, x, y);
            break;
    case Qt::Key_3:
            if (e->state() == NoButton) view->scale(3, 3, false, x, y);
            break;
    case Qt::Key_4:
            if (e->state() == NoButton) view->scale(4, 4, false, x, y);
            break;
    case Qt::Key_5:
            if (e->state() == NoButton) view->scale(5, 5, false, x, y);
            break;
    case Qt::Key_6:
            if (e->state() == NoButton) view->scale(6, 6, false, x, y);
            break;            
    case Qt::Key_7:
            if (e->state() == NoButton) view->scale(7, 7, false, x, y);
            break;
    case Qt::Key_8:
            if (e->state() == NoButton) view->scale(8, 8, false, x, y);
            break;
    case Qt::Key_9:
            if (e->state() == NoButton) view->scale(9, 9, false, x, y);
            break;
    case Qt::Key_0:
            if (e->state() == NoButton) view->scale(10, 10, false, x, y);
            break;
    case Qt::Key_A:
            if (e->state() == AltButton)
               cProp->enableAspectDrawProp(!cProp->doAspectDraw());
    case Qt::Key_B:
             if (e->state() == ShiftButton)
              drawToolBar->setTool(bezierTool, true);
	     else if (e->state() == NoButton)
              pbUtil->switchBrush();
             break;
    case Qt::Key_D:
            if (e->state() == ControlButton)
              ApaintDocUtil::destroyCurLayer(doc);
            else if (e->state() == ShiftButton)
              drawToolBar->setTool(bsTool, true);
            else if (e->state() == NoButton)
              drawToolBar->setTool(eArcTool, true);
            break;
    case Qt::Key_E:
             if (e->state() == ShiftButton)
              {
               explodeTool->doAction();
               if (brushProp->cbStamp->isChecked())
                 doc->refreshLayer(refRec);
	      }
	     else
    	     if (e->state() == NoButton)
	      drawToolBar->setTool(cleanTool, true);
            break;
    case Qt::Key_F:              
             if (e->state() == ShiftButton)
              {
               mirrorxTool->doAction();
               if (brushProp->cbStamp->isChecked())
                doc->refreshLayer(refRec);
              }
             else if (e->state() == NoButton)
              {
               mirroryTool->doAction();
               if (brushProp->cbStamp->isChecked())
                 doc->refreshLayer(refRec);
              }
             break;
    case Qt::Key_G:
    	     if (e->state() == NoButton) drawToolBar->setTool(insRemPixTool, true);
             break;
    case Qt::Key_H:
             if (e->state() == ShiftButton)
               drawToolBar->setTool(vshift, true);
             else if (e->state() == NoButton)
               drawToolBar->setTool(hshift, true);
             break;
    case Qt::Key_I:
             if (e->state() == ShiftButton)
              drawToolBar->setTool(measureTool, true);
	     else if (e->state() == NoButton)
              drawToolBar->setTool(straightLineTool, true);
             break;
    case Qt::Key_J:
             if (e->state() == ShiftButton)
              drawToolBar->setTool(mixTool, true);
             else if (e->state() == NoButton)
              drawToolBar->setTool(sprayTool, true);
             break;
    case Qt::Key_K:
            if (e->state() == ShiftButton)
             drawToolBar->setTool(stTool, true);
	    else if (e->state() == NoButton)
             drawToolBar->setTool(fillTool, true);
            break;
    case Qt::Key_L:
    	    if (e->state() == NoButton)
	     drawToolBar->setTool(freehand, true);
            break;
    case Qt::Key_M:
            if (e->state() == NoButton)
	     drawToolBar->setTool(selTool, true);
            break;
    case Qt::Key_N:
    	    if (e->state() == NoButton)
	     drawToolBar->setTool(pLineTool, true);
            break;
    case Qt::Key_O:
             if (e->state() == ShiftButton)
              drawToolBar->setTool(ellipseTool, true);
             else if (e->state() == NoButton)
              drawToolBar->setTool(circleTool, true);             
             break;
    case Qt::Key_P:
            if (e->state() == ShiftButton)
             drawToolBar->setTool(freeTransTool, true);
	    else if (e->state() == NoButton)
             drawToolBar->setTool(brushTool, true);
            break;
    case Qt::Key_Q:
            if (e->state() == ShiftButton)
              drawToolBar->setTool(quadTool, true);
            else if (e->state() == AltButton)
             {
              drawToolBar->setTool(rectTool, true);            
              drawToolBar->changeTool(rectTool);
             }
            else if (e->state() == NoButton)
             drawToolBar->setTool(rectTool, true);            
            break;            
    case Qt::Key_S:
    	    if (e->state() == NoButton)
    	     brushProp->cbStamp->setChecked(!brushProp->cbStamp->isChecked());
    	    break;
    case Qt::Key_T:
            if (e->state() == ControlButton)
              drawToolBar->setTool(transTool, true);
            else if (e->state() == NoButton)
              drawToolBar->setTool(writeTool, true);
            break;
    case Qt::Key_V:
    	    if (e->state() == NoButton) drawToolBar->setTool(lMoveTool, true);
            break;
    case Qt::Key_W:            
    	    if (e->state() == NoButton) drawToolBar->setTool(magicWandTool, true);
            break;
    case Qt::Key_X:
    	    if (e->state() == NoButton) pbUtil->changeFgBg();
            break;
    case Qt::Key_Y:
            if (e->state() == NoButton)
	     {
	      flipTool->doAction();
              if (brushProp->cbStamp->isChecked())
               doc->refreshLayer(refRec);
	     }
            break;

    case Qt::Key_BracketRight:
             if (e->state() == ControlButton)
              {
               layerSel->moveLayerDown();
               setFocus();
              }
             else if (e->state() == AltButton)
               doc->setActiveLayer(doc->getLayerNum()+1);
//             else if (e->state() & ~ShiftButton == NoButton)
             else
               pbUtil->incLineSize();
             break;
    case Qt::Key_BracketLeft:
             if (e->state() == ControlButton)
              {
               layerSel->moveLayerUp();
               setFocus();
              }
             else if (e->state() == AltButton)
               doc->setActiveLayer(doc->getLayerNum()-1);
//             else if (e->state() & ~ShiftButton == NoButton)
             else
              pbUtil->decLineSize();
             break;
    case Qt::Key_Backspace:
    	    if (e->state() == NoButton)
    	    {
             doc->putUndoStack(DestroyLayer);
             doc->deleteLayer();
    	    }
            break;
    case Qt::Key_Comma:
    	    if ((e->state() & ~ShiftButton) == NoButton)
    	    {
             bListWid->prevBrush();
             setFocus();
    	    }
            break;
    case Qt::Key_Period:
    	    if ((e->state() & ~ShiftButton) == NoButton)
    	    {
             bListWid->nextBrush();
             setFocus();
    	    }
            break;
    case Qt::Key_Delete:
    	    if (e->state() == NoButton)
    	    {
             doc->putUndoStack(DestroyLayer);
             doc->deleteLayer();
    	    }
            break;
    case Qt::Key_F5:
    	    if (e->state() == NoButton) toolTab->setCurrentPage(1);
            break;
    case Qt::Key_F6:
    	    if (e->state() == NoButton) toolTab->setCurrentPage(0);
            break;
    case Qt::Key_F7:
    	    if (e->state() == NoButton) toolTab->setCurrentPage(2);
            break;
    case Qt::Key_F11:
             if (sel && e->state() == NoButton)
              {
               ncol = colP->getColorPos()-1;
               if (ncol<0) ncol = sel->numColors()-1;
               colP->setColorPos(ncol);
              }
             break;
    case Qt::Key_F12:
             if (sel && e->state() == NoButton)
              {
               ncol = (colP->getColorPos()+1)%sel->numColors();
               colP->setColorPos(ncol);
              }
             break;             
  }
}

void ApaintApp::imageResize()
{
 srot->setupDlg(ScaleRotDialog::ScaleType);
 drawToolBar->setTool(transTool, true);
}

void ApaintApp::imageRotate()
{
 srot->setupDlg(ScaleRotDialog::RotateType);
 drawToolBar->setTool(transTool, true);
}

void ApaintApp::protectLayer(bool enable)
{ 
 if (!doc->getLayer()) return;
 undoRedoEnable = !enable;
 layerSel->setEnabled(!enable);
 if (enable)
  {
   saveTool = tCtrl->getActiveTool();
   tCtrl->registerTool(dummyTool);
  }
 else
  tCtrl->registerTool(saveTool); 
  
 drawToolBar->enableToolChange(!enable);
 
 if (enable)
  {
   colorNum->setEnabled(false);
   colorReduce->setEnabled(false);
   colorRemap->setEnabled(false);
   convertMenu->setEnabled(false);
   filtersMenu->setItemEnabled(CONTOUR_AVERAGE, false);
   selfAvoidingMotifsMenu->setEnabled(false);
   randomMotifsMenu->setEnabled(false);
   placeMotifsMenu->setEnabled(false);
   noiseMenu->setEnabled(false);
   filtersMenu->setItemEnabled(CREATE_NOISE_XY, false);
   filtersMenu->setItemEnabled(CREATE_NOISE_X, false);
   filtersMenu->setItemEnabled(CREATE_NOISE_Y, false);
   toolContour->setEnabled(false);
   toolsMenu->setItemEnabled(GUESS_WEAVE_DLG, false);
   editPasteMultiple->setEnabled(false);
   viewRepeat->setEnabled(false);
   editPaste->setEnabled(false);
   imgRotate->setEnabled(false);
   editScale->setEnabled(false);
   editUndo->setEnabled(false);
   editRedo->setEnabled(false);   
  }
 else
  {
   Layer *sel = doc->getLayer();

   if (!sel)
    return;
   
   if (sel->imageBuf.type() == LOAD_TRUE_COL)
    {
     colorRemap->setEnabled(false);
     toolContour->setEnabled(false);
     toolsMenu->setItemEnabled(GUESS_WEAVE_DLG, false);
    }
   else
    {
     colorRemap->setEnabled(true);
     toolContour->setEnabled(true);
     toolsMenu->setItemEnabled(GUESS_WEAVE_DLG, true);
    }

   colorNum->setEnabled(true);
   colorReduce->setEnabled(true);
   convertMenu->setEnabled(true);
   filtersContourAverage->setEnabled(true);
   selfAvoidingMotifsMenu->setEnabled(true);
   randomMotifsMenu->setEnabled(true);
   placeMotifsMenu->setEnabled(true);
   noiseMenu->setEnabled(true);
   filtersMenu->setItemEnabled(CREATE_NOISE_XY, true);
   filtersMenu->setItemEnabled(CREATE_NOISE_X, true);
   filtersMenu->setItemEnabled(CREATE_NOISE_Y, true);
   editPasteMultiple->setEnabled(multipleLayers);
   viewRepeat->setEnabled(true);
   editPaste->setEnabled(canPaste);
   imgRotate->setEnabled(true);
   editScale->setEnabled(true);
   editUndo->setEnabled(canUndo);
   editRedo->setEnabled(canRedo);
 }
}

/********************RULER SLOTS****************************/

void ApaintApp::showRuler(bool enable)
{
 if (enable)
  {
   vruler->show();
   hruler->show();
  }
 else
  {
   vruler->hide();
   hruler->hide();
  }
}

void ApaintApp::rulerStyleChanged(Ruler::MetricStyle style)
{
  if (!mainLayerLoaded)
   return;

  if (viewInteger->isOn())
   {
    vruler->setMetrics(style, (double)doc->mainLayer->imageBuf.ydpi()/1000);
    hruler->setMetrics(style, (double)doc->mainLayer->imageBuf.xdpi()/1000);
   }
  else
   {
    vruler->setMetrics(style, ApaintConfig::getScreenDpiY());
    hruler->setMetrics(style, ApaintConfig::getScreenDpiX());
   }
  vruler->setScale(view->getViewScaleX(), view->getViewScaleY());
  hruler->setScale(view->getViewScaleX(), view->getViewScaleY());
}

void ApaintApp::viewSizeChanged(int w, int h)
{
  vruler->setRange(0, h);
  hruler->setRange(0, w);
}

void ApaintApp::viewMoving(int x, int y)
{
  vruler->setOffset(y);
  hruler->setOffset(x);
}

void ApaintApp::mouseMoveSlot(QMouseEvent *e, QPainter *p)
{
  vruler->setValue(e->pos().y());
  hruler->setValue(e->pos().x());
/*
{
 static int oldX=-1, oldY=-1;

 p->setRasterOp(XorROP);
 p->setPen(QPen(QColor(128, 128, 128), 2, Qt::DotLine));
//printf("moving mouse %d %d\n",e->pos().x(), e->pos().y());
 p->drawLine(0, oldY, 2000, oldY);
 p->drawLine(oldX, 0, oldX, 2000);
 oldX = e->pos().x();
 oldY = e->pos().y();
 p->drawLine(0, oldY, 2000, oldY);
 p->drawLine(oldX, 0, oldX, 2000);
}
*/
}

void ApaintApp::slotAspectChanged(double fx, double fy)
{
  Layer *mainLayer = doc->getMainLayer();
  if (!mainLayer) return;

  if (viewInteger->isOn())
  {
    vruler->setDpi((double)mainLayer->imageBuf.ydpi()/1000);
    hruler->setDpi((double)mainLayer->imageBuf.xdpi()/1000);
  }
  else
  {
    vruler->setDpi(ApaintConfig::getScreenDpiY());
    hruler->setDpi(ApaintConfig::getScreenDpiX());
  }
  vruler->setScale(view->getViewScaleX(), view->getViewScaleY());
  hruler->setScale(view->getViewScaleX(), view->getViewScaleY());
}

void ApaintApp::browseBrushSlot()
{
  // refresh directory history
  getRecentFileNameLists();

  // set directory history for iconDlg
  iconDlg->setDirectoryHistoryList(directories);
  iconDlg->showDlg(imgPath, IconViewDlg::BrushImage); 
}

void ApaintApp::slotLayerBrowse()
{
  // refresh directory history
  getRecentFileNameLists();

  // set directory history for iconDlg
  iconDlg->setDirectoryHistoryList(directories);
  iconDlg->showDlg(imgPath, IconViewDlg::LayerImage); 
}

void ApaintApp::slotFileBrowse()
{
  // refresh directory history
  getRecentFileNameLists();

  // set directory history for iconDlg
  iconDlg->setDirectoryHistoryList(directories);
  iconDlg->showDlg(imgPath, IconViewDlg::MainImage);
}

void ApaintApp::rulerClicked(QPoint* point, Qt::Orientation guideOrient)
{
 if (doc->getMainLayer())
  {
    QRect rect = view->rect();
    
    Guide *guide = new Guide(view, doc, guideOrient);
    
    //zoom button pressed
    connect(view, SIGNAL(zoomChanged(double, double)), guide, SLOT(slotZoomChanged(double, double)));
    //view scrolling
    connect(view, SIGNAL(viewOffsetChanged(int, int)), guide, SLOT(viewMoving(int, int)));
    
    //call object destructor when new document is opened
    connect(doc, SIGNAL(newDocSig()), guide, SLOT(newDocSlot()));
    
    //this was necessary because line in other guides was not drawn correctly without explicit call of update
    //this is not the case with Qt4 lib
    connect(guide, SIGNAL(movedSig()), view, SLOT(repaintGuideSlot()));
    connect(view, SIGNAL(repaintGuideSig()), guide, SLOT(repaintSlot()));
    
    if (guideOrient == Vertical)
     guide->move(point->x(), rect.rTop());
    else 
     guide->move(rect.rLeft(), point->y());
    
    guide->show();
    guide->setBasePos();
  }
}
