/***************************************************************************
                          apaintUtil.cpp  -  description
                             -------------------
    begin                : Thu Apr 15 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaintconfig.h"
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <stdlib.h>
#include <math.h>
#include <qwmatrix.h>
#include "apaintUtil.h"
#include "apaintdoc.h"
#include "apaint.h"


/***************SHOULD NOT INCLUDE ANY CLASS FROM APAINT!!!!*********************/

/*********CommonProp*************************/

static CommonProp *commonPropObject = 0;

CommonProp::CommonProp(QWidget *parent, const char *name):QWidget(parent, name)
{
  vbox = new QVBoxLayout(this);
  cbThickLines = new QCheckBox(this, "cbThickLines");
  cbThickLines->setText(ApaintConfig::getText(640));
  cbThickLines->setFocusPolicy(NoFocus);
  cbDrawInAspect = new QCheckBox(this, "cbDrawAspect");
  cbDrawInAspect->setText(ApaintConfig::getText(734));
  vbox->addWidget(cbThickLines);
  vbox->addWidget(cbDrawInAspect);  
  connect(cbThickLines, SIGNAL(toggled(bool)), this, SIGNAL(setThickLinesSig(bool)));
  connect(cbDrawInAspect, SIGNAL(toggled(bool)), this, SIGNAL(setAspectDrawSig(bool)));
  connect(cbDrawInAspect, SIGNAL(clicked()), this, SIGNAL(aspectDrawChangedSig()));
  setAspectDrawProp(false);
  setThickLinesProp(false);
}

CommonProp::~CommonProp()
{
}

void CommonProp::setThickLinesProp(bool show)
{
 if (show)
  cbThickLines->show();
 else
 {
  cbThickLines->setChecked(false);
  cbThickLines->hide();  
 }
}

void CommonProp::enableAspectDrawProp(bool enable)
{
  cbDrawInAspect->setChecked(enable);
  emit setAspectDrawSig(enable);
  emit aspectDrawChangedSig();
}

void CommonProp::setAspectDrawProp(bool show)
{
  if (show)
  cbDrawInAspect->show();
 else
  cbDrawInAspect->hide();
}

void CommonProp::createObject(QWidget *parent, const char *name)
{
  if (commonPropObject)
   delete commonPropObject;

  commonPropObject = new CommonProp(parent, name);
}

CommonProp *CommonProp::getObject()
{
  return commonPropObject;
}



/*
bool ApaintUtil::executeCommand(char *command)
{
 int status;

 status = system(command);
 if(status<0)
  return(false);
 if (WIFEXITED(status))
  if (WEXITSTATUS(status)==0)
   return(true);
  else
   return(false);
 else
  return(false);
}
*/

QString ApaintUtil::getBaseName(QString &allName)
{
  QStringList bl = QStringList::split(DIR_SEPARATOR, allName);
  if (!bl.count())
   return "";
  return (bl[bl.count()-1]);
}

QString ApaintUtil::getDirName(QString &allName)
{
  QStringList bl = QStringList::split(DIR_SEPARATOR, allName);
  if (!bl.count())
   return "";
  bl.remove(bl.last());
  return (bl.join(DIR_SEPARATOR)); 
}

/***********************************************************************************/

ApaintIntValidator::ApaintIntValidator( int bottom, int top, QObject * parent, const char *name = 0 ):QValidator(parent, name)
{
    b = bottom;
    t = top;
}

ApaintIntValidator::~ApaintIntValidator()
{
}

QValidator::State ApaintIntValidator::validate( QString &input, int & ) const
{
  QString stripped = input.stripWhiteSpace();
  if ( stripped.isEmpty() || (b < 0 && stripped == "-") )
    return Intermediate;
  bool ok;
  long entered = input.toLong( &ok );
  if ( !ok || (entered < 0 && b >= 0) )
    return Invalid;
  else if ( entered < b || entered > t )
    return Invalid;
  else
    return Acceptable;
}

/***********************************************************************************/

ApaintDoubleValidator::ApaintDoubleValidator( double bottom, double top, int decimals, QObject * parent, const char *name = 0 ):
QValidator(parent, name)
{
   b = bottom;
   t = top;
   d = decimals;
}

ApaintDoubleValidator::~ApaintDoubleValidator()
{
  
}

QValidator::State ApaintDoubleValidator::validate( QString &input, int & ) const
{
  QRegExp empty( QString::fromLatin1(" *-?\\.? *") );
  if ( b >= 0 && input.stripWhiteSpace().startsWith(QString::fromLatin1("-")) )
     return Invalid;
  if ( empty.exactMatch(input) )
     return Intermediate;
   bool ok = TRUE;
   double entered = input.toDouble( &ok );
   int nume = input.contains( 'e', FALSE );
   if ( !ok )
   {
     // explicit exponent regexp
     QRegExp expexpexp( QString::fromLatin1("[Ee][+-]?\\d*$") );
     int eeePos = expexpexp.search( input );
     if ( eeePos > 0 && nume == 1 )
     {
        QString mantissa = input.left( eeePos );
        entered = mantissa.toDouble( &ok );
        if ( !ok )
          return Invalid;
     }
     else if ( eeePos == 0 )
     {
        return Intermediate;
     }
     else
     {
        return Invalid;
     }
   }

    int i = input.find( '.' );
    if ( i >= 0 && nume == 0 )
    {
     // has decimal point (but no E), now count digits after that
     i++;
     int j = i;
     while( input[j].isDigit() )
        j++;
     if ( j - i > d )
        return Invalid;
    }

    if ( entered < b || entered > t )
      return Invalid;
    else
      return Acceptable;
}

/***********************************************************************************/

EnterTextDialog::EnterTextDialog(QWidget *parent, const char* name, bool modal):EnterTextDialogUI(parent, name, modal)
{
  btnOK->setText(ApaintConfig::getText(3));
  btnCancel->setText(ApaintConfig::getText(5));
  connect(btnOK, SIGNAL(clicked()), this, SLOT(accept()));
  connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
}


bool  EnterTextDialog::getText(QWidget *parent, QString title, QString label, QString descr, QString &out)
{
  EnterTextDialog *et = new EnterTextDialog(parent, "enterTextDlg", TRUE);
  et->setCaption(title);
  if (descr == "")
    et->lbInfo->hide();
  else
    et->lbInfo->setText(descr);

  if (label == "")
    et->lbText->hide();
  else
    et->lbText->setText(label);

  et->leText->setText(out);
  out = "";
  if (et->exec() == QDialog::Accepted)
  {
   out = et->leText->text();
   delete et;
   return true;
  }
  delete et;
  return false;
}

/************************MatrixUtil Class******************/

/*matrix manipulation functions*/

void MatrixUtil::translatemx(QWMatrix *m, double x, double y)
{
  m->setMatrix(m->m11(), m->m12(), m->m21(), m->m22(), m->dx()+x, m->dy()+y);
}

void MatrixUtil::scalemx(QWMatrix *m, double x, double y)
{
  m->setMatrix(m->m11()*x, m->m12()*y, m->m21()*x, m->m22()*y, m->dx()*x, m->dy()*y);
}

void MatrixUtil::rotatemx(QWMatrix *m, double a, double xyRatio)
{
  double t1, t2;
  double cost, sint;
  double m11, m12, m21, m22, dx, dy;

  cost = cos (a);
  sint = sin (a);

  t1 = m->m11();
  t2 = m->m12();
  m11 = (cost * t1 - sint * t2 * xyRatio);
  m12 = (sint * t1/xyRatio + cost * t2);

  t1 = m->m21();
  t2 = m->m22();
  m21 = (cost * t1 - sint * t2 * xyRatio);
  m22 = (sint*t1/xyRatio + cost*t2);

  t1 = m->dx();
  t2 = m->dy();
  dx= (cost*t1 - sint*t2 * xyRatio);
  dy = (sint*t1/xyRatio + cost*t2);
  m->setMatrix(m11, m12, m21, m22, dx, dy);
}

/* original without density ratio
void MatrixUtil::rotatemx(QWMatrix *m, double a)
{
  double t1, t2;
  double cost, sint;
  double m11, m12, m21, m22, dx, dy;

  cost = cos (a);
  sint = sin (a);

  t1 = m->m11();
  t2 = m->m12();
  m11 = (cost * t1 - sint * t2);
  m12 = (sint * t1 + cost * t2);

  t1 = m->m21();
  t2 = m->m22();
  m21 = (cost * t1 - sint * t2);
  m22 = (sint*t1 + cost*t2);

  t1 = m->dx();
  t2 = m->dy();
  dx= (cost*t1 - sint*t2);
  dy = (sint*t1 + cost*t2);
  m->setMatrix(m11, m12, m21, m22, dx, dy);
}
*/
void MatrixUtil::shearmx(QWMatrix *m, double sx, double sy)
{
  m->setMatrix(m->m11(), sy, sx, m->m22(), m->dx(), m->dy());
}

/**************************VerticalyAlignedWidget**************************/

VerticalyAlignedWidget::VerticalyAlignedWidget(QWidget *parent, const char *name):QWidget(parent, name)
{
 vbox = new QVBoxLayout(this);  
}

VerticalyAlignedWidget::~VerticalyAlignedWidget()
{
  
}

void VerticalyAlignedWidget::addWidget(QWidget *w)
{
  vbox->addWidget(w);
}

/***********************************************************************************/

void ApaintDocUtil::destroyCurLayer(ApaintDoc *doc)
{
  if (!doc) return;
  if (!doc->getLayer()) return;
  if (doc->inTransformMode()) return;
  if (doc->getLayerType() == MainLayer) return;
  if (doc->isProtectMode()) return;

  QRect r=doc->getLayerPos();
  if (doc->isLayerDataLoaded())
  {
     doc->putUndoStack(Unselect);
     doc->collapsSelectedLayer(apaint->transparentPaste());
  }
  doc->refreshLayer(r);
}

void ApaintDocUtil::moveCurLayer(ApaintDoc *doc, int dx, int dy, int border)
{
  QRect prev=doc->getLayerPos();
  doc->moveLayer(dx, dy);
  QRect after=doc->getLayerPos();
  doc->refreshLayer(prev, after, border);
}

/***********************************************************************************/

#define DET2(a,b, c,d) ((a)*(d) - (b)*(c))
#define TOLERANCE 1e-13
#define ZERO(x) ((x)<TOLERANCE && (x)>-TOLERANCE)
#define X(i) quad[i].x()         /* quadrilateral x and y */
#define Y(i) quad[i].y()

double PerspectiveTransform::adjoint(double a[3][3], double b[3][3])
{
  b[0][0] = DET2(a[1][1], a[1][2], a[2][1], a[2][2]);
  b[1][0] = DET2(a[1][2], a[1][0], a[2][2], a[2][0]);
  b[2][0] = DET2(a[1][0], a[1][1], a[2][0], a[2][1]);
  b[0][1] = DET2(a[2][1], a[2][2], a[0][1], a[0][2]);
  b[1][1] = DET2(a[2][2], a[2][0], a[0][2], a[0][0]);
  b[2][1] = DET2(a[2][0], a[2][1], a[0][0], a[0][1]);
  b[0][2] = DET2(a[0][1], a[0][2], a[1][1], a[1][2]);
  b[1][2] = DET2(a[0][2], a[0][0], a[1][2], a[1][0]);
  b[2][2] = DET2(a[0][0], a[0][1], a[1][0], a[1][1]);

  return a[0][0]*b[0][0] + a[0][1]*b[0][1] + a[0][2]*b[0][2];
}

int PerspectiveTransform::quad2rect(double u0,double v0,double u1,double v1, QPoint quad[4], double RQ[3][3] ,double QR[3][3])
{
  int ret;
  double du, dv;

  du = u1-u0;
  dv = v1-v0;
  if (du==0. || dv==0.) {
      return PMAP_BAD;
  }

  /* first find mapping from unit uv square to xy quadrilateral */
  ret = square2quad(quad, RQ);
  if (ret==PMAP_BAD) return PMAP_BAD;

  /* concatenate transform from uv rectangle (u0,v0,u1,v1) to unit square */
  RQ[0][0] /= du;
  RQ[1][0] /= dv;
  RQ[2][0] -= RQ[0][0]*u0 + RQ[1][0]*v0;
  RQ[0][1] /= du;
  RQ[1][1] /= dv;
  RQ[2][1] -= RQ[0][1]*u0 + RQ[1][1]*v0;
  RQ[0][2] /= du;
  RQ[1][2] /= dv;
  RQ[2][2] -= RQ[0][2]*u0 + RQ[1][2]*v0;

  /* now RQ is transform from uv rectangle to xy quadrilateral */
  /* QR = inverse transform, which maps xy to uv */
  if (adjoint(RQ, QR)==0.){}
  return ret;   
}

/*double quad[4][2];       vertices of quadrilateral */
/*double SQ[3][3];         square->quad transform */
int PerspectiveTransform::square2quad(QPoint quad[4], double SQ[3][3])
{
  double px, py;

  px = X(0)-X(1)+X(2)-X(3);
  py = Y(0)-Y(1)+Y(2)-Y(3);

  if (ZERO(px) && ZERO(py)) {         /* affine */
      SQ[0][0] = X(1)-X(0);
      SQ[1][0] = X(2)-X(1);
      SQ[2][0] = X(0);
      SQ[0][1] = Y(1)-Y(0);
      SQ[1][1] = Y(2)-Y(1);
      SQ[2][1] = Y(0);
      SQ[0][2] = 0.;
      SQ[1][2] = 0.;
      SQ[2][2] = 1.;
      return PMAP_AFFINE;
  }
  else {                              /* projective */
      double dx1, dx2, dy1, dy2, del;

      dx1 = X(1)-X(2);
      dx2 = X(3)-X(2);
      dy1 = Y(1)-Y(2);
      dy2 = Y(3)-Y(2);
      del = DET2(dx1,dx2, dy1,dy2);
      if (del==0.)
      {
          return PMAP_BAD;
      }
      SQ[0][2] = DET2(px,dx2, py,dy2)/del;
      SQ[1][2] = DET2(dx1,px, dy1,py)/del;
      SQ[2][2] = 1.;
      SQ[0][0] = X(1)-X(0)+SQ[0][2]*X(1);
      SQ[1][0] = X(3)-X(0)+SQ[1][2]*X(3);
      SQ[2][0] = X(0);
      SQ[0][1] = Y(1)-Y(0)+SQ[0][2]*Y(1);
      SQ[1][1] = Y(3)-Y(0)+SQ[1][2]*Y(3);
      SQ[2][1] = Y(0);
      return PMAP_PROJECTIVE;
  }
}

void PerspectiveTransform::transformPoint(double p[3], double a[3][3], double q[3])

{
    q[0] = p[0]*a[0][0] + p[1]*a[1][0] + p[2]*a[2][0];
    q[1] = p[0]*a[0][1] + p[1]*a[1][1] + p[2]*a[2][1];
    q[2] = p[0]*a[0][2] + p[1]*a[1][2] + p[2]*a[2][2];
}

void PerspectiveTransform::transformPoint(int x, int y, int &tx, int &ty, double mx[3][3])
{
 double p[3], q[3];

 p[0]=x;
 p[1]=y;
 p[2]=1;
 transformPoint(p,mx,q);
 tx=(int)(q[0]/q[2]);
 ty=(int)(q[1]/q[2]);
}
