/***************************************************************************
                          apaintclipboard.cpp  -  description
                             -------------------
    begin                : Wed Jan 28 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaintclipboard.h"
#include "apaintdoc.h"
#include <unistd.h>
#include <stdlib.h>

#define CLIPBOARD_DIR "/tmp/apaintClipboard_"
#define IMAGE_FILE    "image"
#define MASK_FILE     "mask"

static bool dirOk = false;
static QString dirName;

void ApaintClipboard::initClipboard()
{
 QDir dir;
 char *userName = getenv("USER");

 if (!userName)
  userName = strdup("");
 dirName = QString("%1%2/").arg(CLIPBOARD_DIR).arg(userName);
 dir.setPath(dirName);
 if (!dir.exists()) 
   dirOk = dir.mkdir(dirName);
 else
  dirOk = true;
}

void ApaintClipboard::putOnClipboard(Layer *layer)
{
  if (!dirOk)  return;
  if (!layer) return;
  QString path;
  
  path = QString("%1%2").arg(dirName).arg(IMAGE_FILE);
  layer->imageBuf.save(path);
  if (layer->usingMask())
  {
    path = QString("%1%2").arg(dirName).arg(MASK_FILE);
    layer->getMask().save(path, "PNG");
  }
}

void ApaintClipboard::getFromClipboard(ApaintDoc *docp, int posx, int posy)
{
  if (!dirOk)  return;
  if (!docp) return;
  ApaintImage img;  
  QImage mask;
  QString path;
  int px, py;
  QRect ref;
   
  path = QString("%1%2").arg(dirName).arg(IMAGE_FILE);
  if (!img.load(path)) return;
  path = QString("%1%2").arg(dirName).arg(MASK_FILE);
  bool useMask = mask.load(path, "PNG");
  if (useMask)
    mask = mask.convertBitOrder(QImage::LittleEndian);

  px = posx;
  py = posy;
  if (docp->numLayers()>1)
  {
    px = docp->getLayer()->layerPos().x();
    py = docp->getLayer()->layerPos().y();
    docp->putUndoStack(DestroyLayer);
    docp->collapsSelectedLayer(false);
  }
  docp->putUndoStack(CreateLayer);
  if (useMask)
    docp->createNewLayer(img, &mask, px, py);
  else
    docp->createNewLayer(img, 0, px, py);
  docp->refreshLayer();
}

void ApaintClipboard::removeClipboard()
{
  if (!dirOk)  return;
  QDir dir;
  QString path;
  
  path = QString("%1%2").arg(dirName).arg(IMAGE_FILE);
  dir.remove(path);
  path = QString("%1%2").arg(dirName).arg(MASK_FILE);
  dir.remove(path);
  dir.rmdir(dirName);
  
}


