/***************************************************************************
                          apaintview.cpp  -  description
                             -------------------
    begin                : Mon Jan 27 11:15:43 CET 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#include "apaintview.h"
#include <apaintio.h>
#include <qbitmap.h>
#include "apaintconfig.h"
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qtimer.h>
#include "apaintBrushList.h"

#define RECT_SIZE   4

#define min(a,b)   ((a)>(b) ? (b) : (a))
#define max(a,b)   ((a)<(b) ? (b) : (a))

ApaintView::ApaintView(QWidget *parent, ApaintDoc *docp) : QScrollView(parent,"ApaintView", WRepaintNoErase|WResizeNoErase|WStaticContents)
{
  init(DocumentView);
  Q_CHECK_PTR(docp);
  doc=docp; 
  connect(doc, SIGNAL(newDocSig()), this, SLOT(documentChangedSlot()));
  connect(doc, SIGNAL(mainLayerChanged()), this, SLOT(documentChangedSlot()));
}  

ApaintView::ApaintView(QWidget *parent, const char *name): QScrollView(parent,name, WRepaintNoErase|WResizeNoErase|WStaticContents)
{
 init(ImageView);
}

ApaintView::~ApaintView()
{

}

void ApaintView::init(ViewType type)
{
  vType = type;
  io = ApaintIO::getObject();
  brushList = ApaintBrushList::getObject();
  documentLoaded=false;
  repaintFromMoving=true;
  integerView = true;
  antialiasView = false;
  autoScroll = lbtnPressed = rbtnPressed = false;
  mouseX=mouseY=0;
  zoomX= zoomY= mzoomX= mzoomY=1;
  gridOn = false;
  facX = facY = oldFacX = oldFacY = 1;
  resizeContents(1200,800);
  viewport()->setBackgroundMode(NoBackground);
  viewport()->setMouseTracking(true);
  viewport()->setFocusPolicy(QWidget::StrongFocus);
  connect(this, SIGNAL(contentsMoving(int, int)), this, SLOT(contentsMovingSlot(int, int)));

  offsetX = offsetY = 0;
}

void ApaintView::setCanvasPainter(QPainter *p)
{
  p->translate(-contentsX(),-contentsY());
}

void ApaintView::setGridColor(uchar col)
{
 io->setGridColor(col);
}

void ApaintView::setGridSize(int w, int h)
{
 io->setGridSize(w, h);
}

void ApaintView::showGrid(bool show)
{
 gridOn = show;
 if (zoomX>=3 && zoomY>=3)
  repaintContents(0);
}

void ApaintView::setIntegerView(bool enable)
{
  if (integerView == enable) return;
  
  integerView = enable;
  setZoom(mzoomX, mzoomY, true, 0, 0);
}

void ApaintView::setAntialiasView(bool enable)
{
  if (antialiasView == enable) return;
  
  antialiasView = enable;
  setZoom(mzoomX, mzoomY, true, 0, 0);
}

bool ApaintView::antialias() const
{
 return antialiasView;
}

void ApaintView::scaleFactorSlot(double fx, double fy)
{
  oldFacX = facX;
  facX = fx;
  if (facX == 0) facX = 1;
  oldFacY = facY;
  facY = fy;
  if (facY == 0) facY = 1;
  
  setZoom(mzoomX, mzoomY, true, 0, 0);;
}

void ApaintView::leaveEvent(QEvent *e)
{
 if (lbtnPressed || rbtnPressed) return;
 QPainter pc(viewport());
 setCanvasPainter(&pc); 
 emit leaveEventSig(e, &pc);
}

void ApaintView::enterEvent(QEvent *e)
{
 if (lbtnPressed || rbtnPressed) return;
 QPainter pc(viewport());
 setCanvasPainter(&pc);
 emit enterEventSig(e, &pc); 
}

void ApaintView::contentsWheelEvent(QWheelEvent *e)
{

 if (!documentLoaded) return;
 int dir;
 int direction;

 if (e->state()==ControlButton)
 {
    dir = e->delta();
    if (dir>0)
     scaleUp(-1, -1, false);
    else
     scaleDown(-1, -1, false);
 }
 else if (e->state()==ShiftButton)
 {  
   direction=e->delta();
   scrollBy(-direction, 0);
 } 
 else
  QScrollView::contentsWheelEvent(e);
}

void ApaintView::contentsMousePressEvent (QMouseEvent *e)
{
  if (!documentLoaded) return;

  if (e->button()==LeftButton)
   lbtnPressed=true;
  if (e->button() == RightButton)
   rbtnPressed = true;
  QPainter pc(viewport());
  setCanvasPainter(&pc);  
  emit mousePressSig(e, &pc);

}

void ApaintView::contentsMouseReleaseEvent (QMouseEvent *e )
{
  if (!documentLoaded) return;

  if (e->button()==LeftButton)
   lbtnPressed=false;
  if (e->button() == RightButton)
   rbtnPressed = false;
  QPainter pc(viewport());
  setCanvasPainter(&pc);
  emit mouseReleaseSig(e, &pc);
  
}

void ApaintView::contentsMouseDoubleClickEvent (QMouseEvent *e )
{
  if (!documentLoaded) return;
  
  QPainter pc(viewport());
  setCanvasPainter(&pc);  
  emit mouseDoubleClickSig(e, &pc);
}

void ApaintView::contentsMouseMoveEvent (QMouseEvent *e )
{
  if (!documentLoaded) return;

  /*get cur mouse pos*/
  mouseX = e->pos().x();
  mouseY = e->pos().y();
  
  if (autoScroll && ((autoScrollNoButton && !lbtnPressed && !rbtnPressed) || (!autoScrollNoButton && (lbtnPressed || rbtnPressed))) )
   scrollCorner(e->pos().x(), e->pos().y());
  QPainter pc(viewport());
  setCanvasPainter(&pc);    
  emit mouseMoveSig(e, &pc);
}

static uchar blinkColor;
static int numBlinks = 0;
static int numHighlights = 0;

void ApaintView::keyPressEvent( QKeyEvent *e )
{
  if (!documentLoaded) return;
  QPainter pc(viewport());
  setCanvasPainter(&pc);
  emit keyPressSig(e, &pc);
  if (e->key() == Key_PageUp)
   {
    if (e->state() == ControlButton)
      scrollBy(-visibleWidth(), 0);
    else
      scrollBy(0, -visibleWidth());
   }
  else if (e->key() == Key_PageDown)
   {
    if (e->state() == ControlButton)
      scrollBy(visibleWidth(), 0);
    else
      scrollBy(0, visibleWidth());
   }
  else if (e->key() == Key_Home)
    scrollBy(-contentsX(), -contentsY());
  else if (e->key() == Key_End)
    scrollBy(contentsWidth(), contentsHeight());
  else if (e->key() == Key_Asterisk)
   {
    int foundx=-1, foundy=-1;
    int x,y;
    int totalX, totalY;
    uchar *selPtr, selCol;

    if (doc->getLayer()->getImageType() != LOAD_CMAP)
     return;

    if (numHighlights>0)
     {
      numHighlights++;
      return;
     }

    totalX = doc->getLayer()->layerPos().width();
    totalY = doc->getLayer()->layerPos().height();
    selCol = doc->getLayer()->getFgIndx();
    for (y=0; y<totalY; y++)
     {
      selPtr = doc->getLayer()->imageBuf.scanLine(y);
      for (x=0; x<totalX; x++)
       if (*selPtr++ == selCol)
	{ foundx = x; foundy = y; x=totalX; y=totalY; }
     }
    if ((foundx>=0) && (foundy>=0))
     {
      blinkColor = selCol;
      highlightSelColor();
     }
    else
     QApplication::beep();
   }
  else if ((e->key() == Key_Return) || (e->key() == Key_Enter))
   if (doc->getLayer())
    if (doc->getLayer()->getImageType() == LOAD_CMAP)
     {
      int foundx=-1, foundy=-1;
      int x,y;
      int totalX, totalY;
      uchar *selPtr, selCol;
      uchar blinkList[256];

      if (numBlinks) /* don't start new blink until the previous one is over */
       { QApplication::beep(); return; }
      totalX = doc->getLayer()->layerPos().width();
      totalY = doc->getLayer()->layerPos().height();
      selCol = doc->getLayer()->getFgIndx();

      for (x=0; x<256; x++)
       blinkList[x] = doc->getLayer()->isProtectedColor(x);
      blinkList[selCol]=TRUE;

      for (y=0; y<totalY; y++)
       {
        selPtr = doc->getLayer()->imageBuf.scanLine(y);
        for (x=0; x<totalX; x++)
	 if (blinkList[*selPtr++])
	  { foundx = x; foundy = y; x=totalX; y=totalY; }
       }
      if ((foundx>=0) && (foundy>=0))
       {
        blinkColor = selCol;
        if (e->state()==ShiftButton)
	 {
          QRect sel = QRect(foundx, foundy, 10, 10);
          setVisible(sel);
	 }
        refreshSelColor();
        QTimer::singleShot(250, this, SLOT(refreshSelColor()));
        QTimer::singleShot(500, this, SLOT(refreshSelColor()));
        QTimer::singleShot(750, this, SLOT(refreshSelColor()));
       }
      else
       QApplication::beep();
     }
 /*do other*/
  QScrollView::keyPressEvent(e);
} 

/* blink the selected or protected color */
void ApaintView::refreshSelColor()
{
 QRgb* ctab = doc->getLayer()->getColorTable();
 int i;

 for (i=0; i<256; i++)
  if ((i==blinkColor) || doc->getLayer()->isProtectedColor(i))
   ctab[i] = ctab[i] xor ((128 << 16) | (128 << 8) | 128);
 doc->refreshLayer();
 numBlinks++;
 if (numBlinks>3)
  numBlinks=0;
}

/* highlight the selected color */
void ApaintView::highlightSelColor()
{
 QRgb* ctab = doc->getLayer()->getColorTable();
 static QRgb originalCtab[256];
 static int havePalette;
 int i;

 if (numHighlights<=0)
  {
   if (havePalette) 
    {
     for (i=0; i<256; i++)
      ctab[i] = originalCtab[i];
     havePalette = FALSE;
     doc->refreshLayer();
     numHighlights=0;
     return;
    }
   else
    {
     int val, darkHighlight;

     val = (305*qRed(ctab[blinkColor]) + 510*qGreen(ctab[blinkColor]) + 205*qBlue(ctab[blinkColor]) + 512)>>10;
     darkHighlight = (val>64); /* we prefer dark background and light highlight */
     for (i=0; i<256; i++)
      originalCtab[i] = ctab[i];
     for (i=0; i<256; i++)
      if ((i==blinkColor)||(doc->getLayer()->isProtectedColor(i)))
       { /* make the highlight color slightly lighter or darker, but keep the same overall color */
	if (darkHighlight)
	 ctab[i] = ((255-(int)((255-qRed(ctab[i]))*0.75))<<16) + ((255-(int)((255-qGreen(ctab[i]))*0.75))<<8) + (255-(int)((255-qBlue(ctab[i]))*0.75));
	else
	 ctab[i] = ((int)(qRed(ctab[i])*0.75)<<16) + ((int)(qGreen(ctab[i])*0.75)<<8) + (int)(qBlue(ctab[i])*0.75);
       }
      else
       { /* make the other colors significantly darker or lighter */
	if (darkHighlight)
	 ctab[i] = ((qRed(ctab[i])/5)<<16) + ((qGreen(ctab[i])/5)<<8) + (qBlue(ctab[i])/5);
	else
	 ctab[i] = ((255-(255-qRed(ctab[i]))/5)<<16) + ((255-(255-qGreen(ctab[i]))/5)<<8) + (255-(255-qBlue(ctab[i]))/5);
       }
     havePalette = TRUE;
     doc->refreshLayer();
     numHighlights++;
    }
  }
 else
  numHighlights--;
 QTimer::singleShot(1000, this, SLOT(highlightSelColor()));
}

void ApaintView::repaint()
{
 if (!documentLoaded) return;

 QPainter pc(viewport());
 setCanvasPainter(&pc);
 emit repaintSig(&pc);
}

void ApaintView::keyReleaseEvent( QKeyEvent *e)
{
  if (!documentLoaded) return;
  QPainter pc(viewport());
  setCanvasPainter(&pc);
  emit keyReleaseSig(e, &pc);
  QScrollView::keyReleaseEvent(e);

}

void ApaintView::setImage(ApaintImage *imgp)
{
  QRect size;
  if (vType != ImageView) return;
  if (!imgp)
  {
   documentLoaded=false;
   repaintContents();
   return; 
  }
  img = imgp;
  documentLoaded=true;
  size = QRect(0, 0, img->width(), img->height());
  sizeX=size.width();
  sizeY=size.height();
  setZoom(mzoomX, mzoomY, true, 0, 0);
  emit initView();
}
                        
void ApaintView::documentChangedSlot()
{
  if (vType != DocumentView) return;
  QRect size;
  Layer *mainLayer = doc->getMainLayer();
  if (!mainLayer) return;
  if (mainLayer->imageBuf.isNull())
  {
    documentLoaded=false;
    return;
  }
  documentLoaded=true;
  if (mainLayer->isUsingTiles())
    size = mainLayer->layerPos();
  else
    size = mainLayer->realLayerPos();
  sizeX=size.width();
  sizeY=size.height();
  setZoom(mzoomX, mzoomY, true, 0, 0);
  emit initView();
}

void ApaintView::drawContentsDoc(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
  int posx, posy;
  bool resetPen=false;


  if (!clipw || !cliph) return;
  if (!documentLoaded || clipx>zoomSizeX || clipy>zoomSizeY)
  {
    p->fillRect(clipx,clipy,clipw,cliph,gray);
    return;
  }
  emit beforeRefresh(p);
  /*reset it*/
  p->setRasterOp(Qt::CopyROP);
  clipWidth = max(0, min(zoomSizeX-clipx, clipw));
  clipHeight = max(0, min(zoomSizeY-clipy, cliph));
  dstx=(int)(clipx/zoomX);
  dsty=(int)(clipy/zoomY);
  posx=(int)(dstx*zoomX);
  posy=(int)(dsty*zoomY);
  dstx2 = (int)(((int)((clipx+clipWidth)/zoomX)+1)*zoomX);
  if (dstx2 > zoomSizeX) dstx2 = zoomSizeX;
  dsty2 = (int)(((int)((clipy+clipHeight)/zoomY)+1)*zoomY);
  if (dsty2 > zoomSizeY) dsty2 = zoomSizeY;
  dstw = dstx2-posx;
  dsth = dsty2-posy;
  dstx2=min((int)(dstx2/zoomX),sizeX);
  dsty2=min((int)(dsty2/zoomY),sizeY);

  io->fastDocDraw(doc, p, posx, posy, dstw, dsth, zoomX, zoomY, gridOn);
  
  if (doc->mainLayer->isUsingTiles() && doc->mainLayer->doDrawDelimiterLines())
  {
    doc->mainLayer->drawTileLines(p, QRect(contentsX(), contentsY(), contentsWidth(), contentsHeight()), zoomX, zoomY);
    resetPen=true;
  }
  
  for (int i=1; i<doc->numLayers(); i++)
  {
    doc->layerList[i]->drawBorderMain(p, zoomX, zoomY, i == doc->getLayerNum());
    resetPen = true;
    p->setPen(QPen());//becouse of drawing multiple borders pen should be reset!
  }  
  
  /*reset qpainter becouse layer border draw*/
  if (resetPen)
  {
   p->setRasterOp(Qt::CopyROP);
   p->setPen(QPen());
  }

  if (clipWidth!=clipw)
  {
    p->fillRect(clipx+clipWidth,clipy,clipw-clipWidth,cliph,gray);
  }
  if (clipHeight!=cliph)
  {
    p->fillRect(clipx,clipy+clipHeight,clipw,cliph-clipHeight,gray);
  }
  emit drawContentsSig(p, clipx, clipy, clipw, cliph);
  emit afterRefresh(p);
}

void ApaintView::drawContentsImg(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
  int posx, posy;
  int dstx,dsty,dstx2,dsty2, dstw, dsth;
  int clipWidth,clipHeight;

  if (!clipw || !cliph) return;
  if (!documentLoaded || clipx>zoomSizeX || clipy>zoomSizeY)
  {
    p->fillRect(clipx,clipy,clipw,cliph,gray);
    return;
  }
  p->setRasterOp(Qt::CopyROP);
  clipWidth = max(0, min(zoomSizeX-clipx, clipw));
  clipHeight = max(0, min(zoomSizeY-clipy, cliph));
  dstx=(int)(clipx/zoomX);
  dsty=(int)(clipy/zoomY);
  posx=(int)(dstx*zoomX);
  posy=(int)(dsty*zoomY);
  dstx2 = (int)(((int)((clipx+clipWidth)/zoomX)+1)*zoomX);
  if (dstx2 > zoomSizeX) dstx2 = zoomSizeX;
  dsty2 = (int)(((int)((clipy+clipHeight)/zoomY)+1)*zoomY);
  if (dsty2 > zoomSizeY) dsty2 = zoomSizeY;
  dstw = dstx2-posx;
  dsth = dsty2-posy;
  dstx2=min((int)(dstx2/zoomX),sizeX);
  dsty2=min((int)(dsty2/zoomY),sizeY);

  io->fastImageDraw(*img, 0, 0, p, posx, posy, dstw, dsth, zoomX, zoomY);

  if (clipWidth!=clipw)
  {
    p->fillRect(clipx+clipWidth,clipy,clipw-clipWidth,cliph,gray);
  }
  if (clipHeight!=cliph)
  {
    p->fillRect(clipx,clipy+clipHeight,clipw,cliph-clipHeight,gray);
  }  
}
                
void ApaintView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
  if (vType == DocumentView)
    drawContentsDoc(p, clipx, clipy, clipw, cliph);    
  else if (vType == ImageView)
    drawContentsImg(p, clipx, clipy, clipw, cliph);    
}

void ApaintView::setImagePos(int x, int y, double oldZoomX, double oldZoomY)
{
  int newx, newy;
  int cx, cy, dx, dy;

  cx = contentsX();
  cy = contentsY();
  newx=(int)((x/oldZoomX)*zoomX);
  newy=(int)((y/oldZoomY)*zoomY);
  dx = newx -x;
  dy = newy -y; 
  setContentsPos(cx+dx, cy+dy);
}

void ApaintView::setCenterImagePos(double oldZoomX, double oldZoomY)
{
  setImagePos(visibleWidth()/2+contentsX(), visibleHeight()/2+contentsY(), oldZoomX, oldZoomY);
}

void ApaintView::setZoom(double zoomValX, double zoomValY, bool center, int posx, int posy)
{
 double oldZoomX, oldZoomY;
 
  if (!documentLoaded) return;
  viewport()->setUpdatesEnabled(FALSE);
  dirty=true;
  oldZoomX=zoomX;
  oldZoomY=zoomY;
  // THIS IS INTEGER VIEW
  if (integerView)
  {
    if ((zoomValX*facX) >= 1)
      zoomX=(int)(zoomValX*facX+0.5);
    else
      zoomX=zoomValX*facX;
      
    if ((zoomValY*facY) >= 1)
      zoomY=(int)(zoomValY*facY+0.5);
    else
      zoomY=zoomValY*facY;
  }
  else
  {
    zoomX = zoomValX*facX;
    zoomY = zoomValY*facY;
  }

  zoomSizeX=(int)((double)sizeX*zoomX);
  zoomSizeY=(int)((double)sizeY*zoomY);
  mouseX= (int)((mouseX/oldZoomX)*zoomX);
  mouseY= (int)((mouseY/oldZoomY)*zoomY);
  resizeContents(zoomSizeX,zoomSizeY);
  if (center)
    setCenterImagePos(oldZoomX, oldZoomY);
  else
    setImagePos(posx, posy, oldZoomX, oldZoomY);

  viewport()->setUpdatesEnabled(TRUE);
  emit zoomChanged(zoomX, zoomY);
  emit viewZoomChanged(getViewScaleX(), getViewScaleY());
  emit contentsSizeChanged(zoomSizeX, zoomSizeY);
  emit docPropChanged(mzoomX, mzoomY);
  emit scaleFactorChanged();
  repaintContents(0);  
}

double ApaintView::getViewScaleX()
{
  if (integerView)
   return zoomX;

  return (mzoomX*facX);
}

double ApaintView::getViewScaleY()
{
  if (integerView)
    return zoomY;

  return (mzoomY*facY);
}

void ApaintView::refresh(int x, int y, int wid, int hei)
{
  if (x==-1 && y==-1 && wid==-1 && hei==-1)
   repaintContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
  else
   repaintContents(x, y, wid, hei);  
}

void ApaintView::scale(double x, double y,  bool center, int posx, int posy)
{
   mzoomX=x;
   mzoomY=y;
   setZoom(x,y, center, posx, posy);
}


void ApaintView::scaleDown(int posx, int posy, bool center)
{
 double val,nx;

 nx = (double)((int)(mzoomX*10))/10;

 if (nx<=0.2)
  val=0.05;
 else if (nx<=1)
  val=0.1;
 else
  val=1;

 if ((mzoomX-val>=MIN_SCALE) && (mzoomY-val>=MIN_SCALE))
 {
   mzoomX-=val;
   mzoomY-=val;
   if (posx==-1 && posy==-1)
    setZoom(mzoomX, mzoomY, center, mouseX, mouseY);
   else
    setZoom(mzoomX, mzoomY, center, posx, posy);
 }
}

void ApaintView::scaleUp(int posx, int posy, bool center)
{
  double val,nx;
  nx = (double)((int)(mzoomX*10))/10;

  if (nx<0.2)
   val=0.05;
  else if (nx<1)
   val=0.1;
  else
   val=1;

  if ((mzoomX+val<=MAX_SCALE) && (mzoomY+val<=MAX_SCALE))
  {
    mzoomX+=val;
    mzoomY+=val;
    if (posx==-1 && posy==-1)
     setZoom(mzoomX, mzoomY, center, mouseX, mouseY);
    else
     setZoom(mzoomX, mzoomY, center, posx, posy);
  }
}

void ApaintView::setVisible(QRect pos)
{
 int left = (int)(pos.left()*zoomX); 
 int right = (int)(pos.right()*zoomX);
 int top = (int)(pos.top()*zoomY);
 int bottom = (int)(pos.bottom()*zoomY);
 int lx=contentsX();
 int ty=contentsY();
 int rx=contentsX()+visibleWidth();
 int by=contentsY()+visibleHeight();

 if ((right-left)>visibleWidth() || (bottom-top)>visibleHeight())
  return;
 
 if (left<lx)
  scrollBy(left-lx, 0);
 else if (right>rx)
  scrollBy(right-rx,0);

 if (top<ty)
  scrollBy(0, top-ty);
 else if (bottom>by)
  scrollBy(0, bottom-by);
}

void ApaintView::scrollCorner(int x, int y)
{
  int rx, by, lx, ty;

  lx=contentsX();
  ty=contentsY();
  rx=contentsX()+visibleWidth();
  by=contentsY()+visibleHeight();
  if (x>rx-10)
   scrollBy(20,0);
  else if (x<lx+10)
   scrollBy(-20, 0);
   
  if (y>by-10)
   scrollBy(0, 20);
  else if (y<ty+10)
   scrollBy(0, -20);
}

void ApaintView::setViewCur(QCursor &cur)
{
 // if application cursor is in wait state caller action will reset cursor
 if (QApplication::overrideCursor() != 0 && QApplication::overrideCursor()->shape() == Qt::WaitCursor)
  return;

 viewport()->setCursor(cur);

 // application cursor must be set 
 QApplication::setOverrideCursor(cur);
}

void ApaintView::unsetViewCur()
{
 viewport()->unsetCursor();
 // reset of application cursor
 QApplication::restoreOverrideCursor();
}

void ApaintView::setAutoScroll(bool On)
{
 autoScroll=On;
}

QRect ApaintView::getContentsSize()
{
 return QRect((int)(contentsX()/zoomX), (int)(contentsY()/zoomY),
              (int)(QMIN(visibleWidth(), zoomSizeX)/zoomX), (int)(QMIN(visibleHeight(), zoomSizeY)/zoomY));   
}

void ApaintView::contentsMovingSlot(int x, int y)
{
 offsetX = x;
 offsetY = y;
  
 emit viewOffsetChanged(x, y); 
}

void ApaintView::repaintGuideSlot()
{
 emit repaintGuideSig();
}

/*GRID PROPERTIES...*/

GridPropDlg::GridPropDlg(QWidget *parent, const char *name):GridPropDialogUI(parent, name)
{
  setCaption(ApaintConfig::getText(608));
  gbGridSize->setTitle(ApaintConfig::getText(618));
  lbWidth->setText(ApaintConfig::getText(WIDTH_MSG)+":");
  lbHeight->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
  bgColor->setTitle(ApaintConfig::getText(178));
  rbBlack->setText(ApaintConfig::getText(197));
  rbWhite->setText(ApaintConfig::getText(199));
  rbGrey->setText(ApaintConfig::getText(198));
//  btnOk->setText(ApaintConfig::getText(OK_MSG));
  btnClose->setText(ApaintConfig::getText(OK_MSG));
  gridColor = 0;
  connect(bgColor, SIGNAL(clicked(int)), this, SLOT(colorClicked(int)));
//  connect(btnOk, SIGNAL(clicked()), this, SIGNAL(okBtnClicked()));
  connect(btnClose, SIGNAL(clicked()), this, SLOT(closeDlg()));
  connect(sbWid, SIGNAL(valueChanged(int)), this, SLOT(sizeChanged(int)));
  connect(sbHei, SIGNAL(valueChanged(int)), this, SLOT(sizeChanged(int)));
}

GridPropDlg::~GridPropDlg()
{
}

void GridPropDlg::colorClicked(int val)
{
 switch(val)
 {
   case 0: gridColor = 0; break;
   case 1: gridColor = 255; break;
   case 2: gridColor = 127; break;
 }
 emit okBtnClicked();
}

void GridPropDlg::sizeChanged(int val)
{
  emit okBtnClicked();
}

void GridPropDlg::showDlg()
{
  show();
}

void GridPropDlg::closeDlg()
{
  hide();
}
