
/****************************SHOWCOLORREMAP CLASS *************************************/

/***************************************************************************
                          colorpalette.cpp  -  description
                             -------------------
    begin                : Fri Apr 11 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qspinbox.h>
#include <stdlib.h>
#include <qslider.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qtooltip.h>
#include <qpen.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include "dialogColorPaletteUtil.h"
#include "apaintconfig.h"
#include "dialogIconView.h"

extern char *getCommonPNG(const char *pixName);

#define CELL_SIZE_X 27
#define CELL_SIZE_Y 27
#define CELL_OFFSET 1
#define XOFFSET     3
#define YOFFSET     5

#define COLOR_MED           381


class ColNode {
public:
  ColNode(int mc);
  ~ColNode();

  void addTransCol(int num);
  void removeTransCol(int num);
  int getMainColor() {return mainColor;}
  int numChildren() {return pos;}
  int getTransCol(int i)
     { if (pos>0 && i>=0 && i<pos) return transCol[i]; else return 0;}
private:
  bool nodeColExist(int num, int &pos);
  
  int mainColor, pos;
  int transCol[256];
      
};

ColNode::ColNode(int mc)
{
 mainColor = mc;
 pos = 0; 
}

ColNode::~ColNode()
{
  
}

bool ColNode::nodeColExist(int num, int &ptab)
{
  for (int i=0; i<pos; i++)
   if (transCol[i] == num)
   {
    ptab = i; 
    return true;
   }
  return false; 
}

void ColNode::addTransCol(int num)
{
  int ptab;
  
  if (pos<256 && !nodeColExist(num, ptab))
  {
   transCol[pos] = num;
   pos++;
  }
}

void ColNode::removeTransCol(int num)
{ 
  int ptab;
  if (nodeColExist(num, ptab))
  {
   for (int i=ptab; i<pos-1; i++)
   {
    transCol[i] = transCol[i+1];
   }
   pos--;
  }
}

class ColorRemap {
public:
  ColorRemap();
  ~ColorRemap();

  void addMainColor(int num);
  void removeMainColor(int num);
  void removeMainColorPos(int mainPos);
  void addNodeColor(int mainPos, int colNum);
  void removeNodeColor(int mainPos, int num);  
  ColNode *getColNode(int numPos);
  ColNode *first() {return remapNodes.first();}
  ColNode *next() {return remapNodes.next();}
  int mainColorCount() {return remapNodes.count();}
  void reset();
  bool isEmpty();
   bool mainColExist(int num, int &pos);
private:
   
  QPtrList<ColNode> remapNodes;
};

ColorRemap::ColorRemap()
{
 remapNodes.setAutoDelete(true);
}

ColorRemap::~ColorRemap()
{
  
}

void ColorRemap::reset()
{
 remapNodes.clear();
}

bool ColorRemap::isEmpty()
{
 return(remapNodes.isEmpty());
}

bool ColorRemap::mainColExist(int num, int &pos)
{
 ColNode *node;
 int i;

 i=0;
 for (node = remapNodes.first(); node ; node = remapNodes.next())
 {
   if (node->getMainColor() == num)
   {
    pos =  i;
    return true;
   }
   i++;
 }   
 return false;
}

void ColorRemap::addMainColor(int num)
{
  int pos;
  
  ColNode *node = new ColNode(num);
  if (!node)
  {
    qDebug("ColorRemap: NULL pointer!");
    return;
  }
  if (!mainColExist(num, pos))
    remapNodes.append(node);   
}

void ColorRemap::removeMainColor(int num)
{
 int pos;
 
  if (mainColExist(num, pos))
   remapNodes.remove(pos);

}

void ColorRemap::removeMainColorPos(int mainPos)
{
 if (mainPos<0 || mainPos>=(int)remapNodes.count()) return;
 remapNodes.remove(mainPos);
}

void ColorRemap::addNodeColor(int mainPos, int colNum)
{
  
  if (mainPos<0 || mainPos>=(int)remapNodes.count()) return;
  
  (remapNodes.at(mainPos))->addTransCol(colNum);
}

void ColorRemap::removeNodeColor(int mainPos, int num)
{
  if (mainPos<0 || mainPos>=(int)remapNodes.count()) return;
   
  (remapNodes.at(mainPos))->removeTransCol(num);
}

ColNode *ColorRemap::getColNode(int numPos)
{
  if (numPos<0 || numPos>=(int)remapNodes.count()) return 0;

  return(remapNodes.at(numPos));
}

/**************************ShowColorRemap CLASS*********************/


ShowColorRemap::ShowColorRemap(QWidget *parent, const char *name):QScrollView(parent, name)
{
 colMapSel = 0;
 lastMap = 0;
 numCol = 0;
 colMap = new ColorRemap();
 csizex = 0;
 csizey = 0;
 rMenu = new QPopupMenu(this, "rMenu");
 rMenu->insertItem(ApaintConfig::getText(624), 0);
 connect(rMenu, SIGNAL(activated(int)), this, SLOT(rMenuClicked(int)));
 resizeContents(csizex, csizey);
 leftArrPix = QPixmap(getCommonPNG("go-next"));
}

ShowColorRemap::~ShowColorRemap()
{
  
}

QSize ShowColorRemap::sizeHint() const
{
 return (QSize(200, 300));
}

void ShowColorRemap::reset()
{
 colMap->reset();
 colMapSel = 0;
 lastMap = 0;
 csizex = 0;
 csizey = 0; 
 updateContents();
}

bool ShowColorRemap::isEmpty()
{
  return colMap->isEmpty();
}

void ShowColorRemap::updateContentsSize(int mapNum)
{
 QRect s = getColorMapSize(mapNum);
 if (s.width()>csizex)
  csizex = s.width();
 if (s.height()>csizey)
  csizey = s.height();
 resizeContents(csizex, csizey);
 updateContents();
}

void ShowColorRemap::setColorTable(QRgb *colTabel, int numColors)
{
 numCol = numColors;
 if (colTab!=0)
 {
   for (int i=0; i<numCol; i++)
    colTab[i] = colTabel[i];
 }
}

/*return current node width and height from the top of view*/
QRect ShowColorRemap::getColorMapSize(int lastNode)
{
  QRect ns = getNodeMapSize(lastNode);
  return QRect(0, 0, QMAX(ns.width(), CELL_SIZE_X+XOFFSET), (lastNode+1)*(CELL_SIZE_Y+YOFFSET));
}

/*return pos of mapNum node*/
QRect ShowColorRemap::getNodeMapSize(int mapNum)
{
  ColNode *node = colMap->getColNode(mapNum);
  if (!node) return QRect();
  return QRect(0, mapNum*(CELL_SIZE_Y+YOFFSET), (node->numChildren()+2)*(CELL_SIZE_X+XOFFSET), CELL_SIZE_Y);
}


bool ShowColorRemap::colorMapIntersect(int numMap, int clipx, int clipy, int clipw, int cliph, QRect &dstRect)
{
  QRect clip = QRect(clipx, clipy, clipw, cliph);
  QRect mapRect = getNodeMapSize(numMap);
  dstRect = clip.intersect(mapRect);
  return (clip.intersects(mapRect));
}

void ShowColorRemap::drawNumber(QPainter *p,  int x, int y, int w, int h, int n)
{
  int light = qRed(colTab[n])+qGreen(colTab[n])+qBlue(colTab[n]);
  if (light<COLOR_MED)
   p->setPen(QPen(white));
  else
   p->setPen(QPen(black));

  p->drawText(x, y, w, h, AlignCenter, QString("%1").arg(n + ApaintConfig::getFirstPaletteColor()));
}

void ShowColorRemap::drawColorMap(QPainter *p, int mapNum, QRect &clip)
{
  int startx, endx;
  int starty;
  startx = clip.left() / (CELL_SIZE_X+XOFFSET);
  endx = clip.right() / (CELL_SIZE_X+XOFFSET);
  starty = mapNum*(CELL_SIZE_Y+YOFFSET);
  
  ColNode *node = colMap->getColNode(mapNum);
  if (!node) return;
  
  if (startx==0)
  {
   QBrush b = QBrush(colTab[node->getMainColor()]);
   if (colMapSel == mapNum)
    qDrawShadeRect(p, 0, starty, CELL_SIZE_X, CELL_SIZE_Y, colorGroup(), true, 2, 0, &b);
   else
    qDrawShadeRect(p, 0, starty, CELL_SIZE_X, CELL_SIZE_Y, colorGroup(), false, 2, 0, &b);
    
   drawNumber(p, 0, starty, CELL_SIZE_X, CELL_SIZE_Y, node->getMainColor());
   startx++;
  }
  if (startx==1)
  {
    p->drawPixmap((CELL_SIZE_X+XOFFSET), starty, leftArrPix);
    startx++;
  }

  for (int i=startx; i<=endx; i++)
  {
    p->setBrush(QColor(colTab[node->getTransCol(i-2)]));/*-2 becouse we start trans colors from 2nd pos*/
    p->setPen(NoPen);
    p->drawRect(i*(CELL_SIZE_X+XOFFSET), starty, CELL_SIZE_X, CELL_SIZE_Y);
    drawNumber(p, i*(CELL_SIZE_X+XOFFSET), starty, CELL_SIZE_X, CELL_SIZE_Y, node->getTransCol(i-2));
  }
    
}
void ShowColorRemap::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
 QRect dstRect;
  
 for (int i=0; i<colMap->mainColorCount(); i++)
  if (colorMapIntersect(i, clipx, clipy, clipw, cliph, dstRect))
   drawColorMap(p, i, dstRect);
}

void ShowColorRemap::colorMapClicked(int mapNum, int nodeNum, ButtonState bt, QPoint ptClick)
{
 int starty, sel;
  
 if (mapNum<0 || mapNum>=colMap->mainColorCount()) return;

 if (nodeNum==0)
 {
    sel = colMapSel;
    colMapSel = mapNum;   
    starty = sel*(CELL_SIZE_Y+YOFFSET);
    repaintContents(0, starty, CELL_SIZE_X, CELL_SIZE_Y);
    starty = colMapSel*(CELL_SIZE_Y+YOFFSET);
    repaintContents(0, starty, CELL_SIZE_X, CELL_SIZE_Y);
    if (bt == RightButton)
    {
      rMenu->exec(ptClick);
    }
 }
 else
 {   
   int cnum;
   QRect ref =  getNodeMapSize(mapNum);
   ColNode *node = colMap->getColNode(mapNum);
   cnum = node->getTransCol(nodeNum-2); /*first two nodes are not from transCol*/
   colMap->removeNodeColor(mapNum, cnum);
   repaintContents(ref); 
   emit nodeColorSig(cnum);
   emit colorRemoved();
 } 
}

void ShowColorRemap::contentsMousePressEvent (QMouseEvent *e )
{
 int mapNum = e->y() / (CELL_SIZE_Y+YOFFSET);
 int nodeNum = e->x() / (CELL_SIZE_X+XOFFSET);

 colorMapClicked(mapNum, nodeNum, e->button(), e->globalPos());
}

void ShowColorRemap::insertMainColor(int num)
{
 int pos;
 if (colMap->mainColExist(num, pos))
 {
   colorMapClicked(pos, 0, LeftButton, QPoint(0,0));
   return;
 }
 colMap->addMainColor(num);
 colMapSel = lastMap;
 updateContentsSize(lastMap);
 lastMap++;
}

void ShowColorRemap::insertNodeColor(int num)
{
  colMap->addNodeColor(colMapSel, num);
  updateContentsSize(colMapSel);
}

void ShowColorRemap::removeMainColorPos(int numPos)
{
  ColNode *node = colMap->getColNode(numPos);
  for (int i=0; i<node->numChildren(); i++)
    emit nodeColorSig(node->getTransCol(i));
    
  colMap->removeMainColorPos(numPos);
  if (colMapSel == numPos && colMapSel >0)
   colMapSel--;
  lastMap--;
  updateContents();
  emit colorRemoved();
}

void ShowColorRemap::rMenuClicked(int which)
{
  if (which == 0)
  {
    removeMainColorPos(colMapSel);
  }
}

void ShowColorRemap::getRemapArray(RemapArray &rmap)
{
 ColNode *node;
 int i, mainCol;

 rmap.setLinear();
 for (node = colMap->first(); node ; node = colMap->next())
 {
   mainCol = node->getMainColor();
   for (i=0; i<node->numChildren(); i++)
    rmap.setVal(node->getTransCol(i), mainCol);
 }
}

/************************ColorRemapDialog CLASS*****************************************/

ColorRemapDialog::ColorRemapDialog(QWidget *parent, const char *name, ApaintDoc *docp):ColorRemapDialogUI(parent, name)
{

 /*set captions...*/
 setCaption(ApaintConfig::getText(194));
 gbSelCol->setTitle(ApaintConfig::getText(193));
 gbRemapCol->setTitle(ApaintConfig::getText(192));
 btnOk->setText(ApaintConfig::getText(OK_MSG));
 btnPreview->setText(ApaintConfig::getText(158));
 btnCancel->setText(ApaintConfig::getText(86));
 
 gbSelCol->setColumnLayout(0, Qt::Vertical );
 vbox1 = new QVBoxLayout(gbSelCol->layout());
 colSelect = new ColorPalette(gbSelCol, "colSelect");
 colSelect->enableColorDelete(false);
 vbox1->addWidget(colSelect);

 gbRemapCol->setColumnLayout(0, Qt::Vertical );
 vbox2 = new QVBoxLayout(gbRemapCol->layout());
 colRemap = new ShowColorRemap(gbRemapCol, "colRemap");
 vbox2->addWidget(colRemap);
 dlgOn = false;
 doc = docp;
 previewOn = false;
 connect(colSelect, SIGNAL(colorChanged(QRgb, int)), this, SLOT(insertNodeColor(QRgb, int)));
 connect(colRemap, SIGNAL(nodeColorSig(int)), this, SLOT(colorFromRemap(int))); 
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
 connect(btnOk, SIGNAL(clicked()), this, SLOT(okClicked()));
 connect(btnCancel, SIGNAL(clicked()), this, SLOT(cancelClicked()));
 connect(btnPreview, SIGNAL(clicked()), this, SLOT(previewClicked()));
 connect(colRemap, SIGNAL(colorRemoved()), this, SLOT(doUpdate()));

 /*init used colors*/
 for (int i=0; i<256; i++)
  usedColors[i] = true;

 setCaption(ApaintConfig::getText(194));     
}

ColorRemapDialog::~ColorRemapDialog()
{
 
}

void ColorRemapDialog::resetData()
{
 int num;
 
 Layer *selLayer = doc->getLayer();
 if (!dlgOn) return;
 
 colRemap->reset();
 previewOn = false;
 btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));

 if (!selLayer) return;
 
 selLayer->getUsedColors(usedColors, num);
 colSelect->setVisibleColors(usedColors);
}

void ColorRemapDialog::layerChangedSlot(Layer *sel)
{
  if (!sel) return;  
  resetData();
}

void ColorRemapDialog::startDlg()
{
 dlgOn = true;
 setColorTable(doc->getLayerColorTable(), doc->layerNumColors());
 resetData();
 doc->protectLayer(true);
 show();
}


void ColorRemapDialog::doUpdate()
{
 if (previewOn)
  {
    disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doc->undoLayerPreview(false, false);
    connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doAction(true);
  } 
}

void ColorRemapDialog::doUndo(bool refresh)
{
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  doc->undoLayerPreview(false, refresh);
  btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  previewOn = false;
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

void ColorRemapDialog::doAction(bool preview)
{
  RemapArray rmap;
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  if (!preview)
    doc->putUndoStack(ChangeLayerForm);
  colRemap->getRemapArray(rmap);
  doc->layerRemapData(rmap, preview);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

void ColorRemapDialog::okClicked()
{
  if (previewOn)
  {
   doc->putUndoStack(ChangeLayerForm);
   doc->finishLayerPreview(); 
  }
  else
   doAction(false);
  close(); 
}

void ColorRemapDialog::cancelClicked()
{  
  close();
}

void ColorRemapDialog::previewClicked()
{
 if (!previewOn)
 {
  btnPreview->setText(ApaintConfig::getText(26));
  previewOn = true;
  doAction(true);
 }
 else
 {    
  doUndo(true);
 }
}

void ColorRemapDialog::setColorTable(QRgb *colTab, int num)
{
  if (!colTab) return;
  
  colSelect->setColorTable(colTab, num);
  colRemap->setColorTable(colTab, num); 
}


void ColorRemapDialog::insertMainColor(int num)
{
  colRemap->insertMainColor(num);
}

void ColorRemapDialog::insertNodeColor(QRgb rgb, int num)
{
 if (colRemap->isEmpty())  return;
 
 colRemap->insertNodeColor(num);
 colSelect->hideColor(num);
 doUpdate();
}

void ColorRemapDialog::colorFromRemap(int num)
{
 colSelect->showColor(num);
}

void ColorRemapDialog::doOnClose()
{
  dlgOn = false;
  doc->protectLayer(false);
  if (previewOn)
    doUndo(true);  
}

void ColorRemapDialog::closeEvent(QCloseEvent *e)
{
  doOnClose();
  ColorRemapDialogUI::closeEvent(e);
}

void ColorRemapDialog::hideEvent(QHideEvent *e)
{
  doOnClose();
  ColorRemapDialogUI::hideEvent(e);
}

/************************REDUCECOLORDLG CLASS******************************************/

ReduceColorDlg::ReduceColorDlg(QWidget *parent, const char *name, ApaintDoc *docp, QFileDialog *fDlg, IconViewDlg *iDlg):ReduceColDialogUI(parent, name)
{
  setCaption(ApaintConfig::getText(176));
  bgType->setTitle(ApaintConfig::getText(179));
  rbColor->setText(ApaintConfig::getText(178));
  rbBW->setText(ApaintConfig::getText(139));
  gbColTab->setTitle(ApaintConfig::getText(623));
  btnOk->setText(ApaintConfig::getText(OK_MSG));  
  btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  btnCancel->setText(ApaintConfig::getText(86));
  cbDithering->setText(ApaintConfig::getText(140));
  gbLoadPalette->setTitle(ApaintConfig::getText(61));
  btnBrowse->setText(ApaintConfig::getText(657)+"...");
  btnLoad->setText(ApaintConfig::getText(676)+"...");
  btnClear->setPixmap(QPixmap(getCommonPNG("edit-clear")));
  QToolTip::add(btnClear, ApaintConfig::getText(745));
  enableShowColor = true;
  vbox = new QVBoxLayout(gbColTab->layout()); 
  cp = new ColorPalette(gbColTab, name);
  cp->enableColorDelete(true);
  vbox->addWidget(cp);
  palettePath = ApaintConfig::getImgPath();
  Q_CHECK_PTR(fDlg);
  fileDlg = fDlg;
  Q_CHECK_PTR(iDlg);
  iconDlg = iDlg;
  doc = docp;
  previewMode = false;
  isOpen = false;
  sldBW->setTracking(false);

  connect(btnOk, SIGNAL(clicked()), this, SLOT(btnOkClicked()));
  connect(btnPreview, SIGNAL(clicked()), this, SLOT(btnPreviewClicked()));
  connect(btnCancel, SIGNAL(clicked()), this, SLOT(btnCancelClicked()));
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  connect(sldBW, SIGNAL(valueChanged(int)), this, SLOT(setBWBorder(int)));
  connect(bgType, SIGNAL(clicked(int)), this, SLOT(setMode(int)));
  connect(btnLoad, SIGNAL(clicked()), this, SLOT(loadBtnPalette()));
  connect(btnBrowse, SIGNAL(clicked()), this, SLOT(browseBtnPalette()));
  connect(btnClear, SIGNAL(clicked()), this, SLOT(clearBtnPalette()));
  connect(iconDlg, SIGNAL(loadColorPImage(QString&)), this, SLOT(loadPalette(QString&)));
  connect(iconDlg, SIGNAL(loadColormapImage(QString&)), this, SLOT(loadPaletteOK(QString&)));
  connect(iconDlg, SIGNAL(loadColormapDitherImage(QString&)), this, SLOT(loadPaletteDitherOK(QString&)));
  connect(cbDithering, SIGNAL(clicked()), this, SLOT(doUpdate()));
  setMode(ColorMode);
  sldBW->setValue(128);  
  
}

ReduceColorDlg::~ReduceColorDlg()
{

}

void ReduceColorDlg::startDlg()
{
  doc->protectLayer(true);
  resetData();
  isOpen = true;
  show();
}

void ReduceColorDlg::setBWBorder(int num)
{
 bwBorder = num;
 QString bwStr = QString("%1 : %2").arg(ApaintConfig::getText(180)).arg(bwBorder);
 gbBWSet->setTitle(bwStr);
 doUpdate();
  
}

void ReduceColorDlg::setMode(int showMode)
{
  Layer *layer = doc->getLayer();
  if (layer) 
    if (previewMode)
      doUndo(true);

  mode = showMode;

  if (mode == ColorMode)
  {   
   gbColTab->show();
   gbBWSet->hide();
   gbLoadPalette->show();
  }
  else
  {
   gbColTab->hide();
   gbBWSet->show();
   gbLoadPalette->hide();
  }
  
  updateGeometry();
}

void ReduceColorDlg::loadPalette(QString &fname)
{
 ApaintImage ctabImage;
 
 if (!fname.isEmpty())
   if (ctabImage.load(fname))
    if (ctabImage.type() == LOAD_CMAP)
     {
      setColorTable(ctabImage.colorTable(), ctabImage.numColors());
      return;
     }
 QApplication::beep();
}

void ReduceColorDlg::loadPaletteOK(QString &fname)
{
 ApaintImage ctabImage;

 if (!fname.isEmpty())
  if (ctabImage.load(fname))
   if (ctabImage.type() == LOAD_CMAP)
    {
     cbDithering->setChecked(FALSE);
     setColorTable(ctabImage.colorTable(), ctabImage.numColors());
     doAction(false);
     return;
    }
 QApplication::beep();
}

void ReduceColorDlg::loadPaletteDitherOK(QString &fname)
{
 ApaintImage ctabImage;

 if (!fname.isEmpty())
  if (ctabImage.load(fname))
   if (ctabImage.type() == LOAD_CMAP)
    {
     cbDithering->setChecked(TRUE);
     setColorTable(ctabImage.colorTable(), ctabImage.numColors());
     doAction(false);
     return;
    }
 QApplication::beep();
}

void ReduceColorDlg::loadBtnPalette()
{  
 fileDlg->setCaption(ApaintConfig::getText(61));
 fileDlg->setDir(palettePath);
 fileDlg->setSelection("");
 if (fileDlg->exec() == QDialog::Accepted)
 {
   fileName = fileDlg->selectedFile();
   loadPalette(fileName);
 }
 palettePath = fileDlg->dirPath(); 
}

void ReduceColorDlg::browseBtnPalette()
{
  iconDlg->showDlg(palettePath, IconViewDlg::ColorPImage); 
}

void ReduceColorDlg::clearBtnPalette()
{
 int i;

 if (previewMode)
  doUndo(true);

 for (i=0; i<cp->colorNum(); i++)
  cp->hideColor(i);
}

void ReduceColorDlg::layerChangedSlot(Layer *layer)
{
 if (previewMode)
 {
   btnPreview->setText(btnText);
   previewMode = false;
 }
 resetData();
}

void ReduceColorDlg::resetData()
{
  enableShowColor = true;
  if (mode == ColorMode)
  {   
   if (doc->getLayer()->getImageType() == LOAD_TRUE_COL)
    {
       gbDither->show();
       rbBW->setEnabled(false);
    }
    else if (doc->getLayer()->getImageType() == LOAD_CMAP)
    {
      gbDither->hide();
      rbBW->setEnabled(true);
      for (int i=0; i<256; i++)
       usedColors[i] = false;

      cp->setColorTable(doc->getLayerColorTable(), doc->layerNumColors());
      cp->setVisibleColors(usedColors);
    }
  }
  else if (BWMode)
  {
    if (doc->getLayer()->getImageType() == LOAD_TRUE_COL)
    {
      bgType->setButton(0);
      setMode(ColorMode);
      gbDither->show();
      rbBW->setEnabled(false);
    }
  }
  updateGeometry();
}

void ReduceColorDlg::setColorTable(QRgb *ctab, int num)
{
  enableShowColor = false;
  for (int i=0; i<num; i++)
   usedColors[i] = true;

  cp->setColorTable(ctab, num);
  cp->setVisibleColors(usedColors);
  if (previewMode)
   doUpdate();
}

void ReduceColorDlg::showColor(int num)
{
  if (mode != ColorMode) return;
  if (!enableShowColor) return;
  if (!previewMode)
    cp->showColor(num);
}

void ReduceColorDlg::doUpdate()
{
  if (previewMode)
  {
    disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doc->undoLayerPreview(false, false);
    connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doAction(true);    
  }
}

void ReduceColorDlg::doUndo(bool refresh)
{
   disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
   if (mode == ColorMode)   
     doc->undoLayerPreview(false, refresh);
   else
     doc->undoLayerPreview(true, refresh);
   btnPreview->setText(btnText);
   previewMode = false;
   connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
}

void ReduceColorDlg::doAction(bool preview)
{
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  if (mode == ColorMode)
  {
    if (cp->visibleColorNum()>0)
    {
     if (!preview)
       doc->putUndoStack(ChangeLayerForm);
     doc->setLayerColorTable(cp->getTransColorTab(), cp->visibleColorNum(), cbDithering->isChecked(), preview);
    }
  }
  else
  {
    if (!preview)
      doc->putUndoStack(ChangeLayerForm);
    doc->changeLayerToBW(bwBorder,preview);
  }
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
}

void ReduceColorDlg::btnOkClicked()
{
 if (previewMode)
 {
   disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
   btnPreview->setText(btnText);
   previewMode = false;
   if (mode == ColorMode)
   {
     doc->putUndoStack(ChangeLayerForm);   
     doc->finishLayerPreview();
   }
   else 
   {
     /*if BWImage we have to merge colors at the end!*/
     doAction(false);
   }
   connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
 }
 else     
   doAction(false);
 close();
}

void ReduceColorDlg::btnPreviewClicked()
{
  if (!cp->visibleColorNum() && (mode == ColorMode)) return;
  if (previewMode)
   doUndo(true);
  else
  {
   previewMode = true;
   btnText = btnPreview->text();
   btnPreview->setText(ApaintConfig::getText(26));
   doAction(true);
 }
}

void ReduceColorDlg::btnCancelClicked()
{ 
  close();
}

void ReduceColorDlg::doOnClose()
{
  if (previewMode)
     doUndo(true);
  if (!iconDlg->isHidden())
   iconDlg->close();
  doc->protectLayer(false);
  isOpen = false;  
}

void ReduceColorDlg::closeEvent(QCloseEvent *e)
{
 doOnClose();
 ReduceColDialogUI::closeEvent(e);
}

void ReduceColorDlg::hideEvent(QHideEvent *e)
{
  doOnClose();
  ReduceColDialogUI::hideEvent(e);
}

/************************COLNUMDLG CLASS******************************************/

ColNumDlg::ColNumDlg(QWidget* parent , const char* name , ApaintDoc *docp):ColorNumDialogUI(parent, name)
{
  setCaption(ApaintConfig::getText(19));
  lbMaxUsedCol->setText(ApaintConfig::getText(18)+":");
  lbUsedCol->setText(ApaintConfig::getText(17)+":");
  lbNoise->setText(ApaintConfig::getText(607)+":");
  btnOk->setText(ApaintConfig::getText(OK_MSG));
  btnCancel->setText(ApaintConfig::getText(86));
  cbDither->setText(ApaintConfig::getText(140));
  btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  
  ncol = maxCol =0;
  isTrueCol = false;
  dlgOn = false;
  previewMode = false;
  setVisualData(true);
  Q_CHECK_PTR(docp);
  doc = docp;
  //no default buttons argh...
  btnOk->setDefault(false);

  connect(btnOk, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
  connect(btnCancel, SIGNAL(clicked()), this, SLOT(cancelBtnClicked()));
  connect(btnPreview, SIGNAL(clicked()), this, SLOT(previewBtnClicked()));
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  connect(cbDither, SIGNAL(toggled(bool)), this, SLOT(ditherchecked(bool)));
  connect(sbNoise, SIGNAL(valueChanged(int)), this, SLOT(valChanged(int)));
  connect(sbMaxVal, SIGNAL(valueChanged(int)), this, SLOT(valChanged(int)));  
}


ColNumDlg::~ColNumDlg()
{
}


void ColNumDlg::ditherchecked(bool enable)
{
  if (enable)
    sbNoise->setValue(0);
  sbNoise->setEnabled(!enable);
  lbNoise->setEnabled(!enable);
  doUpdate();
}

void ColNumDlg::valChanged(int val)
{
  doUpdate();
}

void ColNumDlg::newDocSlot()
{
  if (previewMode)
  {
   previewMode = false;
   btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  }
}

void ColNumDlg::layerChangedSlot(Layer *sel)
{
 if (previewMode)
 {
   btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
   previewMode = false;
 }
 updateVisual(sel);
}

void ColNumDlg::updateVisual(Layer *layer)
{
  bool usedCol[256];
  int num;

  if (!dlgOn) return;
  if (!layer) return;
  if (layer->getImageType() == LOAD_TRUE_COL)
  {
    ncol = 256;
    maxCol = 256;
    isTrueCol = true;
    showNoiseBox(true);
    cbDither->show();
  }
  else if (layer->getImageType() == LOAD_CMAP)
  {
    layer->getUsedColors(usedCol, num);
    ncol = num;
    maxCol = layer->numColors();
    isTrueCol = false;
    showNoiseBox(false);
    cbDither->hide();
  }
  setVisualData(true); 
}

void ColNumDlg::showNoiseBox(bool enable)
{
  if (enable)
  {
   lbNoise->show();
   sbNoise->show();
  }
  else
  {
   lbNoise->hide();
   sbNoise->hide();    
  }
}

void ColNumDlg::setVisualData(bool setSld)
{
  if (isTrueCol)
  {
    lbNum->setText(ApaintConfig::getText(613));
  }
  else
  {
    lbNum->setText(QString("%1").arg(ncol));
  }
  if (setSld)
    sbMaxVal->setValue(maxCol);
}

void ColNumDlg::startDlg()
{
  dlgOn = true;
  doc->protectLayer(true);
  updateVisual(doc->getLayer());
  show();
}


void ColNumDlg::doUpdate()
{
  if (previewMode)
  {
    disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doc->undoLayerPreview(false, false);
    connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
    doAction(true);
  }
}

void ColNumDlg::doUndo(bool refresh)
{
  if (!previewMode) return;
  int oldNum = sbMaxVal->value();
  previewMode = false;
  btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  doc->undoLayerPreview(false, refresh);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  maxCol = oldNum;
  sbMaxVal->setValue(maxCol);
  
}

void ColNumDlg::doAction(bool preview)
{
  int noise = sbNoise->value();
  bool dither = (doc->getLayer()->getImageType() == LOAD_TRUE_COL) && cbDither->isChecked();
  
  maxCol = sbMaxVal->value();
  if (noise<0 || noise >100) return;

  if (!preview)
    doc->putUndoStack(ChangeLayerForm);
  
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
  if (dither)
    doc->setLayerNumColors(maxCol, 0, dither, preview);
  else
    doc->setLayerNumColors(maxCol, noise, dither, preview);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer*)));
}

void ColNumDlg::previewBtnClicked()
{
  if (previewMode)
  {
    doUndo(true);    
  }
  else
  {
   previewMode = true; 
   btnPreview->setText(ApaintConfig::getText(26)); 
   doAction(true);
  }
}


void ColNumDlg::okBtnClicked()
{
  if (previewMode)
  {
    doc->putUndoStack(ChangeLayerForm);
    doc->finishLayerPreview();
  }
  else
    doAction(false);
  close();  
}

void ColNumDlg::cancelBtnClicked()
{
  close();
}

void ColNumDlg::doOnClose()
{
  if (previewMode)
    doUndo(true);
  doc->protectLayer(false);
  dlgOn = false;  
}

void ColNumDlg::closeEvent(QCloseEvent *e)
{  
  doOnClose();
  ColorNumDialogUI::closeEvent(e);
}

void ColNumDlg::hideEvent(QHideEvent *e)
{
  doOnClose();
  ColorNumDialogUI::hideEvent(e); 
}

