/***************************************************************************
                          layerseldlg.cpp  -  description
                             -------------------
    begin                : Thu Oct 23 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogLayerSelect.h"
#include <qlayout.h>
#include <qgroupbox.h>
#include <qcolor.h>
#include <qlabel.h>
#include "apaintconfig.h"
#include <qtable.h>
#include <qpushbutton.h>
#include <qstringlist.h>
#include <qtooltip.h>
#include <qspinbox.h>

extern char *getCommonPNG(const char *pixName);

/*LayerEntry*/

LayerEntry::LayerEntry(QString name, Layer *layer)
{
  lname = name;
  ladd = layer;
}

LayerEntry::~LayerEntry()
{}

void LayerEntry::setName(QString name)
{
  lname = name;  
}

void LayerEntry::setImage(QImage img)
{
  limg = img;
}

/*LayerSelDlg*/
LayerSelDlg::LayerSelDlg(QWidget *parent, const char *name, ApaintDoc *docp):LayerSelectDialogUI(parent, name)
{
 gbList->setTitle(ApaintConfig::getText(645)+":");
 gbList->setColumnLayout(0, Qt::Vertical );
 gbList->layout()->setSpacing( 1 );
 gbList->layout()->setMargin( 1 );
 epix = QPixmap(getCommonPNG("eye"));
 vbox = new QVBoxLayout(gbList->layout());
 listTable = new QTable(gbList, "Table");
 listTable->setNumRows(0);
 listTable->setNumCols(2);
 listTable->setRowMovingEnabled(false);
 listTable->setSorting(false);
 listTable->setSelectionMode( QTable::SingleRow );
 listTable->setFocusStyle( QTable::FollowStyle );
 listTable->setTopMargin(0);
 listTable->horizontalHeader()->hide();
 listTable->setLeftMargin(0); 
 listTable->verticalHeader()->hide();
 listTable->setColumnWidth(0, 25);
 listTable->setColumnStretchable(1, true);
 listTable->setColumnReadOnly(0, true);
 listTable->setShowGrid(false);
 vbox->addWidget(listTable); 
 Q_CHECK_PTR(docp);
 doc = docp;
 connect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int)));
 connect(listTable, SIGNAL(valueChanged(int, int)), this, SLOT(textChanged(int, int)));
 connect(btnClose, SIGNAL(clicked()), this, SLOT(removeSelected()));
 connect(listTable, SIGNAL(clicked(int , int , int , const QPoint& )), this, SLOT(cellClickedSlot(int , int , int , const QPoint&)));
 connect(btnOpen, SIGNAL(clicked()), this, SIGNAL(loadLayerSig()));
 connect(btnBrowse, SIGNAL(clicked()), this, SIGNAL(browseLayerSig()));
 connect(btnNew, SIGNAL(clicked()), this, SIGNAL(newLayerSig()));
 connect(btnUp, SIGNAL(clicked()), this, SLOT(moveLayerUp()));
 connect(btnDown, SIGNAL(clicked()), this, SLOT(moveLayerDown()));
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(changeToLayer(Layer *)));
 QToolTip::add(btnOpen, ApaintConfig::getText(604));
 QToolTip::add(btnBrowse, ApaintConfig::getText(657));
 QToolTip::add(btnNew, ApaintConfig::getText(653));
 QToolTip::add(btnClose, ApaintConfig::getText(DELETE_LAYER_MSG));
 QToolTip::add(btnUp, ApaintConfig::getText(651));
 QToolTip::add(btnDown, ApaintConfig::getText(652));    
 btnOpen->setPixmap(QPixmap(getCommonPNG("document-open")));
 btnBrowse->setPixmap(QPixmap(getCommonPNG("document-browse")));
 btnNew->setPixmap(QPixmap(getCommonPNG("document-new")));
 btnClose->setPixmap(QPixmap(getCommonPNG("edit-delete")));
 btnDown->setPixmap(QPixmap(getCommonPNG("1downarrow")));
 btnUp->setPixmap(QPixmap(getCommonPNG("1uparrow")));
}

LayerSelDlg::~LayerSelDlg()
{
  
}

void LayerSelDlg::resetAll()
{
 for (int i=0; i<listTable->numRows(); i++)
 {
   listTable->removeRow(0);
 }
   
}

void LayerSelDlg::moveLayerUp()
{  
  if (doc->layerPosUp())
  {
   refreshLayerTab(doc->getLayer());
  }
}

void LayerSelDlg::moveLayerDown()
{  
  if (doc->layerPosDown())
  {   
   refreshLayerTab(doc->getLayer());
  }
}

void LayerSelDlg::removeLayerSlot(int num)
{
  disconnect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int))); 
  int lnum = listTable->numRows()-num-1;
  listTable->removeRow(lnum);
  connect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int))); 
}

void LayerSelDlg::refreshLayerTab(Layer *layer)
{
  disconnect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int)));
  resetAll();  
  listTable->setNumRows(doc->numLayers());
  for (int i=0; i<doc->numLayers();  i++)
  {
   listTable->setText(i, 1, doc->getLayer(doc->numLayers()-i-1)->getLayerName()) ;
   QLabel *elb = new QLabel(this);
   elb->setPixmap(epix);
   listTable->setCellWidget(i, 0, elb);   
  }
  listTable->setCurrentCell(doc->numLayers()-getLayerNum(layer)-1, 0); 
  connect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int)));
}


void LayerSelDlg::setListPos(int num)
{
 int ln = doc->numLayers() - num -1;
 if (ln == listTable->currentRow()) return;
 disconnect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int)));
 listTable->setCurrentCell(ln, 0); 
 connect(listTable, SIGNAL(currentChanged(int, int)), this, SLOT(entryClicked(int, int)));   
}

void LayerSelDlg::changeToLayer(Layer *layer)
{
  int num = getLayerNum(layer);
  if (num != -1)
    setListPos(num);
}

void LayerSelDlg::cellClickedSlot(int row, int col, int button, const QPoint& mp)
{
 if (col == 0)
 {
  Layer *l = doc->getLayer();
  if (l->getType() == MainLayer) return;
  if (l->isHidden()) 
  {
    l->show();
    listTable->cellWidget(row, col)->show();
  }
  else
  {
    l->hide();
    listTable->cellWidget(row, col)->hide(); 
  }
  doc->refreshLayer();
 } 
}

void LayerSelDlg::textChanged(int row, int col)
{
  if (col == 1)
  {
   int num = listTable->numRows()- row - 1;
   doc->getLayer(num)->setLayerName(listTable->text(row, col));
  }
}

void LayerSelDlg::removeSelected()
{
 int num = listTable->numRows()-listTable->currentRow()-1;
 doc->putUndoStack(DestroyLayer);
 doc->deleteLayer(num); 
}

void LayerSelDlg::entryClicked(int row, int col)
{
 int num = listTable->numRows()-row-1;
 disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(changeToLayer(Layer *)));
 
 emit putLayerNum(doc->getLayerNum()); /*put on undo stack*/
 doc->setActiveLayer(num);

 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(changeToLayer(Layer *))); 
}

int LayerSelDlg::getLayerNum(Layer *layer)
{
 Layer *l;
 for (int i = 0; i<doc->numLayers(); i++)
 {
   l = doc->getLayer(i);
   if (l == layer)
   {
    return (i);
   }
 }
 return (-1);
}


LayerSelSimpleWid::LayerSelSimpleWid(QWidget *parent, const char *name, ApaintDoc *docp):QWidget(parent, name)
{
  Q_CHECK_PTR(docp);
  doc = docp;
  lbLayer = new QLabel(ApaintConfig::getText(730)+":", this);
  sbLayer = new QSpinBox(this, name);
  sbLayer->setMinValue(0);
  sbLayer->setMaxValue(0);
  hbox = new QHBoxLayout(this);
  hbox->addWidget(lbLayer);
  hbox->addWidget(sbLayer);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  connect(sbLayer, SIGNAL(valueChanged(int)), this, SLOT(changeToLayer(int)));
}

LayerSelSimpleWid::~LayerSelSimpleWid()
{
  
}

void LayerSelSimpleWid::layerChangedSlot(Layer *layer)
{
  disconnect(sbLayer, SIGNAL(valueChanged(int)), this, SLOT(changeToLayer(int)));
  sbLayer->setMinValue(1);
  sbLayer->setMaxValue(doc->numLayers());
  sbLayer->setValue(doc->getLayerNum()+1);
  connect(sbLayer, SIGNAL(valueChanged(int)), this, SLOT(changeToLayer(int)));
}

void LayerSelSimpleWid::changeToLayer(int num)
{
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));

  emit putLayerNum(doc->getLayerNum()); /*put on undo stack*/
  doc->setActiveLayer(num-1);

  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

