/***************************************************************************
                          dialogPrint.h  -  description
                             -------------------
    begin                : Wed May 19 2004
    copyright            : (C) 2004-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef DIALOG_PRINT_H
#define DIALOG_PRINT_H

#include "dialog_ui_print.h"
#include "dialog_ui_printerSelect.h"
#include "dialog_ui_showProgress.h"
#include <qlistbox.h>

class ApaintDoc;
class ApaintImage;
class PrintDataProducer;
class PreviewDialog;
class RepeatDimDlg;

class ShowProgressDialog: public ShowProgressDialogUI
{
     Q_OBJECT
public:
  ShowProgressDialog(QWidget *parent, const char *name, QPtrList<PrintDataProducer> &pdpList);
  ~ShowProgressDialog();

private slots:
  void updateProgress(PrintDataProducer *pdp);
  void cancelClicked();
private:
  bool doCancel;
  int allSteps;
    
};

class PrinterSelectDialog: public PrinterSelectDialogUI
{
     Q_OBJECT
public:
   PrinterSelectDialog(QWidget *parent, const char *name, ApaintDoc *pdoc);
  ~PrinterSelectDialog();

  QString getPrinter() {return listPrinter->currentText();}
  QString getPrintMode(){ return listPrintMode->currentText();}
  QString getDensityX() {return listDensityX->currentText();}
  QString getDensityY() {return listDensityY->currentText();}
  int getPrinterIndx() { return printerIndx; }
  int getPrintModeIndx() { return modeIndx; }
  int getDensityXIndx() { return denXIndx; }
  int getDensityYIndx() { return denYIndx; }
  void getPrinterLimitWidMM(int &left, int &wid);
  void getPrinterLimitHeiMM(int &top, int &hei);  
  bool saveToFile();
  bool getFileName(QString& name);
  void setFileName(QString& name);
  void setPrinterName(QString name);
  
signals:
  void dataChanged();
  
protected slots:
  void printerChanged(int i);
  void modeChanged(int i);
  void denXChanged(int i);
  void denYChanged(int i);
  void cancelBtnclicked();
  void okBtnClicked();
  void setPrintToFile(bool isOn);
  
private:
  void showDlg();
  void addPrinters();
  void addPrtModes(int i);
  void addDensityX(int i);
  void addDensityY(int i);
  void updateData();
  
  ApaintDoc *doc;
  int printerIndx;
  int modeIndx;
  int denXIndx;
  int denYIndx;
};

class PrintDialog: public PrintDialogUI
{
   Q_OBJECT
public:
  PrintDialog(QWidget *parent, const char *name, ApaintDoc *pdoc);
  ~PrintDialog();

  void showDlg();

protected slots:
 double getPageLeftMM();
 double getPageUpMM();
 double getPageWidMM();
 double getPageHeiMM();
 void setNumPrints(int val);
 void closeDlg();
 void showSetup();
 void setPrinterData();
 void doPrint();
 void doPreview();
 void setFullPage();
 void fitToPageClicked(bool on);
 void pwidChanged(const QString& val);
 void pheiChanged(const QString& val);
 void zoomChanged(int val);
 void oneRepeatClicked(bool on);
 void calcRepeatInfo();
 
private:
  void setPageSizeMinMax(int wid, int hei);
  void setPageLeftMM(double val);
  void setPageUpMM(double val);
  void setPageWidMM(double val);
  void setPageHeiMM(double val);
  void setupDlg();
  void hideZoom(bool enable);
  void setRepeatInfo(double repWid, double repHei);
  void readPrintcapFile(QStringList& list);
  //pwid and phei are in mm
  void calcPageSize(int pl, int pt, int pwid, int phei, int xdpi, int ydpi, int &lb, int &tb, int &wid, int &hei);
  void calcImageSize(ApaintImage *img, int xdpi, int ydpi, int &pwid, int &phei, int &wid, int &hei);
  int getPixelSize(int mm, int dpi);
  bool getBands(int xdpi, int ydpi, int bandHei, int imgType, QPtrList<PrintDataProducer> &pdpList, int &lb, int &tb);
  void setPrintCap(QString pcap);
  
  double pageLeft, pageTop, pageWidth, pageHeight; //paper size stuff
  int numPrints;
  int printer, printMode, densityX, densityY;
  ApaintDoc *doc;
  PrinterSelectDialog *pselDlg;
  QStringList pcapList;
  PreviewDialog *previewDlg;
};

#endif

