/***************************************************************************
                          repeatdimdlg.cpp  -  description
                             -------------------
    begin                : Wed Jun 4 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogRepeatDimen.h"
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <apaintconfig.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include "apaintUtil.h"

RepeatDimDlg::RepeatDimDlg(QWidget* parent, const char* name):RepeatDimDialogUI(parent, name)
{
 setCaption(ApaintConfig::getText(21));

 ApaintDoubleValidator *validDouble = new ApaintDoubleValidator(1.0, 5000.0, 4, this, "doubleValid");
 
 lbThreads->setText(ApaintConfig::getText(188));
 lbDpi->setText(ApaintConfig::getText(196));
 lbSystem->setText(ApaintConfig::getText(619));
 lbWidth->setText(ApaintConfig::getText(WIDTH_MSG)+":");
 lbHeight->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
 buttonOk->setText(ApaintConfig::getText(OK_MSG));
 buttonCancel->setText(ApaintConfig::getText(CANCEL_MSG));

 leMMWid->setValidator(validDouble);
 leMMHei->setValidator(validDouble);
 leDpiWid->setValidator(validDouble);
 leDpiHei->setValidator(validDouble);
 leThreadWid->setValidator(validDouble);
 leThreadHei->setValidator(validDouble);
   
 connect(buttonOk, SIGNAL(clicked()), this, SLOT(okClicked()));
 connect(buttonCancel, SIGNAL(clicked()), this, SLOT(cancelClicked()));
 connect(leMMWid, SIGNAL(textChanged(const QString&)), this, SLOT(mmWidChanged(const QString&)));
 connect(leMMHei, SIGNAL(textChanged(const QString&)), this, SLOT(mmHeiChanged(const QString&)));
 connect(leDpiWid, SIGNAL(textChanged(const QString&)), this, SLOT(dpiWidChanged(const QString&)));
 connect(leDpiHei, SIGNAL(textChanged(const QString&)), this, SLOT(dpiHeiChanged(const QString&)));
 connect(leThreadWid, SIGNAL(textChanged(const QString&)), this, SLOT(threadWidChanged(const QString&)));
 connect(leThreadHei, SIGNAL(textChanged(const QString&)), this, SLOT(threadHeiChanged(const QString&)));
 connect(sbSystemWid, SIGNAL(valueChanged(int)), this, SLOT(systemWidChanged(int)));
 connect(sbSystemHei, SIGNAL(valueChanged(int)), this, SLOT(systemHeiChanged(int)));
 
 xdpi = xmm = xthreads = 0;
 ydpi = ymm = ythreads = 0;
 xsystem = ysystem = 1;
 inChangeMode = false;
 docLoaded = false;
 calcInteger = true;
}

RepeatDimDlg::~RepeatDimDlg()
{
}

void RepeatDimDlg::mmWidChanged(const QString&)
{   
 if (inChangeMode || !docLoaded) return; 
 xmm = leMMWid->text().toDouble();
 xdpi = (selLayer->layerPos().width()/(xmm*xsystem)) * 25.4;
 xthreads = xdpi / 2.54;
 setVisualData(mmwid);
}

void RepeatDimDlg::mmHeiChanged(const QString&)
{
 if (inChangeMode || !docLoaded) return; 
 ymm = leMMHei->text().toDouble();
 ydpi = (selLayer->layerPos().height()/(ymm*ysystem)) * 25.4; 
 ythreads = ydpi / 2.54;
 setVisualData(mmhei);
}

void RepeatDimDlg::dpiWidChanged(const QString&)
{
  if (inChangeMode || !docLoaded) return;
 
  xdpi = leDpiWid->text().toDouble();
  xmm = (selLayer->layerPos().width()/xdpi) * 25.4 * xsystem; 
  xthreads = xdpi / 2.54; 
  setVisualData(dpiwid); 
}

void RepeatDimDlg::dpiHeiChanged(const QString&)
{
  if (inChangeMode || !docLoaded) return;
  ydpi = leDpiHei->text().toDouble();
  ymm = (selLayer->layerPos().height()/ydpi) * 25.4 * ysystem;
  ythreads = ydpi / 2.54;
  setVisualData(dpihei);
}

void RepeatDimDlg::threadWidChanged(const QString&)
{
  if (inChangeMode || !docLoaded) return;  
  xthreads = leThreadWid->text().toDouble();
  xdpi = xthreads * 2.54;
  xmm = (selLayer->layerPos().width()/xdpi) * 25.4 * xsystem;
  setVisualData(twid);
}

void RepeatDimDlg::threadHeiChanged(const QString&)
{
  if (inChangeMode || !docLoaded) return;
  ythreads = leThreadHei->text().toDouble();
  ydpi = ythreads * 2.54;
  ymm = (selLayer->layerPos().height()/ydpi) * 25.4 *ysystem;
  setVisualData(thei);
}

void RepeatDimDlg::systemWidChanged(int val)
{
  if (inChangeMode || !docLoaded) return;
  xsystem = val;
  dpiWidChanged(leDpiWid->text());
}

void RepeatDimDlg::systemHeiChanged(int val)
{
  if (inChangeMode || !docLoaded) return;
  ysystem = val;
  dpiHeiChanged(leDpiHei->text());
}


void RepeatDimDlg::setVisualData(FldChange fld)
{
  inChangeMode =true;
  if (fld!=mmwid)
   leMMWid->setText(QString("%1").arg(xmm));
  if (fld!=mmhei)
    leMMHei->setText(QString("%1").arg(ymm));
  if (fld!=dpiwid)
    leDpiWid->setText(QString("%1").arg(xdpi));
  if (fld!=dpihei)
    leDpiHei->setText(QString("%1").arg(ydpi));
  if (fld!=twid)
    leThreadWid->setText(QString("%1").arg(xthreads));
  if (fld!=thei)
    leThreadHei->setText(QString("%1").arg(ythreads));

  sbSystemWid->setValue(xsystem);
  sbSystemHei->setValue(ysystem);
  
  inChangeMode =false;
}

void RepeatDimDlg::showDlg()
{
 double fx, fy;
  
 if (!docLoaded) return;
 setVisualData(all);
 int ret = exec();
 if (ret == Accepted)
 {
   getData();
   getScaleFactor(fx, fy, false);
   updateImage();
   emit scaleFactor(fx, fy);
   scaleFactorChanged();
 }
}

void RepeatDimDlg::updateImage()
{
  selLayer->imageBuf.setXdpi((int)(xdpi*1000));
  selLayer->imageBuf.setYdpi((int)(ydpi*1000));
  selLayer->origBuf.setXdpi((int)(xdpi*1000));
  selLayer->origBuf.setYdpi((int)(ydpi*1000));

}

void RepeatDimDlg::getData()
{
  xmm = leMMWid->text().toDouble();
  ymm = leMMHei->text().toDouble();
  xdpi = leDpiWid->text().toDouble();
  ydpi = leDpiHei->text().toDouble();
  xthreads = leThreadWid->text().toDouble();
  ythreads = leThreadHei->text().toDouble();
  xsystem = sbSystemWid->value();
  ysystem = sbSystemHei->value();
}

void RepeatDimDlg::okClicked()
{
 accept();
}

void RepeatDimDlg::cancelClicked()
{
 reject();  
}

void RepeatDimDlg::getSystem(int &wid, int &hei)
{
  wid = xsystem;
  hei = ysystem;
}

void RepeatDimDlg::getDensity(double &wid, double &hei)
{
  wid = xthreads;
  hei = ythreads;
}

void RepeatDimDlg::getScaleFactor(double &facX, double &facY, bool forDrawing)
{
  double vxdpi, vydpi;

  facX = 1;
  facY = 1;

  if (calcInteger)
  {
    if (!xsystem || !ysystem) return;

    vxdpi = xdpi / (double)xsystem;
    vydpi = ydpi / (double)ysystem;

    if (!vydpi || !vxdpi ) return;
    if (vxdpi<vydpi)
      facX*= (vydpi/vxdpi);
    else
      facY*= (vxdpi/vydpi);
  }
  else
  {
    facX = (double)ApaintConfig::getScreenDpiX() / (double)xdpi;
    facY = (double)ApaintConfig::getScreenDpiY() / (double)ydpi;
    if (forDrawing)
    {
      if (facX<1)
      {
        facY = facY/facX;
        facX = 1;
      }
      if (facY<1)
      {
        facX = facX/facY;
        facY = 1;
      }      
    }
  }
}

void RepeatDimDlg::setIntegerView(bool enable)
{
  double fx, fy;
  if (!docLoaded) return;  
  if (calcInteger == enable) return;
  
  calcInteger = enable;
  getScaleFactor(fx, fy, false);
  emit scaleFactor(fx, fy);
  scaleFactorChanged();
}

void RepeatDimDlg::layerChanged(Layer *sel)
{
 double fx, fy;

 if (!sel)
 {
   docLoaded = false;
   qDebug("RepeatDimDlg: sel is null!\n");
   return;   
 }
 if (sel->getType()!=MainLayer) return;
 docLoaded = true;
 selLayer = sel;
 xdpi = (double)sel->imageBuf.xdpi()/1000;
 ydpi = (double)sel->imageBuf.ydpi()/1000;
 if (!xdpi) xdpi = 100;
 if (!ydpi) ydpi = 100;
 if (!xsystem) xsystem = 1;
 if (!ysystem) ysystem = 1;
 xmm = (selLayer->layerPos().width()/xdpi) * 25.4 * xsystem;
 ymm = (selLayer->layerPos().height()/ydpi) * 25.4 * ysystem;
 xthreads = xdpi / 2.54;
 ythreads = ydpi / 2.54;
 setVisualData(all);
 getScaleFactor(fx, fy, false);
 emit scaleFactor(fx, fy);
 scaleFactorChanged();
}

