/***************************************************************************
                          icondataview.h  -  description
                             -------------------
    begin                : Mon Jan 5 2004
    copyright            : (C) 2004-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ICONDATAVIEW_H
#define ICONDATAVIEW_H

#include "qrect.h"
#include "qdir.h"
#include <qptrlist.h>
#include <time.h>
#include <stdlib.h>
#include "apaintimage.h"

class QPixmap;  
  
extern "C"{
#include "../../iolib/acolor.h"   
#include "../../iolib/browseio.h"
}
                
class IconData
{
public:
  IconData();
  IconData(QRect iconPosp, QRect textPosp);
  IconData(QString dirName);
  ~IconData();
  void setIconRect(int x, int y, int w, int h);
  void setTextRect(int x, int y, int w, int h);
  QRect getIconPos() {return iconPos;}
  QRect getTextPos() {return textPos;}

  void setImageInfo(imageInfo *info);
  void setDirInfo(dirBrowse *setDirB);  
  imageInfo *getImageInfo(){ return imgInfo; }
  void fillImgInfo();
  bool isIconLoaded();
  void renderIcon(unsigned char *dst, int bpp, int bpl);
  void setIconSize(int x, int y);
  QString getName();
  QString getSize();
  QString getInfo(bool retTrueColor, bool loaderID);
  QString getDTime();
  bool isDir() {return(type == DirType);}
  bool isImage() {return (type == ImageType);}
  int getXDpi();
  int getYDpi();
  int getImageX();
  int getImageY();
  int getNumColors();
  int getNumDirs();
  int getNumFiles();
  void getNumDirsFiles();

private:
 enum IconType {None=0, ImageType, DirType};
 void init();

 QRect iconPos;
 QRect textPos;
 imageInfo *imgInfo;
 dirBrowse *dirB;
 aColor aCol[256];
 int iconx, icony;
 IconType type;
};


enum SortType{ NameSort=0, SizeSort, SizeXSort, SizeYSort, FileSizeSort, DTimeSort, ColorNumSort, FileTypeSort};

class QPtrListIcon: public QPtrList<imageInfo> {
public:
   QPtrListIcon():selectedSort(NameSort){clear();}
   ~QPtrListIcon(){};

   void setSortType(SortType s) {selectedSort = s;}
protected:
  virtual int compareItems( QPtrCollection::Item s1, QPtrCollection::Item s2 );

private:
  SortType selectedSort;
};


class IconList
{ 
public:
  IconList(int numOfCol, int iSizeX, int iSizeY);
  ~IconList();

  /*it is important that this stays in this order*/
  enum FilterType {None=0, SizeX, SizeY, SizeXY, DaysOld, Colors};
  enum FilterOp {More=0, Less, Equal};

  bool setToNewDir(QString dir, int thei, bool showDirs);
  void setSize(int ncol, int sx, int sy);
  int width(){ return (numCol*(iconX));}
  int height(){ return (numRow*(iconY+textY));}
  int iconNum() {return numIcons;}
  IconData* getIconData(int num);
  int getNumCol() {return numCol;}
  int getNumRow() {return numRow;}
  void setIsBrushList(bool enable);
  void setShowLoaderID(bool enable);
  bool isBrushList() { return swIsBrushList; }
  int getTextHeight(){ return textY;};
  QRect getClearArea(){ return clearArea; } /* what is left to clear */
  int getNewPosByFirstCharacter(int first);
  void setSelectedIcon(int x, int y);
  void setSelectedIcon(int num);
  bool validPos(int x, int y);
  int selectedIcon() {return selIcon;}
  QPixmap* getFolderIcon() {return folderIcon;}
  void sortIcons(SortType styp);
  void setFilter(int nFilter, FilterType typ, FilterOp op, int value);
  void setFilterText(QString cmpStr);
  void useFilter(bool enable);
  QString getTotalFileSize();
  int getNumImages() {return numImages;}
  int getNumIcons() {return numIcons;}
  int getNumDirs() {return numDirs;}
  /*-1 is invalid value*/
  int getIconFromPos(int x, int y);
  bool removeIconAndFile(int num);
  bool renameIconAndFile(int num, QString fname);
  QString getFullName(int num);
  void setIncludeSubDir(bool val) { includeSubDir=val; }
  bool getIncludeSubDir() { return includeSubDir; }

private:

  void fillMapTable();
  void sortImages();
  void calcFilter();
  void clearFilter();
  int iconData2FilterNum(int num);

  static const int filterNum = 4;

  IconData *iList;
  imageInfo *allIconsInfo;
  dirBrowse *dirB;
  directoryInfo *dirI;
  QPtrListIcon iconDataMap;
  int numCol, numRow, iconX, iconY, textY;
  int numIcons, numImages, numDirs;
  bool dirLoaded;
  aColor *aCol;
  bool swIsBrushList;
  int textHeight;
  QRect clearArea;
  int selIcon;
  QPixmap *folderIcon;
  SortType selectedSort;
  FilterType ftyp[filterNum];
  FilterOp   fop[filterNum];
  int fvalue[filterNum];
  QString fStr;
  bool usingFilter;
  QString imageDir;
  bool includeSubDir;
};


#endif
