/***************************************************************************
                          toolBezier.h  -  description
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TOOLBEZIER_H_
#define TOOLBEZIER_H_

#include <toolbase.h>
#include <qpainter.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qptrstack.h>
#include "apaintdoc.h"
#include "apaintdrawrepeat.h"

class BezierToolProp;

class BezierTool : public ToolBase  {
  Q_OBJECT
  BezierToolProp *p;
public:
	BezierTool(DrawInRepeat *drawp, ApaintDoc *doc, BezierToolProp *p, PropObject *prop);
	~BezierTool();

 virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
 void begin();
 void end();
 void afterRefresh(QPainter*);
 virtual void showCommonProp(bool enable);
 void resetTool();
 int controlPoints() const;
 bool goEnterMode();
 bool goEditMode();

public slots:

 void slotUndo();
 void slotRedo();
 void newDocSlot();
 void lineSizeChgSlot(const QPen &pen);

signals:
 void putToStack();
 void clearStacks();
 void repaintSig();

private:
  enum OperationType {
	  NONE,
	  INSERT_NODE,
	  REMOVE_NODE,
	  MOVE_NODE,
	  MOVE_HANDLE,
	  MOVE_SEGMENT
  };

 struct UndoRedoObject
 {
  int state;
  QPointArray points;
  int lineWidth;
  int selectedNode;
  int selectedPoint;
 };

 QPtrStack<UndoRedoObject> undoStack;
 QPtrStack<UndoRedoObject> redoStack;
 bool actionStarted;

  void actionComplete();
  void calcPointClick(QPoint mousePosition);
  QPoint calcPoint(int i, float t);
  void moveSegment(QPoint mousePosition, float t);
  float lineSegmentDistance(QPoint mousePosition, float &t);
  float curveDistance(QPoint mousePosition, float &t);
  float distance(QPoint mousePosition, float &t);
  void insertPoint(QPoint mousePosition, float t);
  void removePoint();
  void updateNode();
  void repaint(QPainter *pc, QPainter *pi);
  void drawPoints(QPainter *pc, const QPointArray &points);

  ApaintDoc *doc;
  QPointArray points;
  QPointArray oldPoints;
  QPoint movingVector;

  int selectedPoint;
  int selectedNode;
  enum State { Enter, Edit, Finish } state;
  DrawInRepeat *draw;
  int lineWidth, oldLineWidth;
  bool isActiveTool;

  friend class BezierToolProp;
};

class BezierToolProp : public QButtonGroup
{
 Q_OBJECT

 BezierTool *bezierTool;
 QRadioButton *enterMode;
 QRadioButton *editMode;

public:

 BezierToolProp(QWidget *parent, const char *name);
 void setTool(BezierTool *tool) { bezierTool = tool; }
 void goEnterMode();
 void goEditMode();

public slots:

 void enterModePressed();
 void editModePressed();
};

#endif /* TOOLBEZIER_H_ */
