/***************************************************************************
                          toolInsRemPixels.h  -  description
                             -------------------
    begin                : Fri Mar 19 2004
    copyright            : (C) 2004-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef TOOLINSREMPIX_H
#define TOOLINSREMPIX_H

#include <toolbase.h>
#include <qradiobutton.h>
#include <qpointarray.h>
#include <qcheckbox.h>
#include "dialog_ui_insertPixels.h"

enum DuplicateType { AREA, LINE, COLOR };

class InsertRemovePixelsDialog:public InsertRemovePixelsDialogUI
{
  Q_OBJECT
public:
  enum OpType {InsertHor, InsertVert, RemoveHor, RemoveVert};
  enum SelectType {PointType, VectorType};
  
  InsertRemovePixelsDialog(QWidget *parent, const char *name);
  ~InsertRemovePixelsDialog(){};

  void showDlg();
  void setPos(int posx, int posy);
  void setNum(int val);
  OpType getOp();
  SelectType getSelType();
  int getPos();
  int getNum();
  bool isBefore() {return rbBefore->isChecked();}
  bool isAfter() {return rbAfter->isChecked();}
  bool isFindPath() {return cbFindPath->isChecked();}
  void setPosLimit(int maxH, int maxV);

signals:
  void okBtnClicked();
  void closeSig();
  void selTypeChange(bool enable);
  void duplicateChanged(bool enabled);
  
protected:
  void closeEvent(QCloseEvent *e);
  
private slots:
  void rbBeforeClicked();
  void rbAfterClicked();
  void closeDlg();
  void typeChanged(int typ);
  void cbUseVectorsClicked(bool set);
  void cbFindPathClicked(bool set);
  void colorClicked();
  void areaClicked();
  void lineClicked();

private:  
  int maxHor, maxVert;
};

class DrawInRepeat;
class ApaintDoc;
class Layer;
class Tile;
class ApaintBrushList;


class InsertRemovePixelsTool: public ToolBase
{
 Q_OBJECT
public:
   InsertRemovePixelsTool(ToolType type, DrawInRepeat *drawp, ApaintDoc *docp, InsertRemovePixelsDialog *dlgp,  PropObject *propObj);
   ~InsertRemovePixelsTool(){};

   virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
   virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
   virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
   virtual void begin();
   virtual void end();
   virtual void beforeRefresh(QPainter *p);
   virtual void afterRefresh(QPainter *p);

signals:
  void putUndo(bool horizontal, int pos, int num, bool doInsert);
  void colorTableChanged(Layer *sel);
  void unselect();
  void updateAreaLineButtons(bool);
      
protected slots:
   void doAction();
   void doActionPoint();
   void doActionVector();
   void layerChangedSlot(Layer *layer);
   void selTypeChanged(bool useVect);
   void duplicateChanged(bool doDuplicate);
private:   
  
   void getLeftRightLen(unsigned char *line, int pos, int max, unsigned char pixel, int *lenLeft, int *lenRight);
   void crackLayer(QPointArray &r1, QPointArray &r2);
   void drawScaleLine(QPainter *p, int x1, int y1, int x2, int y2);
   void drawScreenRect(QPainter *p, int x1, int y1, int x2, int y2);
   void refreshPoly(QPainter *p);
   QRect getBorderRect();
   void getFirstLastPoint(QPoint &fp, QPoint &lp, QPoint &wrapPoint);
   void initData();
   void endCurve();
   uchar *getBrushBuffer(int wid, int hei, int dx, int dy);
   void getMinMaxY(QPointArray &pts, int &min, int &max);
   void getMinMaxX(QPointArray &pts, int &min, int &max);
   
   ApaintDoc *doc;
   int firstTile;
   DrawInRepeat *draw;
   InsertRemovePixelsDialog *dlg;
   bool useBgCol, btnPressed, drawStarted, inFirstTile;
   int firstx, firsty, lastx, lasty;   
   QPtrList<QPoint> ptsList;
   ApaintBrushList *bList;  
   QPainter* p_vector;
};

#endif
