/***************************************************************************
                          toolMagicWand.cpp  -  description
                             -------------------
    begin                : Mon Aug 2 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <apaintconfig.h>
#include "toolMagicWand.h"
#include "apaintdrawrepeat.h"
#include "apaintdoc.h"
#include "apaintUtil.h"
#include "apaint.h"

extern char *getCommonPNG(const char *pixName);

#define MAX 10000   /* max depth of stack */

typedef struct {short y, xl, xr, dy;} Segment;

#define PUSH(Y, XL, XR, DY) if ((Y+(DY)>=0) && (sp<stack+MAX) && (Y+(DY)<=hei)) \
    {sp->y = Y; sp->xl = XL; sp->xr = XR; sp->dy = DY; sp++; } /* push new segment on stack */

#define POP(Y, XL, XR, DY) /* pop segment off stack */ \
    {sp--; Y = sp->y+(DY = sp->dy); XL = sp->xl; XR = sp->xr;}
 
MagicToolProp::MagicToolProp(QWidget *parent, const char *name, SelectProp *selectProp):QWidget(parent, name)
{
  vbox = new QVBoxLayout(this);
  hbox = new QHBoxLayout();
  lbTreshold = new QLabel(ApaintConfig::getText(727)+" ", this);
  hbox->addWidget(lbTreshold);
  sbTreshold = new QSpinBox(0, 128, 1, this);
  sbTreshold->setValue(0);
  hbox->addWidget(sbTreshold);
  QSpacerItem* spacer = new QSpacerItem( 5, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
  hbox->addItem(spacer);
  cbWholeImage = new QCheckBox(ApaintConfig::getText(739), this);
  vbox->addWidget(cbWholeImage);
  eightWay = new QCheckBox(ApaintConfig::getText(740), this);
  vbox->addWidget(eightWay);
  this->selectProp = selectProp;
  vbox->addLayout(hbox);
  connect(sbTreshold, SIGNAL(valueChanged(int)), this, SIGNAL(valueChanged(int)));
  connect(cbWholeImage, SIGNAL(toggled(bool)),  this, SIGNAL(selectWholeImage(bool)));
}

MagicToolProp::~MagicToolProp()
{

}

int MagicToolProp::getTreshold()
{
  return sbTreshold->value();
}

bool MagicToolProp::isEightWayOn() const {
  return eightWay->isOn();
}

void MagicToolProp::show()
{
 selectProp->show();
 QWidget::show();
}

void MagicToolProp::hide()
{
 selectProp->hide();
 QWidget::hide();
}

MagicWandTool::MagicWandTool(DrawInRepeat *drawp, ApaintDoc *docp, ToolType type, PropObject *propObj):ToolBase(type, propObj)
{
  mwProp = 0;
  setToggleBtn(QPixmap(getCommonPNG("select-continuous-area")), tr("MagicWand Tool"));
  clickx = clicky = -1;
  if (propObj)
   {
    setUsepWidget(true);
    mwProp = (MagicToolProp *) propObj->pWidget();
    connect(mwProp, SIGNAL(selectWholeImage(bool)), this, SLOT(enableSelectByColor(bool)));
   }
  Q_CHECK_PTR(drawp);
  draw = drawp;
  Q_CHECK_PTR(docp);
  doc = docp;
  layerCreated = false;
  leftBtnDown = rightBtnDown = false;
  doSelectByColor = false;
  state = DrawState;
  allCur =QCursor(Qt::SizeAllCursor);
  mainCur = QCursor();
  normCur = QCursor();
  
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));  
}

MagicWandTool::~MagicWandTool(){
}

void MagicWandTool::layerChangedSlot(Layer *layer)
{
  beginTool(); 
}


/**
* Enable whole image select.
*/

void MagicWandTool::enableSelectByColor(bool enable)
{
  doSelectByColor = enable;
}


void MagicWandTool::begin()
{
  ToolBase::begin();
  beginTool();
  clickx = clicky = -1;
}

void MagicWandTool::beginTool()
{
 if (doc->getLayerType()==SelectionLayer || doc->getLayerType()==ImageLayer)
  {    
   if (doc->getLayerResizeable())
    state = DrawState;
   else
    state = MoveState;
   layerCreated = true;
  }
 else
  {
   state = DrawState;
   layerCreated = false;
  }
}

void MagicWandTool::end()
{
  ToolBase::end();
  endTool();
}

void MagicWandTool::endTool()
{
  if (layerCreated)
    doc->setLayerResizeable(false);
  emit unsetCursor();
}

void MagicWandTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x = (int)(e->x()/zoomX);
  int y = (int)(e->y()/zoomY);
  lastx = x;
  lasty = y;
  bool inReg = true;
  if (state == MoveState)
    inReg = doc->isLayerPointVisible(x, y);
  if (e->button() == LeftButton && inReg)
   {
    apaint->selectionMoving(true, doc->getLayer());
    leftBtnDown = true;
    if (layerCreated)
     doc->putUndoStack(ChangeLayerPos);
   }
  else if (e->button() == RightButton)
   rightBtnDown = true;
}

void MagicWandTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x = (int)(e->x()/zoomX);
  int y = (int)(e->y()/zoomY);

  if (state == DrawState)
   {
    if (leftBtnDown && layerCreated)
     {
      state = MoveState;
      doc->setLayerResizeable(false);
     }
   }
  else if (state == MoveState)
   {
    if (leftBtnDown)
     {
      int dx, dy;

      dx=lastx-x;
      dy=lasty-y;
      ApaintDocUtil::moveCurLayer(doc, dx, dy, QMAX(2, (int)(zoomX/2)));
      lastx=x;
      lasty=y;
     }    
   }
  setupCursor(x, y);
}

void MagicWandTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x = (int)(e->x()/zoomX);
  int y = (int)(e->y()/zoomY);
    
  if (state == DrawState)
   {
    clickx = x;
    clicky = y;
    if (leftBtnDown)
      createLayer(x, y);
    else if (rightBtnDown)
      ApaintDocUtil::destroyCurLayer(doc);
   }
  else if (state == MoveState)
   {
    bool inReg = doc->isLayerPointVisible(x, y);
    if (rightBtnDown && !inReg)
     ApaintDocUtil::destroyCurLayer(doc);
    else
     apaint->selectionMoving(false, doc->getLayer());
   }
  rightBtnDown = leftBtnDown = false;
}

void MagicWandTool::setupCursor(int x, int y)
{  
  bool inReg = doc->isLayerPointVisible(x, y);
  if (inReg && mainCur.handle() != allCur.handle())
   {
    emit setCursor(allCur);
    mainCur = allCur;
   }
  else if (!inReg && mainCur.handle() != normCur.handle())
   {
    emit unsetCursor();
    mainCur = normCur;
   }
}

void MagicWandTool::keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi)
{
  int dx, dy, inc;
  bool keyOK = false;

  dx = 0; dy =0;
  if (e->state() == ShiftButton)
   inc = 10;
  else
   inc = 1;
  if (e->key() == Qt::Key_Left) {dx+=inc; keyOK = true;}
  if (e->key() == Qt::Key_Right) {dx-=inc; keyOK = true;}
  if (e->key() == Qt::Key_Up) {dy+=inc; keyOK = true;}
  if (e->key() == Qt::Key_Down) {dy-=inc; keyOK = true;}

  if (!keyOK) return;

  /* if we have selection, cut it out and move it right away, so we can keep horizontal and/or vertical position if we use arrow keys */ 
  if ((state!=MoveState) && layerCreated)
   {
    apaint->selectionMoving(true, doc->getLayer());
    if (layerCreated)
     doc->putUndoStack(ChangeLayerPos);
    state=MoveState;
   }

  if (state==MoveState)
   {
     ApaintDocUtil::moveCurLayer(doc, dx, dy, QMAX(2, (int)(zoomX/2)));
     lastx+=dx;
     lasty+=dy;
   }
}

void MagicWandTool::createLayer(int x, int y)
{
  if (doc->getLayerType() == SelectionLayer || doc->getLayerType() == ImageLayer)
   ApaintDocUtil::destroyCurLayer(doc);
   
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  QRect br = selLayer->layerPos();
     
  x-=br.x();
  y-=br.y();
  validMask = QImage(selLayer->imageBuf.width(), selLayer->imageBuf.height(), 1, 2, QImage::LittleEndian);
  validMask.fill(0);
  QRect area;
  if (doSelectByColor)
    area = selectByColor(x, y, selLayer->layerPos(), selLayer->imageBuf);
  else
    area = fillBitArea(x, y, selLayer->layerPos(), selLayer->imageBuf);
  if (area.isValid())
  {
//printf("making the area %dx%d size %dx%d\n", area.x(), area.y(), area.width(), area.height());
/* missing logic for handling magic wand over the border */

   //create Layer
   QImage mask = QImage(area.width(), area.height(), 1, 2, QImage::LittleEndian);
   mask.fill(0);
   copy1BitImage(mask, 0, 0, validMask, area.x(), area.y(), area.width(), area.height(), false, 0, 0);
   doc->createNewLayer(&mask, area);
   doc->putUndoStack(CreateLayer);
   //doc->putUndoStack(ChangeLayerPos);
   doc->refreshLayer();
  }
  validMask = QImage();  
}


bool MagicWandTool::validPoint(int xin, int yin, uchar red, uchar green, uchar blue, int treshold, ApaintImage &img)
{
  int x, y;
  QRect pos = doc->getLayerPos();
  if (!pos.contains(xin+pos.x(), yin+pos.y()))
    return false;

  if (doc->getLayer()->isUsingTiles())
   draw->transPoint(xin, yin, &x, &y);
  else
   {
    x = xin;
    y = yin;
   }
  uint col = img.pixel(x,y);
  int dstn = QABS(qRed(col) - red) + QABS(qGreen(col) - green) + QABS(qBlue(col) - blue);
  return  ((dstn <= treshold) && !PIX_ON(validMask, x, y));
}


void MagicWandTool::setPixCol(int xin, int yin)
{
  int x, y;

  if (doc->getLayer()->isUsingTiles())
   draw->transPoint(xin, yin, &x, &y);
  else
   {
    x = xin;
    y = yin;
   }

  SET_PIX(validMask, x, y); 
}

QRect MagicWandTool::fillBitArea(int x, int y, QRect pos, ApaintImage& img)
{
  int l, x1, x2, dy, wid, hei,xmin,ymin,xmax,ymax, xin, yin;
  Segment stack[MAX], *sp = stack;	/* stack of filled segments */
  uint col=0;
  uchar red, green, blue;
  int treshold;
  int eightDirections = mwProp->isEightWayOn();

   wid =pos.width();
   hei =pos.height(); //must use hei variable becouse of PUSH macro

   xmin= wid;
   ymin= hei;
   xmax=ymax=0;

   if (doc->getLayer()->isUsingTiles())
     draw->transPoint(x, y, &xin, &yin);
   else
   {
     xin = x;
     yin = y;
   }

   col = img.pixel(xin,yin); /* read pv at seed point */
   red = qRed(col);
   green = qGreen(col);
   blue =  qBlue(col);
   treshold = mwProp->getTreshold()*3;
   
   if (x<0 || x>wid || y<0 || y>hei) return QRect();
   PUSH(y, x, x, 1);         /* needed in some cases */
   PUSH(y+1, x, x, -1);      /* seed segment (popped 1st) */

   while (sp>stack)
   {
     /* pop segment off stack and fill a neighboring scan line */
     POP(y, x1, x2, dy);
    /*
     * segment of scan line y-dy for x1<=x<=x2 was previously filled,
     * now explore adjacent pixels in scan line y
     */
    if (y>ymax) ymax=y;
    if (y<ymin) ymin=y;
   
    for (x=x1; x>=0 && validPoint(x,y, red, green, blue, treshold, img); x--)
       setPixCol(x, y);

    if (x<xmin) xmin=x;

    if (x>=x1) goto skip;
    l = x+1;
    if (l<x1+eightDirections) PUSH(y, l-eightDirections, x1-1+eightDirections, -dy);   /* leak on left? */
     x = x1+1;
    do
    {
        for (; x<=wid && validPoint(x,y, red, green, blue, treshold, img); x++)
          setPixCol(x, y);

         if (x>xmax) xmax=x;
          PUSH(y, l-eightDirections, x-1+eightDirections, dy);

         if (x>x2+1-eightDirections) PUSH(y, x2+1-eightDirections, x-1+eightDirections, -dy);   /* leak on right? */
    skip:for (x++; x<=x2 && (!validPoint(x, y, red, green, blue, treshold, img)); x++) ;
         l = x;
    }while (x<=x2);
   }
   xmin=QMAX(0,xmin);
   ymin=QMAX(0,ymin);
   return QRect(xmin, ymin, QMAX(0,xmax-xmin), QMAX(0,ymax-ymin)); 
}


/**
* Select by color. Selection is made with every pixel of selected color.
* @param x selected point x-cord.
* @param y selected point y-cord.
* @param pos Area of selection. Whole image in this case.
* @param img Image where you are making a selection.
*/

QRect MagicWandTool::selectByColor(int x, int y, QRect pos, ApaintImage& img)
{
 int wid, hei, xin, yin;
 uint col=0;
 uchar red, green, blue;
 int treshold;

  wid = pos.width();
  hei = pos.height();

  if (doc->getLayer()->isUsingTiles())
    draw->transPoint(x, y, &xin, &yin);
  else
  {
    xin = x;
    yin = y;
  }

  col = img.pixel(xin,yin);
  red = qRed(col);
  green = qGreen(col);
  blue =  qBlue(col);
  treshold = mwProp->getTreshold()*3;

  if (x < 0 || x > wid || y < 0 || y > hei) return QRect();

  for (y = 0; y < hei; y++)
    for (x=0; x < wid; x++)
      if (validPoint(x, y, red, green, blue, treshold, img))
        setPixCol(x, y);

  return pos;
}
