/***************************************************************************
                          selecttool.cpp  -  description
                             -------------------
    begin                : Fri Apr 18 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpainter.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include "apaintconfig.h"
#include "toolSelect.h"
#include <math.h>
#include "apaintUtil.h"
#include "apaint.h"

extern char *getCommonPNG(const char *pixName);

SelectProp::SelectProp(QWidget *parent, const char *name):QWidget(parent, name)
{
 QGridLayout *gbox = new QGridLayout(this, 5, 5);
 alignLeft =  new QPushButton(QPixmap(getCommonPNG("align-left")), NULL, this);
 alignRight =  new QPushButton(QPixmap(getCommonPNG("align-right")), NULL, this);
 alignTop =  new QPushButton(QPixmap(getCommonPNG("align-top")), NULL, this);
 alignBottom =  new QPushButton(QPixmap(getCommonPNG("align-bottom")), NULL, this);
 alignCenterX =  new QPushButton(QPixmap(getCommonPNG("align-centerX")), NULL, this);
 alignCenterY =  new QPushButton(QPixmap(getCommonPNG("align-centerY")), NULL, this);

 lbPosX = new QLabel(this);
 lbPosY = new QLabel(this);
 lbPosX->setText("X:");
 lbPosY->setText("Y:");
 lbPosX->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 lbPosY->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
 sbPosX = new QSpinBox(this);
 sbPosY = new QSpinBox(this);
 sbPosX->setMaxValue(9999999);
 sbPosX->setMinValue(-9999999);
 sbPosY->setMaxValue(9999999);
 sbPosY->setMinValue(-9999999);
 connect(sbPosX, SIGNAL(valueChanged(int)), this, SLOT(posXChanged(int)));
 connect(sbPosY, SIGNAL(valueChanged(int)), this, SLOT(posYChanged(int)));

 transparentMove = new QCheckBox(ApaintConfig::getText(784), this);
 transparentMove->setChecked(TRUE);
 transparentPaste = new QCheckBox(ApaintConfig::getText(785), this);
 connect(transparentMove, SIGNAL(toggled(bool)), this, SLOT(transparentButtonToggled(bool)));
 connect(transparentPaste, SIGNAL(toggled(bool)), this, SLOT(transparentButtonToggled(bool)));

 connect(alignLeft,  SIGNAL(clicked()), this, SLOT(alignLeftClicked()));
 connect(alignRight,  SIGNAL(clicked()), this, SLOT(alignRightClicked()));
 connect(alignTop,  SIGNAL(clicked()), this, SLOT(alignTopClicked()));
 connect(alignBottom,  SIGNAL(clicked()), this, SLOT(alignBottomClicked()));
 connect(alignCenterX,  SIGNAL(clicked()), this, SLOT(alignCenterXClicked()));
 connect(alignCenterY,  SIGNAL(clicked()), this, SLOT(alignCenterYClicked()));

 gbox->addMultiCellWidget(transparentMove, 0, 0, 0, 3);
 gbox->addMultiCellWidget(transparentPaste, 1, 1, 0, 3);
 gbox->addWidget(alignTop, 0, 4);
 gbox->addWidget(alignCenterY, 1, 4);
 gbox->addWidget(alignBottom, 2, 4);

 gbox->addWidget(alignLeft, 2, 0);
 gbox->addWidget(alignCenterX, 2, 1);
 gbox->addWidget(alignRight, 2, 2);

 gbox->addWidget(lbPosX, 3, 0);
 gbox->addMultiCellWidget(sbPosX, 3, 3, 1, 3);
 gbox->addWidget(lbPosY, 4, 0);
 gbox->addMultiCellWidget(sbPosY, 4, 4, 1, 3);
}

SelectProp::~SelectProp()
{
}

void SelectProp::setPosX(int val)
{
 if ( (val>-MAX_IMAGE) &&(val<MAX_IMAGE)) /* this is just so we don't crash on mainLayerToSelection */
  sbPosX->setValue(val);
}

void SelectProp::setPosY(int val)
{
 if ( (val>-MAX_IMAGE) &&(val<MAX_IMAGE)) /* this is just so we don't crash on mainLayerToSelection */
  sbPosY->setValue(val);
}

bool SelectProp::isTransparentMove() const
{
 return transparentMove->isChecked();
}

bool SelectProp::isTransparentPaste() const
{
 return transparentPaste->isChecked();
}

void SelectProp::setAlignEnabled(bool set)
{
 alignLeft->setEnabled(set);
 alignRight->setEnabled(set);
 alignTop->setEnabled(set);
 alignBottom->setEnabled(set);
 alignCenterX->setEnabled(set);
 alignCenterY->setEnabled(set);
 sbPosX->setEnabled(set);
 sbPosY->setEnabled(set);
}

void SelectProp::transparentButtonToggled(bool)
{
 apaint->transparentButtonToggled();
}

void SelectProp::alignLeftClicked(void)
{
 apaint->alignSelection(ALIGN_LEFT);
}

void SelectProp::alignRightClicked(void)
{
 apaint->alignSelection(ALIGN_RIGHT);
}

void SelectProp::alignTopClicked(void)
{
 apaint->alignSelection(ALIGN_TOP);
}

void SelectProp::alignBottomClicked(void)
{
 apaint->alignSelection(ALIGN_BOTTOM);
}

void SelectProp::alignCenterXClicked(void)
{
 apaint->alignSelection(ALIGN_CENTER_X);
}

void SelectProp::alignCenterYClicked(void)
{
 apaint->alignSelection(ALIGN_CENTER_Y);
}

void SelectProp::posXChanged(int val)
{
 apaint->moveSelectionX(val);
}

void SelectProp::posYChanged(int val)
{
 apaint->moveSelectionY(val);
}


SelectTool::SelectTool(ToolType tooltype, SelectType sel, ApaintDoc *docp, PropObject *propObj):ToolBase(tooltype, propObj)
{
 cur=QCursor(ArrowCursor);
 leftBtnDown = rightBtnDown = false;
 setToggleBtn(QPixmap(getCommonPNG("select-rectangular")), QPixmap(getCommonPNG("select-ellipse")));
 selType=sel;
 doc=docp;
 mpos=OutArea;
 startx=lastx=0;
 starty=lasty=0;
 state=DrawState;
 layerCreated=false;
 if (propObj)
  setUsepWidget(true);
 horCur = QCursor(Qt::SizeHorCursor);
 verCur = QCursor(Qt::SizeVerCursor);
 allCur = QCursor(Qt::SizeAllCursor);
 mainCur = QCursor();
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

SelectTool::~SelectTool()
{
  
}

void SelectTool::layerChangedSlot(Layer *layer)
{
 beginTool();
}

void SelectTool::begin()
{
 ToolBase::begin();
 beginTool();
}

void SelectTool::beginTool()
{
 if (doc->getLayerType()==SelectionLayer || doc->getLayerType()==ImageLayer)
 {
   mpos=OutArea;
   if (doc->getLayerResizeable())
    state = DrawState;
   else
    state = MoveState;
   layerCreated = true;
 }
 else
 {
   mpos=OutArea;
   state = DrawState;
   layerCreated = false;
 } 
}

void SelectTool::end()
{
  ToolBase::end();
  endTool();
  emit unsetCursor();
}

void SelectTool::endTool()
{
 if (layerCreated)
  doc->setLayerResizeable(false);
}

void SelectTool::endSelection()
{
 endTool();
}

void SelectTool::drawScreenRect(QPainter *p, int x1, int y1, int x2, int y2)
{
 p->setRasterOp(Qt::XorROP);
 p->setPen(QPen(QColor(128, 128,0),QMAX(2, (int)(zoomX/2)),Qt::DashLine));
 QRect r = selType == Rectangle ? normalRect(x1, y1, x2, y2) : normalEllipse(x1, y1, x2, y2);

 QRect r1 = QRect(QPoint(r.x()*zoomX, r.y()*zoomY), QSize(r.width()*zoomX, r.height()*zoomY));
 if (selType == Rectangle)
   p->drawRect(r1);
 else if (selType == Circle)
  p->drawEllipse(r1);
 emit showSize(r);
}

void SelectTool::beforeRefresh(QPainter *p)
{
 if (state==DrawState && leftBtnDown && !layerCreated)
  drawScreenRect(p, startx, starty, lastx, lasty);    
}

void SelectTool::afterRefresh(QPainter *p)
{
 if (state==DrawState && leftBtnDown && !layerCreated)
  drawScreenRect(p, startx, starty, lastx, lasty);    
}

void SelectTool::selectOp(int x, int y)
{
  if (!layerCreated)
   {
    emit setCursor(cur);
    mainCur = cur;
    return;
   }
  QCursor curL;
  bool doSetCur;

  bool leftCorner, rightCorner, topCorner, bottomCorner;
  double zx, zy;
  int dx, dy;
  zx = (int) zoomX;
  zy = (int) zoomY;
  dx = dy = 3;
  if (zx>5) dx = 2;
  if (zy>5) dy = 2;
  if (zx>7) dx = 1;
  if (zy>7) dy = 1;
    
  if (zoomX<1)
   dx = QMAX(1,QABS((int)(3*log(zoomX))))*3;
  if (zoomY<1)
   dy = QMAX(1,QABS((int)(3*log(zoomY))))*3;

  if (QABS(starty - lasty) < 2 )
   {
    topCorner    = false;
    bottomCorner = false;
   }
  else
   {
    topCorner    = QABS(y - starty)< dy;
    bottomCorner = QABS(y - lasty) < dy;
   }

  if (QABS(startx - lastx) < 2)
   {
    leftCorner = false;
    rightCorner = false;
   }
  else
   {
    leftCorner   = QABS(x - startx)< dx;
    rightCorner  = QABS(x - lastx) < dx;
   }

  int left=QMIN(startx, lastx);
  int right= QMAX(startx, lastx);
  int top=QMIN(starty, lasty);
  int bottom=QMAX(starty, lasty);

  doSetCur = true;
  
  if (leftCorner && y>=top && y<=bottom && state==DrawState)
  {
    curL = horCur;
    mpos=LBorder;
  }
  else if (rightCorner && y>=top && y<=bottom && state==DrawState)
  {
    curL = horCur;
    mpos=RBorder;
  }
  else if (topCorner && x>=left && x<=right && state==DrawState)
  {
    curL = verCur;
    mpos=TBorder;
  }
  else if (bottomCorner && x>=left && x<=right  && state==DrawState)
  {
    curL = verCur;
    mpos=BBorder;
  }
  else if (doc->getLayerPos().contains(x, y, false))
  {
    curL = allCur;
    mpos=InArea;
  }  
  else
  {
    doSetCur = false;
    mpos=OutArea;
  }
  if (doSetCur)
  {
   if (mainCur.handle() != curL.handle())
   { 
    emit setCursor(curL);
    mainCur = curL;
   }
  }
  else
  {   
    // no need to emit setCursor if cursor is not changed
    if (mainCur.handle() != cur.handle())
    {
      emit setCursor(cur);
      mainCur = cur;
    }
  }
}

void SelectTool::doOp(QPainter *pc, int x, int y)
{
 if (state==DrawState)
  drawRec(pc, x, y);
 else if (state==MoveState && mpos !=OutArea)
  {
   int dx, dy;

   dx=lastx-x;
   dy=lasty-y;
   ApaintDocUtil::moveCurLayer(doc, dx, dy, QMAX(2, (int)(zoomX/2)));
   lastx=x;
   lasty=y;
  }
}

void SelectTool::drawRec(QPainter *pc, int x, int y)
{
 drawScreenRect(pc, startx, starty, lastx, lasty);
 switch (mpos)
 {
  case LBorder:
   startx=x;
   drawScreenRect(pc, startx, starty, lastx, lasty);
   break;
  case RBorder:
   lastx=x;
   drawScreenRect(pc, startx, starty, lastx, lasty);
   break;
  case TBorder:
   starty=y;
   drawScreenRect(pc, startx, starty, lastx, lasty);
   break;
  case BBorder:
   lasty=y;
   drawScreenRect(pc, startx, starty, lastx, lasty);
   break;
  case OutArea:
   drawScreenRect(pc, startx, starty, x, y);
   lastx=x;
   lasty=y;
   break;
  case InArea:
   break;
 }
}

void SelectTool::setArea(bool newArea)
{
 QRect r = selType == Rectangle ?
         normalRect(startx, starty, lastx, lasty) :
         normalEllipse(startx, starty, lastx, lasty);

 if (newArea)
  {
   if (selType == Rectangle)
    doc->createNewLayer(r.x(), r.y(), r.width(), r.height(), SelectionLayer, RectangleG, true);
   else
    doc->createNewLayer(r.x(), r.y(), r.width(), r.height(), SelectionLayer, CircleG, true);

    doc->putUndoStack(CreateLayer);
    doc->putUndoStack(ChangeLayerPos);
  }
 else
 {
  if (doc->getLayerType() == SelectionLayer)
   {
    if (selType == Rectangle)
     {
      doc->getLayer()->setGeometry(RectangleG);
      doc->resizeLayer(r.x(), r.y(), r.width(), r.height());
     }
    else if (doc->getLayer()->getGeometry() == CircleG)
     {
      doc->getLayer()->setGeometry(CircleG);
      doc->resizeLayer(r.x(), r.y(), r.width(), r.height());
     }
   }
 }
}

bool SelectTool::pointInImage(int x, int y)
{
 QRect pos = doc->getMainLayer()->layerPos();
 if (pos.contains(x,y))
  return true;
 else
  return false;
}

QPixmap SelectTool::getToolPix()
{
  return (selType == Circle) ? toolPix2 : toolPix;
}

bool SelectTool::doChangeTool(ToolChangeType)
{
 if (!isActive()) return false;

 if (selType == Rectangle) selType = Circle;
 else selType = Rectangle;

 return true;
}


void SelectTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{ 
 int x=(int)(e->x()/zoomX);
 int y=(int)(e->y()/zoomY); 
 
 if (e->button() == LeftButton && pointInImage(x,y))
   leftBtnDown = true;
 else if (e->button() == RightButton)
   rightBtnDown = true;
   
 if (state==DrawState)
  { 
   if (mpos==InArea)
   {
    startx=lastx=x;
    starty=lasty=y; 
    doc->setLayerResizeable(false);
    state = MoveState;
    apaint->selectionMoving(true, doc->getLayer());
    mpos = InArea;/*becouse beginTool is called*/
   }
   else if (mpos==OutArea) 
   {
    startx=lastx=x;
    starty=lasty=y;
    if (doc->getLayerType() == SelectionLayer && rightBtnDown)
     ApaintDocUtil::destroyCurLayer(doc);
   }
  }
 else if (state==MoveState)
  {
   startx=lastx=x;
   starty=lasty=y;
   if (mpos==OutArea && rightBtnDown)
    ApaintDocUtil::destroyCurLayer(doc);
   else
    if (mpos==InArea)
     {
      apaint->selectionMoving(true, doc->getLayer());
      doc->putUndoStack(ChangeLayerPos);
     }
  }
}

void SelectTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{          
 int x=(int)(e->x()/zoomX);
 int y=(int)(e->y()/zoomY);

 /*selection must be smaler than main image*/
 QRect pos = doc->mainLayer->layerPos();
 if (x<0) x=0;
 if (y<0) y=0;
 if (x>=pos.width()) x = pos.width()-1;
 if (y>=pos.height()) y = pos.height()-1;
 if (leftBtnDown)
  {
   drawPointer(pc, e->x(), e->y());
   doOp(pc, x, y);
   drawPointer(pc, e->x(), e->y());
  }
 else 
  selectOp(x, y);

 drawPointer(pc, e->x(), e->y());
}


void SelectTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{ 
 if (state==DrawState && leftBtnDown)
  setArea(!layerCreated);
 leftBtnDown=rightBtnDown=false;
 apaint->selectionMoving(false, doc->getLayer());
}

void SelectTool::keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi)
{
 int dx, dy, inc;
 bool keyOK = false;

 if (doc->getLayerType() == MainLayer)
   return;

 dx = 0; dy =0;
 if (e->state() == ShiftButton)
  inc = 10;
 else
  inc = 1;
 if (e->key() == Qt::Key_Left) {dx+=inc; keyOK = true;}
 if (e->key() == Qt::Key_Right) {dx-=inc; keyOK = true;}
 if (e->key() == Qt::Key_Up) {dy+=inc; keyOK = true;}
 if (e->key() == Qt::Key_Down) {dy-=inc; keyOK = true;}

 if (!keyOK) return;

 if (state == DrawState)
  {
   doc->setLayerResizeable(false);
   state = MoveState;
  }
 else if (state==MoveState && (doc->getLayerType() != MainLayer))
  {
   ApaintDocUtil::moveCurLayer(doc, dx, dy, QMAX(2, (int)(zoomX/2)));
   lastx+=dx;
   lasty+=dy;
  }
}

UnselectTool::UnselectTool(ToolType typ, ApaintDoc *docp):ToolBase(typ, 0)
{
  Q_CHECK_PTR(docp);
  doc = docp;
  setToggleBtn(QPixmap(getCommonPNG("select-drop")), tr("Unselect tool"));
}

void UnselectTool::doAction()
{
 if (doc->isProtectMode())
  return;
 if (doc->getLayerType() != MainLayer)
  ApaintDocUtil::destroyCurLayer(doc);
}
    
