/***************************************************************************
                          transformtool.cpp  -  description
                             -------------------
    begin                : Wed May 14 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolTransform.h"
#include <qpainter.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <math.h>
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define SQR(x) ((x)*(x))
#define NUM_POINTS    4
#define RECT_SIZE     4

TransformTool::TransformTool(ToolType type, ApaintDoc *docp,PropObject *propObj, ScaleRotDialog *pdiag):ToolBase(type, propObj)
{
 cur=QCursor(CrossCursor);
 selLayer = 0;
 opMode = NormalMode;
 setToggleBtn(QPixmap(getCommonPNG("transform-scale")), tr("Transform Tool"));
 rotBmp = new QPixmap(getCommonPNG("object-rotate-right"));
 rotBmp->setMask(*(QBitmap *)rotBmp);
 rotCur = new QCursor(*rotBmp);
 leftBtnDown=false;
 Q_CHECK_PTR(docp);
 doc=docp;
 op=None;
 beginTransform = false;
 doRotate = doScale = doMove =true;
 if (propObj)
 {
  setUsepWidget(true);
 }
 useMouse = false;
 Q_CHECK_PTR(pdiag);
 srot = pdiag;
 connect(srot, SIGNAL(okClicked()), this, SLOT(dialogTransform()));
 connect(srot, SIGNAL(averageClicked()), this, SLOT(dialogTransform()));
 connect(srot, SIGNAL(contourRotateClicked()), this, SLOT(dialogTransform()));
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

TransformTool::~TransformTool()
{
 delete rotBmp;
 delete rotCur; 
}

void TransformTool::layerChangedSlot(Layer *layer)
{
 if (!beginTransform)
  {
   selLayer = layer;
   return;
  }
  
 if (selLayer!=layer)
  {
   if (doc->isLayerValid(selLayer))
    {
     selLayer->endTransform();
     refreshLayer(selLayer->layerPos());
    }
   selLayer = layer;
   setupTool();
  }
  else
   transformIt(0, 0, FILTER_NONE);
}

void TransformTool::refreshLayer(QRect brd)
{
  brd.setRect(brd.x()-3, brd.y()-3, brd.width()+6, brd.height()+6);
  doc->refreshLayer(brd);
}

void TransformTool::setupTool()
{
 Layer *layer = doc->getLayer();
  
 resetData();  
 if (doc->getLayerType()==MainLayer)
  {
    srot->setResizeContents(true);
    opMode = MainLayerMode;
  }
  else
  {
    srot->setResizeContents(!srot->isRotateMode());
    srot->checkResizeContents(true);
    opMode = NormalMode;
  }
  srot->setSize(oldRect.width(), oldRect.height());
  srot->changeAngle(rad_to_deg(mainAngle));
  if (layer)
  {
    if (layer->imageBuf.type() == LOAD_CMAP)
    {
      bool uc[256];
      int nc;
      layer->imageBuf.getUsedColors(uc, nc, layer->usingMask(), &layer->getMask());
      srot->setRotateContour(nc == 2);
      srot->setAverage(true);
    }
    else if (layer->imageBuf.type() == LOAD_TRUE_COL)
    {
      srot->setAverage(false);
      srot->setRotateContour(false);
    }
  }
  doc->putUndoStack(ChangeLayerForm);
  doc->beginLayerTransform(pts);
  refreshLayer(doc->getLayerPos());
}

void TransformTool::begin()
{
  ToolBase::begin();
  beginTransform = true;
  srot->show();
  srot->adjustSize();
  doScale = srot->isScaleMode();
  doRotate = srot->isRotateMode();    
  setupTool();
}

void TransformTool::end()
{
  ToolBase::end();
  if (!beginTransform) return;
  beginTransform = false;
  if (doc->getLayer()->getGeometry()==CircleG)
   doc->getLayer()->setGeometry(RectangleG);
  doc->endLayerTransform();
  refreshLayer(doc->getLayerPos());
  srot->hide();
  emit unsetCursor(); 
}

void TransformTool::resetData()
{
  oldRect = doc->getLayer()->realLayerPos();
  ocx = cx  =(oldRect.left()+oldRect.right())/2;
  ocy = cy  =(oldRect.top()+oldRect.bottom())/2;
  tx = ty =0;
  setCtrlPoints();  
  scalex=scaley=1;
  mainAngle=0;
  leftBtnDown=false;
  mx.reset();
  imxMain.reset();
  imxScale.reset();
  imxRot.reset();
  imxMove.reset();

}

ResizeFilterType TransformTool::uiToFilterType(int uiNum)
{
 switch (uiNum)
  {
   case 0: return FILTER_NONE;
   case 1: return FILTER_BILINEAR;
   case 2: return FILTER_BICUBIC;
   case 3: return FILTER_SMOOTH2;
   case 4: return FILTER_SMOOTH3;
   case 5: return FILTER_SMOOTH4;
  }
 return FILTER_NONE;
}

void TransformTool::setCtrlPoints()
{
  pts = QPointArray(oldRect);  
  transPts = pts.copy();
  rectPts = pts.copy();
  transRect = oldRect;
}

void TransformTool::showProp()
{
  srot->show();
}


void TransformTool::drawScreenRect(QPainter *p)
{
  if (opMode != NormalMode) return;
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  selLayer->drawBorder(p, pts, zoomX, zoomY);

}

/*unRedo object uses to put data on redo back on undo stack*/
void TransformTool::getScaleRotData(double &sx, double &sy, double &angle, int &dstWid, int &dstHei)
{
  sx = scalex;
  sy = scaley;
  angle = mainAngle;
  QRect pos = doc->getLayer()->realLayerPos();
  dstWid = pos.width();
  dstHei = pos.height();
}

ResizeFilterType TransformTool::getFilterType()
{
 return uiToFilterType(srot->getFilterNum());
}

void TransformTool::dialogTransform()
{
  QSize dstSize;
  double sx, sy;

  srot->getSize(dstSize);

  int srcWid = transPts.point(NE).x() - transPts.point(NW).x()+1;
  int srcHei = transPts.point(SE).y() - transPts.point(NE).y()+1;

  sx =  (double)(dstSize.width()) / (double)(srcWid);
  sy =  (double)(dstSize.height()) / (double)(srcHei);

  emit putUndoStack(scalex, scaley, mainAngle, srcWid, srcHei);

  scaleRotImg(sx, sy, deg_to_rad(srot->getAngle()), dstSize.width(), dstSize.height(), false, getFilterType());
  emit layerSizeChanged(QRect(0, 0, dstSize.width(), dstSize.height()));
}

/*function also called from unRedo object to unredo transformation! uses updateDialog*/
void TransformTool::scaleRotImg(double sx, double sy, double angle,
                                int dstWid, int dstHei, bool updateDialog, ResizeFilterType ftyp)
{
 doc->setLayerValidDraw(false);
 refreshLayer(doc->getLayerPos());

 scalex = sx;
 scaley = sy;
 mainAngle = angle;
 transformPoints();
 rectPts = mxScale.map(transPts);
 transformIt(dstWid, dstHei, ftyp);
 doc->setLayerValidDraw(true);
 refreshLayer(doc->getLayerPos());
 if (updateDialog)
  {
   srot->changeSize(rectPts.point(NE).x()-rectPts.point(NW).x(), rectPts.point(SW).y()-rectPts.point(NW).y());
   srot->changeAngle(rad_to_deg(mainAngle));
  }
}

void TransformTool::transformIt(int dstWid, int dstHei, ResizeFilterType ftyp)
{
 QRect newRect;

 QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
 
 doc->transformLayer(mx, mainAngle, dstWid, dstHei, srot->resizeContents(),
                     srot->doRotateContour(), srot->doAverage(), ftyp);
 newRect = doc->getLayerPos();
 doc->refreshLayer(oldRect, newRect, QMAX(2, (int)(zoomX/2)));
 oldRect = newRect;
 
 QApplication::restoreOverrideCursor();
}

void TransformTool::transformPoints()
{
  mx.reset();
  mxRot.reset();
  mxScale.reset();
  mxMove.reset();

  MatrixUtil::translatemx(&mxMove, tx, ty);

  MatrixUtil::translatemx(&mxScale, -transPts.point(NW).x(), -transPts.point(NW).y()) ;
  MatrixUtil::scalemx(&mxScale, scalex, scaley);
  MatrixUtil::translatemx(&mxScale, rectPts.point(NW).x(), rectPts.point(NW).y());

  MatrixUtil::translatemx(&mxRot, -cx, -cy) ;
  MatrixUtil::rotatemx(&mxRot, mainAngle, doc->getXYratio());
  MatrixUtil::translatemx(&mxRot, cx, cy) ;

  imxScale = (mxRot*mxMove).invert();
  imxRot = mxMove.invert();
  mx = mxScale * mxRot * mxMove;
  imxMain = mx.invert();
  pts = mx.map(transPts);  
}

void TransformTool::getOp(int x, int y)
{
 int ix, iy;
 QCursor cur;
 int lenSize = 10;
 
 if (opMode == MainLayerMode)
  {
   op = RotateWithVector;
   return; 
  }
 else if (opMode == NormalMode)
 {
    if (doScale)
    {
       /*check for resize*/
       pointClick = -1;
       if (zoomX<5)
        lenSize = 4;
       else if (zoomX<10)
        lenSize = 2;
       else
        lenSize = 1;

       for (int i=0; i<NUM_POINTS; i++)
       {
         int len = (int)sqrt(SQR(x-pts.point(i).x())+ SQR(y-pts.point(i).y()));

         if (len<lenSize)
         {
        	 pointClick = i;

        	 break;
         }
         else
         {
        	 int j = (i == NUM_POINTS-1) ? 0 : i+1;
        	 len = (int)sqrt(SQR(x-(pts.point(i).x()+pts.point(j).x())/2)+SQR(y-(pts.point(i).y()+pts.point(j).y())/2));

        	 if (len < lenSize)
			 {
				 pointClick = i+4;

				 break;
			 }
         }
       }



       if (pointClick!=-1 && doScale)
       {
          int angleDeg = (int)rad_to_deg(mainAngle);
          if (angleDeg==360) angleDeg = 0;
          if (angleDeg<0) angleDeg+=360;

          switch (pointClick)
          {
			  case NE: case SW:
				  angleDeg += 45;
				  break;
			  case E: case W:
				  angleDeg += 90;
				  break;
			  case SE: case NW:
				  angleDeg += 135;
				  break;
			  case S: case N:
				  break;
          }

          angleDeg = (angleDeg+22)%180 / 45;

          switch (angleDeg)
          {
			  case 0:
				  cur = QCursor(Qt::SizeVerCursor);
				  break;
			  case 1:
				  cur = QCursor(Qt::SizeBDiagCursor);
				  break;
			  case 2:
				  cur = QCursor(Qt::SizeHorCursor);
				  break;
			  case 3:
				  cur = QCursor(Qt::SizeFDiagCursor);
				  break;
          }

          op = Scale;
          emit setCursor(cur);
          return;
       }
    }
    if (doMove)
    {
      /*check for move*/
      imxMain.map(x, y, &ix, &iy);
      if (transRect.contains(ix, iy))
      {
       op = Move;
       cur = QCursor(Qt::SizeAllCursor);
       emit setCursor(cur);
       return;
      }
    }
    /*else rotate*/
    if (doRotate)
    {
      op = Rotate;
    }
    emit unsetCursor();
 }
}


void TransformTool::showOpMove(QPainter *p, int x, int y)
{
 int diffx, diffy;
 
 diffx = x - lastx;
 diffy = y - lasty; 

 drawScreenRect(p);
 tx+=diffx;
 ty+=diffy;
 transformPoints();
 drawScreenRect(p);
}

void TransformTool::showOpScale(QPainter *p, int x, int y)
{
  int diffx, diffy;
  int ilx, ily, ix, iy;   
  
  if (op!=Scale) return;
  drawScreenRect(p);     
 
  imxScale.map(lastx, lasty, &ilx, &ily);
  imxScale.map(x, y, &ix, &iy);

  diffx = ix - ilx;
  diffy = iy - ily; 

  
  switch (pointClick)
  {
   case NW:
    if (srot->keepAspectRatio())
    {
      rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()+diffx); 
      rectPts.setPoint(NE, rectPts.point(NE).x(),rectPts.point(NE).y()+diffx);
      rectPts.setPoint(SW, rectPts.point(SW).x()+diffx,rectPts.point(SW).y());
    }
    else
    {
      rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()+diffy);
      rectPts.setPoint(NE, rectPts.point(NE).x(),rectPts.point(NE).y()+diffy);
      rectPts.setPoint(SW, rectPts.point(SW).x()+diffx,rectPts.point(SW).y());
    }
   break;
   case NE:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()-diffx);  
     rectPts.setPoint(NW, rectPts.point(NW).x(),rectPts.point(NW).y()-diffx);
     rectPts.setPoint(SE, rectPts.point(SE).x()+diffx,rectPts.point(SE).y());
    }
    else
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()+diffy); 
     rectPts.setPoint(NW, rectPts.point(NW).x(),rectPts.point(NW).y()+diffy);
     rectPts.setPoint(SE, rectPts.point(SE).x()+diffx,rectPts.point(SE).y());
    }
   break;
   case SE:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffy, rectPts.point(pointClick).y()+diffy);   
     rectPts.setPoint(NE, rectPts.point(NE).x()+diffy,rectPts.point(NE).y());
     rectPts.setPoint(SW, rectPts.point(SW).x(),rectPts.point(SW).y()+diffy);
    }
    else
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()+diffy); 
     rectPts.setPoint(NE, rectPts.point(NE).x()+diffx,rectPts.point(NE).y());
     rectPts.setPoint(SW, rectPts.point(SW).x(),rectPts.point(SW).y()+diffy);
    }
   break;
   case SW:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()-diffy, rectPts.point(pointClick).y()+diffy);
     rectPts.setPoint(NW, rectPts.point(NW).x()-diffy,rectPts.point(NW).y());
     rectPts.setPoint(SE, rectPts.point(SE).x(),rectPts.point(SE).y()+diffy);
    }
    else
    {
     rectPts.setPoint(pointClick, rectPts.point(pointClick).x()+diffx, rectPts.point(pointClick).y()+diffy); 
     rectPts.setPoint(NW, rectPts.point(NW).x()+diffx,rectPts.point(NW).y());
     rectPts.setPoint(SE, rectPts.point(SE).x(),rectPts.point(SE).y()+diffy);
    }
   break;
   case N:
    if (srot->keepAspectRatio())
    {
	 rectPts.setPoint(NW, rectPts.point(NW).x(),rectPts.point(NW).y()+diffx);
	 rectPts.setPoint(NE, rectPts.point(NE).x(),rectPts.point(NE).y()+diffx);
    }
    else
    {
	 rectPts.setPoint(NW, rectPts.point(NW).x(),rectPts.point(NW).y()+diffy);
	 rectPts.setPoint(NE, rectPts.point(NE).x(),rectPts.point(NE).y()+diffy);
    }
	break;
   case E:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(NE, rectPts.point(NE).x()+diffx,rectPts.point(NE).y());
     rectPts.setPoint(SE, rectPts.point(SE).x()+diffx,rectPts.point(SE).y());
    }
    else
    {
     rectPts.setPoint(NE, rectPts.point(NE).x()+diffx,rectPts.point(NE).y());
     rectPts.setPoint(SE, rectPts.point(SE).x()+diffx,rectPts.point(SE).y());
    }
   break;
   case S:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(SE, rectPts.point(SE).x(),rectPts.point(SE).y()+diffy);
     rectPts.setPoint(SW, rectPts.point(SW).x(),rectPts.point(SW).y()+diffy);
    }
    else
    {
	 rectPts.setPoint(SE, rectPts.point(SE).x(),rectPts.point(SE).y()+diffy);
	 rectPts.setPoint(SW, rectPts.point(SW).x(),rectPts.point(SW).y()+diffy);
    }
   break;
   case W:
    if (srot->keepAspectRatio())
    {
     rectPts.setPoint(NW, rectPts.point(NW).x()+diffx,rectPts.point(NW).y());
     rectPts.setPoint(SW, rectPts.point(SW).x()+diffx,rectPts.point(SW).y());
    }
    else
    {
	 rectPts.setPoint(NW, rectPts.point(NW).x()+diffx,rectPts.point(NW).y());
	 rectPts.setPoint(SW, rectPts.point(SW).x()+diffx,rectPts.point(SW).y());
    }
   break;
  }

  scalex =  (double)(rectPts.point(NE).x() - rectPts.point(NW).x()+1) /
            (double)(transPts.point(NE).x() - transPts.point(NW).x()+1);
            
  scaley = (double)(rectPts.point(SE).y() - rectPts.point(NE).y()+1) /
           (double)(transPts.point(SE).y() - transPts.point(NE).y()+1);


  transformPoints();  
  drawScreenRect(p);
}

void TransformTool::showOpRotate(QPainter *p, int x, int y)
{
  int ilx, ily, ix, iy;

 if (op==Rotate)
 {
   drawScreenRect(p);

   imxRot.map(lastx, lasty, &ilx, &ily);
   imxRot.map(x, y, &ix, &iy);
   
   int x1 = ix  - cx;
   int x2 = ilx - cx;
   int y1 = cy - iy;
   int y2 = cy - ily;

   /*  find the first angle  */
   double angle1 = atan2 (y1, x1);

   /*  find the angle  */
   double angle2 = atan2 (y2, x2);

   double angle = angle2 - angle1;

   if (angle > M_PI || angle < -M_PI)
     angle = angle2 - ((angle1 < 0) ? 2.0* M_PI + angle1 : angle1 - 2.0*M_PI);

   /*  increment the transform tool's angle  */
   mainAngle += angle;

   /*  limit the angle to between 0 and 360 degrees  */
   if (mainAngle < -M_PI)
     mainAngle = 2.0 * M_PI - mainAngle;
   else if (mainAngle > M_PI)
     mainAngle = mainAngle - 2.0 * M_PI;

   srot->setAngle(rad_to_deg(mainAngle));
   transformPoints();
   drawScreenRect(p);
 }
}


void TransformTool::showOpRotateWithVector(QPainter *p, int x, int y)
{
 int dx, dy;
 double angle;
 
 p->drawLine((int)(startx*zoomX), (int)(starty*zoomY), (int)(lastx*zoomX), (int)(lasty*zoomY));

 dx = startx - x;
 dy = starty - y;
 
 if ((QABS(dx) != QABS(dy)) && (dx != 0) && (dy != 0))
 {
   angle = atan2(dx, dy);
   angle = rad_to_deg(angle);

   if ((angle<-45) && (angle>-135))
    angle+=90.0;
   if ((angle>45) && (angle<135))
    angle-=90.0;
   if ((angle>135) && (angle<225))
    angle-=180.0;
   mainAngle = deg_to_rad(angle);
   srot->setAngle(angle);
   transformPoints();
 }
 else
 {
  srot->setAngle(0);
  mainAngle = 0;
  transformPoints();
 }
 p->drawLine((int)(startx*zoomX), (int)(starty*zoomY), (int)(x*zoomX), (int)(y*zoomY));
}

void TransformTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 leftBtnDown=true;
 startx = lastx = (int)(e->x()/zoomX);
 starty = lasty = (int)(e->y()/zoomY);
 emit putUndoStack(scalex, scaley, mainAngle, oldRect.width(), oldRect.height());
}

void TransformTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 int x=(int)(e->x()/zoomX);
 int y=(int)(e->y()/zoomY);

 pc->setRasterOp(NotROP);  

 if (leftBtnDown)
  {
   if (op==Rotate)
    showOpRotate(pc, x, y);
   else if (op==Scale)
    showOpScale(pc, x, y);
   else if (op == Move)
    showOpMove(pc, x, y);
   else if (op == RotateWithVector)
    showOpRotateWithVector(pc, x, y);
  }
 else
  getOp(x, y);

 lastx=x;
 lasty=y;   
}

void TransformTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{  
 int wid, hei;

 QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
 
 wid = rectPts.point(NE).x()-rectPts.point(NW).x()+1;
 hei = rectPts.point(SW).y()-rectPts.point(NW).y()+1;
 
 leftBtnDown=false; 
 doc->transformLayer(mx, mainAngle, wid, hei, srot->resizeContents(),
                     srot->doRotateContour(), srot->doAverage(), getFilterType());
 
 pc->setRasterOp(NotROP);  
 drawScreenRect(pc);  
 QRect newRect = doc->getLayerPos();
 doc->setLayerValidDraw(false);
 doc->refreshLayer(oldRect, newRect, QMAX(2, (int)(zoomX/2)));
 doc->setLayerValidDraw(true);
 drawScreenRect(pc);
 oldRect = doc->getLayer()->realLayerPos();
 srot->changeSize(wid, hei);
 srot->changeAngle(rad_to_deg(mainAngle));
 emit layerSizeChanged(QRect(0, 0, wid, hei));
 
 QApplication::restoreOverrideCursor();
 
}

