/***************************************************************************
                          transformtool.h  -  description
                             -------------------
    begin                : Wed May 14 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRANSFORMTOOL_H
#define TRANSFORMTOOL_H

#include <toolbase.h>
#include <qdialog.h>
#include "apaintdoc.h"
#include "dialogScaleRotate.h"
#include "widgetPenBrushSet.h"
  
#define rad_to_deg(angle) ((angle) * 360.0 / (2.0 * M_PI))
#define deg_to_rad(angle) ((angle) * 2.0* M_PI / 360.0)

    
class QPixmap;
class QCursor;
  
class TransformTool : public ToolBase  {
  Q_OBJECT
public: 
	TransformTool(ToolType type, ApaintDoc *docp, PropObject *propObj, ScaleRotDialog *pdiag);
	~TransformTool();

 virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
 void end();
 void begin();

 ResizeFilterType getFilterType();
 void scaleRotImg(double sx, double sy, double angle, int dstWid, int dstHei, bool updateDialog, ResizeFilterType ftyp);
 void transformIt(int dstWid, int dstHei, ResizeFilterType ftyp);
 void getScaleRotData(double &sx, double &sy, double &angle, int &dstWid, int &dstHei);
  
signals:
  void putUndoStack(double sx, double sy, double angle, int dstWid, int dstHei);
  void setCursor(QCursor &cur);
  void unsetCursor();
  void layerSizeChanged(QRect size);
  
public slots:
  void dialogTransform();
  void showProp();
  void layerChangedSlot(Layer *layer);
  
private:

 enum TransOper{None, Rotate, Scale, Move, RotateWithVector};
 enum OperationMode{NormalMode, MainLayerMode};
 
 void getOp(int x, int y);
 void showOpScale(QPainter *p, int x, int y);
 void showOpRotate(QPainter *p, int x, int y);
 void showOpRotateWithVector(QPainter *p, int x, int y);
 void showOpMove(QPainter *p, int x, int y);
 
 void drawScreenRect(QPainter *p);
 void resetData();
 void transformPoints();
 void setCtrlPoints();
 void setupTool();
 void refreshLayer(QRect brd);
 ResizeFilterType uiToFilterType(int uiNum);
 
 int lastx, lasty, startx, starty;
 int cx, cy, ocx, ocy;
 int tx, ty; /*coords for translating*/
 bool leftBtnDown, beginTransform;
 double mainAngle;
 double scalex, scaley;
 ApaintDoc *doc;
 TransOper op;
 QRect oldRect, transRect;
 QPointArray pts, transPts, rectPts;
 QWMatrix mx, mxRot, mxScale , mxMove, imxRot, imxMove, imxScale, imxMain;
 int pointClick;
 bool useMouse; /*used to determin if we can use mouse to do transformations*/
 ScaleRotDialog *srot;
 QPixmap *rotBmp;
 QCursor *rotCur;
 Layer *selLayer;
 bool doScale, doRotate, doMove;
 OperationMode opMode;
};

#endif
