/***************************************************************************
                          writetool.cpp  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaint.h"
#include "toolWrite.h"
#include "dialogFontSelect.h"
#include <qpainter.h>
#include <qbitmap.h>
#include <qfontmetrics.h>
#include <qregexp.h>
#include <qstringlist.h>
#include <qptrlist.h>
#include "apaintdrawrepeat.h"
#include "widgetPenBrushSet.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

WriteTool::WriteTool(QWidget *parent, ToolType type, DrawInRepeat *drawp, ApaintDoc *docp, ApaintView *viewp, PenBrushUtil *pbUtilp, PropObject *propObj):ToolBase(type, propObj)
{
 drawStart=false;
 cur=QCursor(CrossCursor);
 setToggleBtn(QPixmap(getCommonPNG("draw-text")), tr("Write Tool"));
 Q_CHECK_PTR(docp);
 doc = docp;
 Q_CHECK_PTR(drawp);
 draw = drawp;
 view = viewp;
 if (propObj)
  setUsepWidget(true);
 fontDlg = new TextFontSelect( parent, "fontDlg", false, 0);
 Q_CHECK_PTR( fontDlg );
 connect(fontDlg, SIGNAL(insertBtnClicked()), this, SLOT(insertSlot()));
 connect(fontDlg, SIGNAL(closeSig()), this, SLOT(closeIt()));
 leftBtnDown = false;
 lastx = lasty =0;
 allCur = QCursor(Qt::SizeAllCursor);
 Q_CHECK_PTR(pbUtilp);
 pbUtil = pbUtilp;
}
WriteTool::~WriteTool()
{
}

void WriteTool::closeIt()
{
  end();
}  

void WriteTool::begin()
{
 beginTool(); 
 drawn = false;
}

void WriteTool::end()
{
  ToolBase::end();
  endTool();
}

void WriteTool::beginTool()
{
  fontDlg->showDlg();
}

void WriteTool::endTool()
{
  fontDlg->hide();
}

void WriteTool::showCommonProp(bool enable)
{
  CommonProp *cProp = CommonProp::getObject();
  cProp->setAspectDrawProp(enable);
}

void WriteTool::insertSlot()
{
  ApaintImage dimg;
  QImage mask;
  int dstx, dsty, ytext, fgc, bgc;
  uint i;
  bool useBg;
  QRect cpyRec;
  QPtrList<QSize> sizeTab;
  QSize *nsize;
  sizeTab.setAutoDelete(true);
  QRgb fgCol, bgCol;
  QString rawName;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  Layer *mainLayer = doc->getMainLayer();
  QStringList text = fontDlg->text();
  if (text.isEmpty()) return;
  QFont font = fontDlg->font();
  rawName = font.rawName();
  QFontMetrics fm(font) ;
  QRect size = fontDlg->textSize(true, sizeTab);
  if (!size.isValid()) return;
  useBg = !fontDlg->isBGTransparent();
  fgc = pbUtil->fgIndex();
  bgc = pbUtil->bgIndex();
  fgCol = pbUtil->foreground();
  bgCol = pbUtil->background();
  
  /*set mask*/
  QBitmap bmp = QBitmap(size.width(), size.height());
  QPainter p(&bmp);
  /*reset first*/
  p.setPen(QPen(Qt::color0, 1));
  p.setBrush(QBrush(Qt::color0, Qt::SolidPattern));
  p.drawRect(0, 0, size.width(), size.height());
  /*for mask draw*/
  p.setPen(QPen(Qt::color1, 1));
  p.setBrush(QBrush(Qt::color1, Qt::SolidPattern));
  /*MAKING MASK*/
  p.setFont(font);
  
  ytext = 0;
  int textWidth = 0;

  for (i=0; i<text.count(); i++)
  {
   int width = sizeTab.at(i)->width();

   if (width > textWidth) textWidth = width;
  }

  for (i=0; i<text.count(); i++)
  {
    QString &txt = text[i];
    nsize = sizeTab.at(i);
    ytext += nsize->height() - fm.descent();
//    if (offset != -1) p.drawText(offset, ytext-fm.descent(), text[i]);
//    else
    if (fontDlg->aligning() != Qt::AlignJustify)
    {
     int offset = 0;

     if (fontDlg->aligning() == Qt::AlignLeft) offset = 0;
     else if (fontDlg->aligning() == Qt::AlignRight) offset = textWidth - nsize->width();
     else if (fontDlg->aligning() == Qt::AlignCenter) offset = (textWidth - nsize->width())/2;

     p.drawText(offset, ytext, txt);
    }
    else
    {
     uint i = 0;
     int count = 0;
     int totalWordsWidth = 0;

     while (i < txt.length())
     {
      while(i<txt.length() && txt[i].isSpace()) i++;

      if (i < txt.length())
      {
       QString word;

       count++;
       do
       {
    	word += txt[i];
    	i++;
       } while (i<txt.length() && txt[i].isSpace() == false);

       totalWordsWidth += fm.width(word);
      }
     }

     int space = textWidth - totalWordsWidth;
     int wordNum = 0;
     int offset = 0;

     i = 0;

     while (i<txt.length())
     {
      while(i<txt.length() && txt[i].isSpace())
       i++;

      if (i < txt.length())
       {
	QString word;

	do
	{
         word += txt[i];
    	 i++;
	} while (i < txt.length() && txt[i].isSpace() == false);

	p.drawText(wordNum == 0 ? 0 : offset+space*wordNum/(count-1), ytext, word);
	wordNum++;
	offset += fm.width(word);
       }
     }
    }
  }
  mask = bmp.convertToImage();
  QImage mask1 = cutCorners(mask, cpyRec);
  
  dimg.newImage((int)(mask1.width()/ax), (int)(mask1.height()/ay), mainLayer->imageBuf.depth());
  dimg.setColorTable(mainLayer->imageBuf.colorTable(), mainLayer->imageBuf.numColors());
  /*fill background of image*/
  if (dimg.type() == LOAD_TRUE_COL)
     dimg.fill(pbUtil->background());
  else if (dimg.type() == LOAD_CMAP)
     dimg.fill(pbUtil->bgIndex());

  /*get position for text layer*/
  if (doc->getLayerPos().contains(lastx, lasty))
   {
    dstx=lastx;
    dsty=lasty;
   }
  else
   {
    dstx = 0;
    dsty = 0;
   }

  if (doc->getLayerType() == MainLayer)
   {
    QRect posView= view->getContentsSize();

    dstx = posView.x();
    dsty = posView.y();
   }

  QRect oldPos = doc->getLayerPos();

  if (drawn)
   {
    if (doc->getLayerType() != MainLayer)
     {
      dstx = oldPos.x();
      dsty = oldPos.y();
     }
    doc->putUndoStack(DestroyLayer);
    doc->deleteLayer();
   }

  /*create layer*/  
  doc->putUndoStack(CreateLayer);
  /*if useBg create new mask in createNewLayer*/
  if (useBg)
    doc->createNewLayer(dimg, 0, dstx, dsty);  
  else
   {
    QImage maskAspect = mask1.scale(dimg.width(), dimg.height());
    doc->createNewLayer(dimg, &mask1, dstx, dsty);
   }
  pbUtil->setBgFgIndxCol(fgc, fgCol, bgc, bgCol);
  QRect pos = doc->getLayerPos();

  if (drawn)
   {
    int xmin = QMIN(pos.x(), oldPos.x());
    int xmax = QMAX(pos.right(), oldPos.right());
    int ymin = QMIN(pos.y(), oldPos.y());
    int ymax = QMAX(pos.bottom(), oldPos.bottom());
    pos = QRect(xmin, ymin, xmax-xmin+1, ymax-ymin+1);
   }
  else
   drawn = true;

  /*draw on canvas*/            
  QPainter pw(doc->getLayer()->getPixmap());
  pw.drawImage(0, 0, mask1);

  /*refresh canvas*/
  draw->refreshDirect(pos.x(), pos.y(), pos.right()+1, pos.bottom()+1, true ,true, true);
  sizeTab.clear();
}

void WriteTool::doOp(int x, int y)
{
  if (!inArea) return;
  int dx, dy;

  dx=lastx-x;
  dy=lasty-y;
  QRect prev=doc->getLayerPos();
  doc->moveLayer(dx, dy);
  QRect after=doc->getLayerPos();
  doc->refreshLayer(prev, after, QMAX(2, (int)(zoomX/2)));
  lastx=x;
  lasty=y;
}

void WriteTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x=(int)(e->x()/zoomX);
  int y=(int)(e->y()/zoomY);

  lastx = x;
  lasty = y;
  if (e->button() == LeftButton)
   {
    leftBtnDown = true;
    if (doc->getLayerType() == SelectionLayer)
       doc->putUndoStack(ChangeLayerPos);
   }
  else if (e->button() == RightButton)
  {
   rightBtnDown = true;
   if (doc->getLayerType() == SelectionLayer)
    {
     if (!doc->layerInBox())
       doc->deleteLayer();
      else
      { 
       doc->putUndoStack(DestroyLayer);
       doc->collapsSelectedLayer(apaint->transparentPaste());
      }
     }
  }
}

void WriteTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x=(int)(e->x()/zoomX);
  int y=(int)(e->y()/zoomY);

  if (doc->getLayerType() != SelectionLayer) return;

  QRect pos = doc->mainLayer->layerPos();
  if (x<0) x=0;
  if (y<0) y=0;
  if (x>pos.width()) x = pos.width()-1;
  if (y>pos.height()) y = pos.height()-1;
  if (leftBtnDown)
    doOp(x, y);
  else
  {
    if (doc->getLayerPos().contains(x, y, false))
    {
     inArea = true;
     emit setCursor(allCur);
    }
    else
    {
     inArea = false;
     emit setCursor(cur);
    }
  }
}

void WriteTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  leftBtnDown = rightBtnDown =false;
}

void WriteTool::mouseDoubleClick(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  if (!fontDlg->isVisible())
   fontDlg->showDlg();
}
