/* pbmplus.c */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "print.h"

#ifdef __MINGW32__
#include <windows.h>
#endif

static FILE *fout;
static char *fname;
static int printModeType;

static int initPrint(int printType, int complexImage, int xpixels,
	int ypixels, int xdpi, int ydpi, int xmargin, int ymargin)
{
 int filedes = -1;

 printModeType = printType;

#ifdef __MINGW32__
 char tempPath[MAX_PATH + 1];

 if (GetTempPathA(MAX_PATH + 1, tempPath) == 0)
  {
   // Failed to get temp path.
   perror("plib: failed to get temp path");
   return (PRT_IO_ERROR);
  }

 // This is allocated in advance because of the code below.
 char tempFilename[MAX_PATH + 1];

 fname = tempFilename;
 if (GetTempFileNameA(tempPath, "prt", 1, fname) == 0)
  {
   perror("plib: failed to create tmpfile");
   return(PRT_IO_ERROR);
  }
 fout = fopen(fname, "wb");
 fname = strdup(fname);
#else
 fname = strdup("/tmp/prtXXXXXX");
 if ((filedes=mkstemp(fname)) < 0 )
  return(PRT_IO_ERROR);
 else
  fout = fdopen(filedes, "wb");
#endif

 switch (printModeType)
  {
   case IMG_BW:
    fprintf(fout, "P4\n");
    break;
   case IMG_TRUE_COL:
    fprintf(fout, "P6\n");
    break;
   default:
    return(PRT_INTERNAL_ERROR);
  }
 fprintf(fout, "#Creator: Arahne, d.o.o.\n");
 fprintf(fout, "%d %d\n", xpixels, ypixels);
 if (printModeType!=IMG_BW)
  fprintf(fout, "%d\n", 255);
 return(PRT_OK);
}

static int printBand(int xpixels, char *data, int bpl, int ypos,
	int lines, char bandType, unsigned char *cmap)
{
 int x,y;

 if (data==NULL)
  return(PRT_INTERNAL_ERROR);
 if (printModeType<bandType)
  return(PRT_WRONG_IMG);

 switch (printModeType)
  {
   case IMG_BW: /* just BW image possible, anyway */
    for (y=0; y<lines; y++)
     fwrite(&data[y*bpl], 1, (xpixels+7)/8, fout);     
    break;
   case IMG_TRUE_COL:
    for (y=0; y<lines; y++)
     {
      switch (bandType)
       {
	case IMG_BW:
         for (x=0; x<xpixels; x++)
          if (data[y*bpl+(x>>3)]&(128>>(x&7)))
           { fputc(0, fout); fputc(0, fout); fputc(0, fout);}
          else
           { fputc(255, fout); fputc(255, fout); fputc(255, fout);}
	break;
        case IMG_CMAP:
         for (x=0; x<xpixels; x++)
          fwrite(&cmap[((unsigned char) data[y*bpl+x])*3], 1, 3, fout);
         break;
        case IMG_TRUE_COL_DITH:
        case IMG_TRUE_COL:
         fwrite(&data[y*bpl], 1, xpixels*3, fout);     
        break;
       }
     }
    break;
   default:
    return(PRT_INTERNAL_ERROR);
  }
 return(PRT_OK);
}

static int endPrint(int printStatus)
{
 if (printStatus==PRT_OK)
  {
   fflush(fout);
   fclose(fout);
   printStatus = enqueuePrintFile(fname);
  }
 else
  fclose(fout);
 /* just cleanup; called even if printing fails */
 removeFileFreeFilename(fname, printStatus);
 return(printStatus);
}

/* fill in the preallocated printer modes */
static void getPrtInfo(printerMode *printerMod, int *numModes,
	printerDensity *prtDensityX, int *numDensitX,
	printerDensity *prtDensityY, int *numDensitY,
	int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
	int *dpiVariable, char **prtName, char **profileName)
{
 int i;

 printerMod[0].printType = IMG_TRUE_COL;
 printerMod[1].printType = IMG_BW;

 *numModes=2;

 *numDensitX = *numDensitY = 6;
 for (i=0; i<6; i++)
  {
   prtDensityX[i].pixels = prtDensityY[i].pixels = MAX_PRINT;
   prtDensityX[i].dpi = prtDensityY[i].dpi = 100000*(i+1);
  }

 *nonprintableTop=0;
 *nonprintableBottom=0;
 *nonprintableLeft=0;
 *dpiVariable=TRUE;
 *prtName="PbmPlus";
 *profileName=NULL;
}

/* register PbmPlus "printer" */
void registerPbmPlus()
{
 registerPrinter(*getPrtInfo, *initPrint, *printBand, *endPrint);
}
